@@ -34,12 +34,12 @@
 ## This needs to be run only when the sortcut keys have been changed
 ##
 
-duplicates_xml="<link linkend=\"GuideImageSearchFindingDuplicates\">Duplicates window</link>"
-search_xml="<link linkend=\"GuideImageSearchSearch\">Search window</link>"
-pan_view_xml="<link linkend=\"GuideOtherWindowsPanView\">Pan view window</link>"
-collections_xml="<link linkend=\"GuideCollections\">Collections window</link>"
-image_xml="<link linkend=\"GuideOtherWindowsImageWindow\">Image view window</link>"
-main_window_xml="<link linkend=\"GuideMainWindow\">Main window</link>"
+duplicates_xml="<emphasis role=\"underline\"><link linkend=\"GuideImageSearchFindingDuplicates\">Duplicates window</link></emphasis>"
+search_xml="<emphasis role=\"underline\"><link linkend=\"GuideImageSearchSearch\">Search window</link></emphasis>"
+pan_view_xml="<emphasis role=\"underline\"><link linkend=\"GuideOtherWindowsPanView\">Pan view window</link></emphasis>"
+collections_xml="<emphasis role=\"underline\"><link linkend=\"GuideCollections\">Collections window</link></emphasis>"
+image_xml="<emphasis role=\"underline\"><link linkend=\"GuideOtherWindowsImageWindow\">Image view window</link></emphasis>"
+main_window_xml="<emphasis role=\"underline\"><link linkend=\"GuideMainWindow\">Main window</link></emphasis>"
 
 pre_1_xml="<table frame=\"all\">\n
 <title>\n
@@ -49,6 +49,8 @@ pre_2_xml="
 keyboard shortcuts\n
 </title>\n
 <tgroup cols=\"2\" rowsep=\"1\" colsep=\"1\">\n
+<colspec colname=\"col1\" colwidth=\"1*\"/>
+<colspec colname=\"col2\" colwidth=\"1*\"/>
 <thead>\n
 <row>\n
 <entry>Shortcut</entry>\n
@@ -78,16 +80,19 @@ post_main_window_xml="
 
 # shellcheck disable=SC2016
 awk_window='BEGIN {
-	{FS=","}
+	LINT = "fatal"
+	FS = ","
+	getline
 	while ($0 !~ /^hard_coded_window_keys/) {getline}
-}
-$0~/{static_cast<GdkModifierType>\(0\), 0/ {exit}
+	}
+
+$0~/\{static_cast<GdkModifierType>\(0\), 0/ {exit}
 {
-gsub(/{static_cast<GdkModifierType>\(0\)/, "", $1);
-gsub(/{static_cast<GdkModifierType>\(GDK_CONTROL_MASK \+ GDK_SHIFT_MASK\)/, "Ctrl + Shift +", $1);
-gsub(/{GDK_CONTROL_MASK/, "Ctrl +", $1);
-gsub(/{GDK_SHIFT_MASK/, "Shift +", $1);
-gsub(/{GDK_MOD1_MASK/, "Alt +", $1);
+gsub(/\{static_cast<GdkModifierType>\(0\)/, "", $1);
+gsub(/\{static_cast<GdkModifierType>\(GDK_CONTROL_MASK \+ GDK_SHIFT_MASK\)/, "Ctrl + Shift +", $1);
+gsub(/\{GDK_CONTROL_MASK/, "Ctrl +", $1);
+gsub(/\{GDK_SHIFT_MASK/, "Shift +", $1);
+gsub(/\{GDK_MOD1_MASK/, "Alt +", $1);
 gsub(/ GDK_KEY_/, "", $2);
 gsub(/\047/, "", $2);
 gsub(/N_\(/, "", $3);
@@ -100,14 +105,16 @@ gsub(/"/, "", $3);
 # This assumes that lines beginning with /^  { "/ are the only ones in layout-util.cc containing key shortcuts
 # shellcheck disable=SC2016
 awk_main_window='BEGIN {
-	{FS=","}
-}
+	LINT = "fatal"
+	FS = ","
+	}
+
 $0 ~ /^  { "/ {
 	if ($4 !~ /nullptr/) {
 		{
 		gsub(/^[[:space:]]+|[[:space:]]+$/,"",$4);
 		gsub(/^[[:space:]]+|[[:space:]]+$/,"",$5);
-		gsub(/{0/, "", $4);
+		gsub(/\{0/, "", $4);
 		gsub(/<control>/, "Ctrl + ", $4);
 		gsub(/<alt>/, "Alt + ", $4);
 		gsub(/<shift>/, "Shift + ", $4);
@@ -127,20 +134,20 @@ $0 ~ /^  { "/ {
 }
 '
 
-keys_xml=$(awk "$awk_window" ./src/dupe.cc )
-printf '%b\n' "$pre_1_xml $duplicates_xml $pre_2_xml $keys_xml $post_xml" > ./doc/docbook/GuideReferenceDuplicatesShortcuts.xml
+keys_xml=$(awk -W posix "$awk_window" ./src/dupe.cc )
+printf '%b\n' "$pre_1_xml $duplicates_xml$pre_2_xml $keys_xml $post_xml" > ./doc/docbook/GuideReferenceDuplicatesShortcuts.xml
 
-keys_xml=$(awk "$awk_window" ./src/search.cc )
-printf '%b\n' "$pre_1_xml $search_xml $pre_2_xml $keys_xml $post_xml" > ./doc/docbook/GuideReferenceSearchShortcuts.xml
+keys_xml=$(awk -W posix "$awk_window" ./src/search.cc )
+printf '%b\n' "$pre_1_xml $search_xml$pre_2_xml $keys_xml $post_xml" > ./doc/docbook/GuideReferenceSearchShortcuts.xml
 
-keys_xml=$(awk "$awk_window" ./src/pan-view/pan-view.cc )
-printf '%b\n' "$pre_1_xml $pan_view_xml $pre_2_xml $keys_xml $post_xml" > ./doc/docbook/GuideReferencePanViewShortcuts.xml
+keys_xml=$(awk -W posix "$awk_window" ./src/pan-view/pan-view.cc )
+printf '%b\n' "$pre_1_xml $pan_view_xml$pre_2_xml $keys_xml $post_xml" > ./doc/docbook/GuideReferencePanViewShortcuts.xml
 
-keys_xml=$(awk "$awk_window" ./src/collect-table.cc)
-printf '%b\n' "$pre_1_xml $collections_xml $pre_2_xml $keys_xml $post_xml" > ./doc/docbook/GuideReferenceCollectionsShortcuts.xml
+keys_xml=$(awk -W posix "$awk_window" ./src/collect-table.cc)
+printf '%b\n' "$pre_1_xml $collections_xml$pre_2_xml $keys_xml $post_xml" > ./doc/docbook/GuideReferenceCollectionsShortcuts.xml
 
-keys_xml=$(awk "$awk_window" ./src/img-view.cc)
-printf '%b\n' "$pre_1_xml $image_xml $pre_2_xml $keys_xml $post_xml" > ./doc/docbook/GuideReferenceImageViewShortcuts.xml
+keys_xml=$(awk -W posix "$awk_window" ./src/img-view.cc)
+printf '%b\n' "$pre_1_xml $image_xml$pre_2_xml $keys_xml $post_xml" > ./doc/docbook/GuideReferenceImageViewShortcuts.xml
 
-keys_xml=$(awk "$awk_main_window" ./src/layout-util.cc)
-printf '%b\n' "$pre_1_xml $main_window_xml $pre_2_xml $keys_xml $post_main_window_xml" > ./doc/docbook/GuideReferenceMainWindowShortcuts.xml
+keys_xml=$(awk -W posix "$awk_main_window" ./src/layout-util.cc)
+printf '%b\n' "$pre_1_xml $main_window_xml$pre_2_xml $keys_xml $post_main_window_xml" > ./doc/docbook/GuideReferenceMainWindowShortcuts.xml
