@@ -8,7 +8,7 @@
 if [ "$ACTION" = "download" ]
 then
 	notify-send --expire-time=4000 "$ARGUMENT"  --icon=geeqie
-	geeqie --remote File:"$PWD/$ARGUMENT"
+	geeqie --remote File="$PWD/$ARGUMENT"
 fi
 
 exit 0
