@@ -40,21 +40,23 @@ export XDG_CACHE_HOME
 export XDG_DATA_HOME
 
 rm --recursive --force build
+tmpdir=$(mktemp -d "${TMPDIR:-/tmp}/geeqie.XXXXXXXXXX")
 
 # Check with all options disabled
 meson setup \
 -Darchive=disabled \
 -Dcms=disabled \
--Ddevel=disabled \
 -Ddoxygen=disabled \
 -Ddjvu=disabled \
 -Devince=disabled \
 -Dexecinfo=disabled \
 -Dexiv2=disabled \
+-Dextended_stacktrace=disabled \
 -Dgit=disabled \
 -Dgps-map=disabled \
 -Dgtk4=disabled \
 -Dheif=disabled \
+-Dhelp_pdf=disabled \
 -Dj2k=disabled \
 -Djpeg=disabled \
 -Djpegxl=disabled \
@@ -64,24 +66,50 @@ meson setup \
 -Dpdf=disabled \
 -Dspell=disabled \
 -Dtiff=disabled \
+-Dunit_tests=disabled \
 -Dvideothumbnailer=disabled \
 -Dwebp=disabled \
 -Dyelp-build=disabled \
 build
 
-meson test -C build
+if meson test -C build
+then
+	options_disabled="PASS"
+else
+	options_disabled="FAIL"
+fi
 
-tmpdir=$(mktemp -d "${TMPDIR:-/tmp}/geeqie.XXXXXXXXXX")
-cp ./build/meson-logs/testlog.txt "$tmpdir/testlog-options-disabled.txt"
+cp ./build/meson-logs/meson-log.txt "$tmpdir/testlog-options-disabled.txt"
+cat ./build/meson-logs/testlog.txt >> "$tmpdir/testlog-options-disabled.txt"
 
 rm --recursive --force build
+meson setup -Dunit_tests=enabled build
+if meson test -C build
+then
+	options_enabled="PASS"
+else
+	options_enabled="FAIL"
+fi
 
-meson setup -Ddevel=enabled build
-
-meson test -C build
-
-cp ./build/meson-logs/testlog.txt "$tmpdir/testlog-options-enabled.txt"
+cp ./build/meson-logs/meson-log.txt "$tmpdir/testlog-options-enabled.txt"
+cat ./build/meson-logs/testlog.txt >> "$tmpdir/testlog-options-enabled.txt"
 
 rm -r "$XDG_CONFIG_HOME"
 rm -r "$XDG_CACHE_HOME"
 rm -r "$XDG_DATA_HOME"
+
+printf "\n"
+if [ "$options_disabled" = "PASS" ]
+then
+	printf "%s \033[1;32m PASS \033[0m\n" "$tmpdir/testlog-options-disabled.txt"
+else
+	printf "%s \033[1;31m FAIL \033[0m\n" "$tmpdir/testlog-options-disabled.txt"
+fi
+
+if [ "$options_enabled" = "PASS" ]
+then
+	printf "%s \033[1;32m PASS \033[0m\n" "$tmpdir/testlog-options-enabled.txt"
+else
+	printf "%s \033[1;31m FAIL \033[0m\n" "$tmpdir/testlog-options-enabled.txt"
+fi
+
