@@ -21,7 +21,9 @@
 
 #include "advanced-exif.h"
 
+#include <array>
 #include <cstring>
+#include <string>
 
 #include <gdk/gdk.h>
 #include <glib-object.h>
@@ -39,21 +41,18 @@
 #include "intl.h"
 #include "layout-util.h"
 #include "layout.h"
+#include "main-defines.h"
 #include "misc.h"
 #include "options.h"
 #include "ui-misc.h"
 #include "window.h"
 
-namespace
-{
-
-constexpr gint ADVANCED_EXIF_DATA_COLUMN_WIDTH = 200;
-
-} // namespace
-
 struct ExifData;
 struct ExifItem;
 
+namespace
+{
+
 /*
  *-------------------------------------------------------------------
  * EXIF window
@@ -62,6 +61,8 @@ struct ExifItem;
 
 struct ExifWin
 {
+	GtkWidget *button_close;
+	GtkWidget *button_help;
 	GtkWidget *window;
 	GtkWidget *vbox;
 	GtkWidget *scrolled;
@@ -82,7 +83,7 @@ enum {
 	EXIF_ADVCOL_COUNT
 };
 
-gint display_order [6] = {
+constexpr gint display_order[6] = {
 	EXIF_ADVCOL_DESCRIPTION,
 	EXIF_ADVCOL_VALUE,
 	EXIF_ADVCOL_NAME,
@@ -91,20 +92,19 @@ gint display_order [6] = {
 	EXIF_ADVCOL_ELEMENTS
 };
 
+constexpr gint ADVANCED_EXIF_DATA_COLUMN_WIDTH = 200;
+
+constexpr std::array<GtkTargetEntry, 1> advanced_exif_drag_types{{
+	{ const_cast<gchar *>("text/plain"), 0, TARGET_TEXT_PLAIN }
+}};
+
+} // namespace
+
 static gboolean advanced_exif_row_enabled(const gchar *name)
 {
-	GList *list;
-
 	if (!name) return FALSE;
 
-	list = history_list_get_by_key("exif_extras");
-	while (list)
-		{
-		if (strcmp(name, static_cast<gchar *>(list->data)) == 0) return TRUE;
-		list = list->next;
-	}
-
-	return FALSE;
+	return g_list_find_custom(history_list_get_by_key("exif_extras"), name, reinterpret_cast<GCompareFunc>(strcmp)) ? TRUE : FALSE;
 }
 
 static void advanced_exif_update(ExifWin *ew)
@@ -135,7 +135,7 @@ static void advanced_exif_update(ExifWin *ew)
 		gchar *text;
 		gchar *utf8_text;
 		const gchar *format;
-		gchar *elements;
+		gint elements;
 		gchar *description;
 
 		tag = g_strdup_printf("0x%04x", exif_item_get_tag_id(item));
@@ -144,7 +144,7 @@ static void advanced_exif_update(ExifWin *ew)
 		text = exif_item_get_data_as_text(item, exif);
 		utf8_text = utf8_validate_or_convert(text);
 		g_free(text);
-		elements = g_strdup_printf("%d", exif_item_get_elements(item));
+		elements = exif_item_get_elements(item);
 		description = exif_item_get_description(item);
 		if (!description || *description == '\0')
 			{
@@ -154,16 +154,16 @@ static void advanced_exif_update(ExifWin *ew)
 
 		gtk_list_store_append(store, &iter);
 		gtk_list_store_set(store, &iter,
-				EXIF_ADVCOL_ENABLED, advanced_exif_row_enabled(tag_name),
-				EXIF_ADVCOL_TAG, tag,
-				EXIF_ADVCOL_NAME, tag_name,
-				EXIF_ADVCOL_VALUE, utf8_text,
-				EXIF_ADVCOL_FORMAT, format,
-				EXIF_ADVCOL_ELEMENTS, elements,
-				EXIF_ADVCOL_DESCRIPTION, description, -1);
+		                   EXIF_ADVCOL_ENABLED, advanced_exif_row_enabled(tag_name),
+		                   EXIF_ADVCOL_TAG, tag,
+		                   EXIF_ADVCOL_NAME, tag_name,
+		                   EXIF_ADVCOL_VALUE, utf8_text,
+		                   EXIF_ADVCOL_FORMAT, format,
+		                   EXIF_ADVCOL_ELEMENTS, std::to_string(elements).c_str(),
+		                   EXIF_ADVCOL_DESCRIPTION, description,
+		                   -1);
 		g_free(tag);
 		g_free(utf8_text);
-		g_free(elements);
 		g_free(description);
 		g_free(tag_name);
 		item = exif_get_next_item(exif_original);
@@ -197,11 +197,6 @@ void advanced_exif_set_fd(GtkWidget *window, FileData *fd)
 	advanced_exif_update(ew);
 }
 
-static GtkTargetEntry advanced_exif_drag_types[] = {
-	{ const_cast<gchar *>("text/plain"), 0, TARGET_TEXT_PLAIN }
-};
-static gint n_exif_drag_types = 1;
-
 
 static void advanced_exif_dnd_get(GtkWidget *listview, GdkDragContext *,
 				  GtkSelectionData *selection_data,
@@ -279,12 +274,10 @@ static void advanced_exif_window_get_geometry(ExifWin *ew)
 	if (!ew || !lw) return;
 
 	window = gtk_widget_get_window(ew->window);
-	gdk_window_get_position(window, &lw->options.advanced_exif_window.x, &lw->options.advanced_exif_window.y);
-	lw->options.advanced_exif_window.w = gdk_window_get_width(window);
-	lw->options.advanced_exif_window.h = gdk_window_get_height(window);
+	lw->options.advanced_exif_window = window_get_position_geometry(window);
 }
 
-void advanced_exif_close(ExifWin *ew)
+static void advanced_exif_close(ExifWin *ew)
 {
 	if (!ew) return;
 
@@ -313,43 +306,10 @@ static gboolean advanced_exif_delete_cb(GtkWidget *, GdkEvent *, gpointer data)
 static gint advanced_exif_sort_cb(GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer data)
 {
 	gint n = GPOINTER_TO_INT(data);
-	gint ret = 0;
 
-	switch (n)
-		{
-		case EXIF_ADVCOL_DESCRIPTION:
-		case EXIF_ADVCOL_VALUE:
-		case EXIF_ADVCOL_NAME:
-		case EXIF_ADVCOL_TAG:
-		case EXIF_ADVCOL_FORMAT:
-		case EXIF_ADVCOL_ELEMENTS:
-			{
-			gchar *s1;
-			gchar *s2;
+	if (n < EXIF_ADVCOL_TAG || n > EXIF_ADVCOL_DESCRIPTION) g_return_val_if_reached(0);
 
-			gtk_tree_model_get(model, a, n, &s1, -1);
-			gtk_tree_model_get(model, b, n, &s2, -1);
-
-			if (!s1 || !s2)
-				{
-			  	if (!s1 && !s2) break;
-			  	ret = s1 ? 1 : -1;
-				}
-			else
-				{
-			  	ret = g_utf8_collate(s1, s2);
-				}
-
-			g_free(s1);
-			g_free(s2);
-			}
-			break;
-
-    		default:
-       			g_return_val_if_reached(0);
-		}
-
-	return ret;
+	return gq_gtk_tree_iter_utf8_collate(model, a, b, n);
 }
 
 #if HAVE_GTK4
@@ -407,6 +367,8 @@ static gboolean advanced_exif_keypress(GtkWidget *, GdkEventKey *event, gpointer
 				advanced_exif_close(ew);
 				stop_signal = TRUE;
 				break;
+			default:
+				break;
 			}
 		} // if (event->state & GDK_CONTROL...
 	if (!stop_signal && is_help_key(event))
@@ -442,14 +404,32 @@ static gboolean search_function_cb(GtkTreeModel *model, gint column, const gchar
 	return ret;
 }
 
+static void exif_window_help_cb(GtkWidget *, gpointer)
+{
+	help_window_show("GuideOtherWindowsExif.html");
+}
+
+static void exif_window_close(ExifWin *ew)
+{
+	gq_gtk_widget_destroy(ew->window);
+}
+
+static void exif_window_close_cb(GtkWidget *, gpointer data)
+{
+	auto ew = static_cast<ExifWin *>(data);
+
+	exif_window_close(ew);
+}
+
 GtkWidget *advanced_exif_new(LayoutWindow *lw)
 {
 	ExifWin *ew;
-	GtkListStore *store;
 	GdkGeometry geometry;
+	GtkListStore *store;
 	GtkTreeSortable *sortable;
 	GtkWidget *box;
-	gint n;
+	GtkWidget *button_box;
+	GtkWidget *hbox;
 
 	ew = g_new0(ExifWin, 1);
 
@@ -462,7 +442,7 @@ GtkWidget *advanced_exif_new(LayoutWindow *lw)
 
 	gtk_window_set_resizable(GTK_WINDOW(ew->window), TRUE);
 
-	gtk_window_resize(GTK_WINDOW(ew->window), lw->options.advanced_exif_window.w, lw->options.advanced_exif_window.h);
+	gtk_window_resize(GTK_WINDOW(ew->window), lw->options.advanced_exif_window.width, lw->options.advanced_exif_window.height);
 	if (lw->options.advanced_exif_window.x != 0 && lw->options.advanced_exif_window.y != 0)
 		{
 		gq_gtk_window_move(GTK_WINDOW(ew->window), lw->options.advanced_exif_window.x, lw->options.advanced_exif_window.y);
@@ -496,12 +476,12 @@ GtkWidget *advanced_exif_new(LayoutWindow *lw)
 
 	/* set up sorting */
 	sortable = GTK_TREE_SORTABLE(store);
-	for (n = EXIF_ADVCOL_DESCRIPTION; n <= EXIF_ADVCOL_ELEMENTS; n++)
+	for (gint n = EXIF_ADVCOL_TAG; n <= EXIF_ADVCOL_DESCRIPTION; n++)
 		gtk_tree_sortable_set_sort_func(sortable, n, advanced_exif_sort_cb,
-				  		GINT_TO_POINTER(n), nullptr);
+		                                GINT_TO_POINTER(n), nullptr);
 
 	/* set initial sort order */
-    	gtk_tree_sortable_set_sort_column_id(sortable, EXIF_ADVCOL_NAME, GTK_SORT_ASCENDING);
+	gtk_tree_sortable_set_sort_column_id(sortable, EXIF_ADVCOL_NAME, GTK_SORT_ASCENDING);
 
 	ew->listview = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
 	g_object_unref(store);
@@ -520,9 +500,9 @@ GtkWidget *advanced_exif_new(LayoutWindow *lw)
 	gtk_tree_view_set_search_equal_func(GTK_TREE_VIEW(ew->listview), search_function_cb, ew, nullptr);
 
 	gtk_drag_source_set(ew->listview,
-			   static_cast<GdkModifierType>(GDK_BUTTON1_MASK | GDK_BUTTON2_MASK),
-			   advanced_exif_drag_types, n_exif_drag_types,
-			   static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
+	                    static_cast<GdkModifierType>(GDK_BUTTON1_MASK | GDK_BUTTON2_MASK),
+	                    advanced_exif_drag_types.data(), advanced_exif_drag_types.size(),
+	                    static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
 
 	g_signal_connect(G_OBJECT(ew->listview), "drag_data_get",
 			 G_CALLBACK(advanced_exif_dnd_get), ew);
@@ -545,6 +525,23 @@ GtkWidget *advanced_exif_new(LayoutWindow *lw)
 	gtk_widget_show(ew->listview);
 	gtk_widget_show(ew->scrolled);
 
+	button_box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+	gq_gtk_box_pack_end(GTK_BOX(ew->vbox), button_box, FALSE, FALSE, 0);
+
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+	gtk_box_set_spacing(GTK_BOX(hbox), PREF_PAD_SPACE);
+	gq_gtk_box_pack_end(GTK_BOX(button_box), hbox, FALSE, FALSE, 0);
+
+	ew->button_help = pref_button_new(hbox, GQ_ICON_HELP, _("Help"), G_CALLBACK(exif_window_help_cb), ew);
+	gtk_widget_set_tooltip_text(GTK_WIDGET(ew->button_help), "F1");
+	gtk_widget_set_sensitive(ew->button_help, TRUE);
+
+	ew->button_close = pref_button_new(hbox, GQ_ICON_CLOSE, _("Close"), G_CALLBACK(exif_window_close_cb), ew);
+	gtk_widget_set_tooltip_text(GTK_WIDGET(ew->button_close), _("Ctrl-W"));
+	gtk_widget_set_sensitive(ew->button_close, TRUE);
+
+	gq_gtk_widget_show_all(button_box);
+
 	gtk_widget_show(ew->window);
 	return ew->window;
 }
