@@ -21,6 +21,7 @@
 
 #include "bar-exif.h"
 
+#include <array>
 #include <cstring>
 
 #include <gdk/gdk.h>
@@ -57,7 +58,20 @@ constexpr gint MIN_HEIGHT = 25;
  *-------------------------------------------------------------------
  */
 
-struct PaneExifData;
+struct PaneExifData
+{
+	PaneData pane;
+	GtkWidget *vbox;
+	GtkWidget *widget;
+	GtkSizeGroup *size_group;
+
+	gint min_height;
+
+	gboolean all_hidden;
+	gboolean show_all;
+
+	FileData *fd;
+};
 
 struct ExifEntry
 {
@@ -75,22 +89,6 @@ struct ExifEntry
 	PaneExifData *ped;
 };
 
-
-struct PaneExifData
-{
-	PaneData pane;
-	GtkWidget *vbox;
-	GtkWidget *widget;
-	GtkSizeGroup *size_group;
-
-	gint min_height;
-
-	gboolean all_hidden;
-	gboolean show_all;
-
-	FileData *fd;
-};
-
 struct ConfDialogData
 {
 	GtkWidget *widget; /* pane or entry, devidet by presenceof "pane_data" or "entry_data" */
@@ -121,7 +119,7 @@ void bar_pane_exif_entry_changed(GtkEntry *, gpointer data)
 	g_free(text);
 }
 
-void bar_pane_exif_entry_destroy(GtkWidget *, gpointer data)
+void bar_pane_exif_entry_destroy(gpointer data)
 {
 	auto ee = static_cast<ExifEntry *>(data);
 
@@ -140,7 +138,7 @@ void bar_pane_exif_setup_entry_box(PaneExifData *ped, ExifEntry *ee)
 		gtk_container_remove(GTK_CONTAINER(gtk_widget_get_parent(ee->box)), ee->box);
 		}
 
-	ee->box = horizontal ? gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0) : gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+	ee->box = gtk_box_new(horizontal ? GTK_ORIENTATION_HORIZONTAL : GTK_ORIENTATION_VERTICAL, 0);
 	gq_gtk_container_add(GTK_WIDGET(ee->ebox), ee->box);
 	gtk_widget_show(ee->box);
 
@@ -191,9 +189,7 @@ GtkWidget *bar_pane_exif_add_entry(PaneExifData *ped, const gchar *key, const gc
 	ee->ped = ped;
 
 	ee->ebox = gtk_event_box_new();
-	g_object_set_data(G_OBJECT(ee->ebox), "entry_data", ee);
-	g_signal_connect_after(G_OBJECT(ee->ebox), "destroy",
-			       G_CALLBACK(bar_pane_exif_entry_destroy), ee);
+	g_object_set_data_full(G_OBJECT(ee->ebox), "entry_data", ee, bar_pane_exif_entry_destroy);
 
 	gq_gtk_box_pack_start(GTK_BOX(ped->vbox), ee->ebox, FALSE, FALSE, 0);
 
@@ -358,19 +354,15 @@ void bar_pane_exif_notify_cb(FileData *fd, NotifyType type, gpointer data)
  *-------------------------------------------------------------------
  */
 
-// @todo Use std::array
-constexpr GtkTargetEntry bar_pane_exif_drag_types[] = {
+constexpr std::array<GtkTargetEntry, 2> bar_pane_exif_drag_types{{
 	{ const_cast<gchar *>(TARGET_APP_EXIF_ENTRY_STRING), GTK_TARGET_SAME_APP, TARGET_APP_EXIF_ENTRY },
 	{ const_cast<gchar *>("text/plain"), 0, TARGET_TEXT_PLAIN }
-};
-constexpr gint n_exif_entry_drag_types = 2;
+}};
 
-// @todo Use std::array
-constexpr GtkTargetEntry bar_pane_exif_drop_types[] = {
+constexpr std::array<GtkTargetEntry, 2> bar_pane_exif_drop_types{{
 	{ const_cast<gchar *>(TARGET_APP_EXIF_ENTRY_STRING), GTK_TARGET_SAME_APP, TARGET_APP_EXIF_ENTRY },
 	{ const_cast<gchar *>("text/plain"), 0, TARGET_TEXT_PLAIN }
-};
-constexpr gint n_exif_entry_drop_types = 2;
+}};
 
 
 void bar_pane_exif_entry_dnd_get(GtkWidget *entry, GdkDragContext *,
@@ -464,8 +456,8 @@ void bar_pane_exif_entry_dnd_init(GtkWidget *entry)
 	auto ee = static_cast<ExifEntry *>(g_object_get_data(G_OBJECT(entry), "entry_data"));
 
 	gtk_drag_source_set(entry, static_cast<GdkModifierType>(GDK_BUTTON1_MASK | GDK_BUTTON2_MASK),
-			    bar_pane_exif_drag_types, n_exif_entry_drag_types,
-			    static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
+	                    bar_pane_exif_drag_types.data(), bar_pane_exif_drag_types.size(),
+	                    static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
 	g_signal_connect(G_OBJECT(entry), "drag_data_get",
 			 G_CALLBACK(bar_pane_exif_entry_dnd_get), ee);
 
@@ -478,9 +470,9 @@ void bar_pane_exif_entry_dnd_init(GtkWidget *entry)
 void bar_pane_exif_dnd_init(GtkWidget *pane)
 {
 	gtk_drag_dest_set(pane,
-			  static_cast<GtkDestDefaults>(GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT | GTK_DEST_DEFAULT_DROP),
-			  bar_pane_exif_drop_types, n_exif_entry_drop_types,
-			  static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE));
+	                  static_cast<GtkDestDefaults>(GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT | GTK_DEST_DEFAULT_DROP),
+	                  bar_pane_exif_drop_types.data(), bar_pane_exif_drop_types.size(),
+	                  static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE));
 	g_signal_connect(G_OBJECT(pane), "drag_data_received",
 			 G_CALLBACK(bar_pane_exif_dnd_receive), NULL);
 }
@@ -535,7 +527,7 @@ void bar_pane_exif_edit_ok_cb(GenericDialog *, gpointer data)
 		g_free(ee->key);
 		ee->key = g_strdup(gq_gtk_entry_get_text(GTK_ENTRY(cdd->key_entry)));
 		title = gq_gtk_entry_get_text(GTK_ENTRY(cdd->title_entry));
-		if (!title || strlen(title) == 0)
+		if (!title || title[0] == '\0')
 			{
 			g_free(ee->title);
 			ee->title = exif_get_description_by_key(ee->key);
@@ -769,7 +761,7 @@ void bar_pane_exif_write_config(GtkWidget *pane, GString *outstr, gint indent)
 	WRITE_NL(); WRITE_STRING("</pane_exif>");
 }
 
-void bar_pane_exif_destroy(GtkWidget *, gpointer data)
+void bar_pane_exif_destroy(gpointer data)
 {
 	auto ped = static_cast<PaneExifData *>(data);
 
@@ -780,18 +772,6 @@ void bar_pane_exif_destroy(GtkWidget *, gpointer data)
 	g_free(ped);
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-void bar_pane_exif_size_request_unused(GtkWidget *, GtkRequisition *requisition, gpointer data)
-{
-	auto *ped = static_cast<PaneExifData *>(data);
-	if (requisition->height < ped->min_height)
-		{
-		requisition->height = ped->min_height;
-		}
-}
-#pragma GCC diagnostic pop
-
 void bar_pane_exif_size_allocate(GtkWidget *, GtkAllocation *alloc, gpointer data)
 {
 	auto ped = static_cast<PaneExifData *>(data);
@@ -821,9 +801,7 @@ GtkWidget *bar_pane_exif_new(const gchar *id, const gchar *title, gboolean expan
 	gtk_widget_show(ped->vbox);
 
 	ped->min_height = MIN_HEIGHT;
-	g_object_set_data(G_OBJECT(ped->widget), "pane_data", ped);
-	g_signal_connect_after(G_OBJECT(ped->widget), "destroy",
-			       G_CALLBACK(bar_pane_exif_destroy), ped);
+	g_object_set_data_full(G_OBJECT(ped->widget), "pane_data", ped, bar_pane_exif_destroy);
 	gtk_widget_set_size_request(ped->widget, -1, ped->min_height);
 	g_signal_connect(G_OBJECT(ped->widget), "size-allocate",
 			 G_CALLBACK(bar_pane_exif_size_allocate), ped);
@@ -876,19 +854,6 @@ GList * bar_pane_exif_list()
 	return exif_list;
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-void bar_pane_exif_close_unused(GtkWidget *widget)
-{
-	PaneExifData *ped;
-
-	ped = static_cast<PaneExifData *>(g_object_get_data(G_OBJECT(widget), "pane_data"));
-	if (!ped) return;
-
-	g_object_unref(ped->vbox);
-}
-#pragma GCC diagnostic pop
-
 GtkWidget *bar_pane_exif_new_from_config(const gchar **attribute_names, const gchar **attribute_values)
 {
 	gchar *title = nullptr;
