@@ -21,6 +21,7 @@
 
 #include "bar-gps.h"
 
+#include <array>
 #include <cstring>
 
 #include <cairo.h>
@@ -40,6 +41,7 @@ G_GNUC_END_IGNORE_DEPRECATIONS
 #include "bar.h"
 #include "compat.h"
 #include "debug.h"
+#include "dnd.h"
 #include "filedata.h"
 #include "intl.h"
 #include "layout.h"
@@ -56,11 +58,11 @@ G_GNUC_END_IGNORE_DEPRECATIONS
 namespace
 {
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(ChamplainMapSource, g_object_unref)
+
 constexpr gint THUMB_SIZE = 100;
 constexpr int DIRECTION_SIZE = 300;
 
-} // namespace
-
 #define MARKER_COLOUR 0x00, 0x00, 0xff, 0xff
 #define TEXT_COLOUR 0x00, 0x00, 0x00, 0xff
 #define THUMB_COLOUR 0xff, 0xff, 0xff, 0xff
@@ -101,28 +103,20 @@ struct PaneGPSData
  * drag-and-drop
  *-------------------------------------------------------------------
  */
-enum {
-	TARGET_APP_COLLECTION_MEMBER,
-	TARGET_APP_EXIF_ENTRY,
-	TARGET_APP_KEYWORD_PATH,
-	TARGET_URI_LIST,
-	TARGET_TEXT_PLAIN
-};
 
-static GtkTargetEntry bar_pane_gps_drop_types[] = {
+constexpr std::array<GtkTargetEntry, 2> bar_pane_gps_drop_types{{
 	{ const_cast<gchar *>("text/uri-list"), 0, TARGET_URI_LIST },
 	{ const_cast<gchar *>("text/plain"), 0, TARGET_TEXT_PLAIN }
-};
-static gint n_gps_entry_drop_types = 2;
+}};
 
-static void bar_pane_gps_close_cancel_cb(GenericDialog *, gpointer data)
+void bar_pane_gps_close_cancel_cb(GenericDialog *, gpointer data)
 {
 	auto pgd = static_cast<PaneGPSData *>(data);
 
 	g_list_free(pgd->geocode_list);
 }
 
-static void bar_pane_gps_close_save_cb(GenericDialog *, gpointer data)
+void bar_pane_gps_close_save_cb(GenericDialog *, gpointer data)
 {
 	auto pgd = static_cast<PaneGPSData *>(data);
 	FileData *fd;
@@ -142,10 +136,10 @@ static void bar_pane_gps_close_save_cb(GenericDialog *, gpointer data)
 	g_list_free(pgd->geocode_list);
 }
 
-static void bar_pane_gps_dnd_receive(GtkWidget *pane, GdkDragContext *,
-									  gint x, gint y,
-									  GtkSelectionData *selection_data, guint info,
-									  guint, gpointer)
+void bar_pane_gps_dnd_receive(GtkWidget *pane, GdkDragContext *,
+                              gint x, gint y,
+                              GtkSelectionData *selection_data, guint info,
+                              guint, gpointer)
 {
 	PaneGPSData *pgd;
 	GenericDialog *gd;
@@ -259,20 +253,20 @@ static void bar_pane_gps_dnd_receive(GtkWidget *pane, GdkDragContext *,
 		}
 }
 
-static void bar_pane_gps_dnd_init(gpointer data)
+void bar_pane_gps_dnd_init(gpointer data)
 {
 	auto pgd = static_cast<PaneGPSData *>(data);
 
 	gtk_drag_dest_set(pgd->widget,
-			  static_cast<GtkDestDefaults>(GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT | GTK_DEST_DEFAULT_DROP),
-			  bar_pane_gps_drop_types, n_gps_entry_drop_types,
-			  static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE));
+	                  static_cast<GtkDestDefaults>(GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT | GTK_DEST_DEFAULT_DROP),
+	                  bar_pane_gps_drop_types.data(), bar_pane_gps_drop_types.size(),
+	                  static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE));
 	g_signal_connect(G_OBJECT(pgd->widget), "drag_data_received",
 			 G_CALLBACK(bar_pane_gps_dnd_receive), NULL);
 
 }
 
-static gboolean bar_gps_draw_direction (ClutterCanvas *, cairo_t *cr, gpointer)
+gboolean bar_gps_draw_direction (ClutterCanvas *, cairo_t *cr, gpointer)
 {
 	cairo_set_source_rgb(cr, 255, 0, 0);
 
@@ -285,7 +279,7 @@ static gboolean bar_gps_draw_direction (ClutterCanvas *, cairo_t *cr, gpointer)
 	return TRUE;
 }
 
-static void bar_pane_gps_thumb_done_cb(ThumbLoader *tl, gpointer data)
+void bar_pane_gps_thumb_done_cb(ThumbLoader *tl, gpointer data)
 {
 	FileData *fd;
 	ClutterActor *marker;
@@ -302,12 +296,12 @@ static void bar_pane_gps_thumb_done_cb(ThumbLoader *tl, gpointer data)
 	thumb_loader_free(tl);
 }
 
-static void bar_pane_gps_thumb_error_cb(ThumbLoader *tl, gpointer)
+void bar_pane_gps_thumb_error_cb(ThumbLoader *tl, gpointer)
 {
 	thumb_loader_free(tl);
 }
 
-static gboolean bar_pane_gps_marker_keypress_cb(GtkWidget *widget, ClutterButtonEvent *bevent, gpointer)
+gboolean bar_pane_gps_marker_keypress_cb(GtkWidget *widget, ClutterButtonEvent *bevent, gpointer)
 {
 	FileData *fd;
 	ClutterActor *label_marker;
@@ -442,7 +436,7 @@ static gboolean bar_pane_gps_marker_keypress_cb(GtkWidget *widget, ClutterButton
 	return TRUE;
 }
 
-static gboolean bar_pane_gps_create_markers_cb(gpointer data)
+gboolean bar_pane_gps_create_markers_cb(gpointer data)
 {
 	auto pgd = static_cast<PaneGPSData *>(data);
 	gdouble latitude;
@@ -462,8 +456,8 @@ static gboolean bar_pane_gps_create_markers_cb(gpointer data)
 							static_cast<gdouble>(pgd->selection_count));
 
 	message = g_string_new("");
-	g_string_printf(message, "%i/%i", (pgd->selection_count - g_list_length(pgd->not_added)),
-																			pgd->selection_count);
+	g_string_printf(message, "%u/%i",
+	                pgd->selection_count - g_list_length(pgd->not_added), pgd->selection_count);
 	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(pgd->progress), message->str);
 	g_string_free(message, TRUE);
 
@@ -538,7 +532,7 @@ static gboolean bar_pane_gps_create_markers_cb(gpointer data)
 	return G_SOURCE_REMOVE;
 }
 
-static void bar_pane_gps_update(PaneGPSData *pgd)
+void bar_pane_gps_update(PaneGPSData *pgd)
 {
 	GList *list;
 
@@ -586,7 +580,7 @@ static void bar_pane_gps_update(PaneGPSData *pgd)
 	pgd->num_added = 0;
 }
 
-static void bar_pane_gps_set_map_source(PaneGPSData *pgd, const gchar *map_id)
+void bar_pane_gps_set_map_source(PaneGPSData *pgd, const gchar *map_id)
 {
 	ChamplainMapSource *map_source;
 	ChamplainMapSourceFactory *map_factory;
@@ -602,7 +596,7 @@ static void bar_pane_gps_set_map_source(PaneGPSData *pgd, const gchar *map_id)
 	g_object_unref(map_factory);
 }
 
-static void bar_pane_gps_enable_markers_checked_toggle_cb(GtkWidget *, gpointer data)
+void bar_pane_gps_enable_markers_checked_toggle_cb(GtkWidget *, gpointer data)
 {
 	auto pgd = static_cast<PaneGPSData *>(data);
 
@@ -616,7 +610,7 @@ static void bar_pane_gps_enable_markers_checked_toggle_cb(GtkWidget *, gpointer
 		}
 }
 
-static void bar_pane_gps_centre_map_checked_toggle_cb(GtkWidget *, gpointer data)
+void bar_pane_gps_centre_map_checked_toggle_cb(GtkWidget *, gpointer data)
 {
 	auto pgd = static_cast<PaneGPSData *>(data);
 
@@ -630,7 +624,7 @@ static void bar_pane_gps_centre_map_checked_toggle_cb(GtkWidget *, gpointer data
 		}
 }
 
-static void bar_pane_gps_change_map_cb(GtkWidget *widget, gpointer data)
+void bar_pane_gps_change_map_cb(GtkWidget *widget, gpointer data)
 {
 	auto pgd = static_cast<PaneGPSData *>(data);
 	gchar *mapsource;
@@ -644,7 +638,7 @@ static void bar_pane_gps_change_map_cb(GtkWidget *widget, gpointer data)
 	bar_pane_gps_set_map_source(pgd, mapsource);
 }
 
-static void bar_pane_gps_notify_selection(GtkWidget *bar, gint count)
+void bar_pane_gps_notify_selection(GtkWidget *bar, gint count)
 {
 	PaneGPSData *pgd;
 
@@ -656,7 +650,7 @@ static void bar_pane_gps_notify_selection(GtkWidget *bar, gint count)
 	bar_pane_gps_update(pgd);
 }
 
-static void bar_pane_gps_set_fd(GtkWidget *bar, FileData *fd)
+void bar_pane_gps_set_fd(GtkWidget *bar, FileData *fd)
 {
 	PaneGPSData *pgd;
 
@@ -669,7 +663,7 @@ static void bar_pane_gps_set_fd(GtkWidget *bar, FileData *fd)
 	bar_pane_gps_update(pgd);
 }
 
-static gint bar_pane_gps_event(GtkWidget *bar, GdkEvent *event)
+gint bar_pane_gps_event(GtkWidget *bar, GdkEvent *event)
 {
 	PaneGPSData *pgd;
 
@@ -681,18 +675,17 @@ static gint bar_pane_gps_event(GtkWidget *bar, GdkEvent *event)
 	return FALSE;
 }
 
-static void bar_pane_gps_write_config(GtkWidget *pane, GString *outstr, gint indent)
+const gchar *bar_pane_gps_get_map_id(const PaneGPSData *pgd)
 {
-	PaneGPSData *pgd;
-	gint zoom;
-	ChamplainMapSource *mapsource;
-	const gchar *map_id;
-	gdouble position;
-	gint int_position;
-	gint w;
-	gint h;
+	g_autoptr(ChamplainMapSource) mapsource = nullptr;
+	g_object_get(pgd->gps_view, "map-source", &mapsource, NULL);
 
-	pgd = static_cast<PaneGPSData *>(g_object_get_data(G_OBJECT(pane), "pane_data"));
+	return champlain_map_source_get_id(mapsource);
+}
+
+void bar_pane_gps_write_config(GtkWidget *pane, GString *outstr, gint indent)
+{
+	auto *pgd = static_cast<PaneGPSData *>(g_object_get_data(G_OBJECT(pane), "pane_data"));
 	if (!pgd) return;
 
 	WRITE_NL();
@@ -701,47 +694,39 @@ static void bar_pane_gps_write_config(GtkWidget *pane, GString *outstr, gint ind
 	write_char_option(outstr, indent, "title", gtk_label_get_text(GTK_LABEL(pgd->pane.title)));
 	WRITE_BOOL(pgd->pane, expanded);
 
-	gtk_widget_get_size_request(GTK_WIDGET(pane), &w, &h);
-	pgd->height = h;
-
+	gint w;
+	gtk_widget_get_size_request(GTK_WIDGET(pane), &w, &pgd->height);
 	WRITE_INT(*pgd, height);
 	indent++;
 
-	g_object_get(G_OBJECT(pgd->gps_view), "map-source", &mapsource, NULL);
-	map_id = champlain_map_source_get_id(mapsource);
+	const gchar *map_id = bar_pane_gps_get_map_id(pgd);
 	WRITE_NL();
 	write_char_option(outstr, indent, "map-id", map_id);
 
-	GString *buffer = g_string_new(nullptr);
-
+	gint zoom;
 	g_object_get(G_OBJECT(pgd->gps_view), "zoom-level", &zoom, NULL);
-	g_string_printf(buffer, "%d", zoom);
 	WRITE_NL();
-	write_char_option(outstr, indent, "zoom-level", buffer->str);
+	write_int_option(outstr, indent, "zoom-level", zoom);
 
-	g_object_get(G_OBJECT(pgd->gps_view), "latitude", &position, NULL);
-	int_position = position * 1000000;
-	g_string_printf(buffer, "%i", int_position);
-	WRITE_NL();
-	write_char_option(outstr, indent, "latitude", buffer->str);
-
-	g_object_get(G_OBJECT(pgd->gps_view), "longitude", &position, NULL);
-	int_position = position * 1000000;
-	g_string_printf(buffer, "%i", int_position);
-	WRITE_NL();
-	write_char_option(outstr, indent, "longitude", buffer->str);
+	const auto write_lat_long_option = [pgd, outstr, indent](const gchar *option)
+	{
+		gdouble position;
+		g_object_get(G_OBJECT(pgd->gps_view), option, &position, NULL);
+		const gint int_position = position * 1000000;
+		WRITE_NL();
+		write_int_option(outstr, indent, option, int_position);
+	};
+	write_lat_long_option("latitude");
+	write_lat_long_option("longitude");
 
 	indent--;
 	WRITE_NL();
 	WRITE_STRING("/>");
-
-	g_string_free(buffer, TRUE);
-	g_object_unref(mapsource);
 }
 
-static void bar_pane_gps_slider_changed_cb(GtkScaleButton *slider,
-					   gdouble zoom,
-					   gpointer data)
+void bar_pane_gps_slider_changed_cb(GtkScaleButton *slider,
+                                    gdouble zoom,
+                                    gpointer data)
 {
 	auto pgd = static_cast<PaneGPSData *>(data);
 	GString *message;
@@ -754,7 +739,7 @@ static void bar_pane_gps_slider_changed_cb(GtkScaleButton *slider,
 	g_string_free(message, TRUE);
 
 }
-static void bar_pane_gps_view_state_changed_cb(ChamplainView *view, GParamSpec *, gpointer data)
+void bar_pane_gps_view_state_changed_cb(ChamplainView *view, GParamSpec *, gpointer data)
 {
 	auto pgd = static_cast<PaneGPSData *>(data);
  	ChamplainState status;
@@ -781,7 +766,7 @@ static void bar_pane_gps_view_state_changed_cb(ChamplainView *view, GParamSpec *
 	g_string_free(message, TRUE);
 }
 
-static void bar_pane_gps_notify_cb(FileData *fd, NotifyType type, gpointer data)
+void bar_pane_gps_notify_cb(FileData *fd, NotifyType type, gpointer data)
 {
 	auto pgd = static_cast<PaneGPSData *>(data);
 
@@ -792,26 +777,12 @@ static void bar_pane_gps_notify_cb(FileData *fd, NotifyType type, gpointer data)
 		}
 }
 
-static const gchar *bar_pane_gps_get_map_id(PaneGPSData *pgd)
-{
-	const gchar *map_id;
-	ChamplainMapSource *mapsource;
-
-	g_object_get(G_OBJECT(pgd->gps_view), "map-source", &mapsource, NULL);
-	map_id = champlain_map_source_get_id(mapsource);
-
-	g_object_unref(mapsource);
-
-	return map_id;
-}
-
-static GtkWidget *bar_pane_gps_menu(PaneGPSData *pgd)
+GtkWidget *bar_pane_gps_menu(PaneGPSData *pgd)
 {
 	GtkWidget *menu;
 	GtkWidget *map_centre;
 	ChamplainMapSourceFactory *map_factory;
 	GSList *map_list;
-	ChamplainMapSourceDesc *map_desc;
 	const gchar *current;
 
 	menu = popup_menu_short_lived();
@@ -820,18 +791,18 @@ static GtkWidget *bar_pane_gps_menu(PaneGPSData *pgd)
 	map_list = champlain_map_source_factory_get_registered(map_factory);
 	current = bar_pane_gps_get_map_id(pgd);
 
-	while (map_list)
+	for (GSList *work = map_list; work; work = work->next)
 		{
-		map_desc = static_cast<ChamplainMapSourceDesc *>(map_list->data);
+		auto *map_desc = static_cast<ChamplainMapSourceDesc *>(work->data);
+		const gchar *map_desc_id = champlain_map_source_desc_get_id(map_desc);
 
 		menu_item_add_radio(menu,
 		                    champlain_map_source_desc_get_name(map_desc),
-		                    const_cast<gchar *>(champlain_map_source_desc_get_id(map_desc)),
-		                    strcmp(champlain_map_source_desc_get_id(map_desc), current) == 0,
+		                    const_cast<gchar *>(map_desc_id),
+		                    strcmp(map_desc_id, current) == 0,
 		                    G_CALLBACK(bar_pane_gps_change_map_cb), pgd);
-
-		map_list = g_slist_next(map_list);
 		}
+	g_slist_free(map_list);
 
 	menu_item_add_divider(menu);
 	menu_item_add_check(menu, _("Enable markers"), pgd->enable_markers_checked,
@@ -843,7 +814,6 @@ static GtkWidget *bar_pane_gps_menu(PaneGPSData *pgd)
 		gtk_widget_set_sensitive(map_centre, FALSE);
 		}
 
-	g_slist_free(map_list);
 	g_object_unref(map_factory);
 
 	return menu;
@@ -851,7 +821,7 @@ static GtkWidget *bar_pane_gps_menu(PaneGPSData *pgd)
 
 /* Determine if the map is to be re-centred on the marker when another photo is selected
  */
-static void bar_pane_gps_map_centreing(PaneGPSData *pgd)
+void bar_pane_gps_map_centreing(PaneGPSData *pgd)
 {
 	GenericDialog *gd;
 	GString *message = g_string_new("");
@@ -878,13 +848,13 @@ static void bar_pane_gps_map_centreing(PaneGPSData *pgd)
 }
 
 #if HAVE_GTK4
-static gboolean bar_pane_gps_map_keypress_cb(GtkWidget *, GdkEventButton *bevent, gpointer data)
+gboolean bar_pane_gps_map_keypress_cb(GtkWidget *, GdkEventButton *bevent, gpointer data)
 {
 /* @FIXME GTK4 stub */
 	return FALSE;
 }
 #else
-static gboolean bar_pane_gps_map_keypress_cb(GtkWidget *, GdkEventButton *bevent, gpointer data)
+gboolean bar_pane_gps_map_keypress_cb(GtkWidget *, GdkEventButton *bevent, gpointer data)
 {
 	auto pgd = static_cast<PaneGPSData *>(data);
 	GtkWidget *menu;
@@ -907,11 +877,9 @@ static gboolean bar_pane_gps_map_keypress_cb(GtkWidget *, GdkEventButton *bevent
 	if (bevent->button == MOUSE_BUTTON_LEFT)
 		{
 		clipboard = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
-		geo_coords = g_strdup_printf("%lf %lf",
-							champlain_view_y_to_latitude(
-								CHAMPLAIN_VIEW(pgd->gps_view),bevent->y),
-							champlain_view_x_to_longitude(
-								CHAMPLAIN_VIEW(pgd->gps_view),bevent->x));
+		geo_coords = g_strdup_printf("%f %f",
+		                             champlain_view_y_to_latitude(CHAMPLAIN_VIEW(pgd->gps_view),bevent->y),
+		                             champlain_view_x_to_longitude(CHAMPLAIN_VIEW(pgd->gps_view),bevent->x));
 		gtk_clipboard_set_text(clipboard, geo_coords, -1);
 
 		g_free(geo_coords);
@@ -923,7 +891,7 @@ static gboolean bar_pane_gps_map_keypress_cb(GtkWidget *, GdkEventButton *bevent
 }
 #endif
 
-static void bar_pane_gps_destroy(GtkWidget *, gpointer data)
+void bar_pane_gps_destroy(gpointer data)
 {
 	auto pgd = static_cast<PaneGPSData *>(data);
 
@@ -941,6 +909,7 @@ static void bar_pane_gps_destroy(GtkWidget *, gpointer data)
 	g_free(pgd);
 }
 
+} // namespace
 
 GtkWidget *bar_pane_gps_new(const gchar *id, const gchar *title, const gchar *map_id,
          					const gint zoom, const gdouble latitude, const gdouble longitude,
@@ -979,7 +948,7 @@ GtkWidget *bar_pane_gps_new(const gchar *id, const gchar *title, const gchar *ma
 	view = gtk_champlain_embed_get_view(GTK_CHAMPLAIN_EMBED(gpswidget));
 
 	gq_gtk_box_pack_start(GTK_BOX(vbox), gpswidget, TRUE, TRUE, 0);
-	gtk_container_add(GTK_CONTAINER(frame), vbox);
+	gq_gtk_container_add(GTK_WIDGET(frame), vbox);
 
 	status = gtk_box_new(GTK_ORIENTATION_HORIZONTAL,0);
 #if HAVE_GTK4
@@ -1026,8 +995,7 @@ GtkWidget *bar_pane_gps_new(const gchar *id, const gchar *title, const gchar *ma
 				     NULL);
 	champlain_view_center_on(view, latitude, longitude);
 	pgd->centre_map_checked = TRUE;
-	g_object_set_data(G_OBJECT(pgd->widget), "pane_data", pgd);
-	g_signal_connect(G_OBJECT(pgd->widget), "destroy", G_CALLBACK(bar_pane_gps_destroy), pgd);
+	g_object_set_data_full(G_OBJECT(pgd->widget), "pane_data", pgd, bar_pane_gps_destroy);
 
 	gq_gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
 
