@@ -21,6 +21,7 @@
 
 #include "bar-keywords.h"
 
+#include <array>
 #include <cstdio>
 #include <cstring>
 
@@ -54,7 +55,7 @@ GtkListStore *keyword_store = nullptr;
 void bar_pane_keywords_changed(GtkTextBuffer *buffer, gpointer data);
 
 void autocomplete_keywords_list_load(const gchar *path);
-gboolean autocomplete_keywords_list_save(gchar *path);
+gboolean autocomplete_keywords_list_save(const gchar *path);
 gboolean autocomplete_activate_cb(GtkWidget *widget, gpointer data);
 
 /*
@@ -420,6 +421,8 @@ void bar_pane_keywords_filter_modify(GtkTreeModel *model, GtkTreeIter *iter, GVa
 		case FILTER_KEYWORD_COLUMN_IS_KEYWORD:
 			gtk_tree_model_get_value(keyword_tree, &child_iter, KEYWORD_COLUMN_IS_KEYWORD, value);
 			break;
+		default:
+			break;
 		}
 }
 
@@ -521,20 +524,15 @@ void bar_pane_keywords_changed(GtkTextBuffer *, gpointer data)
  *-------------------------------------------------------------------
  */
 
-// @todo Use std::array
-constexpr GtkTargetEntry bar_pane_keywords_drag_types[] = {
+constexpr std::array<GtkTargetEntry, 2> bar_pane_keywords_drag_types{{
 	{ const_cast<gchar *>(TARGET_APP_KEYWORD_PATH_STRING), GTK_TARGET_SAME_WIDGET, TARGET_APP_KEYWORD_PATH },
 	{ const_cast<gchar *>("text/plain"), 0, TARGET_TEXT_PLAIN }
-};
-constexpr gint n_keywords_drag_types = 2;
+}};
 
-// @todo Use std::array
-constexpr GtkTargetEntry bar_pane_keywords_drop_types[] = {
+constexpr std::array<GtkTargetEntry, 2> bar_pane_keywords_drop_types{{
 	{ const_cast<gchar *>(TARGET_APP_KEYWORD_PATH_STRING), GTK_TARGET_SAME_WIDGET, TARGET_APP_KEYWORD_PATH },
 	{ const_cast<gchar *>("text/plain"), 0, TARGET_TEXT_PLAIN }
-};
-constexpr gint n_keywords_drop_types = 2;
-
+}};
 
 void bar_pane_keywords_dnd_get(GtkWidget *tree_view, GdkDragContext *,
 				     GtkSelectionData *selection_data, guint info,
@@ -1367,7 +1365,7 @@ void bar_pane_keywords_menu_popup(GtkWidget *, PaneKeywordsData *pkd, gint x, gi
 
 		if (keyword)
 			{
-			text = g_strdup_printf(_("Disconnect all Mark Keyword connections"));
+			text = g_strdup_printf("%s", _("Disconnect all Mark Keyword connections"));
 			menu_item_add_icon(menu, text, GQ_ICON_DELETE, G_CALLBACK(bar_pane_keywords_disconnect_marks_cb), pkd);
 			g_free(text);
 			}
@@ -1416,12 +1414,11 @@ gboolean bar_pane_keywords_menu_cb(GtkWidget *widget, GdkEventButton *bevent, gp
  *-------------------------------------------------------------------
  */
 
-void bar_pane_keywords_destroy(GtkWidget *, gpointer data)
+void bar_pane_keywords_destroy(gpointer data)
 {
 	auto pkd = static_cast<PaneKeywordsData *>(data);
-	gchar *path;
 
-	path = g_build_filename(get_rc_dir(), "keywords", NULL);
+	g_autofree gchar *path = g_build_filename(get_rc_dir(), "keywords", NULL);
 	autocomplete_keywords_list_save(path);
 
 	g_list_free_full(pkd->expanded_rows, g_free);
@@ -1472,9 +1469,7 @@ GtkWidget *bar_pane_keywords_new(const gchar *id, const gchar *title, const gcha
 	gq_gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
 
 	pkd->widget = vbox;
-	g_object_set_data(G_OBJECT(pkd->widget), "pane_data", pkd);
-	g_signal_connect(G_OBJECT(pkd->widget), "destroy",
-			 G_CALLBACK(bar_pane_keywords_destroy), pkd);
+	g_object_set_data_full(G_OBJECT(pkd->widget), "pane_data", pkd, bar_pane_keywords_destroy);
 	gtk_widget_set_size_request(pkd->widget, -1, height);
 	gtk_widget_show(hbox);
 
@@ -1579,9 +1574,9 @@ GtkWidget *bar_pane_keywords_new(const gchar *id, const gchar *title, const gcha
 	gtk_tree_view_set_expander_column(GTK_TREE_VIEW(pkd->keyword_treeview), column);
 
 	gtk_drag_source_set(pkd->keyword_treeview,
-			    static_cast<GdkModifierType>(GDK_BUTTON1_MASK | GDK_BUTTON2_MASK),
-			    bar_pane_keywords_drag_types, n_keywords_drag_types,
-			    static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
+	                    static_cast<GdkModifierType>(GDK_BUTTON1_MASK | GDK_BUTTON2_MASK),
+	                    bar_pane_keywords_drag_types.data(), bar_pane_keywords_drag_types.size(),
+	                    static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
 
 	g_signal_connect(G_OBJECT(pkd->keyword_treeview), "drag_data_get",
 			 G_CALLBACK(bar_pane_keywords_dnd_get), pkd);
@@ -1592,9 +1587,9 @@ GtkWidget *bar_pane_keywords_new(const gchar *id, const gchar *title, const gcha
 			 G_CALLBACK(bar_pane_keywords_dnd_end), pkd);
 
 	gtk_drag_dest_set(pkd->keyword_treeview,
-			  static_cast<GtkDestDefaults>(GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT | GTK_DEST_DEFAULT_DROP),
-			  bar_pane_keywords_drop_types, n_keywords_drop_types,
-			  static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE));
+	                  static_cast<GtkDestDefaults>(GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT | GTK_DEST_DEFAULT_DROP),
+	                  bar_pane_keywords_drop_types.data(), bar_pane_keywords_drop_types.size(),
+	                  static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE));
 
 	g_signal_connect(G_OBJECT(pkd->keyword_treeview), "drag_data_received",
 			 G_CALLBACK(bar_pane_keywords_dnd_receive), pkd);
@@ -1682,33 +1677,7 @@ gint autocomplete_sort_iter_compare_func (GtkTreeModel *model,
 									GtkTreeIter *b,
 									gpointer)
 {
-	gint ret = 0;
-	gchar *name1;
-	gchar *name2;
-
-	gtk_tree_model_get(model, a, 0, &name1, -1);
-	gtk_tree_model_get(model, b, 0, &name2, -1);
-
-	if (name1 == nullptr || name2 == nullptr)
-		{
-		if (name1 == nullptr && name2 == nullptr)
-			{
-			ret = 0;
-			}
-		else
-			{
-			ret = (name1 == nullptr) ? -1 : 1;
-			}
-		}
-	else
-		{
-		ret = g_utf8_collate(name1,name2);
-		}
-
-	g_free(name1);
-	g_free(name2);
-
-	return ret;
+	return gq_gtk_tree_iter_utf8_collate(model, a, b, 0);
 }
 
 void autocomplete_keywords_list_load(const gchar *path)
@@ -1767,7 +1736,7 @@ void autocomplete_keywords_list_load(const gchar *path)
 	fclose(f);
 }
 
-gboolean autocomplete_keywords_list_save(gchar *path)
+gboolean autocomplete_keywords_list_save(const gchar *path)
 {
 	SecureSaveInfo *ssi;
 	gchar *pathl;
@@ -1813,20 +1782,6 @@ gboolean autocomplete_keywords_list_save(gchar *path)
  *-------------------------------------------------------------------
  */
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-void bar_pane_keywords_close_unused(GtkWidget *bar)
-{
-	PaneKeywordsData *pkd;
-
-	pkd = static_cast<PaneKeywordsData *>(g_object_get_data(G_OBJECT(bar), "pane_data"));
-	if (!pkd) return;
-
-	g_free(pkd->pane.id);
-	g_object_unref(pkd->widget);
-}
-#pragma GCC diagnostic pop
-
 GtkWidget *bar_pane_keywords_new_from_config(const gchar **attribute_names, const gchar **attribute_values)
 {
 	gchar *id = g_strdup("keywords");
@@ -2001,15 +1956,15 @@ gboolean bar_keywords_autocomplete_focus(LayoutWindow *lw)
 	pane = bar_find_pane_by_id(lw->bar, PANE_KEYWORDS, "keywords");
 
 	children = gtk_container_get_children(GTK_CONTAINER(pane));
-	children = g_list_last(children);
 
-	if (current_focus == children->data)
+	const GList *last_child = g_list_last(children);
+	if (current_focus == last_child->data)
 		{
 		ret = TRUE;
 		}
 	else
 		{
-		gtk_widget_grab_focus(GTK_WIDGET(children->data));
+		gtk_widget_grab_focus(GTK_WIDGET(last_child->data));
 		ret = FALSE;
 		}
 
