@@ -187,13 +187,11 @@ static void bar_sort_undo_set(SortData *sd, GList *src_list, const gchar *dest)
 		g_list_free_full(sd->undo_dest_list, g_free);
 		sd->undo_dest_list=nullptr;
 
-		GList *work = sd->undo_src_list;
-		while(work)
+		for (GList *work = sd->undo_src_list; work; work = work->next)
 			{
-			gchar *filename = g_strdup(filename_from_path(static_cast<const gchar *>(work->data)));
-			gchar *dest_path = g_build_filename(g_strdup(dest), filename, NULL);
-			sd->undo_dest_list = g_list_prepend(sd->undo_dest_list, g_strdup(dest_path));
-			work = work->next;
+			const gchar *filename = filename_from_path(static_cast<gchar *>(work->data));
+			gchar *dest_path = g_build_filename(dest, filename, NULL);
+			sd->undo_dest_list = g_list_prepend(sd->undo_dest_list, dest_path);
 			}
 		sd->undo_dest_list = g_list_reverse(sd->undo_dest_list);
 		}
@@ -217,45 +215,32 @@ static void bar_sort_undo_folder(SortData *sd, GtkWidget *button)
 		{
 		case BAR_SORT_MOVE:
 			{
-			GList *list;
-			gchar *src_dir;
-			gchar *src_path;
-
-			if (sd->undo_src_list)
+			GList *list = nullptr;
+			for (GList *work = sd->undo_dest_list; work; work = work->next)
 				{
-				GList *work = nullptr;
-
-				src_path = g_strdup(static_cast<const gchar *>(sd->undo_src_list->data));
-				src_dir = remove_level_from_path(src_path);
-				list = sd->undo_dest_list;
-				while (list)
-					{
-					work = g_list_prepend(work, file_data_new_group(static_cast<const gchar *>(list->data)));
-					list=list->next;
-					}
-				file_util_move_simple(work, src_dir, sd->lw->window);
-				g_free(src_dir);
-				g_free(src_path);
+				list = g_list_prepend(list, file_data_new_group(static_cast<gchar *>(work->data)));
 				}
+
+			const auto *src_path = static_cast<const gchar *>(sd->undo_src_list->data);
+			g_autofree gchar *src_dir = remove_level_from_path(src_path);
+
+			file_util_move_simple(list, src_dir, sd->lw->window);
 			}
 			break;
 
 		case BAR_SORT_COPY:
 		case BAR_SORT_FILTER:
-			if (sd->undo_src_list)
+			{
+			GList *delete_list = nullptr;
+			for (GList *work = sd->undo_dest_list; work; work = work->next)
 				{
-				GList *delete_list;
-				GList *work = nullptr;
-
-				delete_list = sd->undo_dest_list;
-				while (delete_list)
-					{
-					work = g_list_append(work, file_data_new_group(static_cast<const gchar *>(delete_list->data)));
-					delete_list = delete_list->next;
-					}
-				options->file_ops.safe_delete_enable = TRUE;
-				file_util_delete(nullptr, work, button);
+				delete_list = g_list_append(delete_list, file_data_new_group(static_cast<gchar *>(work->data)));
 				}
+
+			options->file_ops.safe_delete_enable = TRUE;
+
+			file_util_delete(nullptr, delete_list, button);
+			}
 			break;
 
 		default:
@@ -621,7 +606,7 @@ void bar_sort_close(GtkWidget *bar)
 	gq_gtk_widget_destroy(sd->vbox);
 }
 
-static void bar_sort_destroy(GtkWidget *, gpointer data)
+static void bar_sort_destroy(gpointer data)
 {
 	auto sd = static_cast<SortData *>(data);
 
@@ -634,11 +619,6 @@ static void bar_sort_destroy(GtkWidget *, gpointer data)
 	g_free(sd);
 }
 
-static void bar_sort_edit_button_free(gpointer data)
-{
-	g_free(data);
-}
-
 static GtkWidget *bar_sort_new(LayoutWindow *lw, SortActionType action,
 			       SortModeType mode, SortSelectionType selection,
 			       const gchar *filter_key)
@@ -673,9 +653,7 @@ static GtkWidget *bar_sort_new(LayoutWindow *lw, SortActionType action,
 
 	sd->vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, PREF_PAD_GAP);
 	DEBUG_NAME(sd->vbox);
-	g_object_set_data(G_OBJECT(sd->vbox), "bar_sort_data", sd);
-	g_signal_connect(G_OBJECT(sd->vbox), "destroy",
-			 G_CALLBACK(bar_sort_destroy), sd);
+	g_object_set_data_full(G_OBJECT(sd->vbox), "bar_sort_data", sd, bar_sort_destroy);
 
 	label = gtk_label_new(_("Sort Manager"));
 	pref_label_bold(label, TRUE, FALSE);
@@ -733,7 +711,7 @@ static GtkWidget *bar_sort_new(LayoutWindow *lw, SortActionType action,
 					      G_CALLBACK(bar_sort_set_filter_cb), sd);
 		g_signal_connect(G_OBJECT(button), "button_press_event", G_CALLBACK(bar_filter_message_cb), NULL);
 
-		g_object_set_data_full(G_OBJECT(button), "filter_key", key, bar_sort_edit_button_free);
+		g_object_set_data_full(G_OBJECT(button), "filter_key", key, g_free);
 		}
 	g_list_free(editors_list);
 
@@ -820,10 +798,8 @@ void bar_sort_cold_start(LayoutWindow *lw, const gchar **attribute_names, const
 	lw->options.action = static_cast<SortActionType>(action);
 	lw->options.mode = static_cast<SortModeType>(mode);
 	lw->options.selection = static_cast<SortSelectionType>(selection);
-	lw->options.filter_key = g_strdup(filter_key);
+	lw->options.filter_key = filter_key;
 	lw->bar_sort_enabled = enabled;
-
-	g_free(filter_key);
 }
 
 GtkWidget *bar_sort_new_default(LayoutWindow *lw)
