@@ -22,6 +22,9 @@
 #ifndef BAR_SORT_H
 #define BAR_SORT_H
 
+#include <glib.h>
+#include <gtk/gtk.h>
+
 struct LayoutWindow;
 
 GtkWidget *bar_sort_new_default(LayoutWindow *lw);
