@@ -44,12 +44,14 @@
 namespace
 {
 
+constexpr gint SIDEBAR_DEFAULT_WIDTH = 250;
+
 void remove_child_from_parent(gpointer data)
 {
 	gtk_container_remove(GTK_CONTAINER(gtk_widget_get_parent(GTK_WIDGET(data))), GTK_WIDGET(data));
 }
 
-}
+} // namespace
 
 struct KnownPanes
 {
@@ -459,7 +461,7 @@ static void bar_menu_add_popup(GtkWidget *widget)
 }
 
 
-static gboolean bar_menu_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer)
+static gboolean bar_menu_expander_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer)
 {
 	if (bevent->button == MOUSE_BUTTON_RIGHT)
 		{
@@ -606,7 +608,7 @@ void bar_clear(GtkWidget *bar)
 
 	list = gtk_container_get_children(GTK_CONTAINER(bd->vbox));
 
-	g_list_free_full(list, reinterpret_cast<GDestroyNotify>(remove_child_from_parent));
+	g_list_free_full(list, remove_child_from_parent);
 }
 
 void bar_write_config(GtkWidget *bar, GString *outstr, gint indent)
@@ -682,7 +684,7 @@ void bar_add(GtkWidget *bar, GtkWidget *pane)
 
 	gq_gtk_box_pack_start(GTK_BOX(bd->vbox), expander, FALSE, TRUE, 0);
 
-	g_signal_connect(expander, "button_release_event", G_CALLBACK(bar_menu_cb), bd);
+	g_signal_connect(expander, "button_release_event", G_CALLBACK(bar_menu_expander_cb), bd);
 	g_signal_connect(expander, "notify::expanded", G_CALLBACK(bar_expander_cb), pd);
 
 	gq_gtk_container_add(GTK_WIDGET(expander), pane);
@@ -715,19 +717,6 @@ static void bar_size_allocate(GtkWidget *, GtkAllocation *, gpointer data)
 	bd->width = gtk_paned_get_position(GTK_PANED(bd->lw->utility_paned));
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-gint bar_get_width_unused(GtkWidget *bar)
-{
-	BarData *bd;
-
-	bd = static_cast<BarData *>(g_object_get_data(G_OBJECT(bar), "bar_data"));
-	if (!bd) return 0;
-
-	return bd->width;
-}
-#pragma GCC diagnostic pop
-
 void bar_close(GtkWidget *bar)
 {
 	BarData *bd;
@@ -739,7 +728,7 @@ void bar_close(GtkWidget *bar)
 	gq_gtk_widget_destroy(bd->widget);
 }
 
-static void bar_destroy(GtkWidget *, gpointer data)
+static void bar_destroy(gpointer data)
 {
 	auto bd = static_cast<BarData *>(data);
 
@@ -776,15 +765,11 @@ GtkWidget *bar_new(LayoutWindow *lw)
 
 	bd->widget = gtk_box_new(GTK_ORIENTATION_VERTICAL, PREF_PAD_GAP);
 	DEBUG_NAME(bd->widget);
-	g_object_set_data(G_OBJECT(bd->widget), "bar_data", bd);
-	g_signal_connect(G_OBJECT(bd->widget), "destroy",
-			 G_CALLBACK(bar_destroy), bd);
+	g_object_set_data_full(G_OBJECT(bd->widget), "bar_data", bd, bar_destroy);
 
 	g_signal_connect(G_OBJECT(bd->widget), "size-allocate",
 			 G_CALLBACK(bar_size_allocate), bd);
 
-	g_signal_connect(G_OBJECT(bd->widget), "button_release_event", G_CALLBACK(bar_menu_cb), bd);
-
 	bd->width = SIDEBAR_DEFAULT_WIDTH;
 
 	box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
