@@ -22,12 +22,12 @@
 #include "cache-maint.h"
 
 #include <dirent.h>
-#include <sys/types.h>
 
 #include <cstdlib>
 #include <cstring>
 
 #include <glib-object.h>
+#include <gtk/gtk.h>
 
 #include "cache-loader.h"
 #include "cache.h"
@@ -52,10 +52,6 @@
 namespace
 {
 
-constexpr gint PURGE_DIALOG_WIDTH = 400;
-
-} // namespace
-
 struct CMData
 {
 	GList *list;
@@ -71,28 +67,74 @@ struct CMData
 	gboolean remote;
 };
 
+constexpr gint PURGE_DIALOG_WIDTH = 400;
+
+/* sorry for complexity (cm->done_list), but need it to remove empty dirs */
+CMData *cache_maintain_data_new(gboolean clear, gboolean metadata, gboolean remote)
+{
+	const gchar *cache_folder = metadata ? get_metadata_cache_dir() : get_thumbnails_cache_dir();
+	FileData *dir_fd = file_data_new_dir(cache_folder);
+
+	GList *dlist;
+	if (!filelist_read(dir_fd, nullptr, &dlist))
+		{
+		file_data_unref(dir_fd);
+		return nullptr;
+		}
+
+	dlist = g_list_append(dlist, dir_fd);
+
+	auto *cm = g_new0(CMData, 1);
+	cm->list = dlist;
+	cm->done_list = nullptr;
+	cm->clear = clear;
+	cm->metadata = metadata;
+	cm->remote = remote;
+
+	return cm;
+}
+
+void cache_maintain_home_close(CMData *cm)
+{
+	if (cm->idle_id) g_source_remove(cm->idle_id);
+	if (cm->gd) generic_dialog_close(cm->gd);
+	filelist_free(cm->list);
+	g_list_free(cm->done_list);
+	g_free(cm);
+}
+
+} // namespace
+
 /*
  *-----------------------------------------------------------------------------
- * Command line cache maintenance program functions *-----------------------------------------------------------------------------
+ * Command line cache maintenance program functions
+ *-----------------------------------------------------------------------------
  */
 static gchar *cache_maintenance_path = nullptr;
 static GtkStatusIcon *status_icon;
 
-static void cache_maintenance_sim_stop_cb(gpointer)
+static void cache_manager_sim_remote(const gchar *path, gboolean recurse, GSourceFunc destroy_func);
+
+static gboolean cache_maintenance_sim_stop_cb(gpointer data)
 {
+	g_free(data);
 	exit(EXIT_SUCCESS);
+	return G_SOURCE_REMOVE;
 }
 
-static void cache_maintenance_render_stop_cb(gpointer)
+static gboolean cache_maintenance_render_stop_cb(gpointer data)
 {
+	g_free(data);
 	gtk_status_icon_set_tooltip_text(status_icon, _("Geeqie: Creating sim data..."));
-	cache_manager_sim_remote(cache_maintenance_path, TRUE, reinterpret_cast<GDestroyNotify *>(cache_maintenance_sim_stop_cb));
+	cache_manager_sim_remote(cache_maintenance_path, TRUE, cache_maintenance_sim_stop_cb);
+	return G_SOURCE_REMOVE;
 }
 
-static void cache_maintenance_clean_stop_cb(gpointer)
+static void cache_maintenance_clean_stop_cb(gpointer data)
 {
+	cache_maintain_home_close(static_cast<CMData *>(data));
 	gtk_status_icon_set_tooltip_text(status_icon, _("Geeqie: Creating thumbs..."));
-	cache_manager_render_remote(cache_maintenance_path, TRUE, options->thumbnails.cache_into_dirs, reinterpret_cast<GDestroyNotify *>(cache_maintenance_render_stop_cb));
+	cache_manager_render_remote(cache_maintenance_path, TRUE, options->thumbnails.cache_into_dirs, cache_maintenance_render_stop_cb);
 }
 
 static void cache_maintenance_user_cancel_cb()
@@ -123,12 +165,13 @@ void cache_maintenance(const gchar *path)
 {
 	cache_maintenance_path = g_strdup(path);
 
-	status_icon = gtk_status_icon_new_from_pixbuf(pixbuf_inline(PIXBUF_INLINE_ICON));
+	g_autoptr(GdkPixbuf) pixbuf_icon = pixbuf_inline(PIXBUF_INLINE_ICON);
+	status_icon = gtk_status_icon_new_from_pixbuf(pixbuf_icon);
 	gtk_status_icon_set_tooltip_text(status_icon, _("Geeqie: Cleaning thumbs..."));
 	gtk_status_icon_set_visible(status_icon, TRUE);
 	g_signal_connect(G_OBJECT(status_icon), "activate", G_CALLBACK(cache_maintenance_status_icon_activate_cb), NULL);
 
-	cache_maintain_home_remote(FALSE, FALSE, reinterpret_cast<GDestroyNotify *>(cache_maintenance_clean_stop_cb));
+	cache_maintain_home_remote(FALSE, FALSE, cache_maintenance_clean_stop_cb);
 }
 
 /*
@@ -137,21 +180,6 @@ void cache_maintenance(const gchar *path)
  *-------------------------------------------------------------------
  */
 
-static gchar *extension_find_dot(gchar *path)
-{
-	gchar *dot = nullptr;
-
-	if (!path) return nullptr;
-
-	while (*path != '\0')
-		{
-		if (*path == '.') dot = path;
-		path++;
-		}
-
-	return dot;
-}
-
 static gboolean isempty(const gchar *path)
 {
 	DIR *dp;
@@ -178,15 +206,6 @@ static gboolean isempty(const gchar *path)
 	return TRUE;
 }
 
-static void cache_maintain_home_close(CMData *cm)
-{
-	if (cm->idle_id) g_source_remove(cm->idle_id);
-	if (cm->gd) generic_dialog_close(cm->gd);
-	filelist_free(cm->list);
-	g_list_free(cm->done_list);
-	g_free(cm);
-}
-
 static void cache_maintain_home_stop(CMData *cm)
 {
 	if (cm->idle_id)
@@ -263,9 +282,8 @@ static gboolean cache_maintain_home_cb(gpointer data)
 				{
 				auto fd_list = static_cast<FileData *>(work->data);
 				gchar *path_buf = g_strdup(fd_list->path);
-				gchar *dot;
 
-				dot = extension_find_dot(path_buf);
+				gchar *dot = strrchr(path_buf, '.');
 
 				if (dot) *dot = '\0';
 				if ((!cm->metadata && cm->clear) ||
@@ -350,41 +368,14 @@ static void cache_maintain_home_stop_cb(GenericDialog *, gpointer data)
 	cache_maintain_home_stop(cm);
 }
 
-/* sorry for complexity (cm->done_list), but need it to remove empty dirs */
-void cache_maintain_home(gboolean metadata, gboolean clear, GtkWidget *parent)
+static void cache_maintain_home(gboolean metadata, gboolean clear, GtkWidget *parent)
 {
-	CMData *cm;
-	GList *dlist;
-	FileData *dir_fd;
+	CMData *cm = cache_maintain_data_new(clear, metadata, FALSE);
+	if (!cm) return;
+
 	const gchar *msg;
-	const gchar *cache_folder;
 	GtkWidget *hbox;
 
-	if (metadata)
-		{
-		cache_folder = get_metadata_cache_dir();
-		}
-	else
-		{
-		cache_folder = get_thumbnails_cache_dir();
-		}
-
-	dir_fd = file_data_new_dir(cache_folder);
-	if (!filelist_read(dir_fd, nullptr, &dlist))
-		{
-		file_data_unref(dir_fd);
-		return;
-		}
-
-	dlist = g_list_append(dlist, dir_fd);
-
-	cm = g_new0(CMData, 1);
-	cm->list = dlist;
-	cm->done_list = nullptr;
-	cm->clear = clear;
-	cm->metadata = metadata;
-	cm->remote = FALSE;
-
 	if (metadata)
 		{
 		msg = _("Removing old metadata...");
@@ -440,127 +431,64 @@ void cache_maintain_home(gboolean metadata, gboolean clear, GtkWidget *parent)
  *
  *
  */
-void cache_maintain_home_remote(gboolean metadata, gboolean clear, GDestroyNotify *func)
+void cache_maintain_home_remote(gboolean metadata, gboolean clear, GDestroyNotify func)
 {
-	CMData *cm;
-	GList *dlist;
-	FileData *dir_fd;
-	const gchar *cache_folder;
+	CMData *cm = cache_maintain_data_new(clear, metadata, TRUE);
+	if (!cm) return;
 
-	if (metadata)
-		{
-		cache_folder = get_metadata_cache_dir();
-		}
-	else
-		{
-		cache_folder = get_thumbnails_cache_dir();
-		}
-
-	dir_fd = file_data_new_dir(cache_folder);
-	if (!filelist_read(dir_fd, nullptr, &dlist))
-		{
-		file_data_unref(dir_fd);
-		return;
-		}
-
-	dlist = g_list_append(dlist, dir_fd);
-
-	cm = g_new0(CMData, 1);
-	cm->list = dlist;
-	cm->done_list = nullptr;
-	cm->clear = clear;
-	cm->metadata = metadata;
-	cm->remote = TRUE;
-
-	cm->idle_id = g_idle_add_full(G_PRIORITY_LOW, cache_maintain_home_cb, cm, reinterpret_cast<GDestroyNotify>(func));
-}
-
-static void cache_file_move(const gchar *src, const gchar *dest)
-{
-	if (!dest || !src || !isfile(src)) return;
-
-	if (!move_file(src, dest))
-		{
-		DEBUG_1("Failed to move cache file \"%s\" to \"%s\"", src, dest);
-		/* we remove it anyway - it's stale */
-		unlink_file(src);
-		}
+	cm->idle_id = g_idle_add_full(G_PRIORITY_LOW, cache_maintain_home_cb, cm, func);
 }
 
 static void cache_maint_moved(FileData *fd)
 {
-	gchar *base;
-	mode_t mode = 0755;
 	const gchar *src = fd->change->source;
 	const gchar *dest = fd->change->dest;
 
 	if (!src || !dest) return;
 
-	base = cache_get_location(CACHE_TYPE_THUMB, dest, FALSE, &mode);
-	if (recursive_mkdir_if_not_exists(base, mode))
-		{
-		gchar *buf;
-		gchar *d;
+	const auto cache_move = [src, dest](CacheType cache_type)
+	{
+		g_autofree gchar *src_path = cache_find_location(cache_type, src);
+		if (!src_path || !isfile(src_path)) return;
 
-		buf = cache_find_location(CACHE_TYPE_THUMB, src);
-		d = cache_get_location(CACHE_TYPE_THUMB, dest, TRUE, nullptr);
-		cache_file_move(buf, d);
-		g_free(d);
-		g_free(buf);
+		g_autofree gchar *dest_base = cache_create_location(cache_type, dest);
+		if (!dest_base) return;
 
-		buf = cache_find_location(CACHE_TYPE_SIM, src);
-		d = cache_get_location(CACHE_TYPE_SIM, dest, TRUE, nullptr);
-		cache_file_move(buf, d);
-		g_free(d);
-		g_free(buf);
-		}
-	else
-		{
-		log_printf("Failed to create cache dir for move %s\n", base);
-		}
-	g_free(base);
+		g_autofree gchar *dest_path = cache_get_location(cache_type, dest);
+		if (!dest_path) return;
 
-	base = cache_get_location(CACHE_TYPE_METADATA, dest, FALSE, &mode);
-	if (recursive_mkdir_if_not_exists(base, mode))
-		{
-		gchar *buf;
-		gchar *d;
+		if (!move_file(src_path, dest_path))
+			{
+			DEBUG_1("Failed to move cache file \"%s\" to \"%s\"", src_path, dest_path);
+			/* we remove it anyway - it's stale */
+			unlink_file(src_path);
+			}
+	};
 
-		buf = cache_find_location(CACHE_TYPE_METADATA, src);
-		d = cache_get_location(CACHE_TYPE_METADATA, dest, TRUE, nullptr);
-		cache_file_move(buf, d);
-		g_free(d);
-		g_free(buf);
-		}
-	g_free(base);
+	cache_move(CACHE_TYPE_THUMB);
+	cache_move(CACHE_TYPE_SIM);
+	cache_move(CACHE_TYPE_METADATA);
 
 	if (options->thumbnails.enable_caching && options->thumbnails.spec_standard)
 		thumb_std_maint_moved(src, dest);
 }
 
-static void cache_file_remove(const gchar *path)
-{
-	if (path && isfile(path) && !unlink_file(path))
-		{
-		DEBUG_1("Failed to remove cache file %s", path);
-		}
-}
-
 static void cache_maint_removed(FileData *fd)
 {
-	gchar *buf;
+	const auto cache_remove = [fd](CacheType cache_type)
+	{
+		g_autofree gchar *path = cache_find_location(cache_type, fd->path);
+		if (!path || !isfile(path)) return;
 
-	buf = cache_find_location(CACHE_TYPE_THUMB, fd->path);
-	cache_file_remove(buf);
-	g_free(buf);
+		if (!unlink_file(path))
+			{
+			DEBUG_1("Failed to remove cache file %s", path);
+			}
+	};
 
-	buf = cache_find_location(CACHE_TYPE_SIM, fd->path);
-	cache_file_remove(buf);
-	g_free(buf);
-
-	buf = cache_find_location(CACHE_TYPE_METADATA, fd->path);
-	cache_file_remove(buf);
-	g_free(buf);
+	cache_remove(CACHE_TYPE_THUMB);
+	cache_remove(CACHE_TYPE_SIM);
+	cache_remove(CACHE_TYPE_METADATA);
 
 	if (options->thumbnails.enable_caching && options->thumbnails.spec_standard)
 		thumb_std_maint_removed(fd->path);
@@ -568,28 +496,17 @@ static void cache_maint_removed(FileData *fd)
 
 static void cache_maint_copied(FileData *fd)
 {
-	gchar *dest_base;
-	gchar *src_cache;
-	mode_t mode = 0755;
+	g_autofree gchar *src_path = cache_find_location(CACHE_TYPE_METADATA, fd->change->source);
+	if (!src_path) return;
 
-	src_cache = cache_find_location(CACHE_TYPE_METADATA, fd->change->source);
-	if (!src_cache) return;
+	g_autofree gchar *dest_base = cache_create_location(CACHE_TYPE_METADATA, fd->change->dest);
+	if (!dest_base) return;
 
-	dest_base = cache_get_location(CACHE_TYPE_METADATA, fd->change->dest, FALSE, &mode);
-	if (recursive_mkdir_if_not_exists(dest_base, mode))
+	g_autofree gchar *dest_path = cache_get_location(CACHE_TYPE_METADATA, fd->change->dest);
+	if (!copy_file(src_path, dest_path))
 		{
-		gchar *path;
-
-		path = cache_get_location(CACHE_TYPE_METADATA, fd->change->dest, TRUE, nullptr);
-		if (!copy_file(src_cache, path))
-			{
-			DEBUG_1("failed to copy metadata %s to %s", src_cache, path);
-			}
-		g_free(path);
+		DEBUG_1("failed to copy metadata %s to %s", src_path, dest_path);
 		}
-
-	g_free(dest_base);
-	g_free(src_cache);
 }
 
 void cache_notify_cb(FileData *fd, NotifyType type, gpointer)
@@ -639,7 +556,7 @@ struct CacheOpsData
 	GenericDialog *gd;
 	ThumbLoaderStd *tl;
 	CacheLoader *cl;
-	GDestroyNotify *destroy_func; /* Used by the command line prog. functions */
+	GSourceFunc destroy_func; /* Used by the command line prog. functions */
 
 	GList *list;
 	GList *list_dir;
@@ -805,7 +722,7 @@ static gboolean cache_manager_render_file(CacheOpsData *cd)
 
 	if (cd->destroy_func)
 		{
-		g_idle_add(reinterpret_cast<GSourceFunc>(cd->destroy_func), nullptr);
+		g_idle_add(cd->destroy_func, cd);
 		}
 
 	return FALSE;
@@ -958,11 +875,11 @@ static void cache_manager_render_dialog(GtkWidget *widget, const gchar *path)
  * @param path Path to image folder
  * @param recurse
  * @param local Create thumbnails in same folder as images
- * @param func Function called when idle loop function terminates
+ * @param destroy_func Function called when idle loop function terminates
  *
  *
  */
-void cache_manager_render_remote(const gchar *path, gboolean recurse, gboolean local, GDestroyNotify *func)
+void cache_manager_render_remote(const gchar *path, gboolean recurse, gboolean local, GSourceFunc destroy_func)
 {
 	CacheOpsData *cd;
 
@@ -970,7 +887,7 @@ void cache_manager_render_remote(const gchar *path, gboolean recurse, gboolean l
 	cd->recurse = recurse;
 	cd->local = local;
 	cd->remote = TRUE;
-	cd->destroy_func = func;
+	cd->destroy_func = destroy_func;
 
 	cache_manager_render_start_render_remote(cd, path);
 }
@@ -1096,9 +1013,6 @@ static void cache_manager_standard_clean_valid_cb(const gchar *path, gboolean va
 static void cache_manager_standard_clean_start(GenericDialog *, gpointer data)
 {
 	auto cd = static_cast<CacheOpsData *>(data);
-	GList *list;
-	gchar *path;
-	FileData *dir_fd;
 
 	if (!cd->remote)
 	{
@@ -1111,26 +1025,21 @@ static void cache_manager_standard_clean_start(GenericDialog *, gpointer data)
 		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(cd->progress), _("running..."));
 	}
 
-	path = g_build_filename(get_thumbnails_standard_cache_dir(), THUMB_FOLDER_NORMAL, NULL);
-	dir_fd = file_data_new_dir(path);
-	filelist_read(dir_fd, &list, nullptr);
-	cd->list = list;
-	file_data_unref(dir_fd);
-	g_free(path);
+	const auto get_thumbnails_folder_files = [](const gchar *thumb_folder)
+	{
+		g_autofree gchar *path = g_build_filename(get_thumbnails_standard_cache_dir(), thumb_folder, NULL);
+		FileData *dir_fd = file_data_new_dir(path);
 
-	path = g_build_filename(get_thumbnails_standard_cache_dir(), THUMB_FOLDER_LARGE, NULL);
-	dir_fd = file_data_new_dir(path);
-	filelist_read(dir_fd, &list, nullptr);
-	cd->list = g_list_concat(cd->list, list);
-	file_data_unref(dir_fd);
-	g_free(path);
+		GList *list = nullptr;
+		filelist_read(dir_fd, &list, nullptr);
+		file_data_unref(dir_fd);
 
-	path = g_build_filename(get_thumbnails_standard_cache_dir(), THUMB_FOLDER_FAIL, NULL);
-	dir_fd = file_data_new_dir(path);
-	filelist_read(dir_fd, &list, nullptr);
-	cd->list = g_list_concat(cd->list, list);
-	file_data_unref(dir_fd);
-	g_free(path);
+		return list;
+	};
+
+	cd->list = get_thumbnails_folder_files(THUMB_FOLDER_NORMAL);
+	cd->list = g_list_concat(cd->list, get_thumbnails_folder_files(THUMB_FOLDER_LARGE));
+	cd->list = g_list_concat(cd->list, get_thumbnails_folder_files(THUMB_FOLDER_FAIL));
 
 	cd->count_total = g_list_length(cd->list);
 	cd->count_done = 0;
@@ -1241,7 +1150,7 @@ static void cache_manager_main_clear_ok_cb(GenericDialog *, gpointer)
 	cache_maintain_home(FALSE, TRUE, nullptr);
 }
 
-void cache_manager_main_clear_confirm(GtkWidget *parent)
+static void cache_manager_main_clear_confirm(GtkWidget *parent)
 {
 	GenericDialog *gd;
 
@@ -1409,18 +1318,18 @@ static void cache_manager_sim_start_sim_remote(CacheOpsData *cd, const gchar *us
  * @brief Generate .sim files
  * @param path Path to image folder
  * @param recurse
- * @param func Function called when idle loop function terminates
+ * @param destroy_func Function called when idle loop function terminates
  *
  *
  */
-void cache_manager_sim_remote(const gchar *path, gboolean recurse, GDestroyNotify *func)
+static void cache_manager_sim_remote(const gchar *path, gboolean recurse, GSourceFunc destroy_func)
 {
 	CacheOpsData *cd;
 
 	cd = g_new0(CacheOpsData, 1);
 	cd->recurse = recurse;
 	cd->remote = TRUE;
-	cd->destroy_func = func;
+	cd->destroy_func = destroy_func;
 
 	cache_manager_sim_start_sim_remote(cd, path);
 }
@@ -1465,16 +1374,16 @@ static gboolean cache_manager_sim_file(CacheOpsData *cd)
 		return TRUE;
 		}
 
-		if (!cd->remote)
-			{
-			gq_gtk_entry_set_text(GTK_ENTRY(cd->progress), _("done"));
-			}
+	if (!cd->remote)
+		{
+		gq_gtk_entry_set_text(GTK_ENTRY(cd->progress), _("done"));
+		}
 
 	cache_manager_sim_finish(cd);
 
 	if (cd->destroy_func)
 		{
-		g_idle_add(reinterpret_cast<GSourceFunc>(cd->destroy_func), nullptr);
+		g_idle_add(cd->destroy_func, cd);
 		}
 
 	return FALSE;
@@ -1527,7 +1436,7 @@ static void cache_manager_sim_start_cb(GenericDialog *, gpointer data)
 		g_list_free(list_total);
 		cd->count_done = 0;
 
-		while (cache_manager_sim_file(static_cast<CacheOpsData *>(cd)));
+		while (cache_manager_sim_file(cd));
 		}
 
 	g_free(path);
@@ -1639,7 +1548,7 @@ static void cache_manager_cache_maintenance_start_cb(GenericDialog *, gpointer d
 		}
 	else
 		{
-		cmd_line = g_strdup_printf("%s --cache-maintenance \"%s\"", gq_executable_path, path);
+		cmd_line = g_strdup_printf("%s --cache-maintenance=\"%s\"", gq_executable_path, path);
 
 		g_spawn_command_line_async(cmd_line, nullptr);
 
