@@ -23,20 +23,18 @@
 #define CACHE_MAINT_H
 
 #include <glib.h>
-#include <gtk/gtk.h>
 
 #include "typedefs.h"
 
-struct FileData;
+class FileData;
 
-void cache_maintain_home(gboolean metadata, gboolean clear, GtkWidget *parent);
 void cache_notify_cb(FileData *fd, NotifyType type, gpointer data);
 void cache_manager_show();
 
-void cache_maintain_home_remote(gboolean metadata, gboolean clear, GDestroyNotify *func);
+void cache_maintain_home_remote(gboolean metadata, gboolean clear, GDestroyNotify func);
 void cache_manager_standard_process_remote(gboolean clear);
-void cache_manager_render_remote(const gchar *path, gboolean recurse, gboolean local, GDestroyNotify *func);
-void cache_manager_sim_remote(const gchar *path, gboolean recurse, GDestroyNotify *func);
+void cache_manager_render_remote(const gchar *path, gboolean recurse, gboolean local, GSourceFunc destroy_func);
 void cache_maintenance(const gchar *path);
+
 #endif
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
