@@ -66,8 +66,94 @@
 namespace
 {
 
+struct CachePathParts
+{
+	CachePathParts(CacheType cache_type)
+	{
+		if (cache_type == CACHE_TYPE_METADATA || cache_type == CACHE_TYPE_XMP_METADATA)
+			{
+			rc = get_metadata_cache_dir();
+			local = GQ_CACHE_LOCAL_METADATA;
+			use_local_dir = options->metadata.enable_metadata_dirs;
+			}
+		else
+			{
+			rc = get_thumbnails_cache_dir();
+			local = GQ_CACHE_LOCAL_THUMB;
+			use_local_dir = options->thumbnails.cache_into_dirs;
+			}
+
+		switch (cache_type)
+			{
+			case CACHE_TYPE_THUMB:
+				ext = GQ_CACHE_EXT_THUMB;
+				break;
+			case CACHE_TYPE_SIM:
+				ext = GQ_CACHE_EXT_SIM;
+				break;
+			case CACHE_TYPE_METADATA:
+				ext = GQ_CACHE_EXT_METADATA;
+				break;
+			case CACHE_TYPE_XMP_METADATA:
+				ext = GQ_CACHE_EXT_XMP_METADATA;
+				break;
+			}
+	}
+
+	gchar *build_path_local(const gchar *source) const
+	{
+		g_autofree gchar *base = remove_level_from_path(source);
+		g_autofree gchar *name = g_strconcat(filename_from_path(source), ext, nullptr);
+
+		return g_build_filename(base, local, name, nullptr);
+	}
+
+	gchar *build_path_rc(const gchar *source) const
+	{
+		g_autofree gchar *name = g_strconcat(source, ext, nullptr);
+
+		return g_build_filename(rc, name, nullptr);
+	}
+
+	const gchar *rc = nullptr;
+	const gchar *local = nullptr;
+	const gchar *ext = nullptr;
+	gboolean use_local_dir = FALSE;
+};
+
 constexpr gint CACHE_LOAD_LINE_NOISE = 8;
 
+gchar *cache_get_location(CacheType type, const gchar *source, gint include_name, mode_t *mode)
+{
+	if (!source) return nullptr;
+
+	g_autofree gchar *base = remove_level_from_path(source);
+
+	const CachePathParts cache{type};
+
+	g_autofree gchar *name = nullptr;
+	if (include_name)
+		{
+		name = g_strconcat(filename_from_path(source), cache.ext, NULL);
+		}
+
+	gchar *path = nullptr;
+
+	if (cache.use_local_dir && access_file(base, W_OK))
+		{
+		path = g_build_filename(base, cache.local, name, NULL);
+		if (mode) *mode = 0775;
+		}
+
+	if (!path)
+		{
+		path = g_build_filename(cache.rc, base, name, NULL);
+		if (mode) *mode = 0755;
+		}
+
+	return path;
+}
+
 } // namespace
 
 /*
@@ -503,17 +589,6 @@ void cache_sim_data_set_dimensions(CacheData *cd, gint w, gint h)
 	cd->dimensions = TRUE;
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-void cache_sim_data_set_date_unused(CacheData *cd, time_t date)
-{
-	if (!cd) return;
-
-	cd->date = date;
-	cd->have_date = TRUE;
-}
-#pragma GCC diagnostic pop
-
 void cache_sim_data_set_md5sum(CacheData *cd, const guchar digest[16])
 {
 	gint i;
@@ -553,115 +628,34 @@ gboolean cache_sim_data_filled(ImageSimilarityData *sd)
  *-------------------------------------------------------------------
  */
 
-struct CachePathParts
+gchar *cache_create_location(CacheType cache_type, const gchar *source)
 {
-	CachePathParts(CacheType type)
-	{
-		switch (type)
-			{
-			case CACHE_TYPE_THUMB:
-				rc = get_thumbnails_cache_dir();
-				local = GQ_CACHE_LOCAL_THUMB;
-				ext = GQ_CACHE_EXT_THUMB;
-				break;
-			case CACHE_TYPE_SIM:
-				rc = get_thumbnails_cache_dir();
-				local = GQ_CACHE_LOCAL_THUMB;
-				ext = GQ_CACHE_EXT_SIM;
-				break;
-			case CACHE_TYPE_METADATA:
-				rc = get_metadata_cache_dir();
-				local = GQ_CACHE_LOCAL_METADATA;
-				ext = GQ_CACHE_EXT_METADATA;
-				break;
-			case CACHE_TYPE_XMP_METADATA:
-				rc = get_metadata_cache_dir();
-				local = GQ_CACHE_LOCAL_METADATA;
-				ext = GQ_CACHE_EXT_XMP_METADATA;
-				break;
-			}
-	}
+	mode_t mode = 0755;
+	g_autofree gchar *path = cache_get_location(cache_type, source, FALSE, &mode);
 
-	gchar *build_path_local(const gchar *source) const
-	{
-		gchar *base = remove_level_from_path(source);
-		gchar *name = g_strconcat(filename_from_path(source), ext, nullptr);
-		gchar *path = g_build_filename(base, local, name, nullptr);
-		g_free(name);
-		g_free(base);
+	if (!recursive_mkdir_if_not_exists(path, mode))
+		{
+		log_printf("Failed to create cache dir %s\n", path);
+		return nullptr;
+		}
 
-		return path;
-	}
+	return g_steal_pointer(&path);
+}
 
-	gchar *build_path_rc(const gchar *source) const
-	{
-		gchar *name = g_strconcat(source, ext, nullptr);
-		gchar *path = g_build_filename(rc, name, nullptr);
-		g_free(name);
-
-		return path;
-	}
-
-	const gchar *rc = nullptr;
-	const gchar *local = nullptr;
-	const gchar *ext = nullptr;
-};
-
-gchar *cache_get_location(CacheType type, const gchar *source, gint include_name, mode_t *mode)
+gchar *cache_get_location(CacheType cache_type, const gchar *source)
 {
-	gchar *path = nullptr;
-	gchar *base;
-	gchar *name = nullptr;
-
-	if (!source) return nullptr;
-
-	const CachePathParts cache{type};
-
-	base = remove_level_from_path(source);
-	if (include_name)
-		{
-		name = g_strconcat(filename_from_path(source), cache.ext, NULL);
-		}
-
-	if (((type != CACHE_TYPE_METADATA && type != CACHE_TYPE_XMP_METADATA && options->thumbnails.cache_into_dirs) ||
-	     ((type == CACHE_TYPE_METADATA || type == CACHE_TYPE_XMP_METADATA) && options->metadata.enable_metadata_dirs)) &&
-	    access_file(base, W_OK))
-		{
-		path = g_build_filename(base, cache.local, name, NULL);
-		if (mode) *mode = 0775;
-		}
-
-	if (!path)
-		{
-		path = g_build_filename(cache.rc, base, name, NULL);
-		if (mode) *mode = 0755;
-		}
-
-	g_free(base);
-	if (name) g_free(name);
-
-	return path;
+	return cache_get_location(cache_type, source, TRUE, nullptr);
 }
 
 gchar *cache_find_location(CacheType type, const gchar *source)
 {
 	gchar *path;
-	gboolean prefer_local;
 
 	if (!source) return nullptr;
 
 	const CachePathParts cache{type};
 
-	if (type == CACHE_TYPE_METADATA || type == CACHE_TYPE_XMP_METADATA)
-		{
-		prefer_local = options->metadata.enable_metadata_dirs;
-		}
-	else
-		{
-		prefer_local = options->thumbnails.cache_into_dirs;
-		}
-
-	if (prefer_local)
+	if (cache.use_local_dir)
 		{
 		path = cache.build_path_local(source);
 		}
@@ -675,7 +669,7 @@ gchar *cache_find_location(CacheType type, const gchar *source)
 		g_free(path);
 
 		/* try the opposite method if not found */
-		if (!prefer_local)
+		if (!cache.use_local_dir)
 			{
 			path = cache.build_path_local(source);
 			}
@@ -736,53 +730,34 @@ gboolean cache_time_valid(const gchar *cache, const gchar *path)
 
 const gchar *get_thumbnails_cache_dir()
 {
-	static gchar *thumbnails_cache_dir = nullptr;
-
-	if (thumbnails_cache_dir) return thumbnails_cache_dir;
-
-	if (USE_XDG)
-		{
-		thumbnails_cache_dir = g_build_filename(xdg_cache_home_get(),
-								GQ_APPNAME_LC, GQ_CACHE_THUMB, NULL);
-		}
-	else
-		{
-		thumbnails_cache_dir = g_build_filename(get_rc_dir(), GQ_CACHE_THUMB, NULL);
-		}
+#if USE_XDG
+	static gchar *thumbnails_cache_dir = g_build_filename(xdg_cache_home_get(), GQ_APPNAME_LC, GQ_CACHE_THUMB, NULL);
+#else
+	static gchar *thumbnails_cache_dir = g_build_filename(get_rc_dir(), GQ_CACHE_THUMB, NULL);
+#endif
 
 	return thumbnails_cache_dir;
 }
 
 const gchar *get_thumbnails_standard_cache_dir()
 {
-	static gchar *thumbnails_standard_cache_dir = nullptr;
-
-	if (thumbnails_standard_cache_dir) return thumbnails_standard_cache_dir;
-
-	thumbnails_standard_cache_dir = g_build_filename(xdg_cache_home_get(),
-										THUMB_FOLDER_GLOBAL, NULL);
+	static gchar *thumbnails_standard_cache_dir = g_build_filename(xdg_cache_home_get(),
+	                                                               THUMB_FOLDER_GLOBAL, NULL);
 
 	return thumbnails_standard_cache_dir;
 }
 
 const gchar *get_metadata_cache_dir()
 {
-	static gchar *metadata_cache_dir = nullptr;
-
-	if (metadata_cache_dir) return metadata_cache_dir;
-
-	if (USE_XDG)
-		{
-		/* Metadata go to $XDG_DATA_HOME.
-		 * "Keywords and comments, among other things, are irreplaceable and cannot be auto-generated,
-		 * so I don't think they'd be appropriate for the cache directory." -- Omari Stephens on geeqie-devel ml
-		 */
-		metadata_cache_dir = g_build_filename(xdg_data_home_get(), GQ_APPNAME_LC, GQ_CACHE_METADATA, NULL);
-		}
-	else
-		{
-		metadata_cache_dir = g_build_filename(get_rc_dir(), GQ_CACHE_METADATA, NULL);
-		}
+#if USE_XDG
+	/* Metadata go to $XDG_DATA_HOME.
+	 * "Keywords and comments, among other things, are irreplaceable and cannot be auto-generated,
+	 * so I don't think they'd be appropriate for the cache directory." -- Omari Stephens on geeqie-devel ml
+	 */
+	static gchar *metadata_cache_dir = g_build_filename(xdg_data_home_get(), GQ_APPNAME_LC, GQ_CACHE_METADATA, NULL);
+#else
+	static gchar *metadata_cache_dir = g_build_filename(get_rc_dir(), GQ_CACHE_METADATA, NULL);
+#endif
 
 	return metadata_cache_dir;
 }
