@@ -97,28 +97,21 @@ static gpointer parent_class;
 GType
 gqv_cell_renderer_icon_get_type()
 {
-	static GType cell_icon_type = 0;
-
-	if (!cell_icon_type)
-		{
-		static const GTypeInfo cell_icon_info =
-			{
-			sizeof(GQvCellRendererIconClass), /* class_size */
-			nullptr,		/* base_init */
-			nullptr,		/* base_finalize */
-			static_cast<GClassInitFunc>(gqv_cell_renderer_icon_class_init_wrapper), /* class_init */
-			nullptr,		/* class_finalize */
-			nullptr,		/* class_data */
-			sizeof(GQvCellRendererIcon), /* instance_size */
-			0,		/* n_preallocs */
-			reinterpret_cast<GInstanceInitFunc>(gqv_cell_renderer_icon_init_wrapper), /* instance_init */
-			nullptr,		/* value_table */
-			};
-
-		cell_icon_type = g_type_register_static(GTK_TYPE_CELL_RENDERER,
-							"GQvCellRendererIcon",
-							&cell_icon_info, static_cast<GTypeFlags>(0));
-		}
+	static const GTypeInfo cell_icon_info = {
+	    sizeof(GQvCellRendererIconClass), /* class_size */
+	    nullptr,		/* base_init */
+	    nullptr,		/* base_finalize */
+	    static_cast<GClassInitFunc>(gqv_cell_renderer_icon_class_init_wrapper), /* class_init */
+	    nullptr,		/* class_finalize */
+	    nullptr,		/* class_data */
+	    sizeof(GQvCellRendererIcon), /* instance_size */
+	    0,		/* n_preallocs */
+	    reinterpret_cast<GInstanceInitFunc>(gqv_cell_renderer_icon_init_wrapper), /* instance_init */
+	    nullptr,		/* value_table */
+	};
+	static GType cell_icon_type = g_type_register_static(GTK_TYPE_CELL_RENDERER,
+	                                                     "GQvCellRendererIcon",
+	                                                     &cell_icon_info, static_cast<GTypeFlags>(0));
 
 	return cell_icon_type;
 }
@@ -557,7 +550,7 @@ static void gqv_cell_renderer_icon_get_size(GtkCellRenderer    *cell,
 					    gint	       *width,
 					    gint	       *height)
 {
-	auto cellicon = reinterpret_cast<GQvCellRendererIcon *>(cell);
+	GQvCellRendererIcon *cellicon = GQV_CELL_RENDERER_ICON(cell);
 	gint calc_width;
 	gint calc_height;
 	gint xpad;
@@ -638,7 +631,7 @@ static void gqv_cell_renderer_icon_render(GtkCellRenderer *cell,
 
 {
 	GtkStyleContext *context = gtk_widget_get_style_context(widget);
-	auto cellicon = reinterpret_cast<GQvCellRendererIcon *>(cell);
+	GQvCellRendererIcon *cellicon = GQV_CELL_RENDERER_ICON(cell);
 	GdkPixbuf *pixbuf;
 	const gchar *text;
 	GdkRectangle cell_rect;
@@ -806,7 +799,7 @@ static gboolean gqv_cell_renderer_icon_activate(GtkCellRenderer      *cell,
 						const GdkRectangle   *cell_area,
 						GtkCellRendererState)
 {
-	auto cellicon = reinterpret_cast<GQvCellRendererIcon *>(cell);
+	GQvCellRendererIcon *cellicon = GQV_CELL_RENDERER_ICON(cell);
 	GdkEventButton *bevent = &event->button;
 
 	if (cellicon->show_marks &&
