@@ -22,12 +22,19 @@
 #include "collect-io.h"
 
 #include <cstdio>
+#include <cstdlib>
 #include <cstring>
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gdk/gdk.h>
 #include <glib-object.h>
 
+#if defined(__linux__)
+#include <mntent.h>
+#else
+#include <sys/mount.h>
+#endif
+
 #include <config.h>
 
 #include "collect.h"
@@ -40,6 +47,7 @@
 #include "secure-save.h"
 #include "thumb.h"
 #include "ui-fileops.h"
+#include "ui-utildlg.h"
 
 #define GQ_COLLECTION_MARKER "#" GQ_APPNAME
 
@@ -61,6 +69,11 @@ static gint collect_manager_process_action(CollectManagerEntry *entry, gchar **p
 namespace
 {
 
+const size_t GQ_COLLECTION_MARKER_LEN = strlen(GQ_COLLECTION_MARKER);
+
+constexpr gchar gqview_collection_marker[] = "#GQview collection";
+const size_t gqview_collection_marker_len = strlen(gqview_collection_marker);
+
 gboolean scan_geometry(gchar *buffer, GdkRectangle &window)
 {
 	gint nx;
@@ -78,7 +91,7 @@ gboolean scan_geometry(gchar *buffer, GdkRectangle &window)
 	return TRUE;
 }
 
-}
+} // namespace
 
 static gboolean collection_load_private(CollectionData *cd, const gchar *path, CollectionLoadFlags flags)
 {
@@ -123,8 +136,7 @@ static gboolean collection_load_private(CollectionData *cd, const gchar *path, C
 
 	pathl = path_from_utf8(path);
 
-	DEBUG_1("collection load: append=%d flush=%d only_geometry=%d path=%s",
-			  append, flush, only_geometry, pathl);
+	DEBUG_1("collection load: append=%d flush=%d only_geometry=%d path=%s", append, flush, only_geometry, pathl);
 
 	/* load it */
 	f = fopen(pathl, "r");
@@ -151,7 +163,7 @@ static gboolean collection_load_private(CollectionData *cd, const gchar *path, C
 			if (*p == '#')
 				{
 				if (!need_header) continue;
-				if (g_ascii_strncasecmp(p, GQ_COLLECTION_MARKER, strlen(GQ_COLLECTION_MARKER)) == 0)
+				if (g_ascii_strncasecmp(p, GQ_COLLECTION_MARKER, GQ_COLLECTION_MARKER_LEN) == 0)
 					{
 					/* Looks like an official collection, allow unchecked input.
 					 * All this does is allow adding files that may not exist,
@@ -161,14 +173,13 @@ static gboolean collection_load_private(CollectionData *cd, const gchar *path, C
 					has_official_header = TRUE;
 					limit_failures = FALSE;
 					}
-				else if (strncmp(p, "#geometry:", 10 ) == 0 &&
-					 scan_geometry(p + 10, cd->window))
+				else if (strncmp(p, "#geometry:", 10 ) == 0 && scan_geometry(p + 10, cd->window))
 					{
 					has_geometry_header = TRUE;
 					cd->window_read = TRUE;
 					if (only_geometry) break;
 					}
-				else if (g_ascii_strncasecmp(p, "#GQview collection", strlen("#GQview collection")) == 0)
+				else if (g_ascii_strncasecmp(p, gqview_collection_marker, gqview_collection_marker_len) == 0)
 					{
 					/* As 2008/04/15 there is no difference between our collection file format
 					 * and GQview 2.1.5 collection file format so ignore failures as well. */
@@ -231,22 +242,96 @@ static gboolean collection_load_private(CollectionData *cd, const gchar *path, C
 		if (*buffer2)
 			{
 			gboolean valid;
+			gboolean found = FALSE;
 
 			if (!flush)
 				changed |= collect_manager_process_action(entry, &buffer2);
 
 			valid = (buffer2[0] == G_DIR_SEPARATOR && collection_add_check(cd, file_data_new_simple(buffer2), FALSE, TRUE));
-			if (!valid) DEBUG_1("collection invalid file: %s", buffer2);
+			if (!valid)
+				{
+				log_printf("Warning: Collection: %s Invalid file: %s", cd->name, buffer2);
+				DEBUG_1("collection invalid file: %s", buffer2);
+				}
 
 			total++;
 			if (!valid)
 				{
-				fail++;
-				if (limit_failures &&
-				    fail > GQ_COLLECTION_FAIL_MIN &&
-				    fail * 100 / total > GQ_COLLECTION_FAIL_PERCENT)
+				/* If the file path has the prefix home, tmp or usr it was on the local file system and has been deleted. Ignore it. */
+				if (!g_str_has_prefix(buffer2, "/home") && !g_str_has_prefix(buffer2, "/tmp") && !g_str_has_prefix(buffer2, "/usr"))
 					{
-					log_printf("%d invalid filenames in unofficial collection file, closing: %s\n", fail, path);
+					/* The file was on a mounted drive and either has been deleted or the drive is not mounted */
+#if defined(__linux__)
+					struct mntent *mount_entry;
+					FILE *mount_entries;
+
+					mount_entries = setmntent("/proc/mounts", "r");
+					if (mount_entries == nullptr)
+						{
+						/* It is assumed this will never fail */
+						perror("setmntent");
+						exit(EXIT_FAILURE);
+						}
+
+					while (nullptr != (mount_entry = getmntent(mount_entries)))
+						{
+						if (g_strcmp0(mount_entry->mnt_dir, G_DIR_SEPARATOR_S) != 0)
+							{
+							if (g_str_has_prefix(buffer2, mount_entry->mnt_dir))
+								{
+								log_printf("%s was a file on a mounted filesystem but has been deleted: %s", buffer2, cd->name);
+								found = TRUE;
+								break;
+								}
+							}
+						}
+					endmntent(mount_entries);
+#else
+					struct statfs* mounts;
+					int num_mounts = getmntinfo(&mounts, MNT_NOWAIT);
+
+					if (num_mounts < 0)
+						{
+						/* It is assumed this will never fail */
+						perror("setmntent");
+						exit(EXIT_FAILURE);
+						}
+
+					for (int i = 0; i < num_mounts; i++)
+						{
+						if (g_strcmp0(mounts[i].f_mntonname, G_DIR_SEPARATOR_S) != 0)
+							{
+							if (g_str_has_prefix(buffer2, mounts[i].f_mntonname))
+								{
+								log_printf("%s was a file on a mounted filesystem but has been deleted: %s", buffer2, cd->name);
+								found = TRUE;
+								break;
+								}
+							}
+						}
+#endif
+
+					if (!found)
+						{
+						log_printf("%s is a file on an unmounted filesystem: %s", buffer2, cd->path);
+						gchar *text = g_strdup_printf(_("This Collection cannot be opened because it contains a link to a file on a drive which is not yet mounted.\n\nCollection: %s\nFile: %s\n"), cd->path, buffer2);
+						warning_dialog(_("Cannot open Collection"), text, GQ_ICON_DIALOG_WARNING, nullptr);
+						g_free(text);
+
+						collection_window_close_by_collection(cd);
+						success = FALSE;
+						break;
+						}
+					}
+				else
+					{
+					log_printf("%s was a file on local filesystem but has been deleted: %s", buffer2, cd->name);
+					}
+
+				fail++;
+				if (limit_failures && fail > GQ_COLLECTION_FAIL_MIN && fail * 100 / total > GQ_COLLECTION_FAIL_PERCENT)
+					{
+					log_printf("%u invalid filenames in unofficial collection file, closing: %s\n", fail, path);
 					success = FALSE;
 					break;
 					}
@@ -257,8 +342,7 @@ static gboolean collection_load_private(CollectionData *cd, const gchar *path, C
 
 	g_string_free(extended_filename_buffer, TRUE);
 
-	DEBUG_1("collection files: total = %d fail = %d official=%d gqview=%d geometry=%d",
-			  total, fail, has_official_header, has_gqview_header, has_geometry_header);
+	DEBUG_1("collection files: total = %u fail = %u official=%d gqview=%d geometry=%d", total, fail, has_official_header, has_gqview_header, has_geometry_header);
 
 	fclose(f);
 	if (only_geometry) return has_geometry_header;
@@ -615,22 +699,14 @@ static void collect_manager_entry_reset(CollectManagerEntry *entry)
 
 static CollectManagerEntry *collect_manager_get_entry(const gchar *path)
 {
-	GList *work;
+	const auto collect_manager_entry_compare_path = [](gconstpointer data, gconstpointer user_data)
+	{
+		return strcmp(static_cast<const CollectManagerEntry *>(data)->path, static_cast<const gchar *>(user_data));
+	};
 
-	work = collection_manager_entry_list;
-	while (work)
-		{
-		CollectManagerEntry *entry;
-
-		entry = static_cast<CollectManagerEntry *>(work->data);
-		work = work->next;
-		if (strcmp(entry->path, path) == 0)
-			{
-			return entry;
-			}
-		}
-	return nullptr;
+	GList *work = g_list_find_custom(collection_manager_entry_list, path, collect_manager_entry_compare_path);
 
+	return work ? static_cast<CollectManagerEntry *>(work->data) : nullptr;
 }
 
 static void collect_manager_entry_add_action(CollectManagerEntry *entry, CollectManagerAction *action)
@@ -947,6 +1023,13 @@ static void collect_manager_add_action(CollectManagerAction *action)
 	collect_manager_timer_push(FALSE);
 }
 
+/**
+ * @brief These are used to update collections contained in user's collection
+ * folder when moving or renaming files.
+ * also handles:
+ *   deletes file when newpath == NULL
+ *   adds file when oldpath == NULL
+ */
 void collect_manager_moved(FileData *fd)
 {
 	CollectManagerAction *action;
@@ -957,6 +1040,9 @@ void collect_manager_moved(FileData *fd)
 	collect_manager_add_action(action);
 }
 
+/**
+ * @brief Add from a specific collection
+ */
 void collect_manager_add(FileData *fd, const gchar *collection)
 {
 	CollectManagerAction *action;
@@ -978,6 +1064,9 @@ void collect_manager_add(FileData *fd, const gchar *collection)
 	collect_manager_add_action(action);
 }
 
+/**
+ * @brief Removing from a specific collection
+ */
 void collect_manager_remove(FileData *fd, const gchar *collection)
 {
 	CollectManagerAction *action;
@@ -996,6 +1085,9 @@ void collect_manager_remove(FileData *fd, const gchar *collection)
 	collect_manager_add_action(action);
 }
 
+/**
+ * @brief Commit pending operations to disk
+ */
 void collect_manager_flush()
 {
 	collect_manager_timer_push(TRUE);
@@ -1026,14 +1118,6 @@ void collect_manager_notify_cb(FileData *fd, NotifyType type, gpointer)
 		}
 }
 
-static gint collection_manager_sort_cb(gconstpointer a, gconstpointer b)
-{
-	auto char_a = static_cast<const gchar *>(a);
-	auto char_b = static_cast<const gchar *>(b);
-
-	return g_strcmp0(char_a, char_b);
-}
-
 /**
  * @brief Creates sorted list of collections
  * @param[out] names_exc sorted list of collections names excluding extension
@@ -1041,15 +1125,12 @@ static gint collection_manager_sort_cb(gconstpointer a, gconstpointer b)
  * @param[out] paths sorted list of collection paths
  *
  * Lists of type gchar.
- * Used lists must be freed with string_list_free()
+ * Used lists must be freed with data.
  */
 void collect_manager_list(GList **names_exc, GList **names_inc, GList **paths)
 {
 	FileData *dir_fd;
 	GList *list = nullptr;
-	gchar *name;
-	FileData *fd;
-	gchar *filename;
 
 	if (names_exc == nullptr && names_inc == nullptr && paths == nullptr)
 		{
@@ -1060,37 +1141,29 @@ void collect_manager_list(GList **names_exc, GList **names_inc, GList **paths)
 
 	filelist_read(dir_fd, &list, nullptr);
 
-	while (list)
+	for (GList *work = list; work; work = work->next)
 		{
-		fd = static_cast<FileData *>(list->data);
-		filename = g_strdup(filename_from_path(fd->path));
+		auto *fd = static_cast<FileData *>(work->data);
+		const gchar *filename = filename_from_path(fd->path);
 
 		if (file_extension_match(filename, GQ_COLLECTION_EXT))
 			{
-			name = remove_extension_from_path(filename);
-
 			if (names_exc != nullptr)
 				{
-				*names_exc = g_list_insert_sorted(*names_exc, g_strdup(name),
-											collection_manager_sort_cb);
-				*names_exc = g_list_first(*names_exc);
+				*names_exc = g_list_insert_sorted(*names_exc, remove_extension_from_path(filename),
+				                                  reinterpret_cast<GCompareFunc>(g_strcmp0));
 				}
 			if (names_inc != nullptr)
 				{
-				*names_inc = g_list_insert_sorted(*names_inc,filename,
-											collection_manager_sort_cb);
-				*names_inc = g_list_first(*names_inc);
+				*names_inc = g_list_insert_sorted(*names_inc, g_strdup(filename),
+				                                  reinterpret_cast<GCompareFunc>(g_strcmp0));
 				}
 			if (paths != nullptr)
 				{
 				*paths = g_list_insert_sorted(*paths, g_strdup(fd->path),
-											collection_manager_sort_cb);
-				*paths = g_list_first(*paths);
+				                              reinterpret_cast<GCompareFunc>(g_strcmp0));
 				}
-			g_free(name);
 			}
-		list = list->next;
-		g_free(filename);
 		}
 
 	filelist_free(list);
