@@ -27,7 +27,7 @@
 #include "typedefs.h"
 
 struct CollectionData;
-struct FileData;
+class FileData;
 
 enum CollectionLoadFlags {
 	COLLECTION_LOAD_NONE	= 0,
@@ -47,33 +47,12 @@ gboolean collection_save(CollectionData *cd, const gchar *path);
 
 gboolean collection_load_only_geometry(CollectionData *cd, const gchar *path);
 
-
-/**
- * @headerfile collect_manager_moved
- * these are used to update collections contained in user's collection
- * folder when moving or renaming files.
- * also handles:
- *   deletes file when newpath == NULL
- *   adds file when oldpath == NULL
- */
 void collect_manager_moved(FileData *fd);
 
-/**
- * @headerfile collect_manager_add
- * add from a specific collection
- */
 void collect_manager_add(FileData *fd, const gchar *collection);
 
-/**
- * @headerfile collect_manager_remove
- * removing from a specific collection
- */
 void collect_manager_remove(FileData *fd, const gchar *collection);
 
-/**
- * @headerfile collect_manager_flush
- * commit pending operations to disk
- */
 void collect_manager_flush();
 
 void collect_manager_notify_cb(FileData *fd, NotifyType type, gpointer data);
