@@ -21,12 +21,10 @@
 
 #include "collect-table.h"
 
+#include <array>
 #include <cstddef>
 #include <utility>
 
-#include <cairo.h>
-#include <gdk-pixbuf/gdk-pixbuf.h>
-#include <gio/gio.h>
 #include <glib-object.h>
 
 #include "cellrenderericon.h"
@@ -51,6 +49,7 @@
 #include "typedefs.h"
 #include "ui-fileops.h"
 #include "ui-menu.h"
+#include "ui-misc.h"
 #include "ui-tree-edit.h"
 #include "uri-utils.h"
 #include "utilops.h"
@@ -59,6 +58,17 @@
 namespace
 {
 
+enum {
+	CTABLE_COLUMN_POINTER = 0,
+	CTABLE_COLUMN_COUNT
+};
+
+struct ColumnData
+{
+	CollectTable *ct;
+	gint number;
+};
+
 /* between these, the icon width is increased by thumb_max_width / 2 */
 constexpr gint THUMB_MIN_ICON_WIDTH = 128;
 constexpr gint THUMB_MAX_ICON_WIDTH = 150;
@@ -70,16 +80,16 @@ constexpr gint THUMB_BORDER_PADDING = 2;
 constexpr gint COLLECT_TABLE_TIP_DELAY = 500;
 constexpr gint COLLECT_TABLE_TIP_DELAY_PATH = 850;
 
-enum {
-	CTABLE_COLUMN_POINTER = 0,
-	CTABLE_COLUMN_COUNT
-};
+constexpr std::array<GtkTargetEntry, 3> collection_drag_types{{
+	{ const_cast<gchar *>(TARGET_APP_COLLECTION_MEMBER_STRING), 0, TARGET_APP_COLLECTION_MEMBER },
+	{ const_cast<gchar *>("text/uri-list"), 0, TARGET_URI_LIST },
+	{ const_cast<gchar *>("text/plain"), 0, TARGET_TEXT_PLAIN }
+}};
 
-struct ColumnData
-{
-	CollectTable *ct;
-	gint number;
-};
+constexpr std::array<GtkTargetEntry, 2> collection_drop_types{{
+	{ const_cast<gchar *>(TARGET_APP_COLLECTION_MEMBER_STRING), 0, TARGET_APP_COLLECTION_MEMBER },
+	{ const_cast<gchar *>("text/uri-list"), 0, TARGET_URI_LIST }
+}};
 
 inline gboolean info_selected(const CollectInfo *info)
 {
@@ -102,7 +112,7 @@ hard_coded_window_keys collection_window_keys[] = {
 	{GDK_CONTROL_MASK, 'C', N_("Copy")},
 	{GDK_CONTROL_MASK, 'M', N_("Move")},
 	{GDK_CONTROL_MASK, 'R', N_("Rename")},
-	{GDK_CONTROL_MASK, 'D', N_("Move to Trash")},
+	{GDK_CONTROL_MASK, 'D', N_("Move selection to Trash")},
 	{GDK_CONTROL_MASK, 'W', N_("Close window")},
 	{static_cast<GdkModifierType>(0), GDK_KEY_Delete, N_("Remove")},
 	{static_cast<GdkModifierType>(0), GDK_KEY_Return, N_("View")},
@@ -777,6 +787,8 @@ static void collection_table_popup_delete_cb(GtkWidget *, gpointer data)
 
 	options->file_ops.safe_delete_enable = FALSE;
 	file_util_delete(nullptr, collection_table_popup_file_list(ct), ct->listview);
+
+	collection_table_refresh(ct);
 }
 
 static void collection_table_popup_move_to_trash_cb(GtkWidget *, gpointer data)
@@ -785,20 +797,22 @@ static void collection_table_popup_move_to_trash_cb(GtkWidget *, gpointer data)
 
 	options->file_ops.safe_delete_enable = TRUE;
 	file_util_delete(nullptr, collection_table_popup_file_list(ct), ct->listview);
+
+	collection_table_refresh(ct);
 }
 
 static void collection_table_popup_copy_path_cb(GtkWidget *, gpointer data)
 {
 	auto ct = static_cast<CollectTable *>(data);
 
-	file_util_copy_path_list_to_clipboard(collection_table_popup_file_list(ct), TRUE);
+	file_util_path_list_to_clipboard(collection_table_popup_file_list(ct), TRUE, ClipboardAction::COPY);
 }
 
 static void collection_table_popup_copy_path_unquoted_cb(GtkWidget *, gpointer data)
 {
 	auto ct = static_cast<CollectTable *>(data);
 
-	file_util_copy_path_list_to_clipboard(collection_table_popup_file_list(ct), FALSE);
+	file_util_path_list_to_clipboard(collection_table_popup_file_list(ct), FALSE, ClipboardAction::COPY);
 }
 
 static void collection_table_popup_sort_cb(GtkWidget *widget, gpointer data)
@@ -1056,12 +1070,12 @@ static GtkWidget *collection_table_popup_menu(CollectTable *ct, gboolean over_ic
 
 	menu_item_add_divider(menu);
 	menu_item_add_icon_sensitive(menu,
-				options->file_ops.confirm_move_to_trash ? _("Move to Trash...") :
-					_("Move to Trash"), GQ_ICON_DELETE, over_icon,
+				options->file_ops.confirm_move_to_trash ? _("Move selection to Trash...") :
+					_("Move selection to Trash"), GQ_ICON_DELETE, over_icon,
 				G_CALLBACK(collection_table_popup_move_to_trash_cb), ct);
 	menu_item_add_icon_sensitive(menu,
-				options->file_ops.confirm_delete ? _("_Delete...") :
-					_("_Delete"), GQ_ICON_DELETE_SHRED, over_icon,
+				options->file_ops.confirm_delete ? _("_Delete selection...") :
+					_("_Delete selection"), GQ_ICON_DELETE_SHRED, over_icon,
 				G_CALLBACK(collection_table_popup_delete_cb), ct);
 
 	menu_item_add_divider(menu);
@@ -1502,115 +1516,6 @@ static CollectInfo *collection_table_insert_point(CollectTable *ct, gint x, gint
 	return info;
 }
 
-static void collection_table_insert_marker(CollectTable *ct, CollectInfo *info, gboolean enable)
-{
-	gboolean after = FALSE;
-	GdkRectangle cell;
-	GdkWindow *parent;
-	gint x_parent;
-	gint y_parent;
-	GError *error = nullptr;
-	GInputStream *in_stream;
-	GdkPixbuf *pb;
-
-	parent = gtk_widget_get_window(gtk_widget_get_toplevel(ct->listview));
-	gdk_window_get_position(parent, &x_parent, &y_parent);
-
-	if (!enable)
-		{
-		if (ct->marker_window) gdk_window_destroy(ct->marker_window);
-		ct->marker_window = nullptr;
-
-		return;
-		}
-
-	info = collection_table_insert_find(ct, info, &after, &cell, FALSE, 0, 0);
-
-	/* this setting does not take into account (after), but since it is not really used... */
-	ct->marker_info = info;
-
-	if (!ct->marker_window)
-		{
-		GdkWindowAttr attributes;
-		gint attributes_mask;
-
-		in_stream = g_resources_open_stream(GQ_RESOURCE_PATH_ICONS "/gq-marker.xpm", G_RESOURCE_LOOKUP_FLAGS_NONE, &error);
-		pb = gdk_pixbuf_new_from_stream(in_stream, nullptr, &error);
-		g_object_unref(in_stream);
-
-		if (error)
-			{
-			log_printf("warning: collection marker error: %s", error->message);
-			g_error_free(error);
-			return;
-			}
-
-		gint w = gdk_pixbuf_get_width(pb);
-		gint h = gdk_pixbuf_get_height(pb);
-
-		attributes.window_type = GDK_WINDOW_CHILD;
-		attributes.wclass = GDK_INPUT_OUTPUT;
-		attributes.width = w;
-		attributes.height = h;
-		attributes.event_mask = gtk_widget_get_events(ct->listview);
-		attributes_mask = 0;
-
-		ct->marker_window = gdk_window_new(nullptr, &attributes, attributes_mask);
-
-		cairo_region_t *mask;
-		cairo_pattern_t *pattern;
-		cairo_surface_t *img = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, w, h);
-		cairo_t *cr = cairo_create(img);
-		gdk_cairo_set_source_pixbuf(cr, pb, 0, 0);
-		cairo_paint(cr);
-		/** @FIXME this is a hack to get the background color
-		 * pattern = cairo_pattern_create_for_surface(img);
-		 */
-		pattern = cairo_pattern_create_rgb (1.0, 0.0, 0.0);
-		mask = gdk_cairo_region_create_from_surface(img);
-		gdk_window_shape_combine_region(ct->marker_window, mask, 0, 0);
-		gdk_window_set_background_pattern(ct->marker_window, pattern);
-		cairo_region_destroy(mask);
-		cairo_pattern_destroy(pattern);
-		cairo_destroy(cr);
-		cairo_surface_destroy(img);
-		g_object_unref(pb);
-		}
-
-	if (info)
-		{
-		gint x;
-		gint y;
-		gint w;
-		gint h;
-
-		w = gdk_window_get_width(ct->marker_window);
-		h = gdk_window_get_height(ct->marker_window);
-
-		if (!after)
-			{
-			x = cell.x;
-			}
-		else
-			{
-			x = cell.x + cell.width;
-			}
-		x -= (w / 2);
-		y = cell.y + (cell.height / 2) - (h / 2);
-
-		x = x + x_parent;
-		y = y + y_parent;
-
-		gdk_window_move(ct->marker_window, x, y);
-
-		if (!gdk_window_is_visible(ct->marker_window)) gdk_window_show(ct->marker_window);
-		}
-	else
-		{
-		if (gdk_window_is_visible(ct->marker_window)) gdk_window_hide(ct->marker_window);
-		}
-}
-
 /*
  *-------------------------------------------------------------------
  * mouse drag auto-scroll
@@ -1626,7 +1531,6 @@ static void collection_table_motion_update(CollectTable *ct, gint x, gint y, gbo
 	if (drop_event)
 		{
 		tip_unschedule(ct);
-		collection_table_insert_marker(ct, info, TRUE);
 		}
 	else
 		{
@@ -1637,26 +1541,15 @@ static void collection_table_motion_update(CollectTable *ct, gint x, gint y, gbo
 static gboolean collection_table_auto_scroll_idle_cb(gpointer data)
 {
 	auto ct = static_cast<CollectTable *>(data);
-	GdkWindow *window;
-	gint x;
-	gint y;
-	gint w;
-	gint h;
-	GdkSeat *seat;
-	GdkDevice *device;
 
 	if (!ct->drop_idle_id) return G_SOURCE_REMOVE;
 
-	window = gtk_widget_get_window(ct->listview);
-	seat = gdk_display_get_default_seat(gdk_window_get_display(window));
-	device = gdk_seat_get_pointer(seat);
-	gdk_window_get_device_position(window, device, &x, &y, nullptr);
+	GdkWindow *window = gtk_widget_get_window(ct->listview);
 
-	w = gdk_window_get_width(window);
-	h = gdk_window_get_height(window);
-	if (x >= 0 && x < w && y >= 0 && y < h)
+	GdkPoint pos;
+	if (window_get_pointer_position(window, pos))
 		{
-		collection_table_motion_update(ct, x, y, TRUE);
+		collection_table_motion_update(ct, pos.x, pos.y, TRUE);
 		}
 
 	ct->drop_idle_id = 0;
@@ -1685,7 +1578,6 @@ static void collection_table_scroll(CollectTable *ct, gboolean scroll)
 			ct->drop_idle_id = 0;
 			}
 		widget_auto_scroll_stop(ct->listview);
-		collection_table_insert_marker(ct, nullptr, FALSE);
 		}
 	else
 		{
@@ -1967,7 +1859,7 @@ static void collection_table_sync(CollectTable *ct)
 			list = collection_table_add_row(ct, &iter);
 			}
 
-		while (list)
+		for (; list; list = list->next)
 			{
 			CollectInfo *info;
 			if (work)
@@ -1979,11 +1871,8 @@ static void collection_table_sync(CollectTable *ct)
 				{
 				info = nullptr;
 				}
-			if (list)
-				{
-				list->data = info;
-				list = list->next;
-				}
+
+			list->data = info;
 			}
 		}
 
@@ -2259,7 +2148,7 @@ static GtkWidget *collection_table_drop_menu(CollectTable *ct)
 	g_signal_connect(G_OBJECT(menu), "destroy",
 			 G_CALLBACK(collection_table_popup_destroy_cb), ct);
 
-	menu_item_add_stock(menu, _("Dropped list includes folders."), GQ_ICON_DND, nullptr, nullptr);
+	menu_item_add_icon(menu, _("Dropped list includes folders."), GQ_ICON_DIRECTORY, nullptr, nullptr);
 	menu_item_add_divider(menu);
 	menu_item_add_icon(menu, _("_Add contents"), GQ_ICON_OK,
 			    G_CALLBACK(confirm_dir_list_add), ct);
@@ -2279,20 +2168,6 @@ static GtkWidget *collection_table_drop_menu(CollectTable *ct)
  *-------------------------------------------------------------------
  */
 
-static GtkTargetEntry collection_drag_types[] = {
-	{ const_cast<gchar *>(TARGET_APP_COLLECTION_MEMBER_STRING), 0, TARGET_APP_COLLECTION_MEMBER },
-	{ const_cast<gchar *>("text/uri-list"), 0, TARGET_URI_LIST },
-	{ const_cast<gchar *>("text/plain"), 0, TARGET_TEXT_PLAIN }
-};
-static gint n_collection_drag_types = 3;
-
-static GtkTargetEntry collection_drop_types[] = {
-	{ const_cast<gchar *>(TARGET_APP_COLLECTION_MEMBER_STRING), 0, TARGET_APP_COLLECTION_MEMBER },
-	{ const_cast<gchar *>("text/uri-list"), 0, TARGET_URI_LIST }
-};
-static gint n_collection_drop_types = 2;
-
-
 static void collection_table_dnd_get(GtkWidget *, GdkDragContext *,
 				     GtkSelectionData *selection_data, guint info,
 				     guint, gpointer data)
@@ -2312,12 +2187,12 @@ static void collection_table_dnd_get(GtkWidget *, GdkDragContext *,
 		case TARGET_APP_COLLECTION_MEMBER:
 			if (selected)
 				{
-				uri_text = collection_info_list_to_dnd_data(ct->cd, ct->selection, &total);
+				uri_text = collection_info_list_to_dnd_data(ct->cd, ct->selection, total);
 				}
 			else
 				{
 				list = g_list_append(nullptr, ct->click_info);
-				uri_text = collection_info_list_to_dnd_data(ct->cd, list, &total);
+				uri_text = collection_info_list_to_dnd_data(ct->cd, list, total);
 				g_list_free(list);
 				}
 			gtk_selection_data_set(selection_data, gtk_selection_data_get_target(selection_data),
@@ -2359,7 +2234,6 @@ static void collection_table_dnd_receive(GtkWidget *, GdkDragContext *context,
 	DEBUG_1("%s", gtk_selection_data_get_data(selection_data));
 
 	collection_table_scroll(ct, FALSE);
-	collection_table_insert_marker(ct, nullptr, FALSE);
 
 	drop_info = collection_table_insert_point(ct, x, y);
 
@@ -2475,8 +2349,8 @@ static void collection_table_dnd_leave(GtkWidget *, GdkDragContext *, guint, gpo
 static void collection_table_dnd_init(CollectTable *ct)
 {
 	gtk_drag_source_set(ct->listview, static_cast<GdkModifierType>(GDK_BUTTON1_MASK | GDK_BUTTON2_MASK),
-			    collection_drag_types, n_collection_drag_types,
-			    static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
+	                    collection_drag_types.data(), collection_drag_types.size(),
+	                    static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
 	g_signal_connect(G_OBJECT(ct->listview), "drag_data_get",
 			 G_CALLBACK(collection_table_dnd_get), ct);
 	g_signal_connect(G_OBJECT(ct->listview), "drag_begin",
@@ -2485,9 +2359,9 @@ static void collection_table_dnd_init(CollectTable *ct)
 			 G_CALLBACK(collection_table_dnd_end), ct);
 
 	gtk_drag_dest_set(ct->listview,
-			  static_cast<GtkDestDefaults>(GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT | GTK_DEST_DEFAULT_DROP),
-			  collection_drop_types, n_collection_drop_types,
-			  static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_ASK));
+	                  static_cast<GtkDestDefaults>(GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT | GTK_DEST_DEFAULT_DROP),
+	                  collection_drop_types.data(), collection_drop_types.size(),
+	                  static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_ASK));
 	g_signal_connect(G_OBJECT(ct->listview), "drag_motion",
 			 G_CALLBACK(collection_table_dnd_motion), ct);
 	g_signal_connect(G_OBJECT(ct->listview), "drag_leave",
@@ -2644,6 +2518,8 @@ static void collection_table_destroy(GtkWidget *, gpointer data)
 
 	/* If there is no unsaved data, save the window geometry
 	 */
+	/** @FIXME  This code interferes with the code detecting files on unmounted drives. See collection_load_private() in collect-io,cc. If the user wants to save the geometry of an unchanged Collection, just slightly move one of the thumbnails. */
+/*
 	if (!ct->cd->changed)
 		{
 		if (!collection_save(ct->cd, ct->cd->path))
@@ -2651,6 +2527,7 @@ static void collection_table_destroy(GtkWidget *, gpointer data)
 			log_printf("failed saving to collection path: %s\n", ct->cd->path);
 			}
 		}
+*/
 
 	if (ct->popup)
 		{
