@@ -47,6 +47,7 @@
 #include "pixbuf-util.h"
 #include "print.h"
 #include "ui-fileops.h"
+#include "ui-misc.h"
 #include "ui-tree-edit.h"
 #include "ui-utildlg.h"
 #include "utilops.h"
@@ -107,7 +108,7 @@ CollectInfo *collection_info_new(FileData *fd, struct stat *, GdkPixbuf *pixbuf)
 	return ci;
 }
 
-void collection_info_free_thumb(CollectInfo *ci)
+static void collection_info_free_thumb(CollectInfo *ci)
 {
 	if (ci->pixbuf) g_object_unref(ci->pixbuf);
 	ci->pixbuf = nullptr;
@@ -129,19 +130,6 @@ void collection_info_set_thumb(CollectInfo *ci, GdkPixbuf *pixbuf)
 	ci->pixbuf = pixbuf;
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-gboolean collection_info_load_thumb_unused(CollectInfo *ci)
-{
-	if (!ci) return FALSE;
-
-	collection_info_free_thumb(ci);
-
-	log_printf("collection_info_load_thumb not implemented!\n(because an instant thumb loader not implemented)");
-	return FALSE;
-}
-#pragma GCC diagnostic pop
-
 /* an ugly static var, well what ya gonna do ? */
 static SortType collection_list_sort_method = SORT_NAME;
 
@@ -210,7 +198,7 @@ GList *collection_list_sort(GList *list, SortType method)
 	return g_list_sort(list, collection_list_sort_cb);
 }
 
-GList *collection_list_randomize(GList *list)
+static GList *collection_list_randomize(GList *list)
 {
 	guint random;
 	guint length;
@@ -321,19 +309,15 @@ CollectWindow *collection_window_find(CollectionData *cd)
 
 CollectWindow *collection_window_find_by_path(const gchar *path)
 {
-	GList *work;
-
 	if (!path) return nullptr;
 
-	work = collection_window_list;
-	while (work)
-		{
-		auto cw = static_cast<CollectWindow *>(work->data);
-		if (cw->cd->path && strcmp(cw->cd->path, path) == 0) return cw;
-		work = work->next;
-		}
+	const auto collect_window_compare_data_path = [](gconstpointer data, gconstpointer user_data)
+	{
+		return g_strcmp0(static_cast<const CollectWindow *>(data)->cd->path, static_cast<const gchar *>(user_data));
+	};
 
-	return nullptr;
+	GList *work = g_list_find_custom(collection_window_list, path, collect_window_compare_data_path);
+	return work ? static_cast<CollectWindow *>(work->data) : nullptr;
 }
 
 /**
@@ -550,7 +534,7 @@ void collection_path_changed(CollectionData *cd)
 	collection_window_update_title(collection_window_find(cd));
 }
 
-gint collection_to_number(CollectionData *cd)
+gint collection_to_number(const CollectionData *cd)
 {
 	return g_list_index(collection_list, cd);
 }
@@ -560,107 +544,67 @@ CollectionData *collection_from_number(gint n)
 	return static_cast<CollectionData *>(g_list_nth_data(collection_list, n));
 }
 
-CollectionData *collection_from_dnd_data(const gchar *data, GList **list, GList **info_list)
+/**
+ * @brief Pass a NULL pointer to whatever you don't need
+ * use free_selected_list to free list, and
+ * g_list_free to free info_list, which is a list of
+ * CollectInfo pointers into CollectionData
+ */
+ CollectionData *collection_from_dnd_data(const gchar *data, GList **list, GList **info_list)
 {
-	CollectionData *cd;
-	gint collection_number;
-	const gchar *ptr;
-
 	if (list) *list = nullptr;
 	if (info_list) *info_list = nullptr;
 
 	if (strncmp(data, "COLLECTION:", 11) != 0) return nullptr;
 
-	ptr = data + 11;
+	data += 11;
 
-	collection_number = atoi(ptr);
-	cd = collection_from_number(collection_number);
+	gint collection_number = atoi(data);
+	CollectionData *cd = collection_from_number(collection_number);
 	if (!cd) return nullptr;
 
 	if (!list && !info_list) return cd;
 
-	while (*ptr != '\0' && *ptr != '\n' ) ptr++;
-	if (*ptr == '\0') return cd;
-	ptr++;
-
-	while (*ptr != '\0')
+	GStrv numbers = g_strsplit(data, "\n", -1);
+	for (gint i = 1; numbers[i] != nullptr; i++)
 		{
-		guint item_number;
-		CollectInfo *info;
+		if (!numbers[i + 1]) break; // numbers[i] is data after last \n, skip it
 
-		item_number = static_cast<guint>(atoi(ptr));
-		while (*ptr != '\n' && *ptr != '\0') ptr++;
-		if (*ptr == '\0')
-			break;
-
-		while (*ptr == '\n') ptr++;
-
-		info = static_cast<CollectInfo *>(g_list_nth_data(cd->list, item_number));
+		auto item_number = static_cast<guint>(atoi(numbers[i]));
+		auto *info = static_cast<CollectInfo *>(g_list_nth_data(cd->list, item_number));
 		if (!info) continue;
 
 		if (list) *list = g_list_append(*list, file_data_ref(info->fd));
 		if (info_list) *info_list = g_list_append(*info_list, info);
 		}
 
+	g_strfreev(numbers);
 	return cd;
 }
 
-gchar *collection_info_list_to_dnd_data(CollectionData *cd, GList *list, gint *length)
+gchar *collection_info_list_to_dnd_data(const CollectionData *cd, const GList *list, gint &length)
 {
-	GList *work;
-	GList *temp = nullptr;
-	gchar *ptr;
-	gchar *text;
-	gchar *uri_text;
-	gint collection_number;
-
-	*length = 0;
+	length = 0;
 	if (!list) return nullptr;
 
-	collection_number = collection_to_number(cd);
+	gint collection_number = collection_to_number(cd);
 	if (collection_number < 0) return nullptr;
 
-	text = g_strdup_printf("COLLECTION:%d\n", collection_number);
-	*length += strlen(text);
-	temp = g_list_prepend(temp, text);
+	GString *text = g_string_new(nullptr);
+	g_string_printf(text, "COLLECTION:%d\n", collection_number);
 
-	work = list;
-	while (work)
+	for (const GList *work = list; work; work = work->next)
 		{
 		gint item_number = g_list_index(cd->list, work->data);
 
-		work = work->next;
-
 		if (item_number < 0) continue;
 
-		text = g_strdup_printf("%d\n", item_number);
-		temp = g_list_prepend(temp, text);
-		*length += strlen(text);
+		g_string_append_printf(text, "%d\n", item_number);
 		}
 
-	*length += 1; /* ending nul char */
+	length = text->len + 1; /* ending nul char */
 
-	uri_text = static_cast<gchar *>(g_malloc(*length));
-	ptr = uri_text;
-
-	work = g_list_last(temp);
-	while (work)
-		{
-		gint len;
-		auto text = static_cast<gchar *>(work->data);
-
-		work = work->prev;
-
-		len = strlen(text);
-		memcpy(ptr, text, len);
-		ptr += len;
-		}
-
-	ptr[0] = '\0';
-
-	g_list_free_full(temp, g_free);
-
-	return uri_text;
+	return g_string_free(text, FALSE);
 }
 
 gint collection_info_valid(CollectionData *cd, CollectInfo *info)
@@ -1105,9 +1049,7 @@ static void collection_window_get_geometry(CollectWindow *cw)
 
 	cd = cw->cd;
 	window = gtk_widget_get_window(cw->window);
-	gdk_window_get_position(window, &cd->window.x, &cd->window.y);
-	cd->window.width = gdk_window_get_width(window);
-	cd->window.height = gdk_window_get_height(window);
+	cd->window = window_get_position_geometry(window);
 	cd->window_read = TRUE;
 }
 
@@ -1163,7 +1105,6 @@ static void collection_window_insert(CollectWindow *cw, CollectInfo *ci)
 
 	if (!ci->pixbuf) collection_load_thumb_idle(cw->cd);
 	collection_table_file_insert(cw->table, ci);
-	if (!cw) return;
 }
 
 static void collection_window_remove(CollectWindow *cw, CollectInfo *ci)
