@@ -30,7 +30,7 @@
 #include "typedefs.h"
 
 struct CollectTable;
-struct FileData;
+class FileData;
 struct ThumbLoader;
 
 struct CollectInfo
@@ -42,7 +42,6 @@ struct CollectInfo
 
 CollectInfo *collection_info_new(FileData *fd, struct stat *st, GdkPixbuf *pixbuf);
 
-void collection_info_free_thumb(CollectInfo *ci);
 void collection_info_free(CollectInfo *ci);
 
 void collection_info_set_thumb(CollectInfo *ci, GdkPixbuf *pixbuf);
@@ -90,18 +89,11 @@ void collection_unref(CollectionData *cd);
 
 void collection_path_changed(CollectionData *cd);
 
-gint collection_to_number(CollectionData *cd);
+gint collection_to_number(const CollectionData *cd);
 CollectionData *collection_from_number(gint n);
 
-/**
- * @headerfile collection_from_dnd_data
- * pass a NULL pointer to whatever you don't need
- * use free_selected_list to free list, and
- * g_list_free to free info_list, which is a list of
- * CollectInfo pointers into CollectionData
- */
 CollectionData *collection_from_dnd_data(const gchar *data, GList **list, GList **info_list);
-gchar *collection_info_list_to_dnd_data(CollectionData *cd, GList *list, gint *length);
+gchar *collection_info_list_to_dnd_data(const CollectionData *cd, const GList *list, gint &length);
 
 gint collection_info_valid(CollectionData *cd, CollectInfo *info);
 
