@@ -61,9 +61,6 @@ struct ColorManCache {
 	gint refcount;
 };
 
-/* pixels to transform per idle call */
-#define COLOR_MAN_CHUNK_SIZE 81900
-
 
 static void color_man_lib_init()
 {
@@ -299,17 +296,6 @@ static ColorManCache *color_man_cache_get(ColorManProfileType in_type, const gch
  *-------------------------------------------------------------------
  */
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-static void color_man_done_unused(ColorMan *cm, ColorManReturnType type)
-{
-	if (cm->func_done)
-		{
-		cm->func_done(cm, type, cm->func_done_data);
-		}
-}
-#pragma GCC diagnostic pop
-
 void color_man_correct_region(ColorMan *cm, GdkPixbuf *pixbuf, gint x, gint y, gint w, gint h)
 {
 	ColorManCache *cc;
@@ -347,49 +333,6 @@ void color_man_correct_region(ColorMan *cm, GdkPixbuf *pixbuf, gint x, gint y, g
 
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-static gboolean color_man_idle_cb_unused(gpointer data)
-{
-	auto *cm = static_cast<ColorMan *>(data);
-	gint width;
-	gint height;
-	gint rh;
-
-	if (!cm->pixbuf) return FALSE;
-
-	if (cm->imd &&
-	    cm->pixbuf != image_get_pixbuf(cm->imd))
-		{
-		cm->idle_id = 0;
-		color_man_done_unused(cm, COLOR_RETURN_IMAGE_CHANGED);
-		return FALSE;
-		}
-
-	width = gdk_pixbuf_get_width(cm->pixbuf);
-	height = gdk_pixbuf_get_height(cm->pixbuf);
-
-	if (cm->row > height)
-		{
-		if (!cm->incremental_sync && cm->imd)
-			{
-			image_area_changed(cm->imd, 0, 0, width, height);
-			}
-
-		cm->idle_id = 0;
-		color_man_done_unused(cm, COLOR_RETURN_SUCCESS);
-		return FALSE;
-		}
-
-	rh = COLOR_MAN_CHUNK_SIZE / width + 1;
-	color_man_correct_region(cm, cm->pixbuf, 0, cm->row, width, rh);
-	if (cm->incremental_sync && cm->imd) image_area_changed(cm->imd, 0, cm->row, width, rh);
-	cm->row += rh;
-
-	return TRUE;
-}
-#pragma GCC diagnostic pop
-
 static ColorMan *color_man_new_real(ImageWindow *imd, GdkPixbuf *pixbuf,
 				    ColorManProfileType input_type, const gchar *input_file,
 				    guchar *input_data, guint input_data_len,
@@ -429,16 +372,6 @@ ColorMan *color_man_new(ImageWindow *imd, GdkPixbuf *pixbuf,
 				  screen_type, screen_file, screen_data, screen_data_len);
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-void color_man_start_bg_unused(ColorMan *cm, ColorMan::DoneFunc done_func, gpointer done_data)
-{
-	cm->func_done = done_func;
-	cm->func_done_data = done_data;
-	cm->idle_id = g_idle_add(color_man_idle_cb_unused, cm);
-}
-#pragma GCC diagnostic pop
-
 ColorMan *color_man_new_embedded(ImageWindow *imd, GdkPixbuf *pixbuf,
 				 guchar *input_data, guint input_data_len,
 				 ColorManProfileType screen_type, const gchar *screen_file,
