@@ -29,18 +29,16 @@
 #include "pixbuf-util.h"
 
 
-GtkTargetEntry dnd_file_drag_types[] = {
+constexpr std::array<GtkTargetEntry, 2> dnd_file_drag_types{{
 	{ const_cast<gchar *>("text/uri-list"), 0, TARGET_URI_LIST },
 	{ const_cast<gchar *>("text/plain"), 0, TARGET_TEXT_PLAIN }
-};
-gint dnd_file_drag_types_count = 2;
+}};
 
-GtkTargetEntry dnd_file_drop_types[] = {
+constexpr std::array<GtkTargetEntry, 3> dnd_file_drop_types{{
 	{ const_cast<gchar *>(TARGET_APP_COLLECTION_MEMBER_STRING), 0, TARGET_APP_COLLECTION_MEMBER },
 	{ const_cast<gchar *>("text/uri-list"), 0, TARGET_URI_LIST },
 	{ const_cast<gchar *>("text/plain"), 0, TARGET_TEXT_PLAIN },
-};
-gint dnd_file_drop_types_count = 3;
+}};
 
 
 #define DND_ICON_SIZE (options->dnd_icon_size)
@@ -84,37 +82,9 @@ static void pixbuf_draw_border(GdkPixbuf *pixbuf, gint w, gint h)
 		}
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-static void pixbuf_draw_rect_unused(GdkPixbuf *pixbuf, gint x, gint y, gint w, gint h, guint8 val)
-{
-	gboolean alpha;
-	gint rs;
-	guchar *pix;
-	guchar *p;
-	gint i;
-	gint j;
-
-	alpha = gdk_pixbuf_get_has_alpha(pixbuf);
-	rs = gdk_pixbuf_get_rowstride(pixbuf);
-	pix = gdk_pixbuf_get_pixels(pixbuf);
-
-	const gint p_step = alpha ? 4 : 3;
-
-	for (j = 0; j < h; j++)
-		{
-		p = pix + (rs * (y + j)) + (x * p_step);
-		for (i = 0; i < w; i++)
-			{
-			*p = (*p * (256-val)) >> 8; p++;
-			*p = (*p * (256-val)) >> 8; p++;
-			*p = (*p * (256-val)) >> 8; p++;
-			if (alpha) { *p = 255; p++; }
-			}
-		}
-}
-#pragma GCC diagnostic pop
-
+/**
+ * @brief Sets a drag icon to pixbuf, if items is > 1, text is drawn onto icon to indicate value
+ */
 void dnd_set_drag_icon(GtkWidget *widget, GdkDragContext *context, GdkPixbuf *pixbuf, gint items)
 {
 	GdkPixbuf *dest;
@@ -168,13 +138,13 @@ void dnd_set_drag_icon(GtkWidget *widget, GdkDragContext *context, GdkPixbuf *pi
 		lw = CLAMP(lw, 0, w - x - 1);
 		lh = CLAMP(lh, 0, h - y - 1);
 
-		pixbuf_draw_rect_fill(dest, x, y, lw, lh, 128, 128, 128, 255);
+		pixbuf_draw_rect_fill(dest, {x, y, lw, lh}, 128, 128, 128, 255);
 		}
 
 	if (layout)
 		{
-		pixbuf_draw_layout(dest, layout, nullptr, x+1, y+1, 0, 0, 0, 255);
-		pixbuf_draw_layout(dest, layout, nullptr, x, y, 255, 255, 255, 255);
+		pixbuf_draw_layout(dest, layout, x + 1, y + 1, 0, 0, 0, 255);
+		pixbuf_draw_layout(dest, layout, x, y, 255, 255, 255, 255);
 
 		g_object_unref(G_OBJECT(layout));
 		}
