@@ -95,7 +95,7 @@ gboolean editors_finished = FALSE;
 #define DESKTOP_GROUP "Desktop Entry"
 #endif
 
-void editor_description_free(EditorDescription *editor)
+static void editor_description_free(EditorDescription *editor)
 {
 	if (!editor) return;
 
@@ -197,8 +197,6 @@ gboolean editor_read_desktop_file(const gchar *path)
 	gchar *try_exec;
 	GtkTreeIter iter;
 	gboolean category_geeqie = FALSE;
-	GList *work;
-	gboolean disabled;
 
 	if (g_hash_table_lookup(editors, key)) return FALSE; /* the file found earlier wins */
 
@@ -365,22 +363,7 @@ gboolean editor_read_desktop_file(const gchar *path)
 
 	g_key_file_free(key_file);
 
-	if (editor->ignored) return TRUE;
-
-	work = options->disabled_plugins;
-
-	disabled = FALSE;
-	while (work)
-		{
-		if (g_strcmp0(path, static_cast<const gchar *>(work->data)) == 0)
-			{
-			disabled = TRUE;
-			break;
-			}
-		work = work->next;
-		}
-
-	editor->disabled = disabled;
+	editor->disabled = g_list_find_custom(options->disabled_plugins, path, reinterpret_cast<GCompareFunc>(g_strcmp0)) ? TRUE : FALSE;
 
 	gtk_list_store_append(desktop_file_list, &iter);
 	gtk_list_store_set(desktop_file_list, &iter,
@@ -733,9 +716,13 @@ static gchar *editor_command_path_parse(const FileData *fd, gboolean consider_si
 			p = fd->path;
 		else
 			{
+			const auto file_data_compare_ext = [](gconstpointer data, gconstpointer user_data)
+			{
+				return g_ascii_strcasecmp(static_cast<const FileData *>(data)->extension, static_cast<const gchar *>(user_data));
+			};
+
 			while (work)
 				{
-				GList *work2;
 				auto ext = static_cast<gchar *>(work->data);
 				work = work->next;
 
@@ -746,20 +733,19 @@ static gchar *editor_command_path_parse(const FileData *fd, gboolean consider_si
 					break;
 					}
 
-				work2 = consider_sidecars ? fd->sidecar_files : nullptr;
-				while (work2)
+				if (consider_sidecars)
 					{
-					auto sfd = static_cast<FileData *>(work2->data);
-					work2 = work2->next;
-
-					if (g_ascii_strcasecmp(ext, sfd->extension) == 0)
+					GList *work2 = g_list_find_custom(fd->sidecar_files, ext, file_data_compare_ext);
+					if (work2)
 						{
+						auto sfd = static_cast<FileData *>(work2->data);
 						p = sfd->path;
-						break;
 						}
 					}
+
 				if (p) break;
 				}
+
 			if (!p) return nullptr;
 			}
 		}
@@ -1253,6 +1239,8 @@ static EditorFlags editor_command_next_finish(EditorData *ed, gint status)
 		return static_cast<EditorFlags>(editor_errors(ed->flags));
 		case EDITOR_CB_SKIP:
 			return editor_command_done(ed);
+		default:
+			break;
 		}
 
 	return editor_command_next_start(ed);
@@ -1454,16 +1442,4 @@ const gchar *editor_get_error_str(EditorFlags flags)
 	return _("Unknown error.");
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-const gchar *editor_get_name_unused(const gchar *key)
-{
-	auto *editor = static_cast<EditorDescription *>(g_hash_table_lookup(editors, key));
-
-	if (!editor) return nullptr;
-
-	return editor->name;
-}
-#pragma GCC diagnostic pop
-
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
