@@ -24,6 +24,7 @@
 #endif
 
 #include <sys/stat.h>
+#include <sys/types.h>
 
 #include <cmath>
 #include <cstdlib>
@@ -446,6 +447,8 @@ static gchar *exif_build_formatted_Flash(ExifData *exif)
 		case 3:
 			string = g_string_append(string, _("auto"));
 			break;
+		default:
+			break;
 		}
 
 	/* return light (bits 1, 2) */
@@ -661,7 +664,7 @@ static void zd_tz(ZoneDetectResult *results, gchar **timezone, gchar **countryna
 	g_free(timezone_id);
 }
 
-void ZoneDetect_onError(int errZD, int errNative)
+static void ZoneDetect_onError(int errZD, int errNative)
 {
 	log_printf("Error: ZoneDetect %s (0x%08X)\n", ZDGetErrorString(errZD), (unsigned)errNative);
 }
@@ -836,7 +839,7 @@ static gchar *exif_build_formatted_localtime(ExifData *exif)
 /**
  * @brief Gets timezone from GPS lat/long
  * @param[in] exif
- * @returns Timezone string in the form "Europe/London"
+ * @returns Timezone string in the form Europe/London
  *
  *
  */
@@ -1018,13 +1021,13 @@ gchar *exif_get_data_as_text(ExifData *exif, const gchar *key)
 
 static FileCacheData *exif_cache;
 
-void exif_release_cb(FileData *fd)
+static void exif_release_cb(FileData *fd)
 {
 	exif_free(fd->exif);
 	fd->exif = nullptr;
 }
 
-void exif_init_cache()
+static void exif_init_cache()
 {
 	g_assert(!exif_cache);
 	exif_cache = file_cache_new(exif_release_cb, 4);
@@ -1219,15 +1222,15 @@ gchar *metadata_file_info(FileData *fd, const gchar *key, MetadataFormat)
 		}
 	if (strcmp(key, "file.owner") == 0)
 		{
-		return g_strdup(fd->owner);
+		return get_file_owner(fd->path);
 		}
 	if (strcmp(key, "file.group") == 0)
 		{
-		return g_strdup(fd->group);
+		return get_file_group(fd->path);
 		}
 	if (strcmp(key, "file.link") == 0)
 		{
-		return g_strdup(fd->sym_link);
+		return get_symbolic_link(fd->path);
 		}
 	if (strcmp(key, "file.page_no") == 0)
 		{
