@@ -66,7 +66,7 @@
 #include "typedefs.h"
 #include "ui-fileops.h"
 
-struct FileData;
+class FileData;
 
 /*
  *-----------------------------------------------------------------------------
@@ -1184,6 +1184,11 @@ static gint unmap_file(gpointer mapping, gint size)
 	return 0;
 }
 
+/**
+ * @brief exif_read returns processed data (merged from image and sidecar, etc.)
+ * this function gives access to the original data from the image.
+ * original data are part of the processed data and should not be freed separately
+ */
 ExifData *exif_get_original(ExifData *processed)
 {
 	return processed;
@@ -1275,20 +1280,15 @@ ExifData *exif_read(gchar *path, gchar *, GHashTable *)
 
 ExifItem *exif_get_item(ExifData *exif, const gchar *key)
 {
-	GList *work;
-
 	if (!key) return nullptr;
 
-	work = exif->items;
-	while (work)
-		{
-		ExifItem *item;
+	const auto exif_item_compare_marker_key = [](gconstpointer data, gconstpointer user_data)
+	{
+		return g_strcmp0(static_cast<const ExifItem *>(data)->marker->key, static_cast<const gchar *>(user_data));
+	};
 
-		item = static_cast<ExifItem*>(work->data);
-		work = work->next;
-		if (item->marker->key && strcmp(key, item->marker->key) == 0) return item;
-		}
-	return nullptr;
+	GList *work = g_list_find_custom(exif->items, key, exif_item_compare_marker_key);
+	return work ? static_cast<ExifItem*>(work->data) : nullptr;
 }
 
 #define EXIF_DATA_AS_TEXT_MAX_COUNT 16
@@ -1519,6 +1519,9 @@ static void exif_write_item(FILE *f, ExifItem *item, ExifData *exif)
 	g_free(text);
 }
 
+/**
+ * @brief Usually for debugging to stdout
+ */
 void exif_write_data_list(ExifData *exif, FILE *f, gint human_readable_list)
 {
 	if (!f || !exif) return;
