@@ -208,8 +208,8 @@ public:
 
 	~ExifDataOriginal() override
 	{
-		if (cp_data_) g_free(cp_data_);
-		if (pathl_) g_free(pathl_);
+		g_free(cp_data_);
+		g_free(pathl_);
 	}
 
 	Exiv2::Image *image() override
@@ -238,7 +238,7 @@ public:
 
 	void add_jpeg_color_profile(unsigned char *cp_data, guint cp_length) override
 	{
-		if (cp_data_) g_free(cp_data_);
+		g_free(cp_data_);
 		cp_data_ = cp_data;
 		cp_length_ = cp_length;
 	}
@@ -410,8 +410,10 @@ void exif_init()
 	bind_textdomain_codeset (EXV_PACKAGE, "UTF-8");
 #endif
 
+#if !EXIV2_TEST_VERSION(0,28,3)
 #ifdef EXV_ENABLE_BMFF
-	Exiv2::enableBMFF(TRUE);
+	Exiv2::enableBMFF(true);
+#endif
 #endif
 }
 
@@ -506,41 +508,6 @@ ExifItem *exif_get_item(ExifData *exif, const gchar *key)
 	}
 }
 
-ExifItem *exif_add_item(ExifData *exif, const gchar *key)
-{
-	try {
-		Exiv2::Metadatum *item = nullptr;
-		try {
-			Exiv2::ExifKey ekey(key);
-			exif->exifData().add(ekey, nullptr);
-			auto pos = exif->exifData().end(); // a hack, there should be a better way to get the currently added item
-			pos--;
-			item = &*pos;
-		}
-		catch (Exiv2::AnyError& e) {
-			try {
-				Exiv2::IptcKey ekey(key);
-				exif->iptcData().add(ekey, nullptr);
-				auto pos = exif->iptcData().end();
-				pos--;
-				item = &*pos;
-			}
-			catch (Exiv2::AnyError& e) {
-				Exiv2::XmpKey ekey(key);
-				exif->xmpData().add(ekey, nullptr);
-				auto pos = exif->xmpData().end();
-				pos--;
-				item = &*pos;
-			}
-		}
-		return reinterpret_cast<ExifItem *>(item);
-	}
-	catch (Exiv2::AnyError& e) {
-		debug_exception(e);
-		return nullptr;
-	}
-}
-
 
 ExifItem *exif_get_first_item(ExifData *exif)
 {
@@ -551,19 +518,19 @@ ExifItem *exif_get_first_item(ExifData *exif)
 		if (exif->exifIter != exif->exifData().end())
 			{
 			const Exiv2::Metadatum *item = &*exif->exifIter;
-			exif->exifIter++;
+			++exif->exifIter;
 			return (ExifItem *)item;
 			}
 		if (exif->iptcIter != exif->iptcData().end())
 			{
 			const Exiv2::Metadatum *item = &*exif->iptcIter;
-			exif->iptcIter++;
+			++exif->iptcIter;
 			return (ExifItem *)item;
 			}
 		if (exif->xmpIter != exif->xmpData().end())
 			{
 			const Exiv2::Metadatum *item = &*exif->xmpIter;
-			exif->xmpIter++;
+			++exif->xmpIter;
 			return (ExifItem *)item;
 			}
 		return nullptr;
@@ -581,19 +548,19 @@ ExifItem *exif_get_next_item(ExifData *exif)
 		if (exif->exifIter != exif->exifData().end())
 			{
 			const Exiv2::Metadatum *item = &*exif->exifIter;
-			exif->exifIter++;
+			++exif->exifIter;
 			return (ExifItem *)item;
 		}
 		if (exif->iptcIter != exif->iptcData().end())
 			{
 			const Exiv2::Metadatum *item = &*exif->iptcIter;
-			exif->iptcIter++;
+			++exif->iptcIter;
 			return (ExifItem *)item;
 		}
 		if (exif->xmpIter != exif->xmpData().end())
 			{
 			const Exiv2::Metadatum *item = &*exif->xmpIter;
-			exif->xmpIter++;
+			++exif->xmpIter;
 			return (ExifItem *)item;
 		}
 		return nullptr;
