@@ -24,7 +24,7 @@
 #include <glib.h>
 
 struct FileCacheData;
-struct FileData;
+class FileData;
 
 using FileCacheReleaseFunc = void (*)(FileData *);
 
@@ -33,7 +33,6 @@ gboolean file_cache_get(FileCacheData *fc, FileData *fd);
 void file_cache_put(FileCacheData *fc, FileData *fd, gulong size);
 void file_cache_dump(FileCacheData *fc);
 void file_cache_set_size(FileCacheData *fc, gulong size);
-gulong file_cache_get_size(FileCacheData *fc);
 void file_cache_set_max_size(FileCacheData *fc, gulong size);
 
 
