@@ -19,22 +19,12 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#include <config.h>
+#include "format-canon.h"
 
-#ifndef HAVE_EXIV2
-
-#include <cstdio>
 #include <cstring>
-#include <unistd.h>
 
 #include <glib.h>
 
-#include "intl.h"
-
-#include "main.h"
-#include "format-canon.h"
-#include "format-raw.h"
-
 #include "exif.h"
 
 
@@ -162,11 +152,11 @@ gboolean format_canon_raw_crw(guchar *data, const guint len,
 
 	/* CRW header starts with 2 bytes for byte order (always "II", little endian),
 	 * 4 bytes for start of root block,
-	 * and 8 bytes of magic for file type and format "HEAPCCDR"
+	 * and 8 bytes of magic for file type and format 'HEAPCCDR'
 	 * (also 4 bytes for file version, and 8 bytes reserved)
 	 *
 	 * CIFF specification in pdf format is available on some websites,
-	 * search for "CIFFspecV1R03.pdf" or "CIFFspecV1R04.pdf"
+	 * search for 'CIFFspecV1R03.pdf' or 'CIFFspecV1R04.pdf'
 	 */
 	if (len < CRW_HEADER_SIZE ||
 	    memcmp(data, "II", 2) != 0 ||
@@ -528,8 +518,4 @@ gboolean format_canon_makernote(ExifData *exif, guchar *tiff, guint offset,
 	return TRUE;
 }
 
-#else
-using dummy_variable = int;
-#endif
-/* not HAVE_EXIV2 */
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
