@@ -17,22 +17,12 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#include <config.h>
+#include "format-olympus.h"
 
-#ifndef HAVE_EXIV2
-
-#include <cstdio>
 #include <cstring>
-#include <unistd.h>
 
 #include <glib.h>
 
-#include "intl.h"
-
-#include "main.h"
-#include "format-olympus.h"
-#include "format-raw.h"
-
 #include "exif.h"
 
 
@@ -380,8 +370,4 @@ gboolean format_olympus_makernote(ExifData *exif, guchar *tiff, guint offset,
 	return TRUE;
 }
 
-#else
-using dummy_variable = int;
-#endif
-/* not HAVE_EXIV2 */
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
