@@ -59,32 +59,17 @@ enum {
 
 void clear_mouse_cursor(GtkWidget *widget, gint state)
 {
-	GdkWindow *window = gtk_widget_get_window(widget);
-	GdkDisplay *display;
-
-	if (!window) return;
-
-	display = gdk_display_get_default();
-
 	if (state & FULLSCREEN_CURSOR_BUSY)
 		{
-		GdkCursor *cursor;
-
-		cursor = gdk_cursor_new_for_display(display, GDK_WATCH);
-		gdk_window_set_cursor(window, cursor);
-		g_object_unref(G_OBJECT(cursor));
+		widget_set_cursor(widget, GDK_WATCH);
 		}
 	else if (state & FULLSCREEN_CURSOR_NORMAL)
 		{
-		gdk_window_set_cursor(window, nullptr);
+		widget_set_cursor(widget, -1);
 		}
 	else
 		{
-		GdkCursor *cursor;
-
-		cursor = gdk_cursor_new_for_display(display, GDK_BLANK_CURSOR);
-		gdk_window_set_cursor(window, cursor);
-		g_object_unref(G_OBJECT(cursor));
+		widget_set_cursor(widget, GDK_BLANK_CURSOR);
 		}
 }
 
@@ -199,14 +184,7 @@ void fullscreen_image_complete_cb(ImageWindow *, gboolean preload, gpointer data
 
 void fullscreen_saver_deactivate()
 {
-	static gboolean checked = FALSE;
-	static gboolean found = FALSE;
-
-	if (!checked)
-		{
-		checked = TRUE;
-		found = file_in_path(XSCREENSAVER_BINARY);
-		}
+	static gboolean found = file_in_path(XSCREENSAVER_BINARY);
 
 	if (found)
 		{
@@ -377,29 +355,6 @@ GdkRectangle fullscreen_prefs_get_geometry(gint screen_num, GtkWidget *widget, G
 	return geometry;
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-gint fullscreen_prefs_find_screen_for_widget_unused(GtkWidget *widget)
-{
-	GdkScreen *screen;
-	gint monitor;
-	gint n;
-
-	if (!widget || !gtk_widget_get_window(widget)) return 0;
-
-	screen = gtk_widget_get_screen(widget);
-	G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
-	monitor = gdk_screen_get_monitor_at_window(screen, gtk_widget_get_window(widget));
-	G_GNUC_END_IGNORE_DEPRECATIONS;
-
-	n = 100 + monitor + 1;
-
-	DEBUG_1("Screen appears to be %d", n);
-
-	return n;
-}
-#pragma GCC diagnostic pop
-
 enum {
 	FS_MENU_COLUMN_NAME = 0,
 	FS_MENU_COLUMN_VALUE
@@ -458,6 +413,12 @@ FullScreenData *fullscreen_start(GtkWidget *window, ImageWindow *imd,
 
 	fs->cursor_state = FULLSCREEN_CURSOR_HIDDEN;
 
+	fs->osd_flags = image_osd_get(imd);
+	if (options->hide_osd_in_fullscreen)
+		{
+		image_osd_set(imd, OSD_SHOW_NOTHING);
+		}
+
 	fs->normal_window = window;
 	fs->normal_imd = imd;
 
@@ -582,6 +543,11 @@ void fullscreen_stop(FullScreenData *fs)
 {
 	if (!fs) return;
 
+	if (options->hide_osd_in_fullscreen)
+		{
+		image_osd_set(fs->normal_imd, fs->osd_flags);
+		}
+
 	if (fs->saver_block_id) g_source_remove(fs->saver_block_id);
 
 	fullscreen_hide_mouse_disable(fs);
