@@ -24,23 +24,38 @@
 
  */
 
-#include "main.h"
-
-#include "image-load.h"
 #include "image-load-dds.h"
 
-struct ImageLoaderDDS {
-	ImageLoaderBackendCbAreaUpdated area_updated_cb;
-	ImageLoaderBackendCbSize size_cb;
-	ImageLoaderBackendCbAreaPrepared area_prepared_cb;
+#include <sys/types.h>
+
+#include <gdk-pixbuf/gdk-pixbuf.h>
+#include <glib-object.h>
+#include <glib.h>
+
+#include "image-load.h"
+
+namespace
+{
+
+struct ImageLoaderDDS : public ImageLoaderBackend
+{
+public:
+	~ImageLoaderDDS() override;
+
+	void init(AreaUpdatedCb area_updated_cb, SizePreparedCb size_prepared_cb, AreaPreparedCb area_prepared_cb, gpointer data) override;
+	gboolean write(const guchar *buf, gsize &chunk_size, gsize count, GError **error) override;
+	GdkPixbuf *get_pixbuf() override;
+	gchar *get_format_name() override;
+	gchar **get_format_mime_types() override;
+
+private:
+	AreaUpdatedCb area_updated_cb;
 	gpointer data;
+
 	GdkPixbuf *pixbuf;
-	guint requested_width;
-	guint requested_height;
-	gboolean abort;
 };
 
-static void free_buffer(guchar *pixels, gpointer)
+void free_buffer(guchar *pixels, gpointer)
 {
 	g_free(pixels);
 }
@@ -53,13 +68,6 @@ uint ddsGetWidth(unsigned const char * buffer) {
 	return (buffer[16] & 0xFF) | (buffer[17] & 0xFF) << 8 | (buffer[18] & 0xFF) << 16 | (buffer[19] & 0xFF) << 24;
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-uint ddsGetMipmap_unused(unsigned const char * buffer) {
-	return (buffer[28] & 0xFF) | (buffer[29] & 0xFF) << 8 | (buffer[30] & 0xFF) << 16 | (buffer[31] & 0xFF) << 24;
-}
-#pragma GCC diagnostic pop
-
 uint ddsGetPixelFormatFlags(unsigned const char * buffer) {
 	return (buffer[80] & 0xFF) | (buffer[81] & 0xFF) << 8 | (buffer[82] & 0xFF) << 16 | (buffer[83] & 0xFF) << 24;
 }
@@ -89,39 +97,41 @@ uint ddsGetAlphaMask(unsigned const char * buffer) {
 }
 
 // Image Type
-#define DXT1 (0x44585431)
-#define DXT2 (0x44585432)
-#define DXT3 (0x44585433)
-#define DXT4 (0x44585434)
-#define DXT5 (0x44585435)
-#define A1R5G5B5 ((1 << 16) | 2)
-#define X1R5G5B5 ((2 << 16) | 2)
-#define A4R4G4B4 ((3 << 16) | 2)
-#define X4R4G4B4 ((4 << 16) | 2)
-#define R5G6B5 ((5 << 16) | 2)
-#define R8G8B8 ((1 << 16) | 3)
-#define A8B8G8R8 ((1 << 16) | 4)
-#define X8B8G8R8 ((2 << 16) | 4)
-#define A8R8G8B8 ((3 << 16) | 4)
-#define X8R8G8B8 ((4 << 16) | 4)
+enum {
+	DXT1 = (0x44585431),
+	DXT2 = (0x44585432),
+	DXT3 = (0x44585433),
+	DXT4 = (0x44585434),
+	DXT5 = (0x44585435),
+	A1R5G5B5 = ((1 << 16) | 2),
+	X1R5G5B5 = ((2 << 16) | 2),
+	A4R4G4B4 = ((3 << 16) | 2),
+	X4R4G4B4 = ((4 << 16) | 2),
+	R5G6B5 = ((5 << 16) | 2),
+	R8G8B8 = ((1 << 16) | 3),
+	A8B8G8R8 = ((1 << 16) | 4),
+	X8B8G8R8 = ((2 << 16) | 4),
+	A8R8G8B8 = ((3 << 16) | 4),
+	X8R8G8B8 = ((4 << 16) | 4)
+};
 
 // RGBA Masks
-static const uint A1R5G5B5_MASKS[] = { 0x7C00, 0x03E0, 0x001F, 0x8000 };
-static const uint X1R5G5B5_MASKS[] = { 0x7C00, 0x03E0, 0x001F, 0x0000 };
-static const uint A4R4G4B4_MASKS[] = { 0x0F00, 0x00F0, 0x000F, 0xF000 };
-static const uint X4R4G4B4_MASKS[] = { 0x0F00, 0x00F0, 0x000F, 0x0000 };
-static const uint R5G6B5_MASKS[] = { 0xF800, 0x07E0, 0x001F, 0x0000 };
-static const uint R8G8B8_MASKS[] = { 0xFF0000, 0x00FF00, 0x0000FF, 0x000000 };
-static const uint A8B8G8R8_MASKS[] = { 0x000000FF, 0x0000FF00, 0x00FF0000, 0xFF000000 };
-static const uint X8B8G8R8_MASKS[] = { 0x000000FF, 0x0000FF00, 0x00FF0000, 0x00000000 };
-static const uint A8R8G8B8_MASKS[] = { 0x00FF0000, 0x0000FF00, 0x000000FF, 0xFF000000 };
-static const uint X8R8G8B8_MASKS[] = { 0x00FF0000, 0x0000FF00, 0x000000FF, 0x00000000 };
+constexpr uint A1R5G5B5_MASKS[] = { 0x7C00, 0x03E0, 0x001F, 0x8000 };
+constexpr uint X1R5G5B5_MASKS[] = { 0x7C00, 0x03E0, 0x001F, 0x0000 };
+constexpr uint A4R4G4B4_MASKS[] = { 0x0F00, 0x00F0, 0x000F, 0xF000 };
+constexpr uint X4R4G4B4_MASKS[] = { 0x0F00, 0x00F0, 0x000F, 0x0000 };
+constexpr uint R5G6B5_MASKS[] = { 0xF800, 0x07E0, 0x001F, 0x0000 };
+constexpr uint R8G8B8_MASKS[] = { 0xFF0000, 0x00FF00, 0x0000FF, 0x000000 };
+constexpr uint A8B8G8R8_MASKS[] = { 0x000000FF, 0x0000FF00, 0x00FF0000, 0xFF000000 };
+constexpr uint X8B8G8R8_MASKS[] = { 0x000000FF, 0x0000FF00, 0x00FF0000, 0x00000000 };
+constexpr uint A8R8G8B8_MASKS[] = { 0x00FF0000, 0x0000FF00, 0x000000FF, 0xFF000000 };
+constexpr uint X8R8G8B8_MASKS[] = { 0x00FF0000, 0x0000FF00, 0x000000FF, 0x00000000 };
 
 // BIT4 = 17 * index;
-static const uint BIT5[] = { 0, 8, 16, 25, 33, 41, 49, 58, 66, 74, 82, 90, 99, 107, 115, 123, 132, 140, 148, 156, 165, 173, 181, 189, 197, 206, 214, 222, 230, 239, 247, 255 };
-static const uint BIT6[] = { 0, 4, 8, 12, 16, 20, 24, 28, 32, 36, 40, 45, 49, 53, 57, 61, 65, 69, 73, 77, 81, 85, 89, 93, 97, 101, 105, 109, 113, 117, 121, 125, 130, 134, 138, 142, 146, 150, 154, 158, 162, 166, 170, 174, 178, 182, 186, 190, 194, 198, 202, 206, 210, 215, 219, 223, 227, 231, 235, 239, 243, 247, 251, 255 };
+constexpr uint BIT5[] = { 0, 8, 16, 25, 33, 41, 49, 58, 66, 74, 82, 90, 99, 107, 115, 123, 132, 140, 148, 156, 165, 173, 181, 189, 197, 206, 214, 222, 230, 239, 247, 255 };
+constexpr uint BIT6[] = { 0, 4, 8, 12, 16, 20, 24, 28, 32, 36, 40, 45, 49, 53, 57, 61, 65, 69, 73, 77, 81, 85, 89, 93, 97, 101, 105, 109, 113, 117, 121, 125, 130, 134, 138, 142, 146, 150, 154, 158, 162, 166, 170, 174, 178, 182, 186, 190, 194, 198, 202, 206, 210, 215, 219, 223, 227, 231, 235, 239, 243, 247, 251, 255 };
 
-static uint ddsGetType(const unsigned char *buffer) {
+uint ddsGetType(const unsigned char *buffer) {
 	uint type = 0;
 	uint flags = ddsGetPixelFormatFlags(buffer);
 	if ((flags & 0x04) != 0) {
@@ -226,6 +236,8 @@ uint ddsGetDXTColor(uint c0, uint c1, uint a, uint t) {
 	case 1: return ddsGetDXTColor1(c1, a);
 	case 2: return (c0 > c1) ? ddsGetDXTColor2_1(c0, c1, a) : ddsGetDXTColor1_1(c0, c1, a);
 	case 3: return (c0 > c1) ? ddsGetDXTColor2_1(c1, c0, a) : 0;
+	default:
+		break;
 	}
 	return 0;
 }
@@ -313,6 +325,8 @@ int ddsGetDXT5Alpha(uint a0, uint a1, uint t) {
 	case 5: return (3 * a0 + 4 * a1) / 7;
 	case 6: return (2 * a0 + 5 * a1) / 7;
 	case 7: return (a0 + 6 * a1) / 7;
+	default:
+		break;
 	}
 	else switch (t) {
 	case 0: return a0;
@@ -323,6 +337,8 @@ int ddsGetDXT5Alpha(uint a0, uint a1, uint t) {
 	case 5: return (a0 + 4 * a1) / 5;
 	case 6: return 0;
 	case 7: return 255;
+	default:
+		break;
 	}
 	return 0;
 }
@@ -527,9 +543,8 @@ guchar *ddsReadX8R8G8B8(uint width, uint height, const unsigned char *buffer) {
 	return reinterpret_cast<guchar *>(pixels);
 }
 
-static gboolean image_loader_dds_load (gpointer loader, const guchar *buf, gsize, GError **)
+gboolean ImageLoaderDDS::write(const guchar *buf, gsize &chunk_size, gsize count, GError **)
 {
-	auto ld = static_cast<ImageLoaderDDS *>(loader);
 	uint width = ddsGetWidth(buf);
 	uint height = ddsGetHeight(buf);
 	uint type = ddsGetType(buf);
@@ -553,76 +568,48 @@ static gboolean image_loader_dds_load (gpointer loader, const guchar *buf, gsize
 		case X8B8G8R8: pixels = ddsReadX8B8G8R8(width, height, buf); break;
 		case A8R8G8B8: pixels = ddsReadA8R8G8B8(width, height, buf); break;
 		case X8R8G8B8: pixels = ddsReadX8R8G8B8(width, height, buf); break;
+		default:
+			break;
 		}
-		ld->pixbuf = gdk_pixbuf_new_from_data (pixels, GDK_COLORSPACE_RGB, TRUE, 8, width, height, rowstride, free_buffer, nullptr);
-		ld->area_updated_cb(loader, 0, 0, width, height, ld->data);
+		pixbuf = gdk_pixbuf_new_from_data (pixels, GDK_COLORSPACE_RGB, TRUE, 8, width, height, rowstride, free_buffer, nullptr);
+		area_updated_cb(nullptr, 0, 0, width, height, data);
+		chunk_size = count;
 		return TRUE;
 	}
 }
 
-static gpointer image_loader_dds_new(ImageLoaderBackendCbAreaUpdated area_updated_cb, ImageLoaderBackendCbSize size_cb, ImageLoaderBackendCbAreaPrepared area_prepared_cb, gpointer data)
+void ImageLoaderDDS::init(AreaUpdatedCb area_updated_cb, SizePreparedCb, AreaPreparedCb, gpointer data)
 {
-	auto loader = g_new0(ImageLoaderDDS, 1);
-	loader->area_updated_cb = area_updated_cb;
-	loader->size_cb = size_cb;
-	loader->area_prepared_cb = area_prepared_cb;
-	loader->data = data;
-	return loader;
+	this->area_updated_cb = area_updated_cb;
+	this->data = data;
 }
 
-static void image_loader_dds_set_size(gpointer loader, int width, int height)
+GdkPixbuf *ImageLoaderDDS::get_pixbuf()
 {
-	auto ld = static_cast<ImageLoaderDDS *>(loader);
-	ld->requested_width = width;
-	ld->requested_height = height;
+	return pixbuf;
 }
 
-static GdkPixbuf* image_loader_dds_get_pixbuf(gpointer loader)
-{
-	auto ld = static_cast<ImageLoaderDDS *>(loader);
-	return ld->pixbuf;
-}
-
-static gchar* image_loader_dds_get_format_name(gpointer)
+gchar *ImageLoaderDDS::get_format_name()
 {
 	return g_strdup("dds");
 }
-static gchar** image_loader_dds_get_format_mime_types(gpointer)
+
+gchar **ImageLoaderDDS::get_format_mime_types()
 {
 	static const gchar *mime[] = {"image/vnd-ms.dds", nullptr};
 	return g_strdupv(const_cast<gchar **>(mime));
 }
 
-static gboolean image_loader_dds_close(gpointer, GError **)
+ImageLoaderDDS::~ImageLoaderDDS()
 {
-	return TRUE;
+	if (pixbuf) g_object_unref(pixbuf);
 }
 
-static void image_loader_dds_abort(gpointer loader)
-{
-	auto ld = static_cast<ImageLoaderDDS *>(loader);
-	ld->abort = TRUE;
-}
+} // namespace
 
-static void image_loader_dds_free(gpointer loader)
+std::unique_ptr<ImageLoaderBackend> get_image_loader_backend_dds()
 {
-	auto ld = static_cast<ImageLoaderDDS *>(loader);
-	if (ld->pixbuf) g_object_unref(ld->pixbuf);
-	g_free(ld);
-}
-
-void image_loader_backend_set_dds(ImageLoaderBackend *funcs)
-{
-	funcs->loader_new = image_loader_dds_new;
-	funcs->set_size = image_loader_dds_set_size;
-	funcs->load = image_loader_dds_load;
-	funcs->write = nullptr;
-	funcs->get_pixbuf = image_loader_dds_get_pixbuf;
-	funcs->close = image_loader_dds_close;
-	funcs->abort = image_loader_dds_abort;
-	funcs->free = image_loader_dds_free;
-	funcs->get_format_name = image_loader_dds_get_format_name;
-	funcs->get_format_mime_types = image_loader_dds_get_format_mime_types;
+	return std::make_unique<ImageLoaderDDS>();
 }
 
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
