@@ -50,34 +50,49 @@
  *
  */
 
-#include "main.h"
-
-#include "image-load.h"
 #include "image-load-jpegxl.h"
 
-#ifdef HAVE_JPEGXL
-
+#include <cstdint>
+#include <cstdlib>
 #include <memory>
 
-#include "jxl/decode.h"
+#include <gdk-pixbuf/gdk-pixbuf.h>
+#include <glib-object.h>
+#include <glib.h>
+#include <jxl/codestream_header.h>
+#include <jxl/decode.h> //TODO Use decode_cxx.h?
+#include <jxl/types.h>
 
-struct ImageLoaderJPEGXL {
-	ImageLoaderBackendCbAreaUpdated area_updated_cb;
-	ImageLoaderBackendCbSize size_cb;
-	ImageLoaderBackendCbAreaPrepared area_prepared_cb;
+#include "debug.h"
+#include "image-load.h"
+
+namespace
+{
+
+struct ImageLoaderJPEGXL : public ImageLoaderBackend
+{
+public:
+	~ImageLoaderJPEGXL() override;
+
+	void init(AreaUpdatedCb area_updated_cb, SizePreparedCb size_prepared_cb, AreaPreparedCb area_prepared_cb, gpointer data) override;
+	gboolean write(const guchar *buf, gsize &chunk_size, gsize count, GError **error) override;
+	GdkPixbuf *get_pixbuf() override;
+	gchar *get_format_name() override;
+	gchar **get_format_mime_types() override;
+
+private:
+	AreaUpdatedCb area_updated_cb;
 	gpointer data;
+
 	GdkPixbuf *pixbuf;
-	guint requested_width;
-	guint requested_height;
-	gboolean abort;
 };
 
-static void free_buffer(guchar *pixels, gpointer)
+void free_buffer(guchar *pixels, gpointer)
 {
 	g_free(pixels);
 }
 
-static uint8_t *JxlMemoryToPixels(const uint8_t *next_in, size_t size, size_t &xsize, size_t &ysize, size_t &stride)
+uint8_t *JxlMemoryToPixels(const uint8_t *next_in, size_t size, size_t &xsize, size_t &ysize, size_t &stride)
 {
 	std::unique_ptr<JxlDecoder, decltype(&JxlDecoderDestroy)> dec{JxlDecoderCreate(nullptr), JxlDecoderDestroy};
 	if (!dec)
@@ -160,94 +175,61 @@ static uint8_t *JxlMemoryToPixels(const uint8_t *next_in, size_t size, size_t &x
 	return nullptr;
 }
 
-static gboolean image_loader_jpegxl_load(gpointer loader, const guchar *buf, gsize count, GError **)
+gboolean ImageLoaderJPEGXL::write(const guchar *buf, gsize &chunk_size, gsize count, GError **)
 {
-	auto ld = static_cast<ImageLoaderJPEGXL *>(loader);
 	gboolean ret = FALSE;
 	size_t xsize;
 	size_t ysize;
 	size_t stride;
-	uint8_t *decoded = nullptr;
+	uint8_t *pixels = nullptr;
 
-	decoded = JxlMemoryToPixels(buf, count, xsize, ysize, stride);
+	pixels = JxlMemoryToPixels(buf, count, xsize, ysize, stride);
 
-	if (decoded)
+	if (pixels)
 		{
-		ld->pixbuf = gdk_pixbuf_new_from_data(decoded, GDK_COLORSPACE_RGB, TRUE, 8, xsize, ysize, stride, free_buffer, nullptr);
+		pixbuf = gdk_pixbuf_new_from_data(pixels, GDK_COLORSPACE_RGB, TRUE, 8, xsize, ysize, stride, free_buffer, nullptr);
 
-		ld->area_updated_cb(loader, 0, 0, xsize, ysize, ld->data);
+		area_updated_cb(nullptr, 0, 0, xsize, ysize, data);
 
+		chunk_size = count;
 		ret = TRUE;
 		}
 
 	return ret;
 }
 
-static gpointer image_loader_jpegxl_new(ImageLoaderBackendCbAreaUpdated area_updated_cb, ImageLoaderBackendCbSize size_cb, ImageLoaderBackendCbAreaPrepared area_prepared_cb, gpointer data)
+void ImageLoaderJPEGXL::init(AreaUpdatedCb area_updated_cb, SizePreparedCb, AreaPreparedCb, gpointer data)
 {
-	auto loader = g_new0(ImageLoaderJPEGXL, 1);
-	loader->area_updated_cb = area_updated_cb;
-	loader->size_cb = size_cb;
-	loader->area_prepared_cb = area_prepared_cb;
-	loader->data = data;
-	return loader;
+	this->area_updated_cb = area_updated_cb;
+	this->data = data;
 }
 
-static void image_loader_jpegxl_set_size(gpointer loader, int width, int height)
+GdkPixbuf *ImageLoaderJPEGXL::get_pixbuf()
 {
-	auto ld = static_cast<ImageLoaderJPEGXL *>(loader);
-	ld->requested_width = width;
-	ld->requested_height = height;
+	return pixbuf;
 }
 
-static GdkPixbuf* image_loader_jpegxl_get_pixbuf(gpointer loader)
-{
-	auto ld = static_cast<ImageLoaderJPEGXL *>(loader);
-	return ld->pixbuf;
-}
-
-static gchar* image_loader_jpegxl_get_format_name(gpointer)
+gchar *ImageLoaderJPEGXL::get_format_name()
 {
 	return g_strdup("jxl");
 }
 
-static gchar** image_loader_jpegxl_get_format_mime_types(gpointer)
+gchar **ImageLoaderJPEGXL::get_format_mime_types()
 {
 	static const gchar *mime[] = {"image/jxl", nullptr};
 	return g_strdupv(const_cast<gchar **>(mime));
 }
 
-static gboolean image_loader_jpegxl_close(gpointer, GError **)
+ImageLoaderJPEGXL::~ImageLoaderJPEGXL()
 {
-	return TRUE;
+	if (pixbuf) g_object_unref(pixbuf);
 }
 
-static void image_loader_jpegxl_abort(gpointer loader)
+} // namespace
+
+std::unique_ptr<ImageLoaderBackend> get_image_loader_backend_jpegxl()
 {
-	auto ld = static_cast<ImageLoaderJPEGXL *>(loader);
-	ld->abort = TRUE;
+	return std::make_unique<ImageLoaderJPEGXL>();
 }
 
-static void image_loader_jpegxl_free(gpointer loader)
-{
-	auto ld = static_cast<ImageLoaderJPEGXL *>(loader);
-	if (ld->pixbuf) g_object_unref(ld->pixbuf);
-	g_free(ld);
-}
-
-void image_loader_backend_set_jpegxl(ImageLoaderBackend *funcs)
-{
-	funcs->loader_new = image_loader_jpegxl_new;
-	funcs->set_size = image_loader_jpegxl_set_size;
-	funcs->load = image_loader_jpegxl_load;
-	funcs->write = nullptr;
-	funcs->get_pixbuf = image_loader_jpegxl_get_pixbuf;
-	funcs->close = image_loader_jpegxl_close;
-	funcs->abort = image_loader_jpegxl_abort;
-	funcs->free = image_loader_jpegxl_free;
-	funcs->get_format_name = image_loader_jpegxl_get_format_name;
-	funcs->get_format_mime_types = image_loader_jpegxl_get_format_mime_types;
-}
-
-#endif
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
