@@ -18,23 +18,37 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#include "main.h"
-
-#include "image-load.h"
 #include "image-load-zxscr.h"
 
-struct ImageLoaderZXSCR {
-	ImageLoaderBackendCbAreaUpdated area_updated_cb;
-	ImageLoaderBackendCbSize size_cb;
-	ImageLoaderBackendCbAreaPrepared area_prepared_cb;
+#include <gdk-pixbuf/gdk-pixbuf.h>
+#include <glib-object.h>
+#include <glib.h>
+
+#include "debug.h"
+#include "image-load.h"
+
+namespace
+{
+
+struct ImageLoaderZXSCR : public ImageLoaderBackend
+{
+public:
+	~ImageLoaderZXSCR() override;
+
+	void init(AreaUpdatedCb area_updated_cb, SizePreparedCb size_prepared_cb, AreaPreparedCb area_prepared_cb, gpointer data) override;
+	gboolean write(const guchar *buf, gsize &chunk_size, gsize count, GError **error) override;
+	GdkPixbuf *get_pixbuf() override;
+	gchar *get_format_name() override;
+	gchar **get_format_mime_types() override;
+
+private:
+	AreaUpdatedCb area_updated_cb;
 	gpointer data;
+
 	GdkPixbuf *pixbuf;
-	guint requested_width;
-	guint requested_height;
-	gboolean abort;
 };
 
-const guchar palette[2][8][3] = {
+constexpr guchar palette[2][8][3] = {
 	{
 		{0x00, 0x00, 0x00},
 		{0x00, 0x00, 0xbf},
@@ -56,18 +70,25 @@ const guchar palette[2][8][3] = {
 	}
 };
 
-static void free_buffer(guchar *pixels, gpointer)
+void free_buffer(guchar *pixels, gpointer)
 {
 	g_free(pixels);
 }
 
-static gboolean image_loader_zxscr_load(gpointer loader, const guchar *buf, gsize count, GError **)
+gboolean ImageLoaderZXSCR::write(const guchar *buf, gsize &chunk_size, gsize count, GError **)
 {
-	auto ld = static_cast<ImageLoaderZXSCR *>(loader);
 	guint8 *pixels;
-	gint width, height;
-	gint row, col, mrow, pxs, i;
-	guint8 attr, bright, ink, paper;
+	gint width;
+	gint height;
+	gint row;
+	gint col;
+	gint mrow;
+	gint pxs;
+	gint i;
+	guint8 attr;
+	guint8 bright;
+	guint8 ink;
+	guint8 paper;
 	guint8 *ptr;
 
 	if (count != 6144 && count != 6912)
@@ -87,9 +108,9 @@ static gboolean image_loader_zxscr_load(gpointer loader, const guchar *buf, gsiz
 		return FALSE;
 		}
 
-	ld->pixbuf = gdk_pixbuf_new_from_data(pixels, GDK_COLORSPACE_RGB, FALSE, 8, width, height, width * 3, free_buffer, nullptr);
+	pixbuf = gdk_pixbuf_new_from_data(pixels, GDK_COLORSPACE_RGB, FALSE, 8, width, height, width * 3, free_buffer, nullptr);
 
-	if (!ld->pixbuf)
+	if (!pixbuf)
 		{
 		g_free(pixels);
 		DEBUG_1("Insufficient memory to open ZXSCR file");
@@ -138,74 +159,43 @@ static gboolean image_loader_zxscr_load(gpointer loader, const guchar *buf, gsiz
 				}
 			}
 
-	ld->area_updated_cb(loader, 0, 0, width, height, ld->data);
+	area_updated_cb(nullptr, 0, 0, width, height, data);
 
+	chunk_size = count;
 	return TRUE;
 }
 
-static gpointer image_loader_zxscr_new(ImageLoaderBackendCbAreaUpdated area_updated_cb, ImageLoaderBackendCbSize size_cb, ImageLoaderBackendCbAreaPrepared area_prepared_cb, gpointer data)
+void ImageLoaderZXSCR::init(AreaUpdatedCb area_updated_cb, SizePreparedCb, AreaPreparedCb, gpointer data)
 {
-	auto loader = g_new0(ImageLoaderZXSCR, 1);
-	loader->area_updated_cb = area_updated_cb;
-	loader->size_cb = size_cb;
-	loader->area_prepared_cb = area_prepared_cb;
-	loader->data = data;
-	return loader;
+	this->area_updated_cb = area_updated_cb;
+	this->data = data;
 }
 
-static void image_loader_zxscr_set_size(gpointer loader, int width, int height)
+GdkPixbuf *ImageLoaderZXSCR::get_pixbuf()
 {
-	auto ld = static_cast<ImageLoaderZXSCR *>(loader);
-	ld->requested_width = width;
-	ld->requested_height = height;
+	return pixbuf;
 }
 
-static GdkPixbuf *image_loader_zxscr_get_pixbuf(gpointer loader)
-{
-	auto ld = static_cast<ImageLoaderZXSCR *>(loader);
-	return ld->pixbuf;
-}
-
-static gchar *image_loader_zxscr_get_format_name(gpointer)
+gchar *ImageLoaderZXSCR::get_format_name()
 {
 	return g_strdup("zxscr");
 }
 
-static gchar **image_loader_zxscr_get_format_mime_types(gpointer)
+gchar **ImageLoaderZXSCR::get_format_mime_types()
 {
 	static const gchar *mime[] = {"application/octet-stream", nullptr};
 	return g_strdupv(const_cast<gchar **>(mime));
 }
 
-static gboolean image_loader_zxscr_close(gpointer, GError **)
+ImageLoaderZXSCR::~ImageLoaderZXSCR()
 {
-	return TRUE;
+	if (pixbuf) g_object_unref(pixbuf);
 }
 
-static void image_loader_zxscr_abort(gpointer loader)
-{
-	auto ld = static_cast<ImageLoaderZXSCR *>(loader);
-	ld->abort = TRUE;
-}
+} // namespace
 
-static void image_loader_zxscr_free(gpointer loader)
+std::unique_ptr<ImageLoaderBackend> get_image_loader_backend_zxscr()
 {
-	auto ld = static_cast<ImageLoaderZXSCR *>(loader);
-	if (ld->pixbuf) g_object_unref(ld->pixbuf);
-	g_free(ld);
-}
-
-void image_loader_backend_set_zxscr(ImageLoaderBackend *funcs)
-{
-	funcs->loader_new = image_loader_zxscr_new;
-	funcs->set_size = image_loader_zxscr_set_size;
-	funcs->load = image_loader_zxscr_load;
-	funcs->write = nullptr;
-	funcs->get_pixbuf = image_loader_zxscr_get_pixbuf;
-	funcs->close = image_loader_zxscr_close;
-	funcs->abort = image_loader_zxscr_abort;
-	funcs->free = image_loader_zxscr_free;
-	funcs->get_format_name = image_loader_zxscr_get_format_name;
-	funcs->get_format_mime_types = image_loader_zxscr_get_format_mime_types;
+	return std::make_unique<ImageLoaderZXSCR>();
 }
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
