@@ -21,10 +21,7 @@
 
 #include "image-load.h"
 
-#include <fcntl.h>
 #include <sys/mman.h>
-#include <sys/stat.h>
-#include <unistd.h>
 
 #include <cstring>
 
@@ -70,6 +67,7 @@
 #endif
 #include "image-load-webp.h"
 #include "image-load-zxscr.h"
+#include "jpeg-parser.h"
 #include "misc.h"
 #include "options.h"
 #include "typedefs.h"
@@ -104,23 +102,20 @@ static void image_loader_stop(ImageLoader *il);
 
 GType image_loader_get_type()
 {
-	static GType type = 0;
-	if (type == 0)
-		{
-		static const GTypeInfo info = {
-			sizeof(ImageLoaderClass),
-			nullptr,   /* base_init */
-			nullptr,   /* base_finalize */
-			static_cast<GClassInitFunc>(image_loader_class_init_wrapper), /* class_init */
-			nullptr,   /* class_finalize */
-			nullptr,   /* class_data */
-			sizeof(ImageLoader),
-			0,      /* n_preallocs */
-			static_cast<GInstanceInitFunc>(image_loader_init), /* instance_init */
-			nullptr	/* value_table */
-			};
-		type = g_type_register_static(G_TYPE_OBJECT, "ImageLoaderType", &info, static_cast<GTypeFlags>(0));
-		}
+	static const GTypeInfo info = {
+	    sizeof(ImageLoaderClass),
+	    nullptr,   /* base_init */
+	    nullptr,   /* base_finalize */
+	    static_cast<GClassInitFunc>(image_loader_class_init_wrapper), /* class_init */
+	    nullptr,   /* class_finalize */
+	    nullptr,   /* class_data */
+	    sizeof(ImageLoader),
+	    0,      /* n_preallocs */
+	    static_cast<GInstanceInitFunc>(image_loader_init), /* instance_init */
+	    nullptr	/* value_table */
+	};
+	static GType type = g_type_register_static(G_TYPE_OBJECT, "ImageLoaderType", &info, static_cast<GTypeFlags>(0));
+
 	return type;
 }
 
@@ -939,9 +934,6 @@ static gboolean image_loader_begin(ImageLoader *il)
 
 static gboolean image_loader_setup_source(ImageLoader *il)
 {
-	struct stat st;
-	gchar *pathl;
-
 	if (!il || il->backend || il->mapped_file) return FALSE;
 
 	il->mapped_file = nullptr;
@@ -961,20 +953,11 @@ static gboolean image_loader_setup_source(ImageLoader *il)
 			}
 		else
 			{
-			il->mapped_file = libraw_get_preview(il, reinterpret_cast<guint *>(&il->bytes_total));
+			il->mapped_file = libraw_get_preview(il->fd->path, il->bytes_total);
 
 			if (il->mapped_file)
 				{
-				/* Both exiv2 and libraw sometimes return a pointer to a file
-				 * section that is not a jpeg */
-				if (il->mapped_file[0] != 0xFF || il->mapped_file[1] != 0xD8)
-					{
-					il->mapped_file = nullptr;
-					}
-				else
-					{
-					il->preview = IMAGE_LOADER_PREVIEW_LIBRAW;
-					}
+				il->preview = IMAGE_LOADER_PREVIEW_LIBRAW;
 				}
 			}
 
@@ -985,10 +968,10 @@ static gboolean image_loader_setup_source(ImageLoader *il)
 
 			if (il->mapped_file)
 				{
-				/* Both exiv2 and libraw sometimes return a pointer to a file
-				 * section that is not a jpeg */
-				if (il->mapped_file[0] != 0xFF || il->mapped_file[1] != 0xD8)
+				/* exiv2 sometimes returns a pointer to a file section that is not a jpeg */
+				if (!is_jpeg_container(il->mapped_file, il->bytes_total))
 					{
+					exif_free_preview(il->mapped_file);
 					il->mapped_file = nullptr;
 					}
 				else
@@ -1005,32 +988,14 @@ static gboolean image_loader_setup_source(ImageLoader *il)
 		exif_free_fd(il->fd, exif);
 		}
 
-
 	if (!il->mapped_file)
 		{
 		/* normal file */
-		gint load_fd;
+		g_autofree gchar *pathl = path_from_utf8(il->fd->path);
 
-		pathl = path_from_utf8(il->fd->path);
-		load_fd = open(pathl, O_RDONLY | O_NONBLOCK);
-		g_free(pathl);
-		if (load_fd == -1) return FALSE;
-
-		if (fstat(load_fd, &st) == 0)
+		il->mapped_file = map_file(pathl, il->bytes_total);
+		if (!il->mapped_file)
 			{
-			il->bytes_total = st.st_size;
-			}
-		else
-			{
-			close(load_fd);
-			return FALSE;
-			}
-
-		il->mapped_file = static_cast<guchar *>(mmap(nullptr, il->bytes_total, PROT_READ|PROT_WRITE, MAP_PRIVATE, load_fd, 0));
-		close(load_fd);
-		if (il->mapped_file == MAP_FAILED)
-			{
-			il->mapped_file = nullptr;
 			return FALSE;
 			}
 		il->preview = IMAGE_LOADER_PREVIEW_NONE;
@@ -1085,6 +1050,9 @@ static void image_loader_stop(ImageLoader *il)
 
 }
 
+/**
+ * @brief Delay area_ready signals
+ */
 void image_loader_delay_area_ready(ImageLoader *il, gboolean enable)
 {
 	g_mutex_lock(il->data_mutex);
@@ -1269,7 +1237,7 @@ static gboolean image_loader_start_thread(ImageLoader *il)
 	il->can_destroy = FALSE; /* ImageLoader can't be freed until image_loader_thread_run finishes */
 
 	g_thread_pool_push(image_loader_thread_pool, il, nullptr);
-	DEBUG_1("Thread pool num threads: %d", g_thread_pool_get_num_threads(image_loader_thread_pool));
+	DEBUG_1("Thread pool num threads: %u", g_thread_pool_get_num_threads(image_loader_thread_pool));
 
 	return TRUE;
 }
@@ -1301,6 +1269,11 @@ GdkPixbuf *image_loader_get_pixbuf(ImageLoader *il)
 	return ret;
 }
 
+/**
+ * @brief Speed up loading when you only need at most width x height size image,
+ * only the jpeg GdkPixbuf loader benefits from it - so there is no
+ * guarantee that the image will scale down to the requested size..
+ */
 void image_loader_set_requested_size(ImageLoader *il, gint width, gint height)
 {
 	if (!il) return;
@@ -1320,6 +1293,10 @@ void image_loader_set_buffer_size(ImageLoader *il, guint count)
 	g_mutex_unlock(il->data_mutex);
 }
 
+/**
+ * @brief This only has effect if used before image_loader_start()
+ * default is G_PRIORITY_DEFAULT_IDLE
+ */
 void image_loader_set_priority(ImageLoader *il, gint priority)
 {
 	if (!il) return;
