@@ -321,7 +321,7 @@ static GdkPixbuf *image_osd_info_render(OverlayStateData *osd)
 		osd_template_insert(vars, "number", g_strdup_printf("%d", n), OSDT_NO_DUP);
 		osd_template_insert(vars, "total", g_strdup_printf("%d", t), OSDT_NO_DUP);
 		osd_template_insert(vars, "name", const_cast<gchar *>(name), OSDT_NONE);
-		osd_template_insert(vars, "path", const_cast<gchar *>(image_get_path(imd)), OSDT_NONE);
+		osd_template_insert(vars, "path", image_get_path(imd), OSDT_NONE);
 		osd_template_insert(vars, "date", imd->image_fd ? (const_cast<gchar *>(text_from_time(imd->image_fd->date))) : "", OSDT_NONE);
 		osd_template_insert(vars, "size", imd->image_fd ? (text_from_size_abrev(imd->image_fd->size)) : g_strdup(""), OSDT_FREE);
 		osd_template_insert(vars, "zoom", image_zoom_get_as_text(imd), OSDT_FREE);
@@ -461,8 +461,8 @@ static GdkPixbuf *image_osd_info_render(OverlayStateData *osd)
 			pixbuf_set_rect_fill(pixbuf, x, y, w, HISTOGRAM_HEIGHT, 220, 220, 220, 210);
 			histogram_draw(osd->histogram, histmap, pixbuf, x, y, w, HISTOGRAM_HEIGHT);
 			}
-		pixbuf_draw_layout(pixbuf, layout, imd->pr, 5, 5, options->image_overlay.text_red, options->image_overlay.text_green,
-															options->image_overlay.text_blue, options->image_overlay.text_alpha);
+		pixbuf_draw_layout(pixbuf, layout, 5, 5,
+		                   options->image_overlay.text_red, options->image_overlay.text_green, options->image_overlay.text_blue, options->image_overlay.text_alpha);
 	}
 
 	g_object_unref(G_OBJECT(layout));
@@ -489,12 +489,12 @@ static GdkPixbuf *image_osd_info_render(OverlayStateData *osd)
  */
 static GdkPixbuf *image_osd_icon_pixbuf(ImageOSDFlag flag)
 {
-	static GdkPixbuf **icons = nullptr;
-	GdkPixbuf *icon = nullptr;
+	static auto **icons = g_new0(GdkPixbuf *, IMAGE_OSD_COUNT);
 
-	if (!icons) icons = g_new0(GdkPixbuf *, IMAGE_OSD_COUNT);
 	if (icons[flag]) return icons[flag];
 
+	GdkPixbuf *icon = nullptr;
+
 	if (osd_icons[flag].key)
 		{
 		icon = pixbuf_inline(osd_icons[flag].key);
@@ -511,9 +511,9 @@ static GdkPixbuf *image_osd_icon_pixbuf(ImageOSDFlag flag)
 				pixbuf_set_rect(icon, 3, 8, 11, 12,
 						0, 0, 0, 255,
 						3, 0, 3, 0);
-				pixbuf_draw_triangle(icon, 14, 3, 6, 12,
-						     20, 9, 14, 15, 14, 3,
-						     0, 0, 0, 255);
+				pixbuf_draw_triangle(icon, {14, 3, 6, 12},
+				                     {20, 9}, {14, 15}, {14, 3},
+				                     0, 0, 0, 255);
 				break;
 			case IMAGE_OSD_ROTATE_USER:
 				break;
@@ -524,15 +524,15 @@ static GdkPixbuf *image_osd_icon_pixbuf(ImageOSDFlag flag)
 				break;
 			case IMAGE_OSD_FIRST:
 				pixbuf_set_rect(icon, 3, 3, 18, 18, 0, 0, 0, 200, 3, 3, 3, 0);
-				pixbuf_draw_triangle(icon, 6, 5, 12, 6,
-						     12, 5, 18, 11, 6, 11,
-						     0, 0, 0, 255);
+				pixbuf_draw_triangle(icon, {6, 5, 12, 6},
+				                     {12, 5}, {18, 11}, {6, 11},
+				                     0, 0, 0, 255);
 				break;
 			case IMAGE_OSD_LAST:
 				pixbuf_set_rect(icon, 3, 3, 18, 18, 0, 0, 0, 200, 3, 3, 0, 3);
-				pixbuf_draw_triangle(icon, 6, 12, 12, 6,
-						     12, 18, 6, 12, 18, 12,
-						     0, 0, 0, 255);
+				pixbuf_draw_triangle(icon, {6, 12, 12, 6},
+				                     {12, 18}, {6, 12}, {18, 12},
+				                     0, 0, 0, 255);
 				break;
 			case IMAGE_OSD_ICON:
 				pixbuf_set_rect_fill(icon, 11, 3, 3, 12, 0, 0, 0, 255);
@@ -551,17 +551,17 @@ static GdkPixbuf *image_osd_icon_pixbuf(ImageOSDFlag flag)
 static gint image_overlay_add(ImageWindow *imd, GdkPixbuf *pixbuf, gint x, gint y,
 			      OverlayRendererFlags flags)
 {
-	return pixbuf_renderer_overlay_add(reinterpret_cast<PixbufRenderer *>(imd->pr), pixbuf, x, y, flags);
+	return pixbuf_renderer_overlay_add(PIXBUF_RENDERER(imd->pr), pixbuf, x, y, flags);
 }
 
 static void image_overlay_set(ImageWindow *imd, gint id, GdkPixbuf *pixbuf, gint x, gint y)
 {
-	pixbuf_renderer_overlay_set(reinterpret_cast<PixbufRenderer *>(imd->pr), id, pixbuf, x, y);
+	pixbuf_renderer_overlay_set(PIXBUF_RENDERER(imd->pr), id, pixbuf, x, y);
 }
 
 static void image_overlay_remove(ImageWindow *imd, gint id)
 {
-	pixbuf_renderer_overlay_remove(reinterpret_cast<PixbufRenderer *>(imd->pr), id);
+	pixbuf_renderer_overlay_remove(PIXBUF_RENDERER(imd->pr), id);
 }
 
 static void image_osd_icon_show(OverlayStateData *osd, ImageOSDFlag flag)
