@@ -184,7 +184,7 @@ static void image_press_cb(PixbufRenderer *pr, GdkEventButton *event, gpointer d
 
 	if (rect_id)
 		{
-		pixbuf_renderer_overlay_remove(reinterpret_cast<PixbufRenderer *>(imd->pr), rect_id);
+		pixbuf_renderer_overlay_remove(PIXBUF_RENDERER(imd->pr), rect_id);
 		}
 
 	lw = layout_find_by_image(imd);
@@ -217,6 +217,7 @@ static void image_release_cb(PixbufRenderer *, GdkEventButton *event, gpointer d
 static void image_drag_cb(PixbufRenderer *pr, GdkEventMotion *event, gpointer data)
 {
 	auto imd = static_cast<ImageWindow *>(data);
+	gdouble aspect_ratio = 0.0;
 	gint width;
 	gint height;
 	gint rect_width;
@@ -250,10 +251,44 @@ static void image_drag_cb(PixbufRenderer *pr, GdkEventMotion *event, gpointer da
 			image_y_pixel = y_pixel;
 			}
 
+		if (options->rectangle_draw_aspect_ratio != RECTANGLE_DRAW_ASPECT_RATIO_NONE)
+			{
+			switch (options->rectangle_draw_aspect_ratio)
+				{
+				case RECTANGLE_DRAW_ASPECT_RATIO_ONE_ONE:
+					aspect_ratio = 1.0;
+					break;
+				case RECTANGLE_DRAW_ASPECT_RATIO_FOUR_THREE:
+					aspect_ratio = gdouble(4) / 3;
+					break;
+				case RECTANGLE_DRAW_ASPECT_RATIO_THREE_TWO:
+					aspect_ratio = gdouble(3) / 2;
+					break;
+				case RECTANGLE_DRAW_ASPECT_RATIO_SIXTEEN_NINE:
+					aspect_ratio = gdouble(16) / 9;
+					break;
+				default:
+					aspect_ratio = 1.0;
+				}
+			}
+
+		if (options->rectangle_draw_aspect_ratio != RECTANGLE_DRAW_ASPECT_RATIO_NONE)
+			{
+			if (gdouble(image_x_pixel - image_start_x) / (image_y_pixel - image_start_y) < aspect_ratio)
+				{
+				image_x_pixel = image_start_x + ((image_y_pixel - image_start_y) * aspect_ratio);
+				}
+			else
+				{
+				image_y_pixel = image_start_y + ((image_x_pixel - image_start_x) / aspect_ratio);
+				}
+			}
+
 		switch_coords_orientation(imd, image_x_pixel, image_y_pixel, width, height);
+
 		if (rect_id)
 			{
-			pixbuf_renderer_overlay_remove(reinterpret_cast<PixbufRenderer *>(imd->pr), rect_id);
+			pixbuf_renderer_overlay_remove(PIXBUF_RENDERER(imd->pr), rect_id);
 			}
 
 		rect_width = pr->drag_last_x - pixbuf_start_x;
@@ -267,22 +302,31 @@ static void image_drag_cb(PixbufRenderer *pr, GdkEventMotion *event, gpointer da
 			rect_height = 1;
 			}
 
+		if (options->rectangle_draw_aspect_ratio != RECTANGLE_DRAW_ASPECT_RATIO_NONE)
+			{
+			if (gdouble(rect_width) / rect_height < aspect_ratio)
+				{
+				rect_width = gdouble(rect_height) * aspect_ratio;
+				}
+			else
+				{
+				rect_height = rect_width / aspect_ratio;
+				}
+			}
+
 		rect_pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, rect_width, rect_height);
 		pixbuf_set_rect_fill(rect_pixbuf, 0, 0, rect_width, rect_height, 255, 255, 255, 0);
 		pixbuf_set_rect(rect_pixbuf, 1, 1, rect_width-2, rect_height - 2, 0, 0, 0, 255, 1, 1, 1, 1);
-		pixbuf_set_rect(rect_pixbuf, 2, 2, rect_width-4, rect_height - 4, 255, 255, 255, 255, 1, 1, 1, 1);
+		pixbuf_set_rect(rect_pixbuf, 0, 0, rect_width, rect_height, 255, 255, 255, 255, 1, 1, 1, 1);
 
-		rect_id = pixbuf_renderer_overlay_add(reinterpret_cast<PixbufRenderer *>(imd->pr), rect_pixbuf, pixbuf_start_x, pixbuf_start_y, OVL_NORMAL);
+		rect_id = pixbuf_renderer_overlay_add(PIXBUF_RENDERER(imd->pr), rect_pixbuf, pixbuf_start_x, pixbuf_start_y, OVL_NORMAL);
 		}
 
 	pixbuf_renderer_get_scaled_size(pr, &width, &height);
 
 	if (imd->func_drag)
 		{
-		imd->func_drag(imd, event,
-			       static_cast<gfloat>(pr->drag_last_x - event->x) / width,
-			       static_cast<gfloat>(pr->drag_last_y - event->y) / height,
-			       imd->data_button);
+		imd->func_drag(imd, event, static_cast<gfloat>(pr->drag_last_x - event->x) / width, static_cast<gfloat>(pr->drag_last_y - event->y) / height, imd->data_button);
 		}
 }
 
@@ -363,53 +407,38 @@ static void image_zoom_cb(PixbufRenderer *, gdouble, gpointer data)
 
 void image_update_title(ImageWindow *imd)
 {
-	gchar *title = nullptr;
-	gchar *zoom = nullptr;
-	gchar *collection = nullptr;
-	LayoutWindow *lw;
-	gchar *lw_ident = nullptr;
-
 	if (!imd->top_window) return;
 
+	g_autoptr(GString) title = g_string_new(imd->title);
+
+	if (imd->image_fd) title = g_string_append(title, imd->image_fd->name);
+
+	if (imd->title_show_zoom)
+		{
+		g_autofree gchar *buf = image_zoom_get_as_text(imd);
+		g_string_append_printf(title, " [%s]", buf);
+		}
+
 	if (imd->collection && collection_to_number(imd->collection) >= 0)
 		{
 		const gchar *name = imd->collection->name;
 		if (!name) name = _("Untitled");
-		collection = g_strdup_printf(_(" (Collection %s)"), name);
+		g_string_append_printf(title, _(" (Collection %s)"), name);
 		}
 
-	if (imd->title_show_zoom)
+	if (imd->image_fd) title = g_string_append(title, " - ");
+	if (imd->title_right) title = g_string_append(title, imd->title_right);
+
+	if (options->show_window_ids)
 		{
-		gchar *buf = image_zoom_get_as_text(imd);
-		zoom = g_strconcat(" [", buf, "]", NULL);
-		g_free(buf);
+		LayoutWindow *lw = layout_find_by_image(imd);
+		if (lw)
+			{
+			g_string_append_printf(title, " (%s)", lw->options.id);
+			}
 		}
 
-	lw = layout_find_by_image(imd);
-	if (lw)
-		{
-		lw_ident = g_strconcat(" (", lw->options.id, ")", NULL);
-		}
-
-	title = g_strdup_printf("%s%s%s%s%s%s%s",
-		imd->title ? imd->title : "",
-		imd->image_fd ? imd->image_fd->name : "",
-		zoom ? zoom : "",
-		collection ? collection : "",
-		imd->image_fd ? " - " : "",
-		imd->title_right ? imd->title_right : "",
-		options->show_window_ids ? (lw_ident ? lw_ident : "") : ""
-		);
-	if (lw_ident)
-		{
-		g_free(lw_ident);
-		}
-
-	gtk_window_set_title(GTK_WINDOW(imd->top_window), title);
-
-	g_free(title);
-	g_free(zoom);
-	g_free(collection);
+	gtk_window_set_title(GTK_WINDOW(imd->top_window), title->str);
 }
 
 /*
@@ -707,7 +736,7 @@ void image_alter_orientation(ImageWindow *imd, FileData *fd_n, AlterType type)
 	if (imd->image_fd == fd_n && (!options->metadata.write_orientation || options->image.exif_rotate_enable))
 		{
 		imd->orientation = orientation;
-		pixbuf_renderer_set_orientation(reinterpret_cast<PixbufRenderer *>(imd->pr), orientation);
+		pixbuf_renderer_set_orientation(PIXBUF_RENDERER(imd->pr), orientation);
 		}
 }
 
@@ -715,10 +744,10 @@ void image_set_desaturate(ImageWindow *imd, gboolean desaturate)
 {
 	imd->desaturate = desaturate;
 	if (imd->cm || imd->desaturate || imd->overunderexposed)
-		pixbuf_renderer_set_post_process_func(reinterpret_cast<PixbufRenderer *>(imd->pr), image_post_process_tile_color_cb, imd, (imd->cm != nullptr) );
+		pixbuf_renderer_set_post_process_func(PIXBUF_RENDERER(imd->pr), image_post_process_tile_color_cb, imd, (imd->cm != nullptr) );
 	else
-		pixbuf_renderer_set_post_process_func(reinterpret_cast<PixbufRenderer *>(imd->pr), nullptr, nullptr, TRUE);
-	pixbuf_renderer_set_orientation(reinterpret_cast<PixbufRenderer *>(imd->pr), imd->orientation);
+		pixbuf_renderer_set_post_process_func(PIXBUF_RENDERER(imd->pr), nullptr, nullptr, TRUE);
+	pixbuf_renderer_set_orientation(PIXBUF_RENDERER(imd->pr), imd->orientation);
 }
 
 gboolean image_get_desaturate(ImageWindow *imd)
@@ -730,15 +759,15 @@ void image_set_overunderexposed(ImageWindow *imd, gboolean overunderexposed)
 {
 	imd->overunderexposed = overunderexposed;
 	if (imd->cm || imd->desaturate || imd->overunderexposed)
-		pixbuf_renderer_set_post_process_func(reinterpret_cast<PixbufRenderer *>(imd->pr), image_post_process_tile_color_cb, imd, (imd->cm != nullptr) );
+		pixbuf_renderer_set_post_process_func(PIXBUF_RENDERER(imd->pr), image_post_process_tile_color_cb, imd, (imd->cm != nullptr) );
 	else
-		pixbuf_renderer_set_post_process_func(reinterpret_cast<PixbufRenderer *>(imd->pr), nullptr, nullptr, TRUE);
-	pixbuf_renderer_set_orientation(reinterpret_cast<PixbufRenderer *>(imd->pr), imd->orientation);
+		pixbuf_renderer_set_post_process_func(PIXBUF_RENDERER(imd->pr), nullptr, nullptr, TRUE);
+	pixbuf_renderer_set_orientation(PIXBUF_RENDERER(imd->pr), imd->orientation);
 }
 
 void image_set_ignore_alpha(ImageWindow *imd, gboolean ignore_alpha)
 {
-   pixbuf_renderer_set_ignore_alpha(reinterpret_cast<PixbufRenderer *>(imd->pr), ignore_alpha);
+   pixbuf_renderer_set_ignore_alpha(PIXBUF_RENDERER(imd->pr), ignore_alpha);
 }
 
 /*
@@ -838,8 +867,7 @@ static void image_cache_release_cb(FileData *fd)
 
 static FileCacheData *image_get_cache()
 {
-	static FileCacheData *cache = nullptr;
-	if (!cache) cache = file_cache_new(image_cache_release_cb, 1);
+	static FileCacheData *cache = file_cache_new(image_cache_release_cb, 1);
 	file_cache_set_max_size(cache, static_cast<gulong>(options->image.image_cache_max) * 1048576); /* update from options */
 	return cache;
 }
@@ -882,9 +910,7 @@ static void image_load_pixbuf_ready(ImageWindow *imd)
 static void image_load_area_cb(ImageLoader *il, guint x, guint y, guint w, guint h, gpointer data)
 {
 	auto imd = static_cast<ImageWindow *>(data);
-	PixbufRenderer *pr;
-
-	pr = reinterpret_cast<PixbufRenderer *>(imd->pr);
+	PixbufRenderer *pr = PIXBUF_RENDERER(imd->pr);
 
 	if (imd->delay_flip &&
 	    pr->pixbuf != image_loader_get_pixbuf(il))
@@ -938,8 +964,8 @@ static void image_load_size_cb(ImageLoader *, guint width, guint height, gpointe
 {
 	auto imd = static_cast<ImageWindow *>(data);
 
-	DEBUG_1("image_load_size_cb: %dx%d", width, height);
-	pixbuf_renderer_set_size_early(reinterpret_cast<PixbufRenderer *>(imd->pr), width, height);
+	DEBUG_1("image_load_size_cb: %ux%u", width, height);
+	pixbuf_renderer_set_size_early(PIXBUF_RENDERER(imd->pr), width, height);
 }
 
 static void image_load_error_cb(ImageLoader *il, gpointer data)
@@ -1260,9 +1286,9 @@ void image_attach_window(ImageWindow *imd, GtkWidget *window,
 
 	lw = layout_find_by_image(imd);
 
-	if (!(options->image.fit_window_to_image && lw && (lw->options.tools_float || lw->options.tools_hidden))) window = nullptr;
+	if (!options->image.fit_window_to_image || !lw || (!lw->options.tools_float && !lw->options.tools_hidden)) window = nullptr;
 
-	pixbuf_renderer_set_parent(reinterpret_cast<PixbufRenderer *>(imd->pr), reinterpret_cast<GtkWindow *>(window));
+	pixbuf_renderer_set_parent(PIXBUF_RENDERER(imd->pr), GTK_WINDOW(window));
 
 	image_update_title(imd);
 }
@@ -1316,17 +1342,6 @@ void image_set_scroll_func(ImageWindow *imd,
 	imd->data_scroll = data;
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-void image_set_scroll_notify_func_unused(ImageWindow *imd,
-				  void (*func)(ImageWindow *imd, gint x, gint y, gint width, gint height, gpointer data),
-				  gpointer data)
-{
-	imd->func_scroll_notify = func;
-	imd->data_scroll_notify = data;
-}
-#pragma GCC diagnostic pop
-
 void image_set_focus_in_func(ImageWindow *imd,
 			   void (*func)(ImageWindow *, gpointer),
 			   gpointer data)
@@ -1354,7 +1369,9 @@ FileData *image_get_fd(ImageWindow *imd)
 	return imd->image_fd;
 }
 
-/* merely changes path string, does not change the image! */
+/**
+ * @brief Merely changes path string, does not change the image!
+ */
 void image_set_fd(ImageWindow *imd, FileData *fd)
 {
 	if (imd->auto_refresh && imd->image_fd)
@@ -1386,7 +1403,7 @@ gboolean image_get_image_size(ImageWindow *imd, gint *width, gint *height)
 
 GdkPixbuf *image_get_pixbuf(ImageWindow *imd)
 {
-	return pixbuf_renderer_get_pixbuf(reinterpret_cast<PixbufRenderer *>(imd->pr));
+	return pixbuf_renderer_get_pixbuf(PIXBUF_RENDERER(imd->pr));
 }
 
 void image_change_pixbuf(ImageWindow *imd, GdkPixbuf *pixbuf, gdouble zoom, gboolean lazy)
@@ -1429,7 +1446,7 @@ void image_change_pixbuf(ImageWindow *imd, GdkPixbuf *pixbuf, gdouble zoom, gboo
 			}
 		}
 
-	pixbuf_renderer_set_post_process_func(reinterpret_cast<PixbufRenderer *>(imd->pr), nullptr, nullptr, FALSE);
+	pixbuf_renderer_set_post_process_func(PIXBUF_RENDERER(imd->pr), nullptr, nullptr, FALSE);
 	if (imd->cm)
 		{
 		color_man_free(static_cast<ColorMan *>(imd->cm));
@@ -1438,13 +1455,13 @@ void image_change_pixbuf(ImageWindow *imd, GdkPixbuf *pixbuf, gdouble zoom, gboo
 
 	if (lazy)
 		{
-		pixbuf_renderer_set_pixbuf_lazy(reinterpret_cast<PixbufRenderer *>(imd->pr), pixbuf, zoom, imd->orientation, stereo_data);
+		pixbuf_renderer_set_pixbuf_lazy(PIXBUF_RENDERER(imd->pr), pixbuf, zoom, imd->orientation, stereo_data);
 		}
 	else
 		{
-		pixbuf_renderer_set_pixbuf(reinterpret_cast<PixbufRenderer *>(imd->pr), pixbuf, zoom);
-		pixbuf_renderer_set_orientation(reinterpret_cast<PixbufRenderer *>(imd->pr), imd->orientation);
-		pixbuf_renderer_set_stereo_data(reinterpret_cast<PixbufRenderer *>(imd->pr), stereo_data);
+		pixbuf_renderer_set_pixbuf(PIXBUF_RENDERER(imd->pr), pixbuf, zoom);
+		pixbuf_renderer_set_orientation(PIXBUF_RENDERER(imd->pr), imd->orientation);
+		pixbuf_renderer_set_stereo_data(PIXBUF_RENDERER(imd->pr), stereo_data);
 		}
 
 	if (pixbuf) g_object_unref(pixbuf);
@@ -1457,7 +1474,7 @@ void image_change_pixbuf(ImageWindow *imd, GdkPixbuf *pixbuf, gdouble zoom, gboo
 		}
 
 	if (imd->cm || imd->desaturate || imd->overunderexposed)
-		pixbuf_renderer_set_post_process_func(reinterpret_cast<PixbufRenderer *>(imd->pr), image_post_process_tile_color_cb, imd, (imd->cm != nullptr) );
+		pixbuf_renderer_set_post_process_func(PIXBUF_RENDERER(imd->pr), image_post_process_tile_color_cb, imd, (imd->cm != nullptr) );
 
 	image_state_set(imd, IMAGE_STATE_IMAGE);
 }
@@ -1580,9 +1597,9 @@ void image_move_from_image(ImageWindow *imd, ImageWindow *source)
 	pixbuf_renderer_move(PIXBUF_RENDERER(imd->pr), PIXBUF_RENDERER(source->pr));
 
 	if (imd->cm || imd->desaturate || imd->overunderexposed)
-		pixbuf_renderer_set_post_process_func(reinterpret_cast<PixbufRenderer *>(imd->pr), image_post_process_tile_color_cb, imd, (imd->cm != nullptr) );
+		pixbuf_renderer_set_post_process_func(PIXBUF_RENDERER(imd->pr), image_post_process_tile_color_cb, imd, (imd->cm != nullptr) );
 	else
-		pixbuf_renderer_set_post_process_func(reinterpret_cast<PixbufRenderer *>(imd->pr), nullptr, nullptr, TRUE);
+		pixbuf_renderer_set_post_process_func(PIXBUF_RENDERER(imd->pr), nullptr, nullptr, TRUE);
 
 }
 
@@ -1642,9 +1659,9 @@ void image_copy_from_image(ImageWindow *imd, ImageWindow *source)
 	pixbuf_renderer_copy(PIXBUF_RENDERER(imd->pr), PIXBUF_RENDERER(source->pr));
 
 	if (imd->cm || imd->desaturate || imd->overunderexposed)
-		pixbuf_renderer_set_post_process_func(reinterpret_cast<PixbufRenderer *>(imd->pr), image_post_process_tile_color_cb, imd, (imd->cm != nullptr) );
+		pixbuf_renderer_set_post_process_func(PIXBUF_RENDERER(imd->pr), image_post_process_tile_color_cb, imd, (imd->cm != nullptr) );
 	else
-		pixbuf_renderer_set_post_process_func(reinterpret_cast<PixbufRenderer *>(imd->pr), nullptr, nullptr, TRUE);
+		pixbuf_renderer_set_post_process_func(PIXBUF_RENDERER(imd->pr), nullptr, nullptr, TRUE);
 
 }
 
@@ -1653,25 +1670,25 @@ void image_copy_from_image(ImageWindow *imd, ImageWindow *source)
 
 void image_area_changed(ImageWindow *imd, gint x, gint y, gint width, gint height)
 {
-	pixbuf_renderer_area_changed(reinterpret_cast<PixbufRenderer *>(imd->pr), x, y, width, height);
+	pixbuf_renderer_area_changed(PIXBUF_RENDERER(imd->pr), x, y, width, height);
 }
 
 void image_reload(ImageWindow *imd)
 {
-	if (pixbuf_renderer_get_tiles(reinterpret_cast<PixbufRenderer *>(imd->pr))) return;
+	if (pixbuf_renderer_get_tiles(PIXBUF_RENDERER(imd->pr))) return;
 
 	image_change_complete(imd, image_zoom_get(imd));
 }
 
 void image_scroll(ImageWindow *imd, gint x, gint y)
 {
-	pixbuf_renderer_scroll(reinterpret_cast<PixbufRenderer *>(imd->pr), x, y);
+	pixbuf_renderer_scroll(PIXBUF_RENDERER(imd->pr), x, y);
 }
 
 void image_scroll_to_point(ImageWindow *imd, gint x, gint y,
 			   gdouble x_align, gdouble y_align)
 {
-	pixbuf_renderer_scroll_to_point(reinterpret_cast<PixbufRenderer *>(imd->pr), x, y, x_align, y_align);
+	pixbuf_renderer_scroll_to_point(PIXBUF_RENDERER(imd->pr), x, y, x_align, y_align);
 }
 
 void image_get_scroll_center(ImageWindow *imd, gdouble *x, gdouble *y)
@@ -1686,33 +1703,31 @@ void image_set_scroll_center(ImageWindow *imd, gdouble x, gdouble y)
 
 void image_zoom_adjust(ImageWindow *imd, gdouble increment)
 {
-	pixbuf_renderer_zoom_adjust(reinterpret_cast<PixbufRenderer *>(imd->pr), increment);
+	pixbuf_renderer_zoom_adjust(PIXBUF_RENDERER(imd->pr), increment);
 }
 
 void image_zoom_adjust_at_point(ImageWindow *imd, gdouble increment, gint x, gint y)
 {
-	pixbuf_renderer_zoom_adjust_at_point(reinterpret_cast<PixbufRenderer *>(imd->pr), increment, x, y);
+	pixbuf_renderer_zoom_adjust_at_point(PIXBUF_RENDERER(imd->pr), increment, x, y);
 }
 
 void image_zoom_set_limits(ImageWindow *imd, gdouble min, gdouble max)
 {
-	pixbuf_renderer_zoom_set_limits(reinterpret_cast<PixbufRenderer *>(imd->pr), min, max);
+	pixbuf_renderer_zoom_set_limits(PIXBUF_RENDERER(imd->pr), min, max);
 }
 
 void image_zoom_set(ImageWindow *imd, gdouble zoom)
 {
-	pixbuf_renderer_zoom_set(reinterpret_cast<PixbufRenderer *>(imd->pr), zoom);
+	pixbuf_renderer_zoom_set(PIXBUF_RENDERER(imd->pr), zoom);
 }
 
 void image_zoom_set_fill_geometry(ImageWindow *imd, gboolean vertical)
 {
-	PixbufRenderer *pr;
+	PixbufRenderer *pr = PIXBUF_RENDERER(imd->pr);
 	gdouble zoom;
 	gint width;
 	gint height;
 
-	pr = reinterpret_cast<PixbufRenderer *>(imd->pr);
-
 	if (!pixbuf_renderer_get_pixbuf(pr) ||
 	    !pixbuf_renderer_get_image_size(pr, &width, &height)) return;
 
@@ -1735,12 +1750,12 @@ void image_zoom_set_fill_geometry(ImageWindow *imd, gboolean vertical)
 
 gdouble image_zoom_get(ImageWindow *imd)
 {
-	return pixbuf_renderer_zoom_get(reinterpret_cast<PixbufRenderer *>(imd->pr));
+	return pixbuf_renderer_zoom_get(PIXBUF_RENDERER(imd->pr));
 }
 
 gdouble image_zoom_get_real(ImageWindow *imd)
 {
-	return pixbuf_renderer_zoom_get_scale(reinterpret_cast<PixbufRenderer *>(imd->pr));
+	return pixbuf_renderer_zoom_get_scale(PIXBUF_RENDERER(imd->pr));
 }
 
 gchar *image_zoom_get_as_text(ImageWindow *imd)
@@ -1804,30 +1819,12 @@ gdouble image_zoom_get_default(ImageWindow *imd)
 
 /* stereo */
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-gint image_stereo_get_unused(ImageWindow *imd)
-{
-	return pixbuf_renderer_stereo_get((PixbufRenderer *)imd->pr);
-}
-#pragma GCC diagnostic pop
-
 void image_stereo_set(ImageWindow *imd, gint stereo_mode)
 {
 	DEBUG_1("Setting stereo mode %04x for imd %p", stereo_mode, (void *)imd);
-	pixbuf_renderer_stereo_set(reinterpret_cast<PixbufRenderer *>(imd->pr), stereo_mode);
+	pixbuf_renderer_stereo_set(PIXBUF_RENDERER(imd->pr), stereo_mode);
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-void image_stereo_swap_unused(ImageWindow *imd)
-{
-	gint stereo_mode = pixbuf_renderer_stereo_get((PixbufRenderer *)imd->pr);
-	stereo_mode ^= PR_STEREO_SWAP;
-	pixbuf_renderer_stereo_set((PixbufRenderer *)imd->pr, stereo_mode);
-}
-#pragma GCC diagnostic pop
-
 StereoPixbufData image_stereo_pixbuf_get(ImageWindow *imd)
 {
 	return static_cast<StereoPixbufData>(imd->user_stereo);
@@ -1839,11 +1836,12 @@ void image_stereo_pixbuf_set(ImageWindow *imd, StereoPixbufData stereo_mode)
 	image_reload(imd);
 }
 
-/* read ahead */
-
+/**
+ * @brief Read ahead, pass NULL to cancel
+ */
 void image_prebuffer_set(ImageWindow *imd, FileData *fd)
 {
-	if (pixbuf_renderer_get_tiles(reinterpret_cast<PixbufRenderer *>(imd->pr))) return;
+	if (pixbuf_renderer_get_tiles(PIXBUF_RENDERER(imd->pr))) return;
 
 	if (fd)
 		{
@@ -1879,6 +1877,9 @@ static void image_notify_cb(FileData *fd, NotifyType type, gpointer data)
 		}
 }
 
+/**
+ * @brief Auto refresh
+ */
 void image_auto_refresh_enable(ImageWindow *imd, gboolean enable)
 {
 	if (!enable && imd->auto_refresh && imd->image_fd)
@@ -1890,6 +1891,9 @@ void image_auto_refresh_enable(ImageWindow *imd, gboolean enable)
 	imd->auto_refresh = enable;
 }
 
+/**
+ * @brief Allow top window to be resized ?
+ */
 void image_top_window_set_sync(ImageWindow *imd, gboolean allow_sync)
 {
 	imd->top_window_sync = allow_sync;
@@ -1899,7 +1903,7 @@ void image_top_window_set_sync(ImageWindow *imd, gboolean allow_sync)
 
 void image_background_set_color(ImageWindow *imd, GdkRGBA *color)
 {
-	pixbuf_renderer_set_color(reinterpret_cast<PixbufRenderer *>(imd->pr), color);
+	pixbuf_renderer_set_color(PIXBUF_RENDERER(imd->pr), color);
 }
 
 void image_background_set_color_from_options(ImageWindow *imd, gboolean fullscreen)
@@ -1933,9 +1937,7 @@ void image_background_set_color_from_options(ImageWindow *imd, gboolean fullscre
 	image_background_set_color(imd, color);
 }
 
-void image_color_profile_set(ImageWindow *imd,
-			     gint input_type,
-			     gboolean use_image)
+void image_color_profile_set(ImageWindow *imd, gint input_type, gboolean use_image)
 {
 	if (!imd) return;
 
@@ -1948,14 +1950,12 @@ void image_color_profile_set(ImageWindow *imd,
 	imd->color_profile_use_image = use_image;
 }
 
-gboolean image_color_profile_get(ImageWindow *imd,
-			 	 gint *input_type,
-			     	 gboolean *use_image)
+gboolean image_color_profile_get(const ImageWindow *imd, gint &input_type, gboolean &use_image)
 {
 	if (!imd) return FALSE;
 
-	if (input_type) *input_type = imd->color_profile_input;
-	if (use_image) *use_image = imd->color_profile_use_image;
+	input_type = imd->color_profile_input;
+	use_image = imd->color_profile_use_image;
 
 	return TRUE;
 }
@@ -1987,6 +1987,9 @@ gboolean image_color_profile_get_status(ImageWindow *imd, gchar **image_profile,
 
 }
 
+/**
+ * @brief Set delayed page flipping
+ */
 void image_set_delay_flip(ImageWindow *imd, gboolean delay)
 {
 	if (!imd ||
@@ -2008,6 +2011,9 @@ void image_set_delay_flip(ImageWindow *imd, gboolean delay)
 		}
 }
 
+/**
+ * @brief Wallpaper util
+ */
 void image_to_root_window(ImageWindow *, gboolean)
 {
 }
@@ -2018,11 +2024,11 @@ void image_select(ImageWindow *imd, gboolean select)
 
 	if (select)
 		{
-		gtk_widget_set_state(imd->widget, GTK_STATE_SELECTED);
-		gtk_widget_set_state(imd->pr, GTK_STATE_NORMAL); /* do not propagate */
+		gtk_widget_set_state_flags(imd->widget, GTK_STATE_FLAG_SELECTED, TRUE);
+		gtk_widget_set_state_flags(imd->pr, GTK_STATE_FLAG_NORMAL, TRUE); /* do not propagate */
 		}
 	else
-		gtk_widget_set_state(imd->widget, GTK_STATE_NORMAL);
+		gtk_widget_set_state_flags(imd->widget, GTK_STATE_FLAG_NORMAL, TRUE);
 }
 
 void image_set_selectable(ImageWindow *imd, gboolean selectable)
@@ -2068,15 +2074,18 @@ static void image_options_set(ImageWindow *imd)
 
 					NULL);
 
-	pixbuf_renderer_set_parent(reinterpret_cast<PixbufRenderer *>(imd->pr), reinterpret_cast<GtkWindow *>(imd->top_window));
+	pixbuf_renderer_set_parent(PIXBUF_RENDERER(imd->pr), GTK_WINDOW(imd->top_window));
 
 	image_stereo_set(imd, options->stereo.mode);
-	pixbuf_renderer_stereo_fixed_set(reinterpret_cast<PixbufRenderer *>(imd->pr),
+	pixbuf_renderer_stereo_fixed_set(PIXBUF_RENDERER(imd->pr),
 					options->stereo.fixed_w, options->stereo.fixed_h,
 					options->stereo.fixed_x1, options->stereo.fixed_y1,
 					options->stereo.fixed_x2, options->stereo.fixed_y2);
 }
 
+/**
+ * @brief Reset default options
+ */
 void image_options_sync()
 {
 	GList *work;
@@ -2124,7 +2133,7 @@ static void image_destroy_cb(GtkWidget *, gpointer data)
 	image_free(imd);
 }
 
-gboolean selectable_frame_draw_cb(GtkWidget *widget, cairo_t *cr, gpointer)
+static gboolean selectable_frame_draw_cb(GtkWidget *widget, cairo_t *cr, gpointer)
 {
 	GtkAllocation allocation;
 	gtk_widget_get_allocation(widget, &allocation);
@@ -2244,12 +2253,12 @@ ImageWindow *image_new(gboolean frame)
 	return imd;
 }
 
-void image_get_rectangle(gint *x1, gint *y1, gint *x2, gint *y2)
+void image_get_rectangle(gint &x1, gint &y1, gint &x2, gint &y2)
 {
-	*x1 = rect_x1;
-	*y1 = rect_y1;
-	*x2 = rect_x2;
-	*y2 = rect_y2;
+	x1 = rect_x1;
+	y1 = rect_y1;
+	x2 = rect_x2;
+	y2 = rect_y2;
 }
 
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
