@@ -21,6 +21,8 @@
 
 #include "img-view.h"
 
+#include <array>
+
 #include <gdk/gdk.h>
 #include <glib-object.h>
 #include <gtk/gtk.h>
@@ -41,7 +43,6 @@
 #include "layout-util.h"
 #include "layout.h"
 #include "main-defines.h"
-#include "main.h"
 #include "menu.h"
 #include "misc.h"
 #include "options.h"
@@ -442,13 +443,7 @@ static gboolean view_window_key_press_cb(GtkWidget * (widget), GdkEventKey *even
 
 	if (x != 0 || y!= 0)
 		{
-		if (event->state & GDK_SHIFT_MASK)
-			{
-			x *= 3;
-			y *= 3;
-			}
-
-		keyboard_scroll_calc(&x, &y, event);
+		keyboard_scroll_calc(x, y, event);
 		image_scroll(imd, x, y);
 		}
 
@@ -1282,7 +1277,7 @@ static void view_copy_path_cb(GtkWidget *, gpointer data)
 	ImageWindow *imd;
 
 	imd = view_window_active_image(vw);
-	file_util_copy_path_to_clipboard(image_get_fd(imd), TRUE);
+	file_util_copy_path_to_clipboard(image_get_fd(imd), TRUE, ClipboardAction::COPY);
 }
 
 static void view_copy_path_unquoted_cb(GtkWidget *, gpointer data)
@@ -1291,7 +1286,7 @@ static void view_copy_path_unquoted_cb(GtkWidget *, gpointer data)
 	ImageWindow *imd;
 
 	imd = view_window_active_image(vw);
-	file_util_copy_path_to_clipboard(image_get_fd(imd), FALSE);
+	file_util_copy_path_to_clipboard(image_get_fd(imd), FALSE, ClipboardAction::COPY);
 }
 
 static void view_fullscreen_cb(GtkWidget *, gpointer data)
@@ -1333,8 +1328,7 @@ static LayoutWindow *view_new_layout_with_fd(FileData *fd)
 {
 	LayoutWindow *nw;
 
-	nw = layout_new(nullptr, nullptr);
-	layout_sort_set_files(nw, nw->options.file_view_list_sort.method, nw->options.file_view_list_sort.ascend, nw->options.file_view_list_sort.case_sensitive);
+	nw = layout_new_from_default();
 	layout_set_fd(nw, fd);
 	return nw;
 }
@@ -1352,9 +1346,15 @@ static void view_set_layout_path_cb(GtkWidget *, gpointer data)
 
 	lw = layout_find_by_image_fd(imd);
 	if (lw)
+		{
 		layout_set_fd(lw, imd->image_fd);
+		gtk_window_present(GTK_WINDOW(lw->window));
+		}
 	else
+		{
 		view_new_layout_with_fd(imd->image_fd);
+		}
+
 	view_window_close(vw);
 }
 
@@ -1608,7 +1608,7 @@ static GtkWidget *view_confirm_dir_list(ViewWindow *vw, GList *list)
 	g_signal_connect(G_OBJECT(menu), "destroy",
 			 G_CALLBACK(view_dir_list_destroy), d);
 
-	menu_item_add_stock(menu, _("Dropped list includes folders."), GQ_ICON_DND, nullptr, nullptr);
+	menu_item_add_icon(menu, _("Dropped list includes folders."), GQ_ICON_DIRECTORY, nullptr, nullptr);
 	menu_item_add_divider(menu);
 	menu_item_add_icon(menu, _("_Add contents"), GQ_ICON_OK, G_CALLBACK(view_dir_list_add), d);
 	menu_item_add_icon(menu, _("Add contents _recursive"), GQ_ICON_ADD, G_CALLBACK(view_dir_list_recurse), d);
@@ -1736,15 +1736,15 @@ static void view_window_dnd_init(ViewWindow *vw)
 	imd = vw->imd;
 
 	gtk_drag_source_set(imd->pr, GDK_BUTTON2_MASK,
-			    dnd_file_drag_types, dnd_file_drag_types_count,
-			    static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
+	                    dnd_file_drag_types.data(), dnd_file_drag_types.size(),
+	                    static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
 	g_signal_connect(G_OBJECT(imd->pr), "drag_data_get",
 			 G_CALLBACK(view_window_set_dnd_data), vw);
 
 	gtk_drag_dest_set(imd->pr,
-			  static_cast<GtkDestDefaults>(GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP),
-			  dnd_file_drop_types, dnd_file_drop_types_count,
-			  static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
+	                  static_cast<GtkDestDefaults>(GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP),
+	                  dnd_file_drop_types.data(), dnd_file_drop_types.size(),
+	                  static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
 	g_signal_connect(G_OBJECT(imd->pr), "drag_data_received",
 			 G_CALLBACK(view_window_get_dnd_data), vw);
 }
