@@ -21,6 +21,7 @@
 
 #include "layout-image.h"
 
+#include <array>
 #include <cstring>
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
@@ -33,6 +34,7 @@
 
 #include "archives.h"
 #include "collect.h"
+#include "compat.h"
 #include "debug.h"
 #include "dnd.h"
 #include "editors.h"
@@ -60,25 +62,18 @@
 #include "utilops.h"
 #include "view-file.h"
 
+namespace
+{
+
+constexpr gint IMAGE_MIN_WIDTH = 100;
+
+} // namespace
+
 static GtkWidget *layout_image_pop_menu(LayoutWindow *lw);
 static void layout_image_set_buttons(LayoutWindow *lw);
 static gboolean layout_image_animate_new_file(LayoutWindow *lw);
 static void layout_image_animate_update_image(LayoutWindow *lw);
 
-/*
- *----------------------------------------------------------------------------
- * full screen overlay
- *----------------------------------------------------------------------------
- */
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-static void layout_image_overlay_toggle_unused(LayoutWindow *lw)
-{
-	if (!lw) return;
-	image_osd_toggle(lw->image);
-}
-#pragma GCC diagnostic pop
-
 /*
  *----------------------------------------------------------------------------
  * full screen
@@ -333,7 +328,6 @@ static gboolean show_next_frame(gpointer data)
 {
 	auto fd = static_cast<AnimationData*>(data);
 	int delay;
-	PixbufRenderer *pr;
 
 	if(animation_should_continue(fd)==FALSE)
 		{
@@ -341,7 +335,7 @@ static gboolean show_next_frame(gpointer data)
 		return FALSE;
 		}
 
-	pr = reinterpret_cast<PixbufRenderer*>(fd->iw->pr);
+	PixbufRenderer *pr = PIXBUF_RENDERER(fd->iw->pr);
 
 	if (gdk_pixbuf_animation_iter_advance(fd->iter,nullptr)==FALSE)
 		{
@@ -479,7 +473,7 @@ static gboolean layout_image_animate_new_file(LayoutWindow *lw)
 	if (gfstream)
 		{
 		animation->gfstream = gfstream;
-		gdk_pixbuf_animation_new_from_stream_async(reinterpret_cast<GInputStream*>(gfstream), animation->cancellable, animation_async_ready_cb, animation);
+		gdk_pixbuf_animation_new_from_stream_async(G_INPUT_STREAM(gfstream), animation->cancellable, animation_async_ready_cb, animation);
 		}
 	else
 		{
@@ -497,8 +491,8 @@ void layout_image_animate_toggle(LayoutWindow *lw)
 
 	lw->options.animate = !lw->options.animate;
 
-	action = gtk_action_group_get_action(lw->action_group, "Animate");
-	gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(action), lw->options.animate);
+	action = gq_gtk_action_group_get_action(lw->action_group, "Animate");
+	gq_gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(action), lw->options.animate);
 
 	layout_image_animate_new_file(lw);
 }
@@ -552,11 +546,8 @@ static void li_pop_menu_edit_cb(GtkWidget *widget, gpointer data)
 
 static void li_pop_menu_alter_cb(GtkWidget *widget, gpointer data)
 {
-	auto lw = static_cast<LayoutWindow *>(data);
-	AlterType type;
-
-	lw = static_cast<LayoutWindow *>(submenu_item_get_data(widget));
-	type = static_cast<AlterType>GPOINTER_TO_INT(data);
+	auto *lw = static_cast<LayoutWindow *>(submenu_item_get_data(widget));
+	auto type = static_cast<AlterType>GPOINTER_TO_INT(data);
 
 	image_alter_orientation(lw->image, lw->image->image_fd, type);
 }
@@ -598,14 +589,21 @@ static void li_pop_menu_copy_path_cb(GtkWidget *, gpointer data)
 {
 	auto lw = static_cast<LayoutWindow *>(data);
 
-	file_util_copy_path_to_clipboard(layout_image_get_fd(lw), TRUE);
+	file_util_copy_path_to_clipboard(layout_image_get_fd(lw), TRUE, ClipboardAction::COPY);
 }
 
 static void li_pop_menu_copy_path_unquoted_cb(GtkWidget *, gpointer data)
 {
 	auto lw = static_cast<LayoutWindow *>(data);
 
-	file_util_copy_path_to_clipboard(layout_image_get_fd(lw), FALSE);
+	file_util_copy_path_to_clipboard(layout_image_get_fd(lw), FALSE, ClipboardAction::COPY);
+}
+
+static void li_pop_menu_cut_path_cb(GtkWidget *, gpointer data)
+{
+	auto lw = static_cast<LayoutWindow *>(data);
+
+	file_util_copy_path_to_clipboard(layout_image_get_fd(lw), FALSE, ClipboardAction::CUT);
 }
 
 #if HAVE_GTK4
@@ -845,9 +843,10 @@ static GtkWidget *layout_image_pop_menu(LayoutWindow *lw)
 	if (!path) gtk_widget_set_sensitive(item, FALSE);
 	item = menu_item_add(menu, _("_Rename..."), G_CALLBACK(li_pop_menu_rename_cb), lw);
 	if (!path) gtk_widget_set_sensitive(item, FALSE);
-	item = menu_item_add(menu, _("_Copy path to clipboard"), G_CALLBACK(li_pop_menu_copy_path_cb), lw);
-	item = menu_item_add(menu, _("_Copy path unquoted to clipboard"), G_CALLBACK(li_pop_menu_copy_path_unquoted_cb), lw);
+	item = menu_item_add(menu, _("_Copy to clipboard"), G_CALLBACK(li_pop_menu_copy_path_cb), lw);
+	item = menu_item_add(menu, _("_Copy to clipboard (unquoted)"), G_CALLBACK(li_pop_menu_copy_path_unquoted_cb), lw);
 	item = menu_item_add(menu, _("Copy _image to clipboard"), G_CALLBACK(li_pop_menu_copy_image_cb), lw);
+	item = menu_item_add(menu, _("Cut to clipboard"), G_CALLBACK(li_pop_menu_cut_path_cb), lw);
 	if (!path) gtk_widget_set_sensitive(item, FALSE);
 	menu_item_add_divider(menu);
 
@@ -1083,17 +1082,17 @@ static void layout_image_dnd_init(LayoutWindow *lw, gint i)
 	ImageWindow *imd = lw->split_images[i];
 
 	gtk_drag_source_set(imd->pr, GDK_BUTTON2_MASK,
-			    dnd_file_drag_types, dnd_file_drag_types_count,
-			    static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
+	                    dnd_file_drag_types.data(), dnd_file_drag_types.size(),
+	                    static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
 	g_signal_connect(G_OBJECT(imd->pr), "drag_data_get",
 			 G_CALLBACK(layout_image_dnd_get), lw);
 	g_signal_connect(G_OBJECT(imd->pr), "drag_end",
 			 G_CALLBACK(layout_image_dnd_end), lw);
 
 	gtk_drag_dest_set(imd->pr,
-			  static_cast<GtkDestDefaults>(GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP),
-			  dnd_file_drop_types, dnd_file_drop_types_count,
-			  static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
+	                  static_cast<GtkDestDefaults>(GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP),
+	                  dnd_file_drop_types.data(), dnd_file_drop_types.size(),
+	                  static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
 	g_signal_connect(G_OBJECT(imd->pr), "drag_data_received",
 			 G_CALLBACK(layout_image_dnd_receive), lw);
 }
@@ -1227,44 +1226,9 @@ void layout_image_zoom_set_fill_geometry(LayoutWindow *lw, gboolean vertical, gb
 void layout_image_alter_orientation(LayoutWindow *lw, AlterType type)
 {
 	if (!layout_valid(&lw)) return;
-
-	GtkTreeModel *store;
-	GList *work;
-	GtkTreeSelection *selection;
-	GtkTreePath *tpath;
-	FileData *fd_n;
-	GtkTreeIter iter;
-
 	if (!lw || !lw->vf) return;
 
-	if (lw->vf->type == FILEVIEW_ICON)
-		{
-		if (!VFICON(lw->vf)->selection) return;
-		work = VFICON(lw->vf)->selection;
-		}
-	else
-		{
-		selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(lw->vf->listview));
-		work = gtk_tree_selection_get_selected_rows(selection, &store);
-		}
-
-	while (work)
-		{
-		if (lw->vf->type == FILEVIEW_ICON)
-			{
-			fd_n = static_cast<FileData *>(work->data);
-			work = work->next;
-			}
-		else
-			{
-			tpath = static_cast<GtkTreePath *>(work->data);
-			gtk_tree_model_get_iter(store, &iter, tpath);
-			gtk_tree_model_get(store, &iter, VIEW_FILE_COLUMN_POINTER, &fd_n, -1);
-			work = work->next;
-			}
-
-		image_alter_orientation(lw->image, fd_n, type);
-		}
+	vf_selection_foreach(lw->vf, [lw, type](FileData *fd_n) { image_alter_orientation(lw->image, fd_n, type); });
 }
 
 static void image_alter_rating(FileData *fd_n, const gchar *rating)
@@ -1276,44 +1240,9 @@ static void image_alter_rating(FileData *fd_n, const gchar *rating)
 void layout_image_rating(LayoutWindow *lw, const gchar *rating)
 {
 	if (!layout_valid(&lw)) return;
-
-	GtkTreeModel *store;
-	GList *work;
-	GtkTreeSelection *selection;
-	GtkTreePath *tpath;
-	FileData *fd_n;
-	GtkTreeIter iter;
-
 	if (!lw || !lw->vf) return;
 
-	if (lw->vf->type == FILEVIEW_ICON)
-		{
-		if (!VFICON(lw->vf)->selection) return;
-		work = VFICON(lw->vf)->selection;
-		}
-	else
-		{
-		selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(lw->vf->listview));
-		work = gtk_tree_selection_get_selected_rows(selection, &store);
-		}
-
-	while (work)
-		{
-		if (lw->vf->type == FILEVIEW_ICON)
-			{
-			fd_n = static_cast<FileData *>(work->data);
-			work = work->next;
-			}
-		else
-			{
-			tpath = static_cast<GtkTreePath *>(work->data);
-			gtk_tree_model_get_iter(store, &iter, tpath);
-			gtk_tree_model_get(store, &iter, VIEW_FILE_COLUMN_POINTER, &fd_n, -1);
-			work = work->next;
-			}
-
-		image_alter_rating(fd_n, rating);
-		}
+	vf_selection_foreach(lw->vf, [rating](FileData *fd_n) { image_alter_rating(fd_n, rating); });
 }
 
 void layout_image_reset_orientation(LayoutWindow *lw)
@@ -1346,7 +1275,7 @@ void layout_image_reset_orientation(LayoutWindow *lw)
 		 imd->orientation = imd->image_fd->user_orientation;
 		}
 
-	pixbuf_renderer_set_orientation(reinterpret_cast<PixbufRenderer *>(imd->pr), imd->orientation);
+	pixbuf_renderer_set_orientation(PIXBUF_RENDERER(imd->pr), imd->orientation);
 }
 
 void layout_image_set_desaturate(LayoutWindow *lw, gboolean desaturate)
@@ -1370,17 +1299,6 @@ void layout_image_set_overunderexposed(LayoutWindow *lw, gboolean overunderexpos
 	image_set_overunderexposed(lw->image, overunderexposed);
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-gboolean layout_image_get_overunderexposed_unused(LayoutWindow *lw)
-{
-	if (!layout_valid(&lw)) return FALSE;
-
-//	return image_get_overunderexposed(lw->image);
-	return FALSE;
-}
-#pragma GCC diagnostic pop
-
 void layout_image_set_ignore_alpha(LayoutWindow *lw, gboolean ignore_alpha)
 {
    if (!layout_valid(&lw)) return;
@@ -1390,30 +1308,6 @@ void layout_image_set_ignore_alpha(LayoutWindow *lw, gboolean ignore_alpha)
 }
 
 /* stereo */
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-gint layout_image_stereo_get_unused(LayoutWindow *lw)
-{
-	if (!layout_valid(&lw)) return 0;
-
-//	return image_stereo_get(lw->image);
-	return 0;
-}
-
-void layout_image_stereo_set_unused(LayoutWindow *lw, gint stereo_mode)
-{
-	if (!layout_valid(&lw)) return;
-
-	image_stereo_set(lw->image, stereo_mode);
-}
-void layout_image_stereo_swap_unused(LayoutWindow *lw)
-{
-	if (!layout_valid(&lw)) return;
-
-//	image_stereo_swap(lw->image);
-}
-#pragma GCC diagnostic pop
-
 gint layout_image_stereo_pixbuf_get(LayoutWindow *lw)
 {
 	if (!layout_valid(&lw)) return 0;
@@ -1435,16 +1329,6 @@ const gchar *layout_image_get_path(LayoutWindow *lw)
 	return image_get_path(lw->image);
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-const gchar *layout_image_get_name_unused(LayoutWindow *lw)
-{
-	if (!layout_valid(&lw)) return nullptr;
-
-	return image_get_name(lw->image);
-}
-#pragma GCC diagnostic pop
-
 FileData *layout_image_get_fd(LayoutWindow *lw)
 {
 	if (!layout_valid(&lw)) return nullptr;
@@ -1609,18 +1493,14 @@ void layout_image_refresh(LayoutWindow *lw)
 	image_reload(lw->image);
 }
 
-void layout_image_color_profile_set(LayoutWindow *lw,
-				    gint input_type,
-				    gboolean use_image)
+void layout_image_color_profile_set(LayoutWindow *lw, gint input_type, gboolean use_image)
 {
 	if (!layout_valid(&lw)) return;
 
 	image_color_profile_set(lw->image, input_type, use_image);
 }
 
-gboolean layout_image_color_profile_get(LayoutWindow *lw,
-				    	gint *input_type,
-				    	gboolean *use_image)
+gboolean layout_image_color_profile_get(LayoutWindow *lw, gint &input_type, gboolean &use_image)
 {
 	if (!layout_valid(&lw)) return FALSE;
 
@@ -1911,7 +1791,7 @@ static void layout_image_button_cb(ImageWindow *imd, GdkEventButton *event, gpoi
 	switch (event->button)
 		{
 		case MOUSE_BUTTON_LEFT:
-			if (event->button == MOUSE_BUTTON_LEFT && event->type == GDK_2BUTTON_PRESS)
+			if (event->type == GDK_2BUTTON_PRESS)
 				{
 				layout_image_full_screen_toggle(lw);
 				}
@@ -2130,7 +2010,7 @@ static gint num_length(gint num)
 	return len;
 }
 
-void layout_status_update_pixel_cb(PixbufRenderer *pr, gpointer data)
+static void layout_status_update_pixel_cb(PixbufRenderer *pr, gpointer data)
 {
 	auto lw = static_cast<LayoutWindow *>(data);
 	gint x_pixel;
@@ -2153,15 +2033,25 @@ void layout_status_update_pixel_cb(PixbufRenderer *pr, gpointer data)
 		gint r_mouse;
 		gint g_mouse;
 		gint b_mouse;
+		gint a_mouse;
 
 		pixbuf_renderer_get_pixel_colors(pr, x_pixel, y_pixel,
-						 &r_mouse, &g_mouse, &b_mouse);
+						 &r_mouse, &g_mouse, &b_mouse, &a_mouse);
 
-		text = g_strdup_printf(_("[%*d,%*d]: RGB(%3d,%3d,%3d)"),
+		if (gdk_pixbuf_get_has_alpha(pr->pixbuf))
+			{
+			text = g_strdup_printf(_("[%*d,%*d]: RGBA(%3d,%3d,%3d,%3d)"),
+					 num_length(width - 1), x_pixel,
+					 num_length(height - 1), y_pixel,
+					 r_mouse, g_mouse, b_mouse, a_mouse);
+			}
+		else
+			{
+			text = g_strdup_printf(_("[%*d,%*d]: RGB(%3d,%3d,%3d)"),
 					 num_length(width - 1), x_pixel,
 					 num_length(height - 1), y_pixel,
 					 r_mouse, g_mouse, b_mouse);
-
+			}
 		}
 	else
 		{
@@ -2224,7 +2114,7 @@ GtkWidget *layout_image_new(LayoutWindow *lw, gint i)
 	return lw->split_images[i]->widget;
 }
 
-void layout_image_deactivate(LayoutWindow *lw, gint i)
+static void layout_image_deactivate(LayoutWindow *lw, gint i)
 {
 	if (!lw->split_images[i]) return;
 	image_set_update_func(lw->split_images[i], nullptr, nullptr);
@@ -2443,7 +2333,7 @@ static GtkWidget *layout_image_setup_split_triple(LayoutWindow *lw)
 	return lw->split_image_widget;
 }
 
-GtkWidget *layout_image_setup_split_quad(LayoutWindow *lw)
+static GtkWidget *layout_image_setup_split_quad(LayoutWindow *lw)
 {
 	GtkWidget *hpaned;
 	GtkWidget *vpaned1;
