@@ -167,17 +167,13 @@ LayoutWindow *layout_find_by_layout_id(const gchar *id)
 		return nullptr;
 		}
 
-	work = layout_window_list;
-	while (work)
-		{
-		auto lw = static_cast<LayoutWindow *>(work->data);
-		work = work->next;
+	work = g_list_find_custom(layout_window_list, id, reinterpret_cast<GCompareFunc>(layout_compare_options_id));
+	return work ? static_cast<LayoutWindow *>(work->data) : nullptr;
+}
 
-		if (lw->options.id && strcmp(id, lw->options.id) == 0)
-			return lw;
-		}
-
-	return nullptr;
+gint layout_compare_options_id(const LayoutWindow *lw, const gchar *id)
+{
+	return g_strcmp0(lw->options.id, id);
 }
 
 gchar *layout_get_unique_id()
@@ -418,7 +414,7 @@ static GtkWidget *layout_tool_setup(LayoutWindow *lw)
 		gq_gtk_container_add(GTK_WIDGET(scroll_window), menu_toolbar_box);
 		gq_gtk_box_pack_start(GTK_BOX(box), scroll_window, FALSE, FALSE, 0);
 
-		gtk_widget_show_all(scroll_window);
+		gq_gtk_widget_show_all(scroll_window);
 		}
 	else
 		{
@@ -535,14 +531,7 @@ static void layout_sort_button_press_cb(GtkWidget *, gpointer data)
 	menu = submenu_add_sort(nullptr, G_CALLBACK(layout_sort_menu_cb), lw, FALSE, FALSE, TRUE, lw->options.file_view_list_sort.method);
 
 	/* take ownership of menu */
-#ifdef GTK_OBJECT_FLOATING
-	/* GTK+ < 2.10 */
-	g_object_ref(G_OBJECT(menu));
-	gtk_object_sink(GTK_OBJECT(menu));
-#else
-	/* GTK+ >= 2.10 */
 	g_object_ref_sink(G_OBJECT(menu));
-#endif
 
 	/* ascending option */
 	menu_item_add_divider(menu);
@@ -594,12 +583,9 @@ static void layout_zoom_menu_cb(GtkWidget *widget, gpointer data)
 
 static void layout_scroll_menu_cb(GtkWidget *widget, gpointer data)
 {
-	guint scroll_type;
-
 	if (!gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget))) return;
 
-	scroll_type = GPOINTER_TO_UINT(data);
-	options->image.scroll_reset_method = scroll_type;
+	options->image.scroll_reset_method = static_cast<ScrollReset>(GPOINTER_TO_UINT(data));
 	image_options_sync();
 }
 
@@ -618,32 +604,25 @@ static void layout_zoom_button_press_cb(GtkWidget *, gpointer data)
 			lw, FALSE, FALSE, TRUE, options->image.zoom_mode);
 
 	/* take ownership of menu */
-#ifdef GTK_OBJECT_FLOATING
-	/* GTK+ < 2.10 */
-	g_object_ref(G_OBJECT(menu));
-	gtk_object_sink(GTK_OBJECT(menu));
-#else
-	/* GTK+ >= 2.10 */
 	g_object_ref_sink(G_OBJECT(menu));
-#endif
 
 	menu_item_add_divider(menu);
 
 	menu_item_add_radio(menu, _("Scroll to top left corner"),
-			GUINT_TO_POINTER(SCROLL_RESET_TOPLEFT),
-			options->image.scroll_reset_method == SCROLL_RESET_TOPLEFT,
-			G_CALLBACK(layout_scroll_menu_cb),
-			GUINT_TO_POINTER(SCROLL_RESET_TOPLEFT));
+	                    GUINT_TO_POINTER(ScrollReset::TOPLEFT),
+	                    options->image.scroll_reset_method == ScrollReset::TOPLEFT,
+	                    G_CALLBACK(layout_scroll_menu_cb),
+	                    GUINT_TO_POINTER(ScrollReset::TOPLEFT));
 	menu_item_add_radio(menu, _("Scroll to image center"),
-			GUINT_TO_POINTER(SCROLL_RESET_CENTER),
-			options->image.scroll_reset_method == SCROLL_RESET_CENTER,
-			G_CALLBACK(layout_scroll_menu_cb),
-			GUINT_TO_POINTER(SCROLL_RESET_CENTER));
+	                    GUINT_TO_POINTER(ScrollReset::CENTER),
+	                    options->image.scroll_reset_method == ScrollReset::CENTER,
+	                    G_CALLBACK(layout_scroll_menu_cb),
+	                    GUINT_TO_POINTER(ScrollReset::CENTER));
 	menu_item_add_radio(menu, _("Keep the region from previous image"),
-			GUINT_TO_POINTER(SCROLL_RESET_NOCHANGE),
-			options->image.scroll_reset_method == SCROLL_RESET_NOCHANGE,
-			G_CALLBACK(layout_scroll_menu_cb),
-			GUINT_TO_POINTER(SCROLL_RESET_NOCHANGE));
+	                    GUINT_TO_POINTER(ScrollReset::NOCHANGE),
+	                    options->image.scroll_reset_method == ScrollReset::NOCHANGE,
+	                    G_CALLBACK(layout_scroll_menu_cb),
+	                    GUINT_TO_POINTER(ScrollReset::NOCHANGE));
 
 	g_signal_connect(G_OBJECT(menu), "selection_done",
 			 G_CALLBACK(layout_zoom_menu_hide_cb), NULL);
@@ -772,28 +751,20 @@ void layout_status_update_info(LayoutWindow *lw, const gchar *text)
 
 			layout_bars_new_selection(lw, s);
 
+			g_autofree gchar *b = text_from_size_abrev(n_bytes);
+
 			if (s > 0)
 				{
-				gchar *b = text_from_size_abrev(n_bytes);
-				gchar *sb = text_from_size_abrev(s_bytes);
+				g_autofree gchar *sb = text_from_size_abrev(s_bytes);
 				buf = g_strdup_printf(_("%s, %d files (%s, %d)%s"), b, n, sb, s, ss);
-				g_free(b);
-				g_free(sb);
-				g_free(ss);
-				}
-			else if (n > 0)
-				{
-				gchar *b = text_from_size_abrev(n_bytes);
-				buf = g_strdup_printf(_("%s, %d files%s"), b, n, ss);
-				g_free(b);
-				g_free(ss);
 				}
 			else
 				{
-				buf = g_strdup_printf(_("%d files%s"), n, ss);
-				g_free(ss);
+				buf = g_strdup_printf(_("%s, %d files%s"), b, n, ss);
 				}
 
+			g_free(ss);
+
 			text = buf;
 
 			image_osd_update(lw->image);
@@ -802,7 +773,7 @@ void layout_status_update_info(LayoutWindow *lw, const gchar *text)
 			{
 			text = "";
 			}
-	}
+		}
 
 	if (lw->info_status) gtk_label_set_text(GTK_LABEL(lw->info_status), text);
 	g_free(buf);
@@ -835,7 +806,8 @@ void layout_status_update_image(LayoutWindow *lw)
 
 		if (lw->image->unknown)
 			{
-			if (image_get_path(lw->image) && !access_file(image_get_path(lw->image), R_OK))
+			const gchar *filename = image_get_path(lw->image);
+			if (filename && !access_file(filename, R_OK))
 				{
 				text = g_strdup_printf(_("(no read permission) %s bytes"), b);
 				}
@@ -1396,23 +1368,6 @@ void layout_marks_set(LayoutWindow *lw, gboolean enable)
 	layout_list_sync_marks(lw);
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-gboolean layout_thumb_get_unused(LayoutWindow *lw)
-{
-	if (!layout_valid(&lw)) return FALSE;
-
-	return lw->options.show_thumbnails;
-}
-
-gboolean layout_marks_get_unused(LayoutWindow *lw)
-{
-	if (!layout_valid(&lw)) return FALSE;
-
-	return lw->options.show_marks;
-}
-#pragma GCC diagnostic pop
-
 void layout_sort_set_files(LayoutWindow *lw, SortType type, gboolean ascend, gboolean case_sensitive)
 {
 	if (!layout_valid(&lw)) return;
@@ -1437,15 +1392,13 @@ gboolean layout_sort_get(LayoutWindow *lw, SortType *type, gboolean *ascend, gbo
 	return TRUE;
 }
 
-gboolean layout_geometry_get(LayoutWindow *lw, gint *x, gint *y, gint *w, gint *h)
+static gboolean layout_geometry_get(LayoutWindow *lw, GdkRectangle &rect)
 {
 	GdkWindow *window;
 	if (!layout_valid(&lw)) return FALSE;
 
 	window = gtk_widget_get_window(lw->window);
-	gdk_window_get_root_origin(window, x, y);
-	*w = gdk_window_get_width(window);
-	*h = gdk_window_get_height(window);
+	rect = window_get_root_origin_geometry(window);
 
 	return TRUE;
 }
@@ -1515,19 +1468,6 @@ void layout_views_set_sort_dir(LayoutWindow *lw, SortType method, gboolean ascen
 	layout_style_set(lw, -1, nullptr);
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-gboolean layout_views_get_unused(LayoutWindow *lw, DirViewType *dir_view_type, FileViewType *file_view_type)
-{
-	if (!layout_valid(&lw)) return FALSE;
-
-	*dir_view_type = lw->options.dir_view_type;
-	*file_view_type = lw->options.file_view_type;
-
-	return TRUE;
-}
-#pragma GCC diagnostic pop
-
 /*
  *-----------------------------------------------------------------------------
  * location utils
@@ -1588,7 +1528,7 @@ static void layout_location_compute(LayoutLocation l1, LayoutLocation l2,
  *-----------------------------------------------------------------------------
  */
 
-gboolean layout_geometry_get_tools(LayoutWindow *lw, gint *x, gint *y, gint *w, gint *h, gint *divider_pos)
+static gboolean layout_geometry_get_tools(LayoutWindow *lw, GdkRectangle &rect, gint &divider_pos)
 {
 	GdkWindow *window;
 	GtkAllocation allocation;
@@ -1598,24 +1538,22 @@ gboolean layout_geometry_get_tools(LayoutWindow *lw, gint *x, gint *y, gint *w,
 		{
 		/* use the stored values (sort of breaks success return value) */
 
-		*divider_pos = lw->options.float_window.vdivider_pos;
+		divider_pos = lw->options.float_window.vdivider_pos;
 
 		return FALSE;
 		}
 
 	window = gtk_widget_get_window(lw->tools);
-	gdk_window_get_root_origin(window, x, y);
-	*w = gdk_window_get_width(window);
-	*h = gdk_window_get_height(window);
+	rect = window_get_root_origin_geometry(window);
 	gtk_widget_get_allocation(gtk_paned_get_child1(GTK_PANED(lw->tools_pane)), &allocation);
 
 	if (gtk_orientable_get_orientation(GTK_ORIENTABLE(lw->tools_pane)) == GTK_ORIENTATION_VERTICAL)
 		{
-		*divider_pos = allocation.height;
+		divider_pos = allocation.height;
 		}
 	else
 		{
-		*divider_pos = allocation.width;
+		divider_pos = allocation.width;
 		}
 
 	return TRUE;
@@ -1633,17 +1571,14 @@ static gboolean layout_geometry_get_log_window(LayoutWindow *lw, GdkRectangle &l
 		}
 
 	window = gtk_widget_get_window(lw->log_window);
-	gdk_window_get_root_origin(window, &log_window.x, &log_window.y);
-	log_window.width = gdk_window_get_width(window);
-	log_window.height = gdk_window_get_height(window);
+	log_window = window_get_root_origin_geometry(window);
 
 	return TRUE;
 }
 
 static void layout_tools_geometry_sync(LayoutWindow *lw)
 {
-	layout_geometry_get_tools(lw, &lw->options.float_window.x, &lw->options.float_window.y,
-				  &lw->options.float_window.w, &lw->options.float_window.h, &lw->options.float_window.vdivider_pos);
+	layout_geometry_get_tools(lw, lw->options.float_window.rect, lw->options.float_window.vdivider_pos);
 }
 
 static void layout_tools_hide(LayoutWindow *lw, gboolean hide)
@@ -1759,8 +1694,8 @@ static void layout_tools_setup(LayoutWindow *lw, GtkWidget *tools, GtkWidget *fi
 		{
 		if (options->save_window_positions)
 			{
-			gtk_window_set_default_size(GTK_WINDOW(lw->tools), lw->options.float_window.w, lw->options.float_window.h);
-			gq_gtk_window_move(GTK_WINDOW(lw->tools), lw->options.float_window.x, lw->options.float_window.y);
+			gtk_window_set_default_size(GTK_WINDOW(lw->tools), lw->options.float_window.rect.width, lw->options.float_window.rect.height);
+			gq_gtk_window_move(GTK_WINDOW(lw->tools), lw->options.float_window.rect.x, lw->options.float_window.rect.y);
 			}
 		else
 			{
@@ -2369,15 +2304,6 @@ static void startup_path_set_home_cb(GtkWidget *widget, gpointer data)
 	lc->options.startup_path = STARTUP_PATH_HOME;
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-static void layout_config_save_cb_unused(GtkWidget *, gpointer)
-{
-	//layout_config_apply();
-	save_options(options);
-}
-#pragma GCC diagnostic pop
-
 void layout_show_config_window(LayoutWindow *lw)
 {
 	LayoutConfig *lc;
@@ -2425,13 +2351,7 @@ void layout_show_config_window(LayoutWindow *lw)
 	gtk_widget_show(button);
 
 	ct_button = button;
-/*
-	button = pref_button_new(NULL, GQ_ICON_SAVE, _("Save"), FALSE,
-				 G_CALLBACK(layout_config_save_cb), NULL);
-	gq_gtk_container_add(GTK_WIDGET(hbox), button);
-	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
-	gtk_widget_show(button);
-*/
+
 	button = pref_button_new(nullptr, GQ_ICON_HELP, _("Help"),
 				 G_CALLBACK(layout_config_help_cb), lc);
 	gq_gtk_container_add(GTK_WIDGET(hbox), button);
@@ -2521,14 +2441,12 @@ void layout_sync_options_with_current_state(LayoutWindow *lw)
 	lw->options.main_window.maximized =  window_maximized(lw->window);
 	if (!lw->options.main_window.maximized)
 		{
-		layout_geometry_get(lw, &lw->options.main_window.x, &lw->options.main_window.y,
-				    &lw->options.main_window.w, &lw->options.main_window.h);
+		layout_geometry_get(lw, lw->options.main_window.rect);
 		}
 
 	layout_geometry_get_dividers(lw, &lw->options.main_window.hdivider_pos, &lw->options.main_window.vdivider_pos);
 
-	layout_geometry_get_tools(lw, &lw->options.float_window.x, &lw->options.float_window.y,
-				  &lw->options.float_window.w, &lw->options.float_window.h, &lw->options.float_window.vdivider_pos);
+	layout_geometry_get_tools(lw, lw->options.float_window.rect, lw->options.float_window.vdivider_pos);
 
 	lw->options.image_overlay.state = image_osd_get(lw->image);
 	histogram = image_osd_get_histogram(lw->image);
@@ -2740,8 +2658,8 @@ LayoutWindow *layout_new_with_geometry(FileData *dir_fd, LayoutOptions *lop,
 
 	if (options->save_window_positions || isfile(default_path))
 		{
-		gtk_window_set_default_size(GTK_WINDOW(lw->window), lw->options.main_window.w, lw->options.main_window.h);
-		gq_gtk_window_move(GTK_WINDOW(lw->window), lw->options.main_window.x, lw->options.main_window.y);
+		gtk_window_set_default_size(GTK_WINDOW(lw->window), lw->options.main_window.rect.width, lw->options.main_window.rect.height);
+		gq_gtk_window_move(GTK_WINDOW(lw->window), lw->options.main_window.rect.x, lw->options.main_window.rect.y);
 		if (lw->options.main_window.maximized) gtk_window_maximize(GTK_WINDOW(lw->window));
 
 		g_idle_add(move_window_to_workspace_cb, lw);
@@ -2837,10 +2755,10 @@ void layout_write_attributes(LayoutOptions *layout, GString *outstr, gint indent
 	WRITE_NL(); WRITE_UINT(*layout, startup_path);
 	WRITE_SEPARATOR();
 
-	WRITE_NL(); WRITE_INT(*layout, main_window.x);
-	WRITE_NL(); WRITE_INT(*layout, main_window.y);
-	WRITE_NL(); WRITE_INT(*layout, main_window.w);
-	WRITE_NL(); WRITE_INT(*layout, main_window.h);
+	WRITE_NL(); WRITE_INT_FULL("main_window.x", layout->main_window.rect.x);
+	WRITE_NL(); WRITE_INT_FULL("main_window.y", layout->main_window.rect.y);
+	WRITE_NL(); WRITE_INT_FULL("main_window.w", layout->main_window.rect.width);
+	WRITE_NL(); WRITE_INT_FULL("main_window.h", layout->main_window.rect.height);
 	WRITE_NL(); WRITE_BOOL(*layout, main_window.maximized);
 	WRITE_NL(); WRITE_INT(*layout, main_window.hdivider_pos);
 	WRITE_NL(); WRITE_INT(*layout, main_window.vdivider_pos);
@@ -2850,17 +2768,13 @@ void layout_write_attributes(LayoutOptions *layout, GString *outstr, gint indent
 	WRITE_NL(); WRITE_INT(*layout, folder_window.vdivider_pos);
 	WRITE_SEPARATOR();
 
-	WRITE_NL(); WRITE_INT(*layout, float_window.x);
-	WRITE_NL(); WRITE_INT(*layout, float_window.y);
-	WRITE_NL(); WRITE_INT(*layout, float_window.w);
-	WRITE_NL(); WRITE_INT(*layout, float_window.h);
+	WRITE_NL(); WRITE_INT_FULL("float_window.x", layout->float_window.rect.x);
+	WRITE_NL(); WRITE_INT_FULL("float_window.y", layout->float_window.rect.y);
+	WRITE_NL(); WRITE_INT_FULL("float_window.w", layout->float_window.rect.width);
+	WRITE_NL(); WRITE_INT_FULL("float_window.h", layout->float_window.rect.height);
 	WRITE_NL(); WRITE_INT(*layout, float_window.vdivider_pos);
 	WRITE_SEPARATOR();
 
-	WRITE_NL(); WRITE_INT(*layout, properties_window.w);
-	WRITE_NL(); WRITE_INT(*layout, properties_window.h);
-	WRITE_SEPARATOR();
-
 	WRITE_NL(); WRITE_BOOL(*layout, tools_float);
 	WRITE_NL(); WRITE_BOOL(*layout, tools_hidden);
 	WRITE_SEPARATOR();
@@ -2885,26 +2799,26 @@ void layout_write_attributes(LayoutOptions *layout, GString *outstr, gint indent
 	WRITE_NL(); WRITE_INT(*layout, log_window.width);
 	WRITE_NL(); WRITE_INT(*layout, log_window.height);
 
-	WRITE_NL(); WRITE_INT(*layout, preferences_window.x);
-	WRITE_NL(); WRITE_INT(*layout, preferences_window.y);
-	WRITE_NL(); WRITE_INT(*layout, preferences_window.w);
-	WRITE_NL(); WRITE_INT(*layout, preferences_window.h);
+	WRITE_NL(); WRITE_INT_FULL("preferences_window.x", layout->preferences_window.rect.x);
+	WRITE_NL(); WRITE_INT_FULL("preferences_window.y", layout->preferences_window.rect.y);
+	WRITE_NL(); WRITE_INT_FULL("preferences_window.w", layout->preferences_window.rect.width);
+	WRITE_NL(); WRITE_INT_FULL("preferences_window.h", layout->preferences_window.rect.height);
 	WRITE_NL(); WRITE_INT(*layout, preferences_window.page_number);
 
 	WRITE_NL(); WRITE_INT(*layout, search_window.x);
 	WRITE_NL(); WRITE_INT(*layout, search_window.y);
-	WRITE_NL(); WRITE_INT(*layout, search_window.w);
-	WRITE_NL(); WRITE_INT(*layout, search_window.h);
+	WRITE_NL(); WRITE_INT_FULL("search_window.w", layout->search_window.width);
+	WRITE_NL(); WRITE_INT_FULL("search_window.h", layout->search_window.height);
 
 	WRITE_NL(); WRITE_INT(*layout, dupe_window.x);
 	WRITE_NL(); WRITE_INT(*layout, dupe_window.y);
-	WRITE_NL(); WRITE_INT(*layout, dupe_window.w);
-	WRITE_NL(); WRITE_INT(*layout, dupe_window.h);
+	WRITE_NL(); WRITE_INT_FULL("dupe_window.w", layout->dupe_window.width);
+	WRITE_NL(); WRITE_INT_FULL("dupe_window.h", layout->dupe_window.height);
 
 	WRITE_NL(); WRITE_INT(*layout, advanced_exif_window.x);
 	WRITE_NL(); WRITE_INT(*layout, advanced_exif_window.y);
-	WRITE_NL(); WRITE_INT(*layout, advanced_exif_window.w);
-	WRITE_NL(); WRITE_INT(*layout, advanced_exif_window.h);
+	WRITE_NL(); WRITE_INT_FULL("advanced_exif_window.w", layout->advanced_exif_window.width);
+	WRITE_NL(); WRITE_INT_FULL("advanced_exif_window.h", layout->advanced_exif_window.height);
 	WRITE_SEPARATOR();
 
 	WRITE_NL(); WRITE_BOOL(*layout, animate);
@@ -2963,25 +2877,22 @@ void layout_load_attributes(LayoutOptions *layout, const gchar **attribute_names
 
 		/* window positions */
 
-		if (READ_INT(*layout, main_window.x)) continue;
-		if (READ_INT(*layout, main_window.y)) continue;
-		if (READ_INT(*layout, main_window.w)) continue;
-		if (READ_INT(*layout, main_window.h)) continue;
+		if (READ_INT_FULL("main_window.x", layout->main_window.rect.x)) continue;
+		if (READ_INT_FULL("main_window.y", layout->main_window.rect.y)) continue;
+		if (READ_INT_FULL("main_window.w", layout->main_window.rect.width)) continue;
+		if (READ_INT_FULL("main_window.h", layout->main_window.rect.height)) continue;
 		if (READ_BOOL(*layout, main_window.maximized)) continue;
 		if (READ_INT(*layout, main_window.hdivider_pos)) continue;
 		if (READ_INT(*layout, main_window.vdivider_pos)) continue;
 
 		if (READ_INT_CLAMP(*layout, folder_window.vdivider_pos, 1, 1000)) continue;
 
-		if (READ_INT(*layout, float_window.x)) continue;
-		if (READ_INT(*layout, float_window.y)) continue;
-		if (READ_INT(*layout, float_window.w)) continue;
-		if (READ_INT(*layout, float_window.h)) continue;
+		if (READ_INT_FULL("float_window.x", layout->float_window.rect.x)) continue;
+		if (READ_INT_FULL("float_window.y", layout->float_window.rect.y)) continue;
+		if (READ_INT_FULL("float_window.w", layout->float_window.rect.width)) continue;
+		if (READ_INT_FULL("float_window.h", layout->float_window.rect.height)) continue;
 		if (READ_INT(*layout, float_window.vdivider_pos)) continue;
 
-		if (READ_INT(*layout, properties_window.w)) continue;
-		if (READ_INT(*layout, properties_window.h)) continue;
-
 		if (READ_BOOL(*layout, tools_float)) continue;
 		if (READ_BOOL(*layout, tools_hidden)) continue;
 		if (READ_BOOL(*layout, show_info_pixel)) continue;
@@ -3002,26 +2913,26 @@ void layout_load_attributes(LayoutOptions *layout, const gchar **attribute_names
 		if (READ_INT(*layout, log_window.width)) continue;
 		if (READ_INT(*layout, log_window.height)) continue;
 
-		if (READ_INT(*layout, preferences_window.x)) continue;
-		if (READ_INT(*layout, preferences_window.y)) continue;
-		if (READ_INT(*layout, preferences_window.w)) continue;
-		if (READ_INT(*layout, preferences_window.h)) continue;
+		if (READ_INT_FULL("preferences_window.x", layout->preferences_window.rect.x)) continue;
+		if (READ_INT_FULL("preferences_window.y", layout->preferences_window.rect.y)) continue;
+		if (READ_INT_FULL("preferences_window.w", layout->preferences_window.rect.width)) continue;
+		if (READ_INT_FULL("preferences_window.h", layout->preferences_window.rect.height)) continue;
 		if (READ_INT(*layout, preferences_window.page_number)) continue;
 
 		if (READ_INT(*layout, search_window.x)) continue;
 		if (READ_INT(*layout, search_window.y)) continue;
-		if (READ_INT(*layout, search_window.w)) continue;
-		if (READ_INT(*layout, search_window.h)) continue;
+		if (READ_INT_FULL("search_window.w", layout->search_window.width)) continue;
+		if (READ_INT_FULL("search_window.h", layout->search_window.height)) continue;
 
 		if (READ_INT(*layout, dupe_window.x)) continue;
 		if (READ_INT(*layout, dupe_window.y)) continue;
-		if (READ_INT(*layout, dupe_window.w)) continue;
-		if (READ_INT(*layout, dupe_window.h)) continue;
+		if (READ_INT_FULL("dupe_window.w", layout->dupe_window.width)) continue;
+		if (READ_INT_FULL("dupe_window.h", layout->dupe_window.height)) continue;
 
 		if (READ_INT(*layout, advanced_exif_window.x)) continue;
 		if (READ_INT(*layout, advanced_exif_window.y)) continue;
-		if (READ_INT(*layout, advanced_exif_window.w)) continue;
-		if (READ_INT(*layout, advanced_exif_window.h)) continue;
+		if (READ_INT_FULL("advanced_exif_window.w", layout->advanced_exif_window.width)) continue;
+		if (READ_INT_FULL("advanced_exif_window.h", layout->advanced_exif_window.height)) continue;
 
 		if (READ_BOOL(*layout, animate)) continue;
 		if (READ_INT(*layout, workspace)) continue;
@@ -3169,8 +3080,9 @@ LayoutWindow *layout_new_from_default()
 		lw = layout_new_from_config(nullptr, nullptr, TRUE);
 		}
 
+	gchar *id_tmp = g_strdup(layout_get_unique_id());
 	g_free(lw->options.id);
-	lw->options.id = g_strdup(layout_get_unique_id());
+	lw->options.id = id_tmp;
 
 	return lw;
 }
