@@ -29,7 +29,7 @@
 #include "typedefs.h"
 
 struct AnimationData;
-struct FileData;
+class FileData;
 struct FullScreenData;
 struct ImageWindow;
 struct SlideShowData;
@@ -176,7 +176,7 @@ void layout_write_config(LayoutWindow *lw, GString *outstr, gint indent);
 LayoutWindow *layout_find_by_image(ImageWindow *imd);
 LayoutWindow *layout_find_by_image_fd(ImageWindow *imd);
 LayoutWindow *layout_find_by_layout_id(const gchar *id);
-
+gint layout_compare_options_id(const LayoutWindow *lw, const gchar *id);
 
 const gchar *layout_get_path(LayoutWindow *lw);
 gboolean layout_set_path(LayoutWindow *lw, const gchar *path);
@@ -219,7 +219,6 @@ void layout_file_filter_set(LayoutWindow *lw, gboolean enable);
 void layout_sort_set_files(LayoutWindow *lw, SortType type, gboolean ascend, gboolean case_sensitive);
 gboolean layout_sort_get(LayoutWindow *lw, SortType *type, gboolean *ascend, gboolean *case_sensitive);
 
-gboolean layout_geometry_get(LayoutWindow *lw, gint *x, gint *y, gint *w, gint *h);
 gboolean layout_geometry_get_dividers(LayoutWindow *lw, gint *h, gint *v);
 
 void layout_views_set(LayoutWindow *lw, DirViewType dir_view_type, FileViewType file_view_type);
@@ -235,7 +234,6 @@ void layout_styles_update();
 void layout_colors_update();
 
 
-gboolean layout_geometry_get_tools(LayoutWindow *lw, gint *x, gint *y, gint *w, gint *h, gint *divider_pos);
 void layout_tools_float_set(LayoutWindow *lw, gboolean popped, gboolean hidden);
 gboolean layout_tools_float_get(LayoutWindow *lw, gboolean *popped, gboolean *hidden);
 
