@@ -102,7 +102,7 @@ static gboolean key_pressed(GtkWidget *, GdkEventKey *event, LogWindow *logwin)
 
 	if (event && event->keyval == GDK_KEY_F1)
 		{
-		if (strlen(options->log_window.action) > 0)
+		if (options->log_window.action[0] != '\0')
 			{
 			buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(logwin->text));
 
@@ -178,9 +178,7 @@ static void remove_green_bg(LogWindow *logwin)
 	GtkTextIter start_match;
 	GtkTextIter end_match;
 	GtkTextBuffer *buffer;
-	GSList *list;
 	const gchar *text;
-	gchar *tag_name;
 	gint offset;
 
 	text = gq_gtk_entry_get_text(GTK_ENTRY(logwin->search_entry_box));
@@ -189,15 +187,15 @@ static void remove_green_bg(LogWindow *logwin)
 
 	while (gtk_text_iter_forward_search(&start_find, text, GTK_TEXT_SEARCH_VISIBLE_ONLY,  &start_match, &end_match, nullptr))
 		{
-		list = gtk_text_iter_get_tags(&start_match);
-		while (list)
+		GSList *list = gtk_text_iter_get_tags(&start_match);
+		for (GSList *work = list; work; work = work->next)
 			{
-			g_object_get(list->data, "name", &tag_name, NULL);
+			g_autofree gchar *tag_name = nullptr;
+			g_object_get(work->data, "name", &tag_name, NULL);
 			if (g_strcmp0(tag_name, "green_bg") == 0)
 				{
 				gtk_text_buffer_remove_tag_by_name(buffer, "green_bg", &start_match, &end_match);
 				}
-			list = list->next;
 			}
 		g_slist_free(list);
 
@@ -262,7 +260,7 @@ static gboolean search_keypress_event(GtkWidget *, GdkEventKey *, LogWindow *log
 	gtk_text_buffer_get_start_iter(buffer, &start_find);
 
 	text = gq_gtk_entry_get_text(GTK_ENTRY(logwin->search_entry_box));
-	if (strlen(text) == 0)
+	if (text[0] == '\0')
 		{
 		selected = gtk_text_buffer_get_selection_bounds(buffer, &start_sel, &end_sel);
 		if (selected)
@@ -450,7 +448,7 @@ static LogWindow *log_window_create(LayoutWindow *lw)
 	gq_gtk_container_add(GTK_WIDGET(logwin->pause), label) ;
 	gq_gtk_box_pack_start(GTK_BOX(hbox),logwin->pause, FALSE, FALSE, 0) ;
 	g_signal_connect(logwin->pause, "toggled", G_CALLBACK(log_window_pause_cb), logwin);
-	gtk_widget_show_all(logwin->pause);
+	gq_gtk_widget_show_all(logwin->pause);
 
 	logwin->wrap = gtk_toggle_button_new();
 	label = gtk_label_new("Wrap");
@@ -458,7 +456,7 @@ static LogWindow *log_window_create(LayoutWindow *lw)
 	gq_gtk_container_add(GTK_WIDGET(logwin->wrap), label) ;
 	gq_gtk_box_pack_start(GTK_BOX(hbox),logwin->wrap, FALSE, FALSE, 0) ;
 	g_signal_connect(logwin->wrap, "toggled", G_CALLBACK(log_window_line_wrap_cb), logwin);
-	gtk_widget_show_all(logwin->wrap);
+	gq_gtk_widget_show_all(logwin->wrap);
 
 	logwin->timer_data = gtk_toggle_button_new();
 	label = gtk_label_new(_("Timer"));
@@ -470,7 +468,7 @@ static LogWindow *log_window_create(LayoutWindow *lw)
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(logwin->timer_data), TRUE);
 		}
 	g_signal_connect(logwin->timer_data, "toggled", G_CALLBACK(log_window_timer_data_cb), logwin);
-	gtk_widget_show_all(logwin->timer_data);
+	gq_gtk_widget_show_all(logwin->timer_data);
 
 	search_box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	gq_gtk_container_add(GTK_WIDGET(hbox), search_box);
@@ -514,7 +512,7 @@ static LogWindow *log_window_create(LayoutWindow *lw)
 	gtk_widget_set_tooltip_text(GTK_WIDGET(all_button), _("Highlight all"));
 	gq_gtk_box_pack_start(GTK_BOX(search_box), all_button, FALSE, FALSE, 0) ;
 	g_signal_connect(all_button, "toggled", G_CALLBACK(all_keypress_event_cb), logwin);
-	gtk_widget_show_all(all_button);
+	gq_gtk_widget_show_all(all_button);
 	g_object_unref(pixbuf);
 
 	pref_label_new(hbox, _("Filter regexp"));
