@@ -19,47 +19,75 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#include <csignal>
-#include <sys/mman.h>
-
-#ifdef G_OS_UNIX
-#include <pwd.h>
-#endif
-#include <clocale>
-
 #include "main.h"
 
+#include <sys/types.h>
+#include <unistd.h>
+
+#include <clocale>
+#include <csignal>
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
+#include <ctime>
+
+#include <config.h>
+
+#if HAVE_CLUTTER
+#  include <clutter-gtk/clutter-gtk.h>
+#  include <clutter/clutter.h>
+#endif
+
+#if HAVE_EXECINFO_H
+#include <execinfo.h>
+#endif
+
+#include <gdk/gdk.h>
+#include <gio/gio.h>
+#include <glib-object.h>
+#include <gtk/gtk.h>
+
+#ifdef ENABLE_NLS
+#  include <libintl.h>
+#endif
+
+#if HAVE_DEVELOPER
+#include "third-party/backward.h"
+#endif
+
+#include "cache-maint.h"
 #include "cache.h"
-#include "collect.h"
 #include "collect-io.h"
+#include "collect.h"
+#include "compat.h"
+#include "debug.h"
+#include "exif.h"
 #include "filedata.h"
 #include "filefilter.h"
+#include "glua.h"
+#include "histogram.h"
 #include "history-list.h"
 #include "image.h"
 #include "img-view.h"
+#include "intl.h"
 #include "layout-image.h"
 #include "layout-util.h"
+#include "layout.h"
+#include "main-defines.h"
+#include "metadata.h"
 #include "misc.h"
+#include "options.h"
+#include "pixbuf-util.h"
 #include "rcfile.h"
 #include "remote.h"
 #include "secure-save.h"
+#include "third-party/whereami.h"
+#include "thumb.h"
 #include "ui-fileops.h"
 #include "ui-utildlg.h"
-#include "cache-maint.h"
-#include "thumb.h"
-#include "metadata.h"
-#include "exif.h"
-#include "histogram.h"
-#include "pixbuf-util.h"
-#include "glua.h"
-#include "whereami.h"
 
-#ifdef HAVE_CLUTTER
-#include <clutter-gtk/clutter-gtk.h>
-#endif
-
-#ifdef HAVE_EXECINFO_H
-#include <execinfo.h>
+#if ENABLE_UNIT_TESTS
+#  include "gtest/gtest.h"
 #endif
 
 gboolean thumb_format_changed = FALSE;
@@ -76,7 +104,7 @@ gchar *desktop_file_template;
 gchar *instance_identifier;
 
 #if defined(SA_SIGINFO)
-void sig_handler_cb(int signo, siginfo_t *info, void *)
+static void sig_handler_cb(int signo, siginfo_t *info, void *)
 {
 	gchar hex_char[16];
 	const gchar *signal_name = nullptr;
@@ -84,7 +112,7 @@ void sig_handler_cb(int signo, siginfo_t *info, void *)
 	guint64 addr;
 	guint64 char_index;
 	ssize_t len;
-#ifdef HAVE_EXECINFO_H
+#if HAVE_EXECINFO_H
 	gint bt_size;
 	void *bt[1024];
 #endif
@@ -170,7 +198,7 @@ void sig_handler_cb(int signo, siginfo_t *info, void *)
 		len = write(STDERR_FILENO, "\n", 1);
 		}
 
-#ifdef HAVE_EXECINFO_H
+#if HAVE_EXECINFO_H
 	bt_size = backtrace(bt, 1024);
 	backtrace_symbols_fd(bt, bt_size, STDERR_FILENO);
 #endif
@@ -183,7 +211,7 @@ void sig_handler_cb(int signo, siginfo_t *info, void *)
 #else /* defined(SA_SIGINFO) */
 void sig_handler_cb(int)
 {
-#ifdef HAVE_EXECINFO_H
+#if HAVE_EXECINFO_H
 	gint bt_size;
 	void *bt[1024];
 #endif
@@ -191,7 +219,7 @@ void sig_handler_cb(int)
 	write(STDERR_FILENO, "Geeqie fatal error\n", 19);
 	write(STDERR_FILENO, "Signal: Segmentation fault\n", 27);
 
-#ifdef HAVE_EXECINFO_H
+#if HAVE_EXECINFO_H
 	bt_size = backtrace(bt, 1024);
 	backtrace_symbols_fd(bt, bt_size, STDERR_FILENO);
 #endif
@@ -200,53 +228,6 @@ void sig_handler_cb(int)
 }
 #endif /* defined(SA_SIGINFO) */
 
-/*
- *-----------------------------------------------------------------------------
- * keyboard functions
- *-----------------------------------------------------------------------------
- */
-
-void keyboard_scroll_calc(gint *x, gint *y, GdkEventKey *event)
-{
-	static gint delta = 0;
-	static guint32 time_old = 0;
-	static guint keyval_old = 0;
-
-	if (event->state & GDK_CONTROL_MASK)
-		{
-		if (*x < 0) *x = G_MININT / 2;
-		if (*x > 0) *x = G_MAXINT / 2;
-		if (*y < 0) *y = G_MININT / 2;
-		if (*y > 0) *y = G_MAXINT / 2;
-
-		return;
-		}
-
-	if (options->progressive_key_scrolling)
-		{
-		guint32 time_diff;
-
-		time_diff = event->time - time_old;
-
-		/* key pressed within 125ms ? (1/8 second) */
-		if (time_diff > 125 || event->keyval != keyval_old) delta = 0;
-
-		time_old = event->time;
-		keyval_old = event->keyval;
-
-		delta += 2;
-		}
-	else
-		{
-		delta = 8;
-		}
-
-	*x = *x * delta * options->keyboard_scroll_step;
-	*y = *y * delta * options->keyboard_scroll_step;
-}
-
-
-
 /*
  *-----------------------------------------------------------------------------
  * command line parser (private) hehe, who needs popt anyway?
@@ -348,6 +329,16 @@ static void parse_command_line_process_file(const gchar *file_path, gchar **path
 	parse_command_line_add_file(file_path, path, file, list, collection_list);
 }
 
+static void show_invalid_parameters_warning_dialog(const gchar *command_line_errors)
+{
+	g_autoptr(GtkWidget) dialog_warning = gtk_message_dialog_new(nullptr, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
+	                                                             "%s", _("Invalid parameter(s):"));
+	gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog_warning), "%s", command_line_errors);
+	gtk_window_set_title(GTK_WINDOW(dialog_warning), GQ_APPNAME);
+	gq_gtk_window_set_keep_above(GTK_WINDOW(dialog_warning), TRUE);
+	gtk_dialog_run(GTK_DIALOG(dialog_warning));
+}
+
 static void parse_command_line(gint argc, gchar *argv[])
 {
 	GList *list = nullptr;
@@ -359,21 +350,17 @@ static void parse_command_line(gint argc, gchar *argv[])
 	gchar *pwd;
 	gchar *current_dir;
 	gchar *geometry = nullptr;
-	GtkWidget *dialog_warning;
-	GString *command_line_errors = g_string_new(nullptr);
 
 	command_line = g_new0(CommandLine, 1);
-
 	command_line->argc = argc;
 	command_line->argv = argv;
 	command_line->regexp = nullptr;
 
 	if (argc > 1)
 		{
-		gint i;
 		gchar *base_dir = get_current_dir();
-		i = 1;
-		while (i < argc)
+		g_autoptr(GString) command_line_errors = g_string_new(nullptr);
+		for (gint i = 1; i < argc; i++)
 			{
 			gchar *cmd_line = path_to_utf8(argv[i]);
 			gchar *cmd_all = g_build_filename(base_dir, cmd_line, NULL);
@@ -416,12 +403,12 @@ static void parse_command_line(gint argc, gchar *argv[])
 				{
 				/* do nothing but do not produce warnings */
 				}
-			else if (strcmp(cmd_line, "+t") == 0 ||
+			else if (strcmp(cmd_line, "-T") == 0 ||
 				 strcmp(cmd_line, "--with-tools") == 0)
 				{
 				command_line->tools_show = TRUE;
 
-				remote_list = g_list_append(remote_list, g_strdup("+t"));
+				remote_list = g_list_append(remote_list, g_strdup("-T"));
 				}
 			else if (strcmp(cmd_line, "-t") == 0 ||
 				 strcmp(cmd_line, "--without-tools") == 0)
@@ -458,26 +445,26 @@ static void parse_command_line(gint argc, gchar *argv[])
 					remote_list = remote_build_list(remote_list, argc - i, &argv[i], &remote_errors);
 					}
 				}
-			else if ((strcmp(cmd_line, "+w") == 0) ||
+			else if ((strcmp(cmd_line, "-w") == 0) ||
 						strcmp(cmd_line, "--show-log-window") == 0)
 				{
 				command_line->log_window_show = TRUE;
 				}
-			else if (strncmp(cmd_line, "-o:", 3) == 0)
+			else if (strncmp(cmd_line, "-o", 2) == 0)
 				{
-				command_line->log_file = g_strdup(cmd_line + 3);
+				command_line->log_file = g_strdup(cmd_line + 2);
 				}
-			else if (strncmp(cmd_line, "--log-file:", 11) == 0)
+			else if (strncmp(cmd_line, "--log-file=", 11) == 0)
 				{
 				command_line->log_file = g_strdup(cmd_line + 11);
 				}
-			else if (strncmp(cmd_line, "-g:", 3) == 0)
+			else if (strncmp(cmd_line, "-g", 2) == 0)
 				{
-				set_regexp(g_strdup(cmd_line+3));
+				set_regexp(g_strdup(cmd_line + 2));
 				}
-			else if (strncmp(cmd_line, "-grep:", 6) == 0)
+			else if (strncmp(cmd_line, "--grep=", 7) == 0)
 				{
-				set_regexp(g_strdup(cmd_line+3));
+				set_regexp(g_strdup(cmd_line + 7));
 				}
 			else if (strncmp(cmd_line, "-n", 2) == 0)
 				{
@@ -487,12 +474,6 @@ static void parse_command_line(gint argc, gchar *argv[])
 				{
 				command_line->new_instance = TRUE;
 				}
-			else if (strcmp(cmd_line, "-rh") == 0 ||
-				 strcmp(cmd_line, "--remote-help") == 0)
-				{
-				remote_help();
-				exit(0);
-				}
 			else if (strcmp(cmd_line, "--blank") == 0)
 				{
 				command_line->startup_blank = TRUE;
@@ -500,8 +481,8 @@ static void parse_command_line(gint argc, gchar *argv[])
 			else if (strcmp(cmd_line, "-v") == 0 ||
 				 strcmp(cmd_line, "--version") == 0)
 				{
-				printf_term(FALSE, "%s %s GTK%d\n", GQ_APPNAME, VERSION, gtk_major_version);
-				exit(0);
+				printf_term(FALSE, "%s %s GTK%u\n", GQ_APPNAME, VERSION, gtk_major_version);
+				exit(EXIT_SUCCESS);
 				}
 			else if (strcmp(cmd_line, "-h") == 0 ||
 				 strcmp(cmd_line, "--help") == 0)
@@ -510,24 +491,23 @@ static void parse_command_line(gint argc, gchar *argv[])
 				printf_term(FALSE, _("Usage: %s [options] [path]\n\n"), GQ_APPNAME_LC);
 				print_term(FALSE, _("Valid options:\n"));
 				print_term(FALSE, _("      --blank                      start with blank file list\n"));
-				print_term(FALSE, _("      --cache-maintenance <path>   run cache maintenance in non-GUI mode\n"));
+				print_term(FALSE, _("      --cache-maintenance=<path>   run cache maintenance in non-GUI mode\n"));
 				print_term(FALSE, _("      --disable-clutter            disable use of Clutter library (i.e. GPU accel.)\n"));
 				print_term(FALSE, _("  -f, --fullscreen                 start in full screen mode\n"));
 				print_term(FALSE, _("      --geometry=WxH+XOFF+YOFF     set main window location\n"));
 				print_term(FALSE, _("  -h, --help                       show this message\n"));
 				print_term(FALSE, _("  -l, --list [files] [collections] open collection window for command line\n"));
 				print_term(FALSE, _("  -n, --new-instance               open a new instance of Geeqie *\n"));
-				print_term(FALSE, _("  -o:, --log-file:<file>           save log data to file\n"));
+				print_term(FALSE, _("  -o, --log-file=<file>            save log data to file\n"));
 				print_term(FALSE, _("  -r, --remote                     send following commands to open window\n"));
-				print_term(FALSE, _("  -rh, --remote-help               print remote command list\n"));
 				print_term(FALSE, _("  -s, --slideshow                  start in slideshow mode\n"));
-				print_term(FALSE, _("  +t, --with-tools                 force show of tools\n"));
+				print_term(FALSE, _("  -T, --with-tools                 force show of tools\n"));
 				print_term(FALSE, _("  -t, --without-tools              force hide of tools\n"));
 				print_term(FALSE, _("  -v, --version                    print version info\n"));
-				print_term(FALSE, _("  +w, --show-log-window            show log window\n"));
+				print_term(FALSE, _("  -w, --show-log-window            show log window\n"));
 #ifdef DEBUG
-				print_term(FALSE, _("      --debug[=level]              turn on debug output\n"));
-				print_term(FALSE, _("  -g:, --grep:<regexp>             filter debug output\n"));
+				print_term(FALSE, _("      --debug=[level]              turn on debug output\n"));
+				print_term(FALSE, _("  -g, --grep=<regexp>              filter debug output\n"));
 #endif
 
 				print_term(FALSE, "\n");
@@ -535,29 +515,20 @@ static void parse_command_line(gint argc, gchar *argv[])
 
 				remote_help();
 
-
-				exit(0);
+				exit(EXIT_SUCCESS);
 				}
 			else if (!remote_do)
 				{
-				command_line_errors = g_string_append(command_line_errors, cmd_line);
-				command_line_errors = g_string_append(command_line_errors, "\n");
+				g_string_append_printf(command_line_errors, is_remote_command(cmd_line) ? _("%s\n\nThis is a --remote command option\n") : _("%s\n\nThis option is unknown\n"), cmd_line);
 				}
 
 			g_free(cmd_all);
 			g_free(cmd_line);
-			i++;
 			}
 
 		if (command_line_errors->len > 0)
 			{
-			dialog_warning = gtk_message_dialog_new(nullptr, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, "%s", "Invalid parameter(s):");
-			gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog_warning), "%s", command_line_errors->str);
-			gtk_window_set_title(GTK_WINDOW(dialog_warning), GQ_APPNAME);
-			gq_gtk_window_set_keep_above(GTK_WINDOW(dialog_warning), TRUE);
-			gtk_dialog_run(GTK_DIALOG(dialog_warning));
-			g_object_unref(dialog_warning);
-			g_string_free(command_line_errors, TRUE);
+			show_invalid_parameters_warning_dialog(command_line_errors->str);
 
 			exit(EXIT_FAILURE);
 			}
@@ -601,24 +572,16 @@ static void parse_command_line(gint argc, gchar *argv[])
 		{
 		if (remote_errors)
 			{
-			GList *work = remote_errors;
+			g_autoptr(GString) command_line_errors = g_string_new(nullptr);
 
-			while (work)
+			for (GList *work = remote_errors; work; work = work->next)
 				{
 				auto opt = static_cast<gchar *>(work->data);
 
-				command_line_errors = g_string_append(command_line_errors, opt);
-				command_line_errors = g_string_append(command_line_errors, "\n");
-				work = work->next;
+				g_string_append_printf(command_line_errors, "%s\n", opt);
 				}
 
-			dialog_warning = gtk_message_dialog_new(nullptr, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, "%s", "Invalid parameter(s):");
-			gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog_warning), "%s", command_line_errors->str);
-			gtk_window_set_title(GTK_WINDOW(dialog_warning), GQ_APPNAME);
-			gq_gtk_window_set_keep_above(GTK_WINDOW(dialog_warning), TRUE);
-			gtk_dialog_run(GTK_DIALOG(dialog_warning));
-			g_object_unref(dialog_warning);
-			g_string_free(command_line_errors, TRUE);
+			show_invalid_parameters_warning_dialog(command_line_errors->str);
 
 			exit(EXIT_FAILURE);
 			}
@@ -627,7 +590,7 @@ static void parse_command_line(gint argc, gchar *argv[])
 		 * for use by any remote command that needs it
 		 */
 		current_dir = g_get_current_dir();
-		pwd = g_strconcat("--PWD:", current_dir, NULL);
+		pwd = g_strconcat("--PWD=", current_dir, NULL);
 		remote_list = g_list_prepend(remote_list, pwd);
 
 		remote_control(argv[0], remote_list, command_line->path, list, command_line->collection_list);
@@ -666,28 +629,24 @@ static void parse_command_line_for_debug_option(gint argc, gchar *argv[])
 {
 #ifdef DEBUG
 	const gchar *debug_option = "--debug";
-	gint len = strlen(debug_option);
+	const gint len = strlen(debug_option);
 
-	if (argc > 1)
+	for (gint i = 1; i < argc; i++)
 		{
-		gint i;
-
-		for (i = 1; i < argc; i++)
+		// TODO(xsdg): Replace this with a regex match.  Simpler and less error-prone.
+		const gchar *cmd_line = argv[i];
+		if (strncmp(cmd_line, debug_option, len) == 0)
 			{
-			const gchar *cmd_line = argv[i];
-			if (strncmp(cmd_line, debug_option, len) == 0)
-				{
-				gint cmd_line_len = strlen(cmd_line);
+			const gint cmd_line_len = strlen(cmd_line);
 
-				/* we now increment the debug state for verbosity */
-				if (cmd_line_len == len)
-					debug_level_add(1);
-				else if (cmd_line[len] == '=' && g_ascii_isdigit(cmd_line[len+1]))
-					{
-					gint n = atoi(cmd_line + len + 1);
-					if (n < 0) n = 1;
-					debug_level_add(n);
-					}
+			/* we now increment the debug state for verbosity */
+			if (cmd_line_len == len)
+				debug_level_add(1);
+			else if (cmd_line[len] == '=' && g_ascii_isdigit(cmd_line[len+1]))
+				{
+				gint n = atoi(cmd_line + len + 1);
+				if (n < 0) n = 1;
+				debug_level_add(n);
 				}
 			}
 		}
@@ -696,118 +655,113 @@ static void parse_command_line_for_debug_option(gint argc, gchar *argv[])
 #endif
 }
 
-#ifdef HAVE_CLUTTER
-static gboolean parse_command_line_for_clutter_option(gint argc, gchar *argv[])
+static gboolean search_command_line_for_option(const gint argc, const gchar* const argv[], const gchar* option_name)
 {
-	const gchar *clutter_option = "--disable-clutter";
-	gint len = strlen(clutter_option);
-	gboolean ret = FALSE;
+	const gint name_len = strlen(option_name);
 
-	if (argc > 1)
+	for (gint i = 1; i < argc; i++)
 		{
-		gint i;
-
-		for (i = 1; i < argc; i++)
+		const gchar *current_arg = argv[i];
+		// TODO(xsdg): This actually only checks prefixes.  We should
+		// probably replace this with strcmp, since strlen already has
+		// the shortcomings of strcmp (as compared to strncmp).
+		//
+		// That said, people may be unknowingly relying on the lenience
+		// of this parsing strategy, so that's also something to consider.
+		if (strncmp(current_arg, option_name, name_len) == 0)
 			{
-			const gchar *cmd_line = argv[i];
-			if (strncmp(cmd_line, clutter_option, len) == 0)
-				{
-				ret = TRUE;
-				}
+			return TRUE;
 			}
 		}
 
-	return ret;
+	return FALSE;
+}
+
+static gboolean search_command_line_for_unit_test_option(gint argc, gchar *argv[])
+{
+	return search_command_line_for_option(argc, argv, "--run-unit-tests");
+}
+
+#if HAVE_CLUTTER
+static gboolean search_command_line_for_clutter_option(gint argc, gchar *argv[])
+{
+	return search_command_line_for_option(argc, argv, "--disable-clutter");
 }
 #endif
 
 static gboolean parse_command_line_for_cache_maintenance_option(gint argc, gchar *argv[])
 {
-	const gchar *cache_maintenance_option = "--cache-maintenance";
-	gint len = strlen(cache_maintenance_option);
-	gboolean ret = FALSE;
+	const gchar *cache_maintenance_option = "--cache-maintenance=";
+	const gint len = strlen(cache_maintenance_option);
 
 	if (argc >= 2)
 		{
 		const gchar *cmd_line = argv[1];
 		if (strncmp(cmd_line, cache_maintenance_option, len) == 0)
 			{
-			ret = TRUE;
+			return TRUE;
 			}
 		}
 
-	return ret;
+	return FALSE;
 }
 
 static void process_command_line_for_cache_maintenance_option(gint argc, gchar *argv[])
 {
-	gchar *rc_path;
-	gchar *folder_path = nullptr;
-	gsize size;
-	gsize i = 0;
-	gchar *buf_config_file;
-	gint diff_count;
-
-	if (argc >= 3)
-		{
-		folder_path = expand_tilde(argv[2]);
-
-		if (isdir(folder_path))
-			{
-			rc_path = g_build_filename(get_rc_dir(), RC_FILE_NAME, NULL);
-
-			if (isfile(rc_path))
-				{
-				if (g_file_get_contents(rc_path, &buf_config_file, &size, nullptr))
-					{
-					while (i < size)
-						{
-						diff_count = strncmp("</global>", &buf_config_file[i], 9);
-						if (diff_count == 0)
-							{
-							break;
-							}
-						i++;
-						}
-					/* Load only the <global> section */
-					load_config_from_buf(buf_config_file, i + 9, FALSE);
-
-					if (options->thumbnails.enable_caching)
-						{
-						cache_maintenance(folder_path);
-						}
-					else
-						{
-						print_term(TRUE, "Caching not enabled\n");
-						exit(EXIT_FAILURE);
-						}
-					g_free(buf_config_file);
-					}
-				else
-					{
-					print_term(TRUE, g_strconcat(_("Cannot load "), rc_path, "\n", NULL));
-					exit(EXIT_FAILURE);
-					}
-				}
-			else
-				{
-				print_term(TRUE, g_strconcat(_("Configuration file path "), rc_path, _(" is not a file\n"), NULL));
-				exit(EXIT_FAILURE);
-				}
-			g_free(rc_path);
-			}
-		else
-			{
-			print_term(TRUE, g_strconcat(argv[2], _(" is not a folder\n"), NULL));
-			exit(EXIT_FAILURE);
-			}
-		g_free(folder_path);
-		}
-	else
+	if (argc < 2)
 		{
 		print_term(TRUE, _("No path parameter given\n"));
 		exit(EXIT_FAILURE);
 		}
+
+	const gchar *cache_maintenance_option = "--cache-maintenance=";
+	const gint len = strlen(cache_maintenance_option);
+
+	g_autofree gchar *folder_path = expand_tilde(argv[1] + len);
+	if (!isdir(folder_path))
+		{
+		print_term(TRUE, g_strconcat(argv[1] + len, _(" is not a folder\n"), NULL));
+		exit(EXIT_FAILURE);
+		}
+
+	g_autofree gchar *rc_path = g_build_filename(get_rc_dir(), RC_FILE_NAME, NULL);
+	if (!isfile(rc_path))
+		{
+		print_term(TRUE, g_strconcat(_("Configuration file path "), rc_path, _(" is not a file\n"), NULL));
+		exit(EXIT_FAILURE);
+		}
+
+	g_autofree gchar *buf_config_file = nullptr;
+	gsize size;
+	if (!g_file_get_contents(rc_path, &buf_config_file, &size, nullptr))
+		{
+		print_term(TRUE, g_strconcat(_("Cannot load "), rc_path, "\n", NULL));
+		exit(EXIT_FAILURE);
+		}
+
+	/* Load only the <global> section */
+	const gchar *global_tag_end = "</global>";
+	const gint global_tag_end_len = strlen(global_tag_end);
+
+	gsize global_section_size = size;
+	for (gsize i = 0; i < size; i++)
+		{
+		if (strncmp(global_tag_end, buf_config_file + i, global_tag_end_len) == 0)
+			{
+			global_section_size = i + global_tag_end_len;
+			break;
+			}
+		}
+
+	load_config_from_buf(buf_config_file, global_section_size, FALSE);
+
+	if (!options->thumbnails.enable_caching)
+		{
+		print_term(TRUE, "Caching not enabled\n");
+		exit(EXIT_FAILURE);
+		}
+
+	cache_maintenance(folder_path);
 }
 
 /*
@@ -822,27 +776,33 @@ static void process_command_line_for_cache_maintenance_option(gint argc, gchar *
 static void setup_env_path()
 {
 	const gchar *old_path = g_getenv("PATH");
-	gchar *path = g_strconcat(gq_bindir, ":", old_path, NULL);
-        g_setenv("PATH", path, TRUE);
-	g_free(path);
+	g_autofree gchar *path = g_strconcat(gq_bindir, ":", old_path, NULL);
+	g_setenv("PATH", path, TRUE);
+}
+
+static const gchar *get_history_path()
+{
+#if USE_XDG
+	static gchar *history_path = g_build_filename(xdg_data_home_get(), GQ_APPNAME_LC, RC_HISTORY_NAME, NULL);
+#else
+	static gchar *history_path = g_build_filename(get_rc_dir(), RC_HISTORY_NAME, NULL);
+#endif
+
+	return history_path;
 }
 
 static void keys_load()
 {
-	gchar *path;
+	const gchar *path = get_history_path();
 
-	path = g_build_filename(get_rc_dir(), RC_HISTORY_NAME, NULL);
 	history_list_load(path);
-	g_free(path);
 }
 
 static void keys_save()
 {
-	gchar *path;
+	const gchar *path = get_history_path();
 
-	path = g_build_filename(get_rc_dir(), RC_HISTORY_NAME, NULL);
 	history_list_save(path);
-	g_free(path);
 }
 
 static void marks_load()
@@ -889,7 +849,8 @@ static void gq_accel_map_print(
 {
 	GString *gstring = g_string_new(changed ? nullptr : "; ");
 	auto ssi = static_cast<SecureSaveInfo *>(data);
-	gchar *tmp, *name;
+	gchar *tmp;
+	gchar *name;
 
 	g_string_append(gstring, "(gtk_accel_path \"");
 
@@ -1100,7 +1061,7 @@ static gint exit_confirm_dlg()
 	g_free(msg);
 	msg = g_strdup_printf(_("Quit %s"), GQ_APPNAME);
 
-	message = g_string_new(NULL);
+	message = g_string_new(nullptr);
 
 	if (collection_window_modified_exists())
 		{
@@ -1136,58 +1097,14 @@ void exit_program()
 
 	if (metadata_write_queue_confirm(FALSE, exit_program_write_metadata_cb, nullptr)) return;
 
-	options->marks_save ? marks_save(TRUE) : marks_save(FALSE);
+	marks_save(options->marks_save);
 
 	if (exit_confirm_dlg()) return;
 
 	exit_program_final();
 }
 
-/* This code attempts to handle situation when a file mmaped by image_loader
- * or by exif loader is truncated by some other process.
- * This code is incorrect according to POSIX, because:
- *
- *   mmap is not async-signal-safe and thus may not be called from a signal handler
- *
- *   mmap must be called with a valid file descriptor.  POSIX requires that
- *   a fildes argument of -1 must cause mmap to return EBADF.
- *
- * See https://github.com/BestImageViewer/geeqie/issues/1052 for discussion of
- * an alternative approach.
- */
-/** @FIXME this probably needs some better ifdefs. Please report any compilation problems */
-/** @FIXME This section needs revising */
-
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-#if defined(SIGBUS) && defined(SA_SIGINFO)
-static void sigbus_handler_cb_unused(int, siginfo_t *info, void *)
-{
-	/*
-	 * @FIXME Design and implement a POSIX-acceptable approach,
-	 * after first documenting the sitations where SIGBUS occurs.
-	 * See https://github.com/BestImageViewer/geeqie/issues/1052 for discussion
-	 */
-
-	DEBUG_1("SIGBUS %p NOT HANDLED", info->si_addr);
-	exit(EXIT_FAILURE);
-}
-#endif
-
-static void setup_sigbus_handler_unused(void)
-{
-#if defined(SIGBUS) && defined(SA_SIGINFO)
-	struct sigaction sigbus_action;
-	sigfillset(&sigbus_action.sa_mask);
-	sigbus_action.sa_sigaction = sigbus_handler_cb_unused;
-	sigbus_action.sa_flags = SA_SIGINFO;
-
-	sigaction(SIGBUS, &sigbus_action, NULL);
-#endif
-}
-#pragma GCC diagnostic pop
-
-#ifndef HAVE_DEVELOPER
+#if !HAVE_DEVELOPER
 static void setup_sig_handler()
 {
 	struct sigaction sigsegv_action;
@@ -1279,6 +1196,19 @@ static void create_application_paths()
 
 gint main(gint argc, gchar *argv[])
 {
+	// We handle unit tests here because it takes the place of running the
+	// rest of the app.
+	if (search_command_line_for_unit_test_option(argc, argv))
+	{
+#if ENABLE_UNIT_TESTS
+		testing::InitGoogleTest(&argc, argv);
+		return RUN_ALL_TESTS();
+#else
+		fprintf(stderr, "Unit tests are not enabled in this build.\n");
+		return 1;
+#endif
+	}
+
 	CollectionData *cd = nullptr;
 	CollectionData *first_collection = nullptr;
 	gboolean disable_clutter = FALSE;
@@ -1295,7 +1225,7 @@ gint main(gint argc, gchar *argv[])
 	gdk_threads_enter();
 
 	/* seg. fault handler */
-#ifdef HAVE_DEVELOPER
+#if HAVE_DEVELOPER
 	backward::SignalHandling sh{};
 #else
 	setup_sig_handler();
@@ -1317,18 +1247,13 @@ gint main(gint argc, gchar *argv[])
 
 	exif_init();
 
-#ifdef HAVE_LUA
+#if HAVE_LUA
 	lua_init();
 #endif
 
 	/* setup random seed for random slideshow */
 	srand(time(nullptr));
 
-#if 0
-	/* See later comment; this handler leads to UB. */
-	setup_sigbus_handler();
-#endif
-
 	/* register global notify functions */
 	file_data_register_notify_func(cache_notify_cb, nullptr, NOTIFY_PRIORITY_HIGH);
 	file_data_register_notify_func(thumb_notify_cb, nullptr, NOTIFY_PRIORITY_HIGH);
@@ -1341,8 +1266,8 @@ gint main(gint argc, gchar *argv[])
 
 	parse_command_line_for_debug_option(argc, argv);
 	DEBUG_1("%s main: gtk_init", get_exec_time());
-#ifdef HAVE_CLUTTER
-	if (parse_command_line_for_clutter_option(argc, argv))
+#if HAVE_CLUTTER
+	if (search_command_line_for_clutter_option(argc, argv))
 		{
 		disable_clutter	= TRUE;
 		gtk_init(&argc, &argv);
@@ -1353,7 +1278,7 @@ gint main(gint argc, gchar *argv[])
 			{
 			log_printf("Can't initialize clutter-gtk.\nStart Geeqie with the option \"geeqie --disable-clutter\"");
 			runcmd("zenity --error --title=\"Geeqie\" --text \"Can't initialize clutter-gtk.\n\nStart Geeqie with the option:\n geeqie --disable-clutter\" --width=300");
-			exit(1);
+			exit(EXIT_FAILURE);
 			}
 		}
 #else
@@ -1366,7 +1291,7 @@ gint main(gint argc, gchar *argv[])
 		log_printf("!!! This is a friendly warning.\n");
 		log_printf("!!! The version of GTK+ in use now is older than when %s was compiled.\n", GQ_APPNAME);
 		log_printf("!!!  compiled with GTK+-%d.%d\n", GTK_MAJOR_VERSION, GTK_MINOR_VERSION);
-		log_printf("!!!   running with GTK+-%d.%d\n", gtk_major_version, gtk_minor_version);
+		log_printf("!!!   running with GTK+-%u.%u\n", gtk_major_version, gtk_minor_version);
 		log_printf("!!! %s may quit unexpectedly with a relocation error.\n", GQ_APPNAME);
 		}
 
@@ -1423,7 +1348,7 @@ gint main(gint argc, gchar *argv[])
 			filter_rebuild();
 			}
 
-	#ifdef HAVE_CLUTTER
+	#if HAVE_CLUTTER
 	/** @FIXME For the background of this see:
 	 * https://github.com/BestImageViewer/geeqie/issues/397
 	 * The feature CLUTTER_FEATURE_SWAP_EVENTS indictates if the
@@ -1601,7 +1526,7 @@ gint main(gint argc, gchar *argv[])
 			{
 			appimage_notification();
 			}
-		else if (g_strstr_len(gq_executable_path, -1, "AppRun"))
+		else if (g_strstr_len(gq_executable_path, -1, "squashfs-root"))
 			{
 			/* Probably running an extracted AppImage */
 			appimage_notification();
