@@ -28,7 +28,6 @@
 #include "collect.h"
 #include "editors.h"
 #include "intl.h"
-#include "main-defines.h"
 #include "pixbuf-util.h"
 #include "ui-menu.h"
 
@@ -257,7 +256,7 @@ GtkWidget *submenu_add_dir_sort(GtkWidget *menu, GCallback func, gpointer data,
 	return submenu;
 }
 
-gchar *zoom_type_get_text(ZoomMode method)
+static gchar *zoom_type_get_text(ZoomMode method)
 {
 	switch (method)
 		{
@@ -459,8 +458,8 @@ GtkWidget *submenu_add_collections(GtkWidget *menu, GtkWidget **menu_item,
 	submenu = gtk_menu_new();
 	g_object_set_data(G_OBJECT(submenu), "submenu_data", data);
 
-	menu_item_add_stock_sensitive(submenu, _("New collection"),
-					GQ_ICON_COLLECTION, TRUE, G_CALLBACK(func), GINT_TO_POINTER(-1));
+	menu_item_add_icon_sensitive(submenu, _("New collection"),
+	                             PIXBUF_INLINE_COLLECTION, TRUE, G_CALLBACK(func), GINT_TO_POINTER(-1));
 	menu_item_add_divider(submenu);
 
 	collect_manager_list(&collection_list,nullptr,nullptr);
