@@ -29,6 +29,7 @@
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
+#include <string>
 
 #include <glib-object.h>
 
@@ -59,11 +60,16 @@ enum MetadataKey {
 	MK_COMMENT
 };
 
+struct MetadataCacheEntry {
+	gchar *key;
+	GList *values;
+};
+
 /* If contents change, keep GuideOptionsMetadata.xml up to date */
 /**
  *  @brief Tags that will be written to all files in a group - selected by: options->metadata.sync_grouped_files, Preferences/Metadata/Write The Same Description Tags To All Grouped Sidecars
  */
-constexpr std::array<const gchar *, 21> group_keys{
+constexpr std::array<const gchar *, 22> group_keys{
 	"Xmp.dc.title",
 	"Xmp.photoshop.Urgency",
 	"Xmp.photoshop.Category",
@@ -85,8 +91,28 @@ constexpr std::array<const gchar *, 21> group_keys{
 	"Xmp.dc.rights",
 	"Xmp.dc.description",
 	"Xmp.photoshop.CaptionWriter",
+	"Xmp.xmp.Rating",
 };
 
+gint metadata_cache_entry_compare_key(const MetadataCacheEntry *entry, const gchar *key)
+{
+	return strcmp(entry->key, key);
+}
+
+void metadata_cache_entry_free(MetadataCacheEntry *entry)
+{
+	if (!entry) return;
+
+	g_free(entry->key);
+	g_list_free_full(entry->values, g_free);
+	g_free(entry);
+}
+
+void string_list_free(gpointer data)
+{
+	g_list_free_full(static_cast<GList *>(data), g_free);
+}
+
 inline gboolean is_keywords_separator(gchar c)
 {
 	return c == ','
@@ -110,82 +136,64 @@ static gboolean metadata_file_read(gchar *path, GList **keywords, gchar **commen
  *-------------------------------------------------------------------
  */
 
-/* fd->cached metadata list of lists
-   each particular list contains key as a first entry, then the values
-*/
+/* fd->cached_metadata list of MetadataCacheEntry */
 
 static void metadata_cache_update(FileData *fd, const gchar *key, const GList *values)
 {
 	GList *work;
 
-	work = fd->cached_metadata;
-	while (work)
+	work = g_list_find_custom(fd->cached_metadata, key, reinterpret_cast<GCompareFunc>(metadata_cache_entry_compare_key));
+	if (work)
 		{
-		auto entry = static_cast<GList *>(work->data);
-		auto entry_key = static_cast<gchar *>(entry->data);
+		/* key found - just replace values */
+		auto *entry = static_cast<MetadataCacheEntry *>(work->data);
 
-		if (strcmp(entry_key, key) == 0)
-			{
-			/* key found - just replace values */
-			GList *old_values = entry->next;
-			entry->next = nullptr;
-			old_values->prev = nullptr;
-			g_list_free_full(old_values, g_free);
-			work->data = g_list_append(entry, string_list_copy(values));
-			DEBUG_1("updated %s %s\n", key, fd->path);
-			return;
-			}
-		work = work->next;
+		g_list_free_full(entry->values, g_free);
+		entry->values = string_list_copy(values);
+		DEBUG_1("updated %s %s\n", key, fd->path);
+		return;
 		}
 
 	/* key not found - prepend new entry */
-	fd->cached_metadata = g_list_prepend(fd->cached_metadata,
-				g_list_prepend(string_list_copy(values), g_strdup(key)));
-	DEBUG_1("added %s %s\n", key, fd->path);
+	auto *entry = g_new0(MetadataCacheEntry, 1);
+	entry->key = g_strdup(key);
+	entry->values = string_list_copy(values);
 
+	fd->cached_metadata = g_list_prepend(fd->cached_metadata, entry);
+	DEBUG_1("added %s %s\n", key, fd->path);
 }
 
 static const GList *metadata_cache_get(FileData *fd, const gchar *key)
 {
 	GList *work;
 
-	work = fd->cached_metadata;
-	while (work)
+	work = g_list_find_custom(fd->cached_metadata, key, reinterpret_cast<GCompareFunc>(metadata_cache_entry_compare_key));
+	if (work)
 		{
-		auto entry = static_cast<GList *>(work->data);
-		auto entry_key = static_cast<gchar *>(entry->data);
+		/* key found */
+		auto *entry = static_cast<MetadataCacheEntry *>(work->data);
 
-		if (strcmp(entry_key, key) == 0)
-			{
-			/* key found */
-			DEBUG_1("found %s %s\n", key, fd->path);
-			return entry;
-			}
-		work = work->next;
+		DEBUG_1("found %s %s\n", key, fd->path);
+		return entry->values;
 		}
-	return nullptr;
 	DEBUG_1("not found %s %s\n", key, fd->path);
+	return nullptr;
 }
 
 static void metadata_cache_remove(FileData *fd, const gchar *key)
 {
 	GList *work;
 
-	work = fd->cached_metadata;
-	while (work)
+	work = g_list_find_custom(fd->cached_metadata, key, reinterpret_cast<GCompareFunc>(metadata_cache_entry_compare_key));
+	if (work)
 		{
-		auto entry = static_cast<GList *>(work->data);
-		auto entry_key = static_cast<gchar *>(entry->data);
+		/* key found */
+		auto *entry = static_cast<MetadataCacheEntry *>(work->data);
 
-		if (strcmp(entry_key, key) == 0)
-			{
-			/* key found */
-			g_list_free_full(entry, g_free);
-			fd->cached_metadata = g_list_delete_link(fd->cached_metadata, work);
-			DEBUG_1("removed %s %s\n", key, fd->path);
-			return;
-			}
-		work = work->next;
+		metadata_cache_entry_free(entry);
+		fd->cached_metadata = g_list_delete_link(fd->cached_metadata, work);
+		DEBUG_1("removed %s %s\n", key, fd->path);
+		return;
 		}
 	DEBUG_1("not removed %s %s\n", key, fd->path);
 }
@@ -194,19 +202,11 @@ void metadata_cache_free(FileData *fd)
 {
 	if (fd->cached_metadata) DEBUG_1("freed %s\n", fd->path);
 
-	g_list_free_full(fd->cached_metadata, [](gpointer data)
-		{
-		auto entry = static_cast<GList *>(data);
-		g_list_free_full(entry, g_free);
-		});
+	g_list_free_full(fd->cached_metadata, reinterpret_cast<GDestroyNotify>(metadata_cache_entry_free));
 	fd->cached_metadata = nullptr;
 }
 
 
-
-
-
-
 /*
  *-------------------------------------------------------------------
  * write queue
@@ -255,24 +255,6 @@ gboolean metadata_write_queue_remove(FileData *fd)
 	return TRUE;
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-gboolean metadata_write_queue_remove_list_unused(GList *list)
-{
-	GList *work;
-	gboolean ret = TRUE;
-
-	work = list;
-	while (work)
-		{
-		auto *fd = static_cast<FileData *>(work->data);
-		work = work->next;
-		ret = ret && metadata_write_queue_remove(fd);
-		}
-	return ret;
-}
-#pragma GCC diagnostic pop
-
 void metadata_notify_cb(FileData *fd, NotifyType type, gpointer)
 {
 	if (type & (NOTIFY_REREAD | NOTIFY_CHANGE))
@@ -396,7 +378,7 @@ gboolean metadata_write_list(FileData *fd, const gchar *key, const GList *values
 {
 	if (!fd->modified_xmp)
 		{
-		fd->modified_xmp = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, reinterpret_cast<GDestroyNotify>(string_list_free));
+		fd->modified_xmp = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, string_list_free);
 		}
 	g_hash_table_insert(fd->modified_xmp, g_strdup(key), string_list_copy(const_cast<GList *>(values)));
 
@@ -440,10 +422,7 @@ gboolean metadata_write_string(FileData *fd, const gchar *key, const char *value
 
 gboolean metadata_write_int(FileData *fd, const gchar *key, guint64 value)
 {
-	gchar string[50];
-
-	g_snprintf(string, sizeof(string), "%llu", static_cast<unsigned long long>(value));
-	return metadata_write_string(fd, key, string);
+	return metadata_write_string(fd, key, std::to_string(static_cast<unsigned long long>(value)).c_str());
 }
 
 /*
@@ -558,7 +537,7 @@ static gboolean metadata_file_read(gchar *path, GList **keywords, gchar **commen
 				{
 				while (*ptr != '\n' && *ptr != '\0') ptr++;
 				*ptr = '\0';
-				if (strlen(s_buf) > 0)
+				if (s_buf[0] != '\0')
 					{
 					gchar *kw = utf8_validate_or_convert(s_buf);
 
@@ -688,21 +667,21 @@ GList *metadata_read_list(FileData *fd, const gchar *key, MetadataFormat format)
 {
 	ExifData *exif;
 	GList *list = nullptr;
-	const GList *cache_entry;
+	const GList *cache_values;
 	if (!fd) return nullptr;
 
 	/* unwritten data override everything */
 	if (fd->modified_xmp && format == METADATA_PLAIN)
 		{
-	        list = static_cast<GList *>(g_hash_table_lookup(fd->modified_xmp, key));
+		list = static_cast<GList *>(g_hash_table_lookup(fd->modified_xmp, key));
 		if (list) return string_list_copy(list);
 		}
 
 
 	if (format == METADATA_PLAIN && strcmp(key, KEYWORD_KEY) == 0
-	    && (cache_entry = metadata_cache_get(fd, key)))
+	    && (cache_values = metadata_cache_get(fd, key)))
 		{
-		return string_list_copy(cache_entry->next);
+		return string_list_copy(cache_values);
 		}
 
 	/*
@@ -917,7 +896,7 @@ gboolean metadata_write_GPS_coord(FileData *fd, const gchar *key, gdouble value)
 			}
 		setlocale(LC_ALL, "C");
 
-		coordinate = g_strdup_printf("%i,%lf,%s", deg, min, ref);
+		coordinate = g_strdup_printf("%i,%f,%s", deg, min, ref);
 		metadata_write_string(fd, key, coordinate );
 
 		setlocale(LC_ALL, saved_locale);
@@ -949,7 +928,7 @@ gboolean metadata_append_list(FileData *fd, const gchar *key, const GList *value
 /**
  * @see find_string_in_list
  */
-gchar *find_string_in_list_utf8nocase(GList *list, const gchar *string)
+static gchar *find_string_in_list_utf8nocase(GList *list, const gchar *string)
 {
 	gchar *string_casefold = g_utf8_casefold(string, -1);
 
@@ -982,7 +961,7 @@ gchar *find_string_in_list_utf8nocase(GList *list, const gchar *string)
 /**
  * @see find_string_in_list
  */
-gchar *find_string_in_list_utf8case(GList *list, const gchar *string)
+static gchar *find_string_in_list_utf8case(GList *list, const gchar *string)
 {
 	while (list)
 		{
@@ -1011,7 +990,7 @@ gchar *find_string_in_list_utf8case(GList *list, const gchar *string)
  * @see find_string_in_list_utf8case
  * @see find_string_in_list_utf8nocase
  */
-gchar *find_string_in_list(GList *list, const gchar *string)
+static gchar *find_string_in_list(GList *list, const gchar *string)
 {
 	if (options->metadata.keywords_case_sensitive)
 		return find_string_in_list_utf8case(list, string);
@@ -1111,8 +1090,8 @@ gboolean meta_data_set_keyword_mark(FileData *fd, gint, gboolean value, gpointer
 void meta_data_connect_mark_with_keyword(GtkTreeModel *keyword_tree, GtkTreeIter *kw_iter, gint mark)
 {
 
-	FileDataGetMarkFunc get_mark_func;
-	FileDataSetMarkFunc set_mark_func;
+	FileData::GetMarkFunc get_mark_func;
+	FileData::SetMarkFunc set_mark_func;
 	gpointer mark_func_data;
 
 	gint i;
@@ -1139,13 +1118,12 @@ void meta_data_connect_mark_with_keyword(GtkTreeModel *keyword_tree, GtkTreeIter
 	if (mark >= 0 && mark < FILEDATA_MARKS_SIZE)
 		{
 		GList *path;
-		gchar *mark_str;
 		path = keyword_tree_get_path(keyword_tree, kw_iter);
-		file_data_register_mark_func(mark, meta_data_get_keyword_mark, meta_data_set_keyword_mark, path, reinterpret_cast<GDestroyNotify>(string_list_free));
+		file_data_register_mark_func(mark, meta_data_get_keyword_mark, meta_data_set_keyword_mark, path, string_list_free);
 
-		mark_str = g_strdup_printf("%d", (mark < 9 ? mark : -1) + 1);
-		gtk_tree_store_set(GTK_TREE_STORE(keyword_tree), kw_iter, KEYWORD_COLUMN_MARK, mark_str, -1);
-		g_free(mark_str);
+		gtk_tree_store_set(GTK_TREE_STORE(keyword_tree), kw_iter,
+		                   KEYWORD_COLUMN_MARK, std::to_string((mark < 9) ? (mark + 1) : 0).c_str(),
+		                   -1);
 		}
 }
 
@@ -1394,20 +1372,8 @@ static gboolean keyword_tree_is_set_casefold(GtkTreeModel *keyword_tree, GtkTree
 
 		if (keyword_get_is_keyword(keyword_tree, &iter))
 			{
-			GList *work = casefold_list;
-			gboolean found = FALSE;
 			gchar *iter_casefold = keyword_get_casefold(keyword_tree, &iter);
-			while (work)
-				{
-				auto casefold = static_cast<const gchar *>(work->data);
-				work = work->next;
-
-				if (strcmp(iter_casefold, casefold) == 0)
-					{
-					found = TRUE;
-					break;
-					}
-				}
+			GList *found = g_list_find_custom(casefold_list, iter_casefold, reinterpret_cast<GCompareFunc>(strcmp));
 			g_free(iter_casefold);
 			if (!found) return FALSE;
 			}
@@ -1441,20 +1407,8 @@ static gboolean keyword_tree_is_set_casefull(GtkTreeModel *keyword_tree, GtkTree
 
 		if (keyword_get_is_keyword(keyword_tree, &iter))
 			{
-			GList *work = kw_list;
-			gboolean found = FALSE;
 			gchar *iter_name = keyword_get_name(keyword_tree, &iter);
-			while (work)
-				{
-				auto name = static_cast<const gchar *>(work->data);
-				work = work->next;
-
-				if (strcmp(iter_name, name) == 0)
-					{
-					found = TRUE;
-					break;
-					}
-				}
+			GList *found = g_list_find_custom(kw_list, iter_name, reinterpret_cast<GCompareFunc>(strcmp));
 			g_free(iter_name);
 			if (!found) return FALSE;
 			}
@@ -1862,7 +1816,7 @@ void keyword_tree_write_config(GString *outstr, gint indent)
 	WRITE_NL(); WRITE_STRING("</keyword_tree>");
 }
 
- void keyword_tree_node_disconnect_marks(GtkTreeModel *keyword_tree, GtkTreeIter *iter_ptr)
+static void keyword_tree_node_disconnect_marks(GtkTreeModel *keyword_tree, GtkTreeIter *iter_ptr)
 {
 	GtkTreeIter iter = *iter_ptr;
 
@@ -1893,9 +1847,9 @@ void keyword_tree_disconnect_marks()
 
 GtkTreeIter *keyword_add_from_config(GtkTreeStore *keyword_tree, GtkTreeIter *parent, const gchar **attribute_names, const gchar **attribute_values)
 {
-	gchar *name = nullptr;
+	g_autofree gchar *name = nullptr;
 	gboolean is_kw = TRUE;
-	gchar *mark_str = nullptr;
+	g_autofree gchar *mark_str = nullptr;
 
 	while (*attribute_names)
 		{
@@ -1908,6 +1862,7 @@ GtkTreeIter *keyword_add_from_config(GtkTreeStore *keyword_tree, GtkTreeIter *pa
 
 		log_printf("unknown attribute %s = %s\n", option, value);
 		}
+
 	if (name && name[0])
 		{
 		GtkTreeIter iter;
@@ -1927,10 +1882,9 @@ GtkTreeIter *keyword_add_from_config(GtkTreeStore *keyword_tree, GtkTreeIter *pa
 											&iter, i - 1);
 			}
 
-		g_free(name);
 		return gtk_tree_iter_copy(&iter);
 		}
-	g_free(name);
+
 	return nullptr;
 }
 
