@@ -31,8 +31,9 @@
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
-#include <memory>
 
+#include <glib-object.h>
+#include <grp.h>
 #include <langinfo.h>
 #include <pwd.h>
 
@@ -107,6 +108,25 @@ gint utf8_compare(const gchar *s1, const gchar *s2, gboolean case_sensitive)
 	return ret;
 }
 
+gint gq_gtk_tree_iter_utf8_collate(GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gint sort_column_id)
+{
+	g_autofree gchar *str_a = nullptr;
+	gtk_tree_model_get(model, a,
+	                   sort_column_id, &str_a,
+	                   -1);
+
+	g_autofree gchar *str_b = nullptr;
+	gtk_tree_model_get(model, b,
+	                   sort_column_id, &str_b,
+	                   -1);
+
+	if (str_a && str_b) return g_utf8_collate(str_a, str_b);
+
+	if (!str_a && !str_b) return 0;
+
+	return (!str_a) ? -1 : 1;
+}
+
 /* Borrowed from gtkfilesystemunix.c */
 gchar *expand_tilde(const gchar *filename)
 {
@@ -159,44 +179,34 @@ gchar *expand_tilde(const gchar *filename)
 
 #define GEOCODE_NAME "geocode-parameters.awk"
 
-gchar *decode_geo_script(const gchar *path_dir, const gchar *input_text)
+static gchar *decode_geo_script(const gchar *path_dir, const gchar *input_text)
 {
-	std::unique_ptr<gchar, decltype(&g_free)> message{nullptr, g_free};
-	gchar *path = g_build_filename(path_dir, GEOCODE_NAME, NULL);
-	gchar *cmd = g_strconcat("echo \'", input_text, "\'  | awk -f ", path, NULL);
-
-	if (g_file_test(path, G_FILE_TEST_EXISTS))
+	g_autofree gchar *path = g_build_filename(path_dir, GEOCODE_NAME, NULL);
+	if (!g_file_test(path, G_FILE_TEST_EXISTS))
 		{
-		gchar buf[BUFSIZE];
-		FILE *fp;
-
-		if ((fp = popen(cmd, "r")) == nullptr)
-			{
-			message.reset(g_strconcat("Error: opening pipe\n", input_text, NULL));
-			}
-		else
-			{
-			while (fgets(buf, BUFSIZE, fp))
-				{
-				DEBUG_1("Output: %s", buf);
-				}
-
-			message.reset(g_strconcat(buf, NULL));
-
-			if(pclose(fp))
-				{
-				message.reset(g_strconcat("Error: Command not found or exited with error status\n", input_text, NULL));
-				}
-			}
-		}
-	else
-		{
-		message.reset(g_strconcat(input_text, NULL));
+		return g_strconcat(input_text, NULL);
 		}
 
-	g_free(path);
-	g_free(cmd);
-	return message.release();
+	g_autofree gchar *cmd = g_strconcat("echo \'", input_text, "\'  | awk -W posix -f ", path, NULL);
+
+	FILE *fp = popen(cmd, "r");
+	if (!fp)
+		{
+		return g_strconcat("Error: opening pipe\n", input_text, NULL);
+		}
+
+	gchar buf[BUFSIZE];
+	while (fgets(buf, BUFSIZE, fp))
+		{
+		DEBUG_1("Output: %s", buf);
+		}
+
+	if(pclose(fp))
+		{
+		return g_strconcat("Error: Command not found or exited with error status\n", input_text, NULL);
+		}
+
+	return g_strconcat(buf, NULL);
 }
 
 gchar *decode_geo_parameters(const gchar *input_text)
@@ -335,6 +345,8 @@ gchar *date_get_abbreviated_day_name(gint day)
 		case 7:
 		abday = g_strdup(nl_langinfo(ABDAY_7));
 		break;
+		default:
+			break;
 		}
 
 	return abday;
@@ -362,6 +374,56 @@ gchar *convert_rating_to_stars(gint rating)
 	return g_strdup("");
 }
 
+gchar *get_file_group(const gchar *path_utf8)
+{
+	struct passwd *user;
+	gchar *ret;
+
+	struct stat st;
+
+	if (!stat_utf8(path_utf8, &st))
+		{
+		return nullptr;
+		}
+
+	user = getpwuid(st.st_uid);
+	if (!user)
+		{
+		ret = g_strdup_printf("%u", st.st_uid);
+		}
+	else
+		{
+		ret = g_strdup(user->pw_name);
+		}
+
+	return ret;
+}
+
+gchar *get_file_owner(const gchar *path_utf8)
+{
+	struct group *group;
+	gchar *ret;
+
+	struct stat st;
+
+	if (!stat_utf8(path_utf8, &st))
+		{
+		return nullptr;
+		}
+
+	group = getgrgid(st.st_gid);
+	if (!group)
+		{
+		ret = g_strdup_printf("%u", st.st_gid);
+		}
+	else
+		{
+		ret = g_strdup(group->gr_name);
+		}
+
+	return ret;
+}
+
 gchar *get_symbolic_link(const gchar *path_utf8)
 {
 	gchar *sl;
@@ -443,7 +505,9 @@ void gq_gtk_grid_attach_default(GtkGrid *grid, GtkWidget *child, guint left_atta
 	gtk_grid_attach(grid, child, left_attach, top_attach, right_attach - left_attach, bottom_attach - top_attach);
 }
 
-/* this overrides the low default of a GtkCellRenderer from 100 to CELL_HEIGHT_OVERRIDE, something sane for our purposes */
+/**
+ * @brief This overrides the low default of a GtkCellRenderer from 100 to CELL_HEIGHT_OVERRIDE, something sane for our purposes
+ */
 void cell_renderer_height_override(GtkCellRenderer *renderer)
 {
 	GParamSpec *spec;
@@ -458,4 +522,29 @@ void cell_renderer_height_override(GtkCellRenderer *renderer)
 		}
 }
 
+/**
+ * @brief Set cursor for widget's window.
+ * @param widget Widget for which cursor is set
+ * @param icon Cursor type from GdkCursorType.
+ *        Value -1 means using the cursor of its parent window.
+ * @todo Use std::optional for icon since C++17 instead of special -1 value
+ */
+void widget_set_cursor(GtkWidget *widget, gint icon)
+{
+	GdkWindow *window = gtk_widget_get_window(widget);
+	if (!window) return;
+
+	GdkCursor *cursor = nullptr;
+
+	if (icon != -1)
+		{
+		GdkDisplay *display = gdk_display_get_default();
+		cursor = gdk_cursor_new_for_display(display, static_cast<GdkCursorType>(icon));
+		}
+
+	gdk_window_set_cursor(window, cursor);
+
+	if (cursor) g_object_unref(cursor);
+}
+
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
