@@ -25,8 +25,11 @@
 #include <gdk/gdk.h>
 #include <glib.h>
 
+#include "image.h"
 #include "typedefs.h"
 
+enum TextPosition : gint;
+
 struct SecureSaveInfo;
 
 #define COLOR_PROFILE_INPUTS 4
@@ -86,12 +89,14 @@ struct ConfOptions
 	gint dnd_icon_size;
 	DnDAction dnd_default_action;
 	gint clipboard_selection;
+	RectangleDrawAspectRatio rectangle_draw_aspect_ratio;
 
 	gboolean save_window_positions;
 	gboolean use_saved_window_positions_for_new_windows;
 	gboolean save_window_workspace;
 	gboolean tools_restore_state;
 	gboolean save_dialog_window_positions;
+	gboolean hide_window_decorations;
 	gboolean show_window_ids;
 
 	gint log_window_lines;
@@ -105,6 +110,7 @@ struct ConfOptions
 	gboolean collections_duplicates;
 	gboolean collections_on_top;
 	gboolean hide_window_in_fullscreen;
+	gboolean hide_osd_in_fullscreen;
 
 	gchar *help_search_engine;
 
@@ -165,7 +171,7 @@ struct ConfOptions
 	/* image */
 	struct {
 		gboolean exif_rotate_enable;
-		guint scroll_reset_method;
+		ScrollReset scroll_reset_method;
 		gboolean fit_window_to_image;
 		gboolean limit_window_size;
 		gint max_window_size;
@@ -384,8 +390,8 @@ struct ConfOptions
 		gboolean show_image_text;
 		gboolean show_page_text;
 		gchar *page_text;
-		gint image_text_position;
-		gint page_text_position;
+		TextPosition image_text_position;
+		TextPosition page_text_position;
 		gchar *template_string;
 	} printer;
 
@@ -486,17 +492,14 @@ struct LayoutOptions
 	DirViewType dir_view_type;
 	FileViewType file_view_type;
 
-	struct {
+	struct SortParams
+	{
 		SortType method;
 		gboolean ascend;
 		gboolean case_sensitive;
-	} dir_view_list_sort;
-
-	struct {
-		SortType method;
-		gboolean ascend;
-		gboolean case_sensitive;
-	} file_view_list_sort;
+	};
+	SortParams dir_view_list_sort;
+	SortParams file_view_list_sort;
 
 	gboolean show_thumbnails;
 	gboolean show_marks;
@@ -507,20 +510,14 @@ struct LayoutOptions
 	gboolean ignore_alpha;
 
 	struct {
-		gint w;
-		gint h;
-		gint x;
-		gint y;
+		GdkRectangle rect;
 		gboolean maximized;
 		gint hdivider_pos;
 		gint vdivider_pos;
 	} main_window;
 
 	struct {
-		gint w;
-		gint h;
-		gint x;
-		gint y;
+		GdkRectangle rect;
 		gint vdivider_pos;
 	} float_window;
 
@@ -528,11 +525,6 @@ struct LayoutOptions
 		gint vdivider_pos;
 	} folder_window;
 
-	struct {
-		gint w;
-		gint h;
-	} properties_window;
-
 	struct {
 		guint state;
 		gint histogram_channel;
@@ -542,33 +534,15 @@ struct LayoutOptions
 	GdkRectangle log_window;
 
 	struct {
-		gint w;
-		gint h;
-		gint x;
-		gint y;
+		GdkRectangle rect;
 		gint page_number;
 	} preferences_window;
 
-	struct {
-		gint w;
-		gint h;
-		gint x;
-		gint y;
-	} search_window;
+	GdkRectangle search_window;
 
-	struct {
-		gint w;
-		gint h;
-		gint x;
-		gint y;
-	} dupe_window;
+	GdkRectangle dupe_window;
 
-	struct {
-		gint w;
-		gint h;
-		gint x;
-		gint y;
-	} advanced_exif_window;
+	GdkRectangle advanced_exif_window;
 
 	gboolean tools_float;
 	gboolean tools_hidden;
