@@ -24,6 +24,9 @@
 #include <cmath>
 #include <cstring>
 #include <ctime>
+#include <string>
+
+#include <gdk/gdk.h>
 
 #include "debug.h"
 #include "filedata.h"
@@ -33,7 +36,6 @@
 #include "pan-util.h"
 #include "pan-view-filter.h"
 #include "pan-view.h"
-#include "pixbuf-util.h"
 #include "typedefs.h"
 
 namespace
@@ -80,16 +82,8 @@ void pan_calendar_update(PanWindow *pw, PanItem *pi_day)
 	PanItem *pi;
 	GList *list;
 	GList *work;
-	gint x1;
-	gint y1;
-	gint x2;
-	gint y2;
-	gint x3;
-	gint y3;
 	gint x;
 	gint y;
-	gint w;
-	gint h;
 	gint grid;
 	gint column;
 
@@ -181,19 +175,14 @@ void pan_calendar_update(PanWindow *pw, PanItem *pi_day)
 			}
 		}
 
-	x1 = pi_day->x + pi_day->width - 8;
-	y1 = pi_day->y + 8;
-	x2 = pbox->x + 1;
-	y2 = pbox->y + MIN(42, pbox->height);
-	x3 = pbox->x + 1;
-	y3 = MAX(pbox->y, y2 - 30);
-	util_clip_triangle(x1, y1, x2, y2, x3, y3,
-	                   x, y, w, h);
+	GdkPoint c1{pi_day->x + pi_day->width - 8, pi_day->y + 8};
+	GdkPoint c2{pbox->x + 1, pbox->y + MIN(42, pbox->height)};
+	GdkPoint c3{pbox->x + 1, MAX(pbox->y, c2.y - 30)};
 
-	pi = pan_item_tri_new(pw, nullptr, x, y, w, h,
-			      x1, y1, x2, y2, x3, y3,
-			      PAN_CAL_POPUP_COLOR);
-	pan_item_tri_border(pi, PAN_BORDER_1 | PAN_BORDER_3, PAN_CAL_POPUP_BORDER_COLOR);
+	pi = pan_item_tri_new(pw,
+	                      c1, c2, c3,
+	                      PAN_CAL_POPUP_COLOR,
+	                      PAN_BORDER_1 | PAN_BORDER_3, PAN_CAL_POPUP_BORDER_COLOR);
 	pan_item_set_key(pi, "day_bubble");
 	pan_item_added(pw, pi);
 
@@ -203,7 +192,7 @@ void pan_calendar_update(PanWindow *pw, PanItem *pi_day)
 	pan_layout_resize(pw);
 }
 
-void pan_calendar_compute(PanWindow *pw, FileData *dir_fd, gint *width, gint *height)
+void pan_calendar_compute(PanWindow *pw, FileData *dir_fd, gint &width, gint &height)
 {
 	GList *list;
 	GList *work;
@@ -275,8 +264,8 @@ void pan_calendar_compute(PanWindow *pw, FileData *dir_fd, gint *width, gint *he
 		end_month = pan_date_value(fd->date, PAN_DATE_LENGTH_MONTH);
 		}
 
-	*width = PAN_BOX_BORDER * 2;
-	*height = PAN_BOX_BORDER * 2;
+	width = PAN_BOX_BORDER * 2;
+	height = PAN_BOX_BORDER * 2;
 
 	x = PAN_BOX_BORDER;
 	y = PAN_BOX_BORDER;
@@ -402,10 +391,9 @@ void pan_calendar_compute(PanWindow *pw, FileData *dir_fd, gint *width, gint *he
 				pi->y = pi_day->y + (pi_day->height - pi->height) / 2;
 				}
 
-			buf = g_strdup_printf("%d", day);
-			pi_day_number = pan_item_text_new(pw, x + 4, y + 4, buf, static_cast<PanTextAttrType>(PAN_TEXT_ATTR_BOLD | PAN_TEXT_ATTR_HEADING),
-					  PAN_BORDER_3, PAN_CAL_DAY_TEXT_COLOR);
-			g_free(buf);
+			pi_day_number = pan_item_text_new(pw, x + 4, y + 4, std::to_string(day).c_str(),
+			                                  static_cast<PanTextAttrType>(PAN_TEXT_ATTR_BOLD | PAN_TEXT_ATTR_HEADING),
+			                                  PAN_BORDER_3, PAN_CAL_DAY_TEXT_COLOR);
 
 			day_of_week = date_get_first_day_of_week() + col;
 			if (day_of_week > 7) day_of_week = day_of_week - 7;
@@ -441,8 +429,8 @@ void pan_calendar_compute(PanWindow *pw, FileData *dir_fd, gint *width, gint *he
 			}
 		}
 
-	*width += grid;
-	*height = MAX(*height, grid + PAN_BOX_BORDER * 2 * 2);
+	width += grid;
+	height = MAX(height, grid + PAN_BOX_BORDER * 2 * 2);
 
 	g_list_free(list);
 }
