@@ -21,8 +21,11 @@
 
 #include "pan-folder.h"
 
+#include <algorithm>
 #include <cmath>
 
+#include <gdk/gdk.h>
+
 #include "filedata.h"
 #include "pan-item.h"
 #include "pan-types.h"
@@ -30,7 +33,7 @@
 #include "pan-view-filter.h"
 #include "typedefs.h"
 
-static void pan_flower_size(PanWindow *pw, gint *width, gint *height)
+static void pan_flower_size(PanWindow *pw, gint &width, gint &height)
 {
 	GList *work;
 	gint x1;
@@ -75,20 +78,18 @@ static void pan_flower_size(PanWindow *pw, gint *width, gint *height)
 
 		if (pi->type == PAN_ITEM_TRIANGLE && pi->data)
 			{
-			gint *coord;
+			auto *coord = static_cast<GdkPoint *>(pi->data);
 
-			coord = static_cast<gint *>(pi->data);
-			coord[0] -= x1;
-			coord[1] -= y1;
-			coord[2] -= x1;
-			coord[3] -= y1;
-			coord[4] -= x1;
-			coord[5] -= y1;
+			for (gint i = 0; i < 3; ++i)
+				{
+				coord[i].x -= x1;
+				coord[i].y -= y1;
+				}
 			}
 		}
 
-	if (width) *width = x2 - x1;
-	if (height) *height = y2 - y1;
+	width = x2 - x1;
+	height = y2 - y1;
 }
 
 struct FlowerGroup {
@@ -124,8 +125,6 @@ static void pan_flower_move(FlowerGroup *group, gint x, gint y)
 	group->y += y;
 }
 
-#define PI 3.14159265
-
 static void pan_flower_position(FlowerGroup *group, FlowerGroup *parent,
 							     gint *result_x, gint *result_y)
 {
@@ -134,10 +133,10 @@ static void pan_flower_position(FlowerGroup *group, FlowerGroup *parent,
 	gint radius;
 	gdouble a;
 
-	radius = parent->circumference / (2*PI);
+	radius = parent->circumference / (2*G_PI);
 	radius = MAX(radius, parent->diameter / 2 + group->diameter / 2);
 
-	a = 2*PI * group->diameter / parent->circumference;
+	a = 2*G_PI * group->diameter / parent->circumference;
 
 	x = static_cast<gint>(static_cast<gdouble>(radius) * cos(parent->angle + a / 2));
 	y = static_cast<gint>(static_cast<gdouble>(radius) * sin(parent->angle + a / 2));
@@ -179,32 +178,13 @@ static void pan_flower_build(PanWindow *pw, FlowerGroup *group, FlowerGroup *par
 
 	if (parent)
 		{
-		PanItem *pi;
-		gint px;
-		gint py;
-		gint gx;
-		gint gy;
-		gint x1;
-		gint y1;
-		gint x2;
-		gint y2;
+		GdkPoint cp{parent->x + parent->width / 2, parent->y + parent->height / 2};
+		GdkPoint cg{group->x + group->width / 2, group->y + group->height / 2};
 
-		px = parent->x + parent->width / 2;
-		py = parent->y + parent->height / 2;
-
-		gx = group->x + group->width / 2;
-		gy = group->y + group->height / 2;
-
-		x1 = MIN(px, gx);
-		y1 = MIN(py, gy);
-
-		x2 = MAX(px, gx + 5);
-		y2 = MAX(py, gy + 5);
-
-		pi = pan_item_tri_new(pw, nullptr, x1, y1, x2 - x1, y2 - y1,
-				      px, py, gx, gy, gx + 5, gy + 5,
-				      {255, 40, 40, 128});
-		pan_item_tri_border(pi, PAN_BORDER_1 | PAN_BORDER_3, {255, 0, 0, 128});
+		pan_item_tri_new(pw,
+		                 cp, cg, {cg.x + 5, cg.y + 5},
+		                 {255, 40, 40, 128},
+		                 PAN_BORDER_1 | PAN_BORDER_3, {255, 0, 0, 128});
 		}
 
 	pw->list = g_list_concat(group->items, pw->list);
@@ -352,8 +332,8 @@ static FlowerGroup *pan_flower_group(PanWindow *pw, FileData *dir_fd, gint x, gi
 }
 
 void pan_flower_compute(PanWindow *pw, FileData *dir_fd,
-			gint *width, gint *height,
-			gint *scroll_x, gint *scroll_y)
+                        gint &width, gint &height,
+                        gint &scroll_x, gint &scroll_y)
 {
 	FlowerGroup *group;
 	GList *list;
@@ -367,16 +347,16 @@ void pan_flower_compute(PanWindow *pw, FileData *dir_fd,
 	if (list)
 		{
 		auto pi = static_cast<PanItem *>(list->data);
-		*scroll_x = pi->x + pi->width / 2;
-		*scroll_y = pi->y + pi->height / 2;
+		scroll_x = pi->x + pi->width / 2;
+		scroll_y = pi->y + pi->height / 2;
 		}
 	g_list_free(list);
 }
 
 static void pan_folder_tree_path(PanWindow *pw, FileData *dir_fd,
-				 gint *x, gint *y, gint *level,
-				 PanItem *parent,
-				 gint *width, gint *height)
+                                 gint &x, gint &y, gint level,
+                                 PanItem *parent,
+                                 gint &width, gint &height)
 {
 	GList *f;
 	GList *d;
@@ -392,23 +372,23 @@ static void pan_folder_tree_path(PanWindow *pw, FileData *dir_fd,
 
 	pan_filter_fd_list(&f, pw->filter_ui->filter_elements, pw->filter_ui->filter_classes);
 
-	*x = PAN_BOX_BORDER + ((*level) * MAX(PAN_BOX_BORDER, PAN_THUMB_GAP));
+	x = PAN_BOX_BORDER + (level * MAX(PAN_BOX_BORDER, PAN_THUMB_GAP));
 
-	pi_box = pan_item_text_new(pw, *x, *y, dir_fd->path, PAN_TEXT_ATTR_NONE,
-				   PAN_BORDER_3,
-				   {PAN_TEXT_COLOR, 255});
+	pi_box = pan_item_text_new(pw, x, y, dir_fd->path, PAN_TEXT_ATTR_NONE,
+	                           PAN_BORDER_3,
+	                           {PAN_TEXT_COLOR, 255});
 
-	*y += pi_box->height;
+	y += pi_box->height;
 
 	pi_box = pan_item_box_new(pw, file_data_ref(dir_fd),
-				  *x, *y,
-				  PAN_BOX_BORDER, PAN_BOX_BORDER,
-				  PAN_BOX_OUTLINE_THICKNESS,
-				  {PAN_BOX_COLOR, PAN_BOX_ALPHA},
-				  {PAN_BOX_OUTLINE_COLOR, PAN_BOX_OUTLINE_ALPHA});
+	                          x, y,
+	                          PAN_BOX_BORDER, PAN_BOX_BORDER,
+	                          PAN_BOX_OUTLINE_THICKNESS,
+	                          {PAN_BOX_COLOR, PAN_BOX_ALPHA},
+	                          {PAN_BOX_OUTLINE_COLOR, PAN_BOX_OUTLINE_ALPHA});
 
-	*x += PAN_BOX_BORDER;
-	*y += PAN_BOX_BORDER;
+	x += PAN_BOX_BORDER;
+	y += PAN_BOX_BORDER;
 
 	work = f;
 	while (work)
@@ -421,21 +401,21 @@ static void pan_folder_tree_path(PanWindow *pw, FileData *dir_fd,
 
 		if (pw->size > PAN_IMAGE_SIZE_THUMB_LARGE)
 			{
-			pi = pan_item_image_new(pw, fd, *x, *y, 10, 10);
-			*x += pi->width + PAN_THUMB_GAP;
+			pi = pan_item_image_new(pw, fd, x, y, 10, 10);
+			x += pi->width + PAN_THUMB_GAP;
 			if (pi->height > y_height) y_height = pi->height;
 			}
 		else
 			{
-			pi = pan_item_thumb_new(pw, fd, *x, *y);
-			*x += PAN_THUMB_SIZE + PAN_THUMB_GAP;
+			pi = pan_item_thumb_new(pw, fd, x, y);
+			x += PAN_THUMB_SIZE + PAN_THUMB_GAP;
 			y_height = PAN_THUMB_SIZE;
 			}
 
 		pan_item_size_by_item(pi_box, pi, PAN_BOX_BORDER);
 		}
 
-	if (f) *y = pi_box->y + pi_box->height;
+	if (f) y = pi_box->y + pi_box->height;
 
 	g_list_free(f);
 
@@ -449,9 +429,7 @@ static void pan_folder_tree_path(PanWindow *pw, FileData *dir_fd,
 
 		if (!pan_is_ignored(fd->path, pw->ignore_symlinks))
 			{
-			*level = *level + 1;
-			pan_folder_tree_path(pw, fd, x, y, level, pi_box, width, height);
-			*level = *level - 1;
+			pan_folder_tree_path(pw, fd, x, y, level + 1, pi_box, width, height);
 			}
 		}
 
@@ -459,29 +437,21 @@ static void pan_folder_tree_path(PanWindow *pw, FileData *dir_fd,
 
 	pan_item_size_by_item(parent, pi_box, PAN_BOX_BORDER);
 
-	if (*y < pi_box->y + pi_box->height + PAN_BOX_BORDER)
-		*y = pi_box->y + pi_box->height + PAN_BOX_BORDER;
+	y = std::max(y, pi_box->y + pi_box->height + PAN_BOX_BORDER);
 
 	pan_item_size_coordinates(pi_box, PAN_BOX_BORDER, width, height);
 }
 
-void pan_folder_tree_compute(PanWindow *pw, FileData *dir_fd, gint *width, gint *height)
+void pan_folder_tree_compute(PanWindow *pw, FileData *dir_fd, gint &width, gint &height)
 {
 	gint x;
 	gint y;
-	gint level;
-	gint w;
-	gint h;
 
-	level = 0;
 	x = PAN_BOX_BORDER;
 	y = PAN_BOX_BORDER;
-	w = PAN_BOX_BORDER * 2;
-	h = PAN_BOX_BORDER * 2;
+	width = PAN_BOX_BORDER * 2;
+	height = PAN_BOX_BORDER * 2;
 
-	pan_folder_tree_path(pw, dir_fd, &x, &y, &level, nullptr, &w, &h);
-
-	if (width) *width = w;
-	if (height) *height = h;
+	pan_folder_tree_path(pw, dir_fd, x, y, 0, nullptr, width, height);
 }
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
