@@ -24,12 +24,12 @@
 
 #include <glib.h>
 
-struct FileData;
+class FileData;
 struct PanWindow;
 
 void pan_flower_compute(PanWindow *pw, FileData *dir_fd,
-			gint *width, gint *height,
-			gint *scroll_x, gint *scroll_y);
-void pan_folder_tree_compute(PanWindow *pw, FileData *dir_fd, gint *width, gint *height);
+                        gint &width, gint &height,
+                        gint &scroll_x, gint &scroll_y);
+void pan_folder_tree_compute(PanWindow *pw, FileData *dir_fd, gint &width, gint &height);
 
 #endif
