@@ -21,26 +21,33 @@
 
 #include "pan-item.h"
 
+#include <algorithm>
 #include <cstring>
 
 #include <glib-object.h>
 #include <gtk/gtk.h>
 #include <pango/pango.h>
 
-#include "cache.h"
 #include "filedata.h"
 #include "image.h"
 #include "pan-types.h"
+#include "pan-view.h"
 #include "pixbuf-util.h"
 #include "ui-misc.h"
 
 namespace
 {
 
+struct PanItemBoxShadow
+{
+	gint offset;
+	gint fade;
+};
+
 constexpr gint PAN_OUTLINE_THICKNESS = 1;
-#define PAN_OUTLINE_ALPHA 180
-#define PAN_OUTLINE_COLOR_1 255, 255, 255, PAN_OUTLINE_ALPHA
-#define PAN_OUTLINE_COLOR_2 64, 64, 64, PAN_OUTLINE_ALPHA
+constexpr guint8 PAN_OUTLINE_ALPHA = 180;
+constexpr PanColor PAN_OUTLINE_COLOR_1{255, 255, 255, PAN_OUTLINE_ALPHA};
+constexpr PanColor PAN_OUTLINE_COLOR_2{64, 64, 64, PAN_OUTLINE_ALPHA};
 
 /* popup info box */
 constexpr PanColor PAN_POPUP_TEXT_COLOR{0, 0, 0, 225};
@@ -108,12 +115,12 @@ void pan_item_size_by_item(PanItem *pi, PanItem *child, gint border)
 		pi->height = child->y + child->height + border - pi->y;
 }
 
-void pan_item_size_coordinates(PanItem *pi, gint border, gint *w, gint *h)
+void pan_item_size_coordinates(PanItem *pi, gint border, gint &w, gint &h)
 {
 	if (!pi) return;
 
-	if (*w < pi->x + pi->width + border) *w = pi->x + pi->width + border;
-	if (*h < pi->y + pi->height + border) *h = pi->y + pi->height + border;
+	w = std::max(w, pi->x + pi->width + border);
+	h = std::max(h, pi->y + pi->height + border);
 }
 
 
@@ -124,7 +131,7 @@ void pan_item_size_coordinates(PanItem *pi, gint border, gint *w, gint *h)
  */
 
 PanItem *pan_item_box_new(PanWindow *pw, FileData *fd, gint x, gint y, gint width, gint height,
-                          gint border_size, const PanColor &base, const PanColor &bord)
+                          gint border_size, PanColor base, PanColor bord)
 {
 	PanItem *pi;
 
@@ -148,20 +155,18 @@ PanItem *pan_item_box_new(PanWindow *pw, FileData *fd, gint x, gint y, gint widt
 
 void pan_item_box_shadow(PanItem *pi, gint offset, gint fade)
 {
-	gint *shadow;
-
 	if (!pi || pi->type != PAN_ITEM_BOX) return;
 
-	shadow = static_cast<gint *>(pi->data);
+	auto *shadow = static_cast<PanItemBoxShadow *>(pi->data);
 	if (shadow)
 		{
-		pi->width -= shadow[0];
-		pi->height -= shadow[0];
+		pi->width -= shadow->offset;
+		pi->height -= shadow->offset;
 		}
 
-	shadow = g_new0(gint, 2);
-	shadow[0] = offset;
-	shadow[1] = fade;
+	shadow = g_new0(PanItemBoxShadow, 1);
+	shadow->offset = offset;
+	shadow->fade = fade;
 
 	pi->width += offset;
 	pi->height += offset;
@@ -170,93 +175,63 @@ void pan_item_box_shadow(PanItem *pi, gint offset, gint fade)
 	pi->data = shadow;
 }
 
-gint pan_item_box_draw(PanWindow *, PanItem *pi, GdkPixbuf *pixbuf, PixbufRenderer *,
-		       gint x, gint y, gint width, gint height)
+gboolean pan_item_box_draw(PanWindow *, PanItem *pi, GdkPixbuf *pixbuf, PixbufRenderer *,
+                           gint x, gint y, gint width, gint height)
 {
 	gint bw;
 	gint bh;
-	gint *shadow;
-	gint rx;
-	gint ry;
-	gint rw;
-	gint rh;
 
 	bw = pi->width;
 	bh = pi->height;
 
-	shadow = static_cast<gint *>(pi->data);
+	auto *shadow = static_cast<PanItemBoxShadow *>(pi->data);
 	if (shadow)
 		{
-		bw -= shadow[0];
-		bh -= shadow[0];
+		bw -= shadow->offset;
+		bh -= shadow->offset;
 
 		if (pi->color.a > 254)
 			{
-			pixbuf_draw_shadow(pixbuf, pi->x - x + bw, pi->y - y + shadow[0],
-					   shadow[0], bh - shadow[0],
-					   pi->x - x + shadow[0], pi->y - y + shadow[0], bw, bh,
-					   shadow[1],
-					   PAN_SHADOW_COLOR, PAN_SHADOW_ALPHA);
-			pixbuf_draw_shadow(pixbuf, pi->x - x + shadow[0], pi->y - y + bh,
-					   bw, shadow[0],
-					   pi->x - x + shadow[0], pi->y - y + shadow[0], bw, bh,
-					   shadow[1],
-					   PAN_SHADOW_COLOR, PAN_SHADOW_ALPHA);
+			pixbuf_draw_shadow(pixbuf,
+			                   {pi->x - x + bw, pi->y - y + shadow->offset, shadow->offset, bh - shadow->offset},
+			                   pi->x - x + shadow->offset, pi->y - y + shadow->offset, bw, bh,
+			                   shadow->fade,
+			                   PAN_SHADOW_COLOR, PAN_SHADOW_ALPHA);
+			pixbuf_draw_shadow(pixbuf,
+			                   {pi->x - x + shadow->offset, pi->y - y + bh, bw, shadow->offset},
+			                   pi->x - x + shadow->offset, pi->y - y + shadow->offset, bw, bh,
+			                   shadow->fade,
+			                   PAN_SHADOW_COLOR, PAN_SHADOW_ALPHA);
 			}
 		else
 			{
 			gint a;
 			a = pi->color.a * PAN_SHADOW_ALPHA >> 8;
-			pixbuf_draw_shadow(pixbuf, pi->x - x + shadow[0], pi->y - y + shadow[0],
-					   bw, bh,
-					   pi->x - x + shadow[0], pi->y - y + shadow[0], bw, bh,
-					   shadow[1],
-					   PAN_SHADOW_COLOR, a);
+			pixbuf_draw_shadow(pixbuf,
+			                   {pi->x - x + shadow->offset, pi->y - y + shadow->offset, bw, bh},
+			                   pi->x - x + shadow->offset, pi->y - y + shadow->offset, bw, bh,
+			                   shadow->fade,
+			                   PAN_SHADOW_COLOR, a);
 			}
 		}
 
-	if (util_clip_region(x, y, width, height,
-			     pi->x, pi->y, bw, bh,
-			     &rx, &ry, &rw, &rh))
-		{
-		pixbuf_draw_rect_fill(pixbuf,
-				      rx - x, ry - y, rw, rh,
-				      pi->color.r, pi->color.g, pi->color.b, pi->color.a);
-		}
-	if (util_clip_region(x, y, width, height,
-			     pi->x, pi->y, bw, pi->border,
-			     &rx, &ry, &rw, &rh))
-		{
-		pixbuf_draw_rect_fill(pixbuf,
-				      rx - x, ry - y, rw, rh,
-				      pi->color2.r, pi->color2.g, pi->color2.b, pi->color2.a);
-		}
-	if (util_clip_region(x, y, width, height,
-			     pi->x, pi->y + pi->border, pi->border, bh - pi->border * 2,
-			     &rx, &ry, &rw, &rh))
-		{
-		pixbuf_draw_rect_fill(pixbuf,
-				      rx - x, ry - y, rw, rh,
-				      pi->color2.r, pi->color2.g, pi->color2.b, pi->color2.a);
-		}
-	if (util_clip_region(x, y, width, height,
-			     pi->x + bw - pi->border, pi->y + pi->border,
-			     pi->border, bh - pi->border * 2,
-			     &rx, &ry, &rw, &rh))
-		{
-		pixbuf_draw_rect_fill(pixbuf,
-				      rx - x, ry - y, rw, rh,
-				      pi->color2.r, pi->color2.g, pi->color2.b, pi->color2.a);
-		}
-	if (util_clip_region(x, y, width, height,
-			     pi->x, pi->y + bh - pi->border,
-			     bw,  pi->border,
-			     &rx, &ry, &rw, &rh))
-		{
-		pixbuf_draw_rect_fill(pixbuf,
-				      rx - x, ry - y, rw, rh,
-				      pi->color2.r, pi->color2.g, pi->color2.b, pi->color2.a);
-		}
+	const GdkRectangle request_rect{x, y, width, height};
+	const auto draw_rect_if_intersect = [pixbuf, &request_rect, x, y](GdkRectangle box_rect, PanColor color)
+	{
+		GdkRectangle r;
+		if (!gdk_rectangle_intersect(&request_rect, &box_rect, &r)) return;
+
+		r.x -= x;
+		r.y -= y;
+		pixbuf_draw_rect_fill(pixbuf, r, color.r, color.g, color.b, color.a);
+	};
+
+	draw_rect_if_intersect({pi->x, pi->y, bw, bh}, pi->color);
+
+	draw_rect_if_intersect({pi->x, pi->y, bw, pi->border}, pi->color2);
+	draw_rect_if_intersect({pi->x, pi->y + pi->border, pi->border, bh - pi->border * 2}, pi->color2);
+	draw_rect_if_intersect({pi->x + bw - pi->border, pi->y + pi->border, pi->border, bh - pi->border * 2}, pi->color2);
+	draw_rect_if_intersect({pi->x, pi->y + bh - pi->border, bw, pi->border}, pi->color2);
 
 	return FALSE;
 }
@@ -268,91 +243,65 @@ gint pan_item_box_draw(PanWindow *, PanItem *pi, GdkPixbuf *pixbuf, PixbufRender
  *-----------------------------------------------------------------------------
  */
 
-PanItem *pan_item_tri_new(PanWindow *pw, FileData *, gint x, gint y, gint width, gint height,
-                          gint x1, gint y1, gint x2, gint y2, gint x3, gint y3,
-                          const PanColor &color)
+PanItem *pan_item_tri_new(PanWindow *pw,
+                          GdkPoint c1, GdkPoint c2, GdkPoint c3,
+                          PanColor color,
+                          gint borders, PanColor border_color)
 {
-	PanItem *pi;
-	gint *coord;
+	GdkRectangle tri_rect = util_triangle_bounding_box(c1, c2, c3);
 
-	pi = g_new0(PanItem, 1);
+	auto *pi = g_new0(PanItem, 1);
 	pi->type = PAN_ITEM_TRIANGLE;
-	pi->x = x;
-	pi->y = y;
-	pi->width = width;
-	pi->height = height;
-
+	pi->x = tri_rect.x;
+	pi->y = tri_rect.y;
+	pi->width = tri_rect.width;
+	pi->height = tri_rect.height;
 	pi->color = color;
 
-	coord = g_new0(gint, 6);
-	coord[0] = x1;
-	coord[1] = y1;
-	coord[2] = x2;
-	coord[3] = y2;
-	coord[4] = x3;
-	coord[5] = y3;
+	auto *coord = g_new0(GdkPoint, 3);
+	coord[0] = c1;
+	coord[1] = c2;
+	coord[2] = c3;
 
 	pi->data = coord;
 
-	pi->border = PAN_BORDER_NONE;
+	pi->border = borders;
+	pi->color2 = border_color;
 
 	pw->list = g_list_prepend(pw->list, pi);
 
 	return pi;
 }
 
-void pan_item_tri_border(PanItem *pi, gint borders, const PanColor &color)
+gboolean pan_item_tri_draw(PanWindow *, PanItem *pi, GdkPixbuf *pixbuf, PixbufRenderer *,
+                           gint x, gint y, gint width, gint height)
 {
-	if (!pi || pi->type != PAN_ITEM_TRIANGLE) return;
+	const GdkRectangle request_rect{x, y, width, height};
+	const GdkRectangle pi_rect{pi->x, pi->y, pi->width, pi->height};
+	GdkRectangle r;
 
-	pi->border = borders;
-
-	pi->color2 = color;
-}
-
-gint pan_item_tri_draw(PanWindow *, PanItem *pi, GdkPixbuf *pixbuf, PixbufRenderer *, gint x, gint y, gint width, gint height)
-{
-	gint rx;
-	gint ry;
-	gint rw;
-	gint rh;
-
-	if (util_clip_region(x, y, width, height,
-			     pi->x, pi->y, pi->width, pi->height,
-			     &rx, &ry, &rw, &rh) && pi->data)
+	if (pi->data && gdk_rectangle_intersect(&request_rect, &pi_rect, &r))
 		{
-		auto coord = static_cast<gint *>(pi->data);
-		pixbuf_draw_triangle(pixbuf,
-				     rx - x, ry - y, rw, rh,
-				     coord[0] - x, coord[1] - y,
-				     coord[2] - x, coord[3] - y,
-				     coord[4] - x, coord[5] - y,
-				     pi->color.r, pi->color.g, pi->color.b, pi->color.a);
+		auto coord = static_cast<GdkPoint *>(pi->data);
+		r.x -= x;
+		r.y -= y;
+		pixbuf_draw_triangle(pixbuf, r,
+		                     {coord[0].x - x, coord[0].y - y},
+		                     {coord[1].x - x, coord[1].y - y},
+		                     {coord[2].x - x, coord[2].y - y},
+		                     pi->color.r, pi->color.g, pi->color.b, pi->color.a);
 
-		if (pi->border & PAN_BORDER_1)
-			{
-			pixbuf_draw_line(pixbuf,
-					 rx - x, ry - y, rw, rh,
-					 coord[0] - x, coord[1] - y,
-					 coord[2] - x, coord[3] - y,
-					 pi->color2.r, pi->color2.g, pi->color2.b, pi->color2.a);
-			}
-		if (pi->border & PAN_BORDER_2)
-			{
-			pixbuf_draw_line(pixbuf,
-					 rx - x, ry - y, rw, rh,
-					 coord[2] - x, coord[3] - y,
-					 coord[4] - x, coord[5] - y,
-					 pi->color2.r, pi->color2.g, pi->color2.b, pi->color2.a);
-			}
-		if (pi->border & PAN_BORDER_3)
-			{
-			pixbuf_draw_line(pixbuf,
-					 rx - x, ry - y, rw, rh,
-					 coord[4] - x, coord[5] - y,
-					 coord[0] - x, coord[1] - y,
-					 pi->color2.r, pi->color2.g, pi->color2.b, pi->color2.a);
-			}
+		const auto draw_line = [pixbuf, &r, x, y, &color = pi->color2](GdkPoint start, GdkPoint end)
+		{
+			pixbuf_draw_line(pixbuf, r,
+			                 start.x - x, start.y - y,
+			                 end.x - x, end.y - y,
+			                 color.r, color.g, color.b, color.a);
+		};
+
+		if (pi->border & PAN_BORDER_1) draw_line(coord[0], coord[1]);
+		if (pi->border & PAN_BORDER_2) draw_line(coord[1], coord[2]);
+		if (pi->border & PAN_BORDER_3) draw_line(coord[2], coord[0]);
 		}
 
 	return FALSE;
@@ -421,7 +370,7 @@ static void pan_item_text_compute_size(PanItem *pi, GtkWidget *widget)
 }
 
 PanItem *pan_item_text_new(PanWindow *pw, gint x, gint y, const gchar *text,
-                           PanTextAttrType attr, PanBorderType border, const PanColor &color)
+                           PanTextAttrType attr, PanBorderType border, PanColor color)
 {
 	PanItem *pi;
 
@@ -443,14 +392,15 @@ PanItem *pan_item_text_new(PanWindow *pw, gint x, gint y, const gchar *text,
 	return pi;
 }
 
-gint pan_item_text_draw(PanWindow *, PanItem *pi, GdkPixbuf *pixbuf, PixbufRenderer *pr, gint x, gint y, gint, gint)
+gboolean pan_item_text_draw(PanWindow *, PanItem *pi, GdkPixbuf *pixbuf, PixbufRenderer *pr,
+                            gint x, gint y, gint, gint)
 {
 	PangoLayout *layout;
 
-	layout = pan_item_text_layout(pi, reinterpret_cast<GtkWidget *>(pr));
-	pixbuf_draw_layout(pixbuf, layout, reinterpret_cast<GtkWidget *>(pr),
-			   pi->x - x + pi->border, pi->y - y + pi->border,
-			   pi->color.r, pi->color.g, pi->color.b, pi->color.a);
+	layout = pan_item_text_layout(pi, GTK_WIDGET(pr));
+	pixbuf_draw_layout(pixbuf, layout,
+	                   pi->x - x + pi->border, pi->y - y + pi->border,
+	                   pi->color.r, pi->color.g, pi->color.b, pi->color.a);
 	g_object_unref(G_OBJECT(layout));
 
 	return FALSE;
@@ -481,127 +431,104 @@ PanItem *pan_item_thumb_new(PanWindow *pw, FileData *fd, gint x, gint y)
 	return pi;
 }
 
-gint pan_item_thumb_draw(PanWindow *pw, PanItem *pi, GdkPixbuf *pixbuf, PixbufRenderer *, gint x, gint y, gint width, gint height)
+gboolean pan_item_thumb_draw(PanWindow *pw, PanItem *pi, GdkPixbuf *pixbuf, PixbufRenderer *,
+                             gint x, gint y, gint width, gint height)
 {
-	gint tx;
-	gint ty;
-	gint tw;
-	gint th;
-	gint rx;
-	gint ry;
-	gint rw;
-	gint rh;
+	const GdkRectangle request_rect{x, y, width, height};
+	GdkRectangle thumb_rect;
+	GdkRectangle r;
 
 	if (pi->pixbuf)
 		{
-		tw = gdk_pixbuf_get_width(pi->pixbuf);
-		th = gdk_pixbuf_get_height(pi->pixbuf);
+		gint tw = gdk_pixbuf_get_width(pi->pixbuf);
+		gint th = gdk_pixbuf_get_height(pi->pixbuf);
 
-		tx = pi->x + (pi->width - tw) / 2;
-		ty = pi->y + (pi->height - th) / 2;
+		gint tx = pi->x + (pi->width - tw) / 2;
+		gint ty = pi->y + (pi->height - th) / 2;
 
 		if (gdk_pixbuf_get_has_alpha(pi->pixbuf))
 			{
-			if (util_clip_region(x, y, width, height,
-					     tx + PAN_SHADOW_OFFSET, ty + PAN_SHADOW_OFFSET, tw, th,
-					     &rx, &ry, &rw, &rh))
+			thumb_rect = {tx + PAN_SHADOW_OFFSET, ty + PAN_SHADOW_OFFSET, tw, th};
+			if (gdk_rectangle_intersect(&request_rect, &thumb_rect, &r))
 				{
 				pixbuf_draw_shadow(pixbuf,
-						   rx - x, ry - y, rw, rh,
-						   tx + PAN_SHADOW_OFFSET - x, ty + PAN_SHADOW_OFFSET - y, tw, th,
-						   PAN_SHADOW_FADE,
-						   PAN_SHADOW_COLOR, PAN_SHADOW_ALPHA);
+				                   {r.x - x, r.y - y, r.width, r.height},
+				                   tx + PAN_SHADOW_OFFSET - x, ty + PAN_SHADOW_OFFSET - y, tw, th,
+				                   PAN_SHADOW_FADE,
+				                   PAN_SHADOW_COLOR, PAN_SHADOW_ALPHA);
 				}
 			}
 		else
 			{
-			if (util_clip_region(x, y, width, height,
-					     tx + tw, ty + PAN_SHADOW_OFFSET,
-					     PAN_SHADOW_OFFSET, th - PAN_SHADOW_OFFSET,
-					     &rx, &ry, &rw, &rh))
+			thumb_rect = {tx + tw, ty + PAN_SHADOW_OFFSET, PAN_SHADOW_OFFSET, th - PAN_SHADOW_OFFSET};
+			if (gdk_rectangle_intersect(&request_rect, &thumb_rect, &r))
 				{
 				pixbuf_draw_shadow(pixbuf,
-						   rx - x, ry - y, rw, rh,
-						   tx + PAN_SHADOW_OFFSET - x, ty + PAN_SHADOW_OFFSET - y, tw, th,
-						   PAN_SHADOW_FADE,
-						   PAN_SHADOW_COLOR, PAN_SHADOW_ALPHA);
+				                   {r.x - x, r.y - y, r.width, r.height},
+				                   tx + PAN_SHADOW_OFFSET - x, ty + PAN_SHADOW_OFFSET - y, tw, th,
+				                   PAN_SHADOW_FADE,
+				                   PAN_SHADOW_COLOR, PAN_SHADOW_ALPHA);
 				}
-			if (util_clip_region(x, y, width, height,
-					     tx + PAN_SHADOW_OFFSET, ty + th, tw, PAN_SHADOW_OFFSET,
-					     &rx, &ry, &rw, &rh))
+
+			thumb_rect = {tx + PAN_SHADOW_OFFSET, ty + th, tw, PAN_SHADOW_OFFSET};
+			if (gdk_rectangle_intersect(&request_rect, &thumb_rect, &r))
 				{
 				pixbuf_draw_shadow(pixbuf,
-						   rx - x, ry - y, rw, rh,
-						   tx + PAN_SHADOW_OFFSET - x, ty + PAN_SHADOW_OFFSET - y, tw, th,
-						   PAN_SHADOW_FADE,
-						   PAN_SHADOW_COLOR, PAN_SHADOW_ALPHA);
+				                   {r.x - x, r.y - y, r.width, r.height},
+				                   tx + PAN_SHADOW_OFFSET - x, ty + PAN_SHADOW_OFFSET - y, tw, th,
+				                   PAN_SHADOW_FADE,
+				                   PAN_SHADOW_COLOR, PAN_SHADOW_ALPHA);
 				}
 			}
 
-		if (util_clip_region(x, y, width, height,
-				     tx, ty, tw, th,
-				     &rx, &ry, &rw, &rh))
+		thumb_rect = {tx, ty, tw, th};
+		if (gdk_rectangle_intersect(&request_rect, &thumb_rect, &r))
 			{
-			gdk_pixbuf_composite(pi->pixbuf, pixbuf, rx - x, ry - y, rw, rh,
+			gdk_pixbuf_composite(pi->pixbuf, pixbuf, r.x - x, r.y - y, r.width, r.height,
 					     static_cast<gdouble>(tx) - x,
 					     static_cast<gdouble>(ty) - y,
 					     1.0, 1.0, GDK_INTERP_NEAREST,
 					     255);
 			}
 
-		if (util_clip_region(x, y, width, height,
-				     tx, ty, tw, PAN_OUTLINE_THICKNESS,
-				     &rx, &ry, &rw, &rh))
-			{
-			pixbuf_draw_rect_fill(pixbuf,
-					      rx - x, ry - y, rw, rh,
-					      PAN_OUTLINE_COLOR_1);
-			}
-		if (util_clip_region(x, y, width, height,
-				     tx, ty, PAN_OUTLINE_THICKNESS, th,
-				     &rx, &ry, &rw, &rh))
-			{
-			pixbuf_draw_rect_fill(pixbuf,
-					      rx - x, ry - y, rw, rh,
-					      PAN_OUTLINE_COLOR_1);
-			}
-		if (util_clip_region(x, y, width, height,
-				     tx + tw - PAN_OUTLINE_THICKNESS, ty +  PAN_OUTLINE_THICKNESS,
-				     PAN_OUTLINE_THICKNESS, th - PAN_OUTLINE_THICKNESS,
-				     &rx, &ry, &rw, &rh))
-			{
-			pixbuf_draw_rect_fill(pixbuf,
-					      rx - x, ry - y, rw, rh,
-					      PAN_OUTLINE_COLOR_2);
-			}
-		if (util_clip_region(x, y, width, height,
-				     tx +  PAN_OUTLINE_THICKNESS, ty + th - PAN_OUTLINE_THICKNESS,
-				     tw - PAN_OUTLINE_THICKNESS * 2, PAN_OUTLINE_THICKNESS,
-				     &rx, &ry, &rw, &rh))
-			{
-			pixbuf_draw_rect_fill(pixbuf,
-					      rx - x, ry - y, rw, rh,
-					      PAN_OUTLINE_COLOR_2);
-			}
+		const auto draw_rect_if_intersect = [pixbuf, &request_rect, x, y](GdkRectangle thumb_rect, PanColor color)
+		{
+			GdkRectangle r;
+			if (!gdk_rectangle_intersect(&request_rect, &thumb_rect, &r)) return;
+
+			r.x -= x;
+			r.y -= y;
+			pixbuf_draw_rect_fill(pixbuf, r, color.r, color.g, color.b, color.a);
+		};
+
+		thumb_rect = {tx, ty, tw, PAN_OUTLINE_THICKNESS};
+		draw_rect_if_intersect(thumb_rect, PAN_OUTLINE_COLOR_1);
+
+		thumb_rect = {tx, ty, PAN_OUTLINE_THICKNESS, th};
+		draw_rect_if_intersect(thumb_rect, PAN_OUTLINE_COLOR_1);
+
+		thumb_rect = {tx + tw - PAN_OUTLINE_THICKNESS, ty + PAN_OUTLINE_THICKNESS,
+		              PAN_OUTLINE_THICKNESS, th - PAN_OUTLINE_THICKNESS};
+		draw_rect_if_intersect(thumb_rect, PAN_OUTLINE_COLOR_2);
+
+		thumb_rect = {tx + PAN_OUTLINE_THICKNESS, ty + th - PAN_OUTLINE_THICKNESS,
+		              tw - PAN_OUTLINE_THICKNESS * 2, PAN_OUTLINE_THICKNESS};
+		draw_rect_if_intersect(thumb_rect, PAN_OUTLINE_COLOR_2);
 		}
 	else
 		{
-		tw = pi->width - PAN_SHADOW_OFFSET * 2;
-		th = pi->height - PAN_SHADOW_OFFSET * 2;
-		tx = pi->x + PAN_SHADOW_OFFSET;
-		ty = pi->y + PAN_SHADOW_OFFSET;
-
-		if (util_clip_region(x, y, width, height,
-				     tx, ty, tw, th,
-				     &rx, &ry, &rw, &rh))
+		thumb_rect = {pi->x + PAN_SHADOW_OFFSET, pi->y + PAN_SHADOW_OFFSET,
+		              pi->width - PAN_SHADOW_OFFSET * 2, pi->height - PAN_SHADOW_OFFSET * 2 };
+		if (gdk_rectangle_intersect(&request_rect, &thumb_rect, &r))
 			{
 			gint d;
 
+			r.x -= x;
+			r.y -= y;
 			d = (pw->size <= PAN_IMAGE_SIZE_THUMB_NONE) ? 2 : 8;
-			pixbuf_draw_rect_fill(pixbuf,
-					      rx - x, ry - y, rw, rh,
-					      PAN_SHADOW_COLOR,
-					      PAN_SHADOW_ALPHA / d);
+			pixbuf_draw_rect_fill(pixbuf, r,
+			                      PAN_SHADOW_COLOR,
+			                      PAN_SHADOW_ALPHA / d);
 			}
 		}
 
@@ -615,47 +542,19 @@ gint pan_item_thumb_draw(PanWindow *pw, PanItem *pi, GdkPixbuf *pixbuf, PixbufRe
  *-----------------------------------------------------------------------------
  */
 
-static void pan_item_image_find_size(PanWindow *pw, PanItem *pi, gint w, gint h)
-{
-	GList *work;
-
-	pi->width = w;
-	pi->height = h;
-
-	if (!pi->fd) return;
-
-	work = pw->cache_list;
-	while (work)
-		{
-		PanCacheData *pc;
-
-		pc = static_cast<PanCacheData *>(work->data);
-		work = work->next;
-
-		if (pc->cd && pc->cd->dimensions &&
-		    pc->fd && pc->fd == pi->fd)
-			{
-			pi->width = MAX(1, pc->cd->width * pw->image_size / 100);
-			pi->height = MAX(1, pc->cd->height * pw->image_size / 100);
-
-			pw->cache_list = g_list_remove(pw->cache_list, pc);
-			cache_sim_data_free(pc->cd);
-			file_data_unref(pc->fd);
-			g_free(pc);
-			return;
-			}
-		}
-}
-
 PanItem *pan_item_image_new(PanWindow *pw, FileData *fd, gint x, gint y, gint w, gint h)
 {
 	PanItem *pi;
 
+	pan_cache_get_image_size(pw, fd, w, h);
+
 	pi = g_new0(PanItem, 1);
 	pi->type = PAN_ITEM_IMAGE;
 	pi->fd = fd;
 	pi->x = x;
 	pi->y = y;
+	pi->width = w;
+	pi->height = h;
 
 	pi->color.a = 255;
 
@@ -664,37 +563,34 @@ PanItem *pan_item_image_new(PanWindow *pw, FileData *fd, gint x, gint y, gint w,
 	pi->color2.b = 0;
 	pi->color2.a = PAN_SHADOW_ALPHA / 2;
 
-	pan_item_image_find_size(pw, pi, w, h);
-
 	pw->list = g_list_prepend(pw->list, pi);
 
 	return pi;
 }
 
-gint pan_item_image_draw(PanWindow *, PanItem *pi, GdkPixbuf *pixbuf, PixbufRenderer *, gint x, gint y, gint width, gint height)
+gboolean pan_item_image_draw(PanWindow *, PanItem *pi, GdkPixbuf *pixbuf, PixbufRenderer *,
+                             gint x, gint y, gint width, gint height)
 {
-	gint rx;
-	gint ry;
-	gint rw;
-	gint rh;
+	const GdkRectangle request_rect{x, y, width, height};
+	const GdkRectangle pi_rect{pi->x, pi->y, pi->width, pi->height};
+	GdkRectangle r;
 
-	if (util_clip_region(x, y, width, height,
-			     pi->x, pi->y, pi->width, pi->height,
-			     &rx, &ry, &rw, &rh))
+	if (gdk_rectangle_intersect(&request_rect, &pi_rect, &r))
 		{
+		r.x -= x;
+		r.y -= y;
+
 		if (pi->pixbuf)
 			{
-			gdk_pixbuf_composite(pi->pixbuf, pixbuf, rx - x, ry - y, rw, rh,
-					     static_cast<gdouble>(pi->x) - x,
-					     static_cast<gdouble>(pi->y) - y,
-					     1.0, 1.0, GDK_INTERP_NEAREST,
-					     pi->color.a);
+			gdk_pixbuf_composite(pi->pixbuf, pixbuf, r.x, r.y, r.width, r.height,
+			                     static_cast<gdouble>(pi->x) - x,
+			                     static_cast<gdouble>(pi->y) - y,
+			                     1.0, 1.0, GDK_INTERP_NEAREST,
+			                     pi->color.a);
 			}
 		else
 			{
-			pixbuf_draw_rect_fill(pixbuf,
-					      rx - x, ry - y, rw, rh,
-					      pi->color2.r, pi->color2.g, pi->color2.b, pi->color2.a);
+			pixbuf_draw_rect_fill(pixbuf, r, pi->color2.r, pi->color2.g, pi->color2.b, pi->color2.a);
 			}
 		}
 
@@ -864,126 +760,72 @@ PanItem *pan_item_find_by_coord(PanWindow *pw, PanItemType type,
  *-----------------------------------------------------------------------------
  */
 
-PanTextAlignment *pan_text_alignment_new(PanWindow *pw, gint x, gint y, const gchar *key)
+PanTextAlignment::PanTextAlignment(PanWindow *pw, gint x, gint y, const gchar *key)
+	: pw(pw)
+	, x(x)
+	, y(y)
+	, key(g_strdup(key))
 {
-	PanTextAlignment *ta;
-
-	ta = g_new0(PanTextAlignment, 1);
-
-	ta->pw = pw;
-	ta->x = x;
-	ta->y = y;
-	ta->key = g_strdup(key);
-
-	return ta;
 }
 
-void pan_text_alignment_free(PanTextAlignment *ta)
+PanTextAlignment::~PanTextAlignment()
 {
-	if (!ta) return;
-
-	g_list_free(ta->column1);
-	g_list_free(ta->column2);
-	g_free(ta->key);
-	g_free(ta);
+	g_free(key);
 }
 
-PanItem *pan_text_alignment_add(PanTextAlignment *ta, const gchar *label, const gchar *text)
+void PanTextAlignment::add(const gchar *label, const gchar *text)
 {
-	PanItem *item;
+	Items items;
 
 	if (label)
 		{
-		item = pan_item_text_new(ta->pw, ta->x, ta->y, label,
-					 PAN_TEXT_ATTR_BOLD, PAN_BORDER_NONE, PAN_POPUP_TEXT_COLOR);
-		pan_item_set_key(item, ta->key);
+		items.label = pan_item_text_new(pw, x, y, label,
+		                                PAN_TEXT_ATTR_BOLD, PAN_BORDER_NONE, PAN_POPUP_TEXT_COLOR);
+		pan_item_set_key(items.label, key);
 		}
-	else
-		{
-		item = nullptr;
-		}
-	ta->column1 = g_list_append(ta->column1, item);
 
 	if (text)
 		{
-		item = pan_item_text_new(ta->pw, ta->x, ta->y, text,
-					 PAN_TEXT_ATTR_NONE, PAN_BORDER_NONE, PAN_POPUP_TEXT_COLOR);
-		pan_item_set_key(item, ta->key);
+		items.text = pan_item_text_new(pw, x, y, text,
+		                               PAN_TEXT_ATTR_NONE, PAN_BORDER_NONE, PAN_POPUP_TEXT_COLOR);
+		pan_item_set_key(items.text, key);
 		}
-	else
-		{
-		item = nullptr;
-		}
-	ta->column2 = g_list_append(ta->column2, item);
 
-	return item;
+	columns.push_back(items);
 }
 
-void pan_text_alignment_calc(PanTextAlignment *ta, PanItem *box)
+void PanTextAlignment::calc(PanItem *box)
 {
-	gint cw1;
-	gint cw2;
-	gint x;
-	gint y;
-	GList *work1;
-	GList *work2;
-
-	cw1 = 0;
-	cw2 = 0;
-
-	work1 = ta->column1;
-	while (work1)
+	gint label_column_width = 0;
+	for (const Items &items : columns)
 		{
-		PanItem *p;
-
-		p = static_cast<PanItem *>(work1->data);
-		work1 = work1->next;
-
-		if (p && p->width > cw1) cw1 = p->width;
+		if (items.label) label_column_width = std::max(label_column_width, items.label->width);
 		}
 
-	work2 = ta->column2;
-	while (work2)
+	gint y = this->y;
+	for (Items &items : columns)
 		{
-		PanItem *p;
-
-		p = static_cast<PanItem *>(work2->data);
-		work2 = work2->next;
-
-		if (p && p->width > cw2) cw2 = p->width;
-		}
-
-	x = ta->x;
-	y = ta->y;
-	work1 = ta->column1;
-	work2 = ta->column2;
-	while (work1 && work2)
-		{
-		PanItem *p1;
-		PanItem *p2;
+		PanItem *pi_label = items.label;
+		PanItem *pi_text = items.text;
 		gint height = 0;
 
-		p1 = static_cast<PanItem *>(work1->data);
-		p2 = static_cast<PanItem *>(work2->data);
-		work1 = work1->next;
-		work2 = work2->next;
-
-		if (p1)
+		if (pi_label)
 			{
-			p1->x = x;
-			p1->y = y;
-			pan_item_size_by_item(box, p1, PREF_PAD_BORDER);
-			height = p1->height;
-			}
-		if (p2)
-			{
-			p2->x = x + cw1 + PREF_PAD_SPACE;
-			p2->y = y;
-			pan_item_size_by_item(box, p2, PREF_PAD_BORDER);
-			if (height < p2->height) height = p2->height;
+			pi_label->x = x;
+			pi_label->y = y;
+			pan_item_size_by_item(box, pi_label, PREF_PAD_BORDER);
+			height = pi_label->height;
 			}
 
-		if (!p1 && !p2) height = PREF_PAD_GROUP;
+		if (pi_text)
+			{
+			pi_text->x = x + label_column_width + PREF_PAD_SPACE;
+			pi_text->y = y;
+			pan_item_size_by_item(box, pi_text, PREF_PAD_BORDER);
+			height = std::max(height, pi_text->height);
+			}
+
+		if (!pi_label && !pi_text) height = PREF_PAD_GROUP;
 
 		y += height;
 		}
