@@ -21,6 +21,8 @@
 
 #include "pan-view.h"
 
+#include <algorithm>
+#include <array>
 #include <cmath>
 #include <cstring>
 
@@ -29,6 +31,7 @@
 #include <glib-object.h>
 #include <gtk/gtk.h>
 
+#include "../pan-view.h"
 #include "bar-exif.h"
 #include "cache-loader.h"
 #include "cache.h"
@@ -46,7 +49,6 @@
 #include "layout-util.h"
 #include "layout.h"
 #include "main-defines.h"
-#include "main.h"
 #include "menu.h"
 #include "metadata.h"
 #include "options.h"
@@ -74,6 +76,16 @@
 namespace
 {
 
+struct PanCacheData {
+	FileData *fd;
+	CacheData *cd;
+};
+
+struct PanGrid {
+	GdkRectangle rect;
+	GList *list;
+};
+
 constexpr gint PAN_WINDOW_DEFAULT_WIDTH = 720;
 constexpr gint PAN_WINDOW_DEFAULT_HEIGHT = 500;
 
@@ -94,6 +106,15 @@ constexpr guint8 PAN_POPUP_ALPHA = 255;
 constexpr PanColor PAN_POPUP_COLOR{255, 255, 225, PAN_POPUP_ALPHA};
 constexpr PanColor PAN_POPUP_BORDER_COLOR{0, 0, 0, PAN_POPUP_ALPHA};
 
+void pan_cache_data_free(PanCacheData *pc)
+{
+	if (!pc) return;
+
+	cache_sim_data_free(pc->cd);
+	file_data_unref(pc->fd);
+	g_free(pc);
+}
+
 } // namespace
 
 #define PAN_PREF_GROUP		"pan_view_options"
@@ -361,43 +382,35 @@ static gboolean pan_window_request_tile_cb(PixbufRenderer *pr, gint x, gint y,
 	auto pw = static_cast<PanWindow *>(data);
 	GList *list;
 	GList *work;
-	gint i;
+	const GdkRectangle request_rect{x, y, width, height};
+	GdkRectangle pan_grid_rect;
 
 	pixbuf_set_rect_fill(pixbuf,
 			     0, 0, width, height,
 			     PAN_BACKGROUND_COLOR);
 
-	for (i = (x / PAN_GRID_SIZE) * PAN_GRID_SIZE; i < x + width; i += PAN_GRID_SIZE)
-		{
-		gint rx;
-		gint ry;
-		gint rw;
-		gint rh;
+	const auto draw_rect_if_intersect = [pixbuf, &request_rect, x, y](GdkRectangle pan_grid_rect)
+	{
+		GdkRectangle r;
+		if (!gdk_rectangle_intersect(&request_rect, &pan_grid_rect, &r)) return;
 
-		if (util_clip_region(x, y, width, height,
-				     i, y, 1, height,
-				     &rx, &ry, &rw, &rh))
-			{
-			pixbuf_draw_rect_fill(pixbuf,
-					      rx - x, ry - y, rw, rh,
-					      PAN_GRID_COLOR);
-			}
+		r.x -= x;
+		r.y -= y;
+		pixbuf_draw_rect_fill(pixbuf, r, PAN_GRID_COLOR);
+	};
+
+	pan_grid_rect = request_rect;
+	pan_grid_rect.width = 1;
+	for (pan_grid_rect.x = (x / PAN_GRID_SIZE) * PAN_GRID_SIZE; pan_grid_rect.x < x + width; pan_grid_rect.x += PAN_GRID_SIZE)
+		{
+		draw_rect_if_intersect(pan_grid_rect);
 		}
-	for (i = (y / PAN_GRID_SIZE) * PAN_GRID_SIZE; i < y + height; i += PAN_GRID_SIZE)
-		{
-		gint rx;
-		gint ry;
-		gint rw;
-		gint rh;
 
-		if (util_clip_region(x, y, width, height,
-				     x, i, width, 1,
-				     &rx, &ry, &rw, &rh))
-			{
-			pixbuf_draw_rect_fill(pixbuf,
-					      rx - x, ry - y, rw, rh,
-					      PAN_GRID_COLOR);
-			}
+	pan_grid_rect = request_rect;
+	pan_grid_rect.height = 1;
+	for (pan_grid_rect.y = (y / PAN_GRID_SIZE) * PAN_GRID_SIZE; pan_grid_rect.y < y + height; pan_grid_rect.y += PAN_GRID_SIZE)
+		{
+		draw_rect_if_intersect(pan_grid_rect);
 		}
 
 	list = pan_layout_intersect(pw, x, y, width, height);
@@ -597,27 +610,22 @@ static void pan_window_zoom_limit(PanWindow *pw)
  *-----------------------------------------------------------------------------
  */
 
-static gint pan_cache_sort_file_cb(gpointer a, gpointer b)
+static gint pan_cache_sort_file_cb(gconstpointer a, gconstpointer b, gpointer data)
 {
-	auto pca = static_cast<PanCacheData *>(a);
-	auto pcb = static_cast<PanCacheData *>(b);
-	return filelist_sort_compare_filedata(pca->fd, pcb->fd);
-}
-GList *pan_cache_sort(GList *list, SortType method, gboolean ascend, gboolean case_sensitive)
-{
-	return filelist_sort_full(list, method, ascend, case_sensitive, reinterpret_cast<GCompareFunc>(pan_cache_sort_file_cb));
+	auto pca = static_cast<const PanCacheData *>(a);
+	auto pcb = static_cast<const PanCacheData *>(b);
+	auto settings = static_cast<FileData::FileList::SortSettings *>(data);
+	return filelist_sort_compare_filedata(pca->fd, pcb->fd, settings);
 }
 
+GList *pan_cache_sort(GList *list, SortType method, gboolean ascend, gboolean case_sensitive)
+{
+	return filelist_sort_full(list, method, ascend, case_sensitive, pan_cache_sort_file_cb);
+}
 
 static void pan_cache_free(PanWindow *pw)
 {
-	g_list_free_full(pw->cache_list, [](gpointer data)
-		{
-		auto pc = static_cast<PanCacheData *>(data);
-		cache_sim_data_free(pc->cd);
-		file_data_unref(pc->fd);
-		g_free(pc);
-		});
+	g_list_free_full(pw->cache_list, reinterpret_cast<GDestroyNotify>(pan_cache_data_free));
 	pw->cache_list = nullptr;
 
 	filelist_free(pw->cache_todo);
@@ -736,6 +744,28 @@ void pan_cache_sync_date(PanWindow *pw, GList *list)
 	g_list_free(haystack);
 }
 
+void pan_cache_get_image_size(PanWindow *pw, const FileData *fd, gint &w, gint &h)
+{
+	if (!fd) return;
+
+	const auto pan_cache_data_cd_dimensions_compare_fd = [](gconstpointer data, gconstpointer user_data)
+	{
+		auto *pc = static_cast<const PanCacheData *>(data);
+		return (pc->cd && pc->cd->dimensions && pc->fd == user_data) ? 0 : 1;
+	};
+
+	GList *work = g_list_find_custom(pw->cache_list, fd, pan_cache_data_cd_dimensions_compare_fd);
+	if (!work) return;
+
+	auto *pc = static_cast<PanCacheData *>(work->data);
+
+	w = std::max(1, pc->cd->width * pw->image_size / 100);
+	h = std::max(1, pc->cd->height * pw->image_size / 100);
+
+	pw->cache_list = g_list_remove(pw->cache_list, pc);
+	pan_cache_data_free(pc);
+}
+
 /*
  *-----------------------------------------------------------------------------
  * item grid
@@ -758,7 +788,6 @@ static void pan_grid_clear(PanWindow *pw)
 
 static void pan_grid_build(PanWindow *pw, gint width, gint height, gint grid_size)
 {
-	GList *work;
 	gint col;
 	gint row;
 	gint cw;
@@ -795,54 +824,38 @@ static void pan_grid_build(PanWindow *pw, gint width, gint height, gint grid_siz
 			PanGrid *pg;
 
 			pg = g_new0(PanGrid, 1);
-			pg->x = i * cw / 2;
-			pg->y = j * ch / 2;
-			pg->w = cw;
-			pg->h = ch;
+			pg->rect.x = i * cw / 2;
+			pg->rect.y = j * ch / 2;
+			pg->rect.width = cw;
+			pg->rect.height = ch;
 
 			pw->list_grid = g_list_prepend(pw->list_grid, pg);
 
-			DEBUG_1("grid section: %d,%d (%dx%d)", pg->x, pg->y, pg->w, pg->h);
+			DEBUG_1("grid section: %d,%d (%dx%d)", pg->rect.x, pg->rect.y, pg->rect.width, pg->rect.height);
 			}
 		}
 
-	work = pw->list;
-	while (work)
+	for (GList *work = pw->list; work; work = work->next)
 		{
-		PanItem *pi;
-		GList *grid;
+		auto *pi = static_cast<PanItem *>(work->data);
 
-		pi = static_cast<PanItem *>(work->data);
-		work = work->next;
+		// @todo use GdkRectangle in PanItem
+		const GdkRectangle pi_rect{pi->x, pi->y, pi->width, pi->height};
 
-		grid = pw->list_grid;
-		while (grid)
+		for (GList *grid = pw->list_grid; grid; grid = grid->next)
 			{
-			PanGrid *pg;
-			gint rx;
-			gint ry;
-			gint rw;
-			gint rh;
+			auto *pg = static_cast<PanGrid *>(grid->data);
 
-			pg = static_cast<PanGrid *>(grid->data);
-			grid = grid->next;
-
-			if (util_clip_region(pi->x, pi->y, pi->width, pi->height,
-					     pg->x, pg->y, pg->w, pg->h,
-					     &rx, &ry, &rw, &rh))
+			if (gdk_rectangle_intersect(&pi_rect, &pg->rect, nullptr))
 				{
 				pg->list = g_list_prepend(pg->list, pi);
 				}
 			}
 		}
 
-	work = pw->list_grid;
-	while (work)
+	for (GList *grid = pw->list_grid; grid; grid = grid->next)
 		{
-		PanGrid *pg;
-
-		pg = static_cast<PanGrid *>(work->data);
-		work = work->next;
+		auto *pg = static_cast<PanGrid *>(grid->data);
 
 		pg->list = g_list_reverse(pg->list);
 		}
@@ -887,8 +900,8 @@ static void pan_window_items_free(PanWindow *pw)
  */
 
 static void pan_layout_compute(PanWindow *pw, FileData *dir_fd,
-			       gint *width, gint *height,
-			       gint *scroll_x, gint *scroll_y)
+                               gint &width, gint &height,
+                               gint &scroll_x, gint &scroll_y)
 {
 	pan_window_items_free(pw);
 
@@ -937,10 +950,10 @@ static void pan_layout_compute(PanWindow *pw, FileData *dir_fd,
 			break;
 		}
 
-	*width = 0;
-	*height = 0;
-	*scroll_x = 0;
-	*scroll_y = 0;
+	width = 0;
+	height = 0;
+	scroll_x = 0;
+	scroll_y = 0;
 
 	switch (pw->layout)
 		{
@@ -964,29 +977,18 @@ static void pan_layout_compute(PanWindow *pw, FileData *dir_fd,
 
 	pan_cache_free(pw);
 
-	DEBUG_1("computed %d objects", g_list_length(pw->list));
+	DEBUG_1("computed %u objects", g_list_length(pw->list));
 }
 
 static GList *pan_layout_intersect_l(GList *list, GList *item_list,
-				     gint x, gint y, gint width, gint height)
+                                     GdkRectangle rect)
 {
-	GList *work;
-
-	work = item_list;
-	while (work)
+	for (GList *work = item_list; work; work = work->next)
 		{
-		PanItem *pi;
-		gint rx;
-		gint ry;
-		gint rw;
-		gint rh;
+		auto *pi = static_cast<PanItem *>(work->data);
+		const GdkRectangle pi_rect = {pi->x, pi->y, pi->width, pi->height};
 
-		pi = static_cast<PanItem *>(work->data);
-		work = work->next;
-
-		if (util_clip_region(x, y, width, height,
-				     pi->x, pi->y, pi->width, pi->height,
-				     &rx, &ry, &rw, &rh))
+		if (gdk_rectangle_intersect(&rect, &pi_rect, nullptr))
 			{
 			list = g_list_prepend(list, pi);
 			}
@@ -998,31 +1000,29 @@ static GList *pan_layout_intersect_l(GList *list, GList *item_list,
 GList *pan_layout_intersect(PanWindow *pw, gint x, gint y, gint width, gint height)
 {
 	GList *list = nullptr;
-	GList *grid;
-	PanGrid *pg = nullptr;
+	const GdkRectangle rect{x, y, width, height};
 
-	grid = pw->list_grid;
-	while (grid && !pg)
+	const auto pan_grid_contains_rect = [](gconstpointer data, gconstpointer user_data) -> gint
+	{
+		const auto *pg = static_cast<const PanGrid *>(data);
+		const auto *rect = static_cast<const GdkRectangle *>(user_data);
+
+		GdkRectangle intersection;
+		if (!gdk_rectangle_intersect(&pg->rect, rect, &intersection)) return 1;
+
+		return gdk_rectangle_equal(rect, &intersection) ? 0 : 1;
+	};
+
+	list = pan_layout_intersect_l(list, pw->list, rect);
+
+	GList *grid = g_list_find_custom(pw->list_grid, &rect, pan_grid_contains_rect);
+	if (grid)
 		{
-		pg = static_cast<PanGrid *>(grid->data);
-		grid = grid->next;
-
-		if (x < pg->x || x + width > pg->x + pg->w ||
-		    y < pg->y || y + height > pg->y + pg->h)
-			{
-			pg = nullptr;
-			}
-		}
-
-	list = pan_layout_intersect_l(list, pw->list, x, y, width, height);
-
-	if (pg)
-		{
-		list = pan_layout_intersect_l(list, pg->list, x, y, width, height);
+		list = pan_layout_intersect_l(list, static_cast<PanGrid *>(grid->data)->list, rect);
 		}
 	else
 		{
-		list = pan_layout_intersect_l(list, pw->list_static, x, y, width, height);
+		list = pan_layout_intersect_l(list, pw->list_static, rect);
 		}
 
 	return list;
@@ -1115,7 +1115,7 @@ static gint pan_layout_update_idle_cb(gpointer data)
 			}
 		}
 
-	pan_layout_compute(pw, pw->dir_fd, &width, &height, &scroll_x, &scroll_y);
+	pan_layout_compute(pw, pw->dir_fd, width, height, scroll_x, scroll_y);
 
 	pan_window_zoom_limit(pw);
 
@@ -1186,7 +1186,7 @@ static void pan_layout_set_fd(PanWindow *pw, FileData *dir_fd)
  *-----------------------------------------------------------------------------
  */
 
-FileData *pan_menu_click_fd(PanWindow *pw)
+static FileData *pan_menu_click_fd(PanWindow *pw)
 {
 	if (pw->click_pi && pw->click_pi->fd) return pw->click_pi->fd;
 	return nullptr;
@@ -1250,12 +1250,7 @@ static gboolean pan_window_key_press_cb(GtkWidget *widget, GdkEventKey *event, g
 
 		if (x != 0 || y!= 0)
 			{
-			if (event->state & GDK_SHIFT_MASK)
-				{
-				x *= 3;
-				y *= 3;
-				}
-			keyboard_scroll_calc(&x, &y, event);
+			keyboard_scroll_calc(x, y, event);
 			pixbuf_renderer_scroll(pr, x, y);
 			}
 		}
@@ -1383,6 +1378,8 @@ static gboolean pan_window_key_press_cb(GtkWidget *widget, GdkEventKey *event, g
 					break;
 					stop_signal = FALSE;
 					break;
+				default:
+					break;
 				}
 			}
 		}
@@ -1401,7 +1398,7 @@ static gboolean pan_window_key_press_cb(GtkWidget *widget, GdkEventKey *event, g
  *-----------------------------------------------------------------------------
  */
 
-static void pan_info_add_exif(PanTextAlignment *ta, FileData *fd)
+static void pan_info_add_exif(PanTextAlignment &ta, FileData *fd)
 {
 	GList *exif_list;
 	gchar *text;
@@ -1421,7 +1418,7 @@ static void pan_info_add_exif(PanTextAlignment *ta, FileData *fd)
 		text = metadata_read_string(fd, key, METADATA_FORMATTED);
 		if (text && text[0] != '\0')
 			{
-			pan_text_alignment_add(ta, title, text);
+			ta.add(title, text);
 			}
 
 		g_free(text);
@@ -1430,23 +1427,34 @@ static void pan_info_add_exif(PanTextAlignment *ta, FileData *fd)
 	g_list_free_full(exif_list, g_free);
 }
 
+static void pan_info_calc_text_alignment(PanWindow *pw, PanItem *pbox, FileData *fd)
+{
+	PanTextAlignment ta{pw, pbox->x + PREF_PAD_BORDER, pbox->y + PREF_PAD_BORDER, "info"};
+
+	ta.add(_("Filename:"), fd->name);
+
+	gchar *buf = remove_level_from_path(fd->path);
+	ta.add(_("Location:"), buf);
+	g_free(buf);
+
+	ta.add(_("Date:"), text_from_time(fd->date));
+
+	buf = text_from_size(fd->size);
+	ta.add(_("Size:"), buf);
+	g_free(buf);
+
+	if (pw->info_includes_exif)
+		{
+		pan_info_add_exif(ta, fd);
+		}
+
+	ta.calc(pbox);
+}
 
 void pan_info_update(PanWindow *pw, PanItem *pi)
 {
-	PanTextAlignment *ta;
 	PanItem *pbox;
 	PanItem *p;
-	gchar *buf;
-	gint x1;
-	gint y1;
-	gint x2;
-	gint y2;
-	gint x3;
-	gint y3;
-	gint x;
-	gint y;
-	gint w;
-	gint h;
 
 	if (pw->click_pi == pi) return;
 	if (pi && !pi->fd) pi = nullptr;
@@ -1462,52 +1470,27 @@ void pan_info_update(PanWindow *pw, PanItem *pi)
 				PAN_POPUP_BORDER, PAN_POPUP_COLOR, PAN_POPUP_BORDER_COLOR);
 	pan_item_set_key(pbox, "info");
 
+	GdkPoint c1{pi->x + pi->width - 8, pi->y + 8};
 	if (pi->type == PAN_ITEM_THUMB && pi->pixbuf)
 		{
-		w = gdk_pixbuf_get_width(pi->pixbuf);
-		h = gdk_pixbuf_get_height(pi->pixbuf);
+		gint w = gdk_pixbuf_get_width(pi->pixbuf);
+		gint h = gdk_pixbuf_get_height(pi->pixbuf);
 
-		x1 = pi->x + pi->width - (pi->width - w) / 2 - 8;
-		y1 = pi->y + (pi->height - h) / 2 + 8;
-		}
-	else
-		{
-		x1 = pi->x + pi->width - 8;
-		y1 = pi->y + 8;
+		c1.x -= (pi->width - w) / 2;
+		c1.y += (pi->height - h) / 2;
 		}
 
-	x2 = pbox->x + 1;
-	y2 = pbox->y + 36;
-	x3 = pbox->x + 1;
-	y3 = pbox->y + 12;
-	util_clip_triangle(x1, y1, x2, y2, x3, y3,
-	                   x, y, w, h);
+	GdkPoint c2{pbox->x + 1, pbox->y + 36};
+	GdkPoint c3{pbox->x + 1, pbox->y + 12};
 
-	p = pan_item_tri_new(pw, nullptr, x, y, w, h,
-			     x1, y1, x2, y2, x3, y3,
-			     PAN_POPUP_COLOR);
-	pan_item_tri_border(p, PAN_BORDER_1 | PAN_BORDER_3, PAN_POPUP_BORDER_COLOR);
+	p = pan_item_tri_new(pw,
+	                     c1, c2, c3,
+	                     PAN_POPUP_COLOR,
+	                     PAN_BORDER_1 | PAN_BORDER_3, PAN_POPUP_BORDER_COLOR);
 	pan_item_set_key(p, "info");
 	pan_item_added(pw, p);
 
-	ta = pan_text_alignment_new(pw, pbox->x + PREF_PAD_BORDER, pbox->y + PREF_PAD_BORDER, "info");
-
-	pan_text_alignment_add(ta, _("Filename:"), pi->fd->name);
-	buf = remove_level_from_path(pi->fd->path);
-	pan_text_alignment_add(ta, _("Location:"), buf);
-	g_free(buf);
-	pan_text_alignment_add(ta, _("Date:"), text_from_time(pi->fd->date));
-	buf = text_from_size(pi->fd->size);
-	pan_text_alignment_add(ta, _("Size:"), buf);
-	g_free(buf);
-
-	if (pw->info_includes_exif)
-		{
-		pan_info_add_exif(ta, pi->fd);
-		}
-
-	pan_text_alignment_calc(ta, pbox);
-	pan_text_alignment_free(ta);
+	pan_info_calc_text_alignment(pw, pbox, pi->fd);
 
 	pan_item_box_shadow(pbox, PAN_SHADOW_OFFSET * 2, PAN_SHADOW_FADE * 2);
 	pan_item_added(pw, pbox);
@@ -1537,6 +1520,8 @@ void pan_info_update(PanWindow *pw, PanItem *pi)
 				case PAN_IMAGE_SIZE_100:
 					scale = 100;
 					break;
+				default:
+					break;
 				}
 
 			iw = MAX(1, iw * scale / 100);
@@ -1902,7 +1887,7 @@ static void pan_window_new_real(FileData *dir_fd)
 
 	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	DEBUG_NAME(vbox);
-	gtk_container_add(GTK_CONTAINER(pw->window), vbox);
+	gq_gtk_container_add(GTK_WIDGET(pw->window), vbox);
 	gtk_widget_show(vbox);
 
 	box = pref_box_new(vbox, FALSE, GTK_ORIENTATION_HORIZONTAL, PREF_PAD_SPACE);
@@ -1951,19 +1936,19 @@ static void pan_window_new_real(FileData *dir_fd)
 	vbox_imd_widget = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	hbox_imd_widget = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 
-	gtk_box_pack_start(GTK_BOX(vbox_imd_widget), pw->imd->widget, true, true, 0);
+	gq_gtk_box_pack_start(GTK_BOX(vbox_imd_widget), pw->imd->widget, true, true, 0);
 
 	pw->scrollbar_h = gtk_scrollbar_new(GTK_ORIENTATION_HORIZONTAL, nullptr);
 	g_signal_connect(G_OBJECT(pw->scrollbar_h), "value_changed",  G_CALLBACK(pan_window_scrollbar_h_value_cb), pw);
-	gtk_box_pack_start(GTK_BOX(vbox_imd_widget), pw->scrollbar_h, false, false, 0);
+	gq_gtk_box_pack_start(GTK_BOX(vbox_imd_widget), pw->scrollbar_h, false, false, 0);
 
-	gtk_box_pack_start(GTK_BOX(hbox_imd_widget), vbox_imd_widget, true, true, 0);
+	gq_gtk_box_pack_start(GTK_BOX(hbox_imd_widget), vbox_imd_widget, true, true, 0);
 
 	pw->scrollbar_v = gtk_scrollbar_new(GTK_ORIENTATION_VERTICAL, nullptr);
 	g_signal_connect(G_OBJECT(pw->scrollbar_v), "value_changed", G_CALLBACK(pan_window_scrollbar_v_value_cb), pw);
-	gtk_box_pack_start(GTK_BOX(hbox_imd_widget), pw->scrollbar_v, false, false, 0);
+	gq_gtk_box_pack_start(GTK_BOX(hbox_imd_widget), pw->scrollbar_v, false, false, 0);
 
-	gtk_box_pack_start(GTK_BOX(vbox), hbox_imd_widget, true, true, 0);
+	gq_gtk_box_pack_start(GTK_BOX(vbox), hbox_imd_widget, true, true, 0);
 
 	gtk_widget_show(GTK_WIDGET(hbox_imd_widget));
 	gtk_widget_show(GTK_WIDGET(pw->imd->widget));
@@ -1997,7 +1982,7 @@ static void pan_window_new_real(FileData *dir_fd)
 	gtk_widget_show(frame);
 
 	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, PREF_PAD_SPACE);
-	gtk_container_add(GTK_CONTAINER(frame), hbox);
+	gq_gtk_container_add(GTK_WIDGET(frame), hbox);
 	gtk_widget_show(hbox);
 
 	pref_spacer(hbox, 0);
@@ -2011,7 +1996,7 @@ static void pan_window_new_real(FileData *dir_fd)
 	gtk_widget_show(frame);
 
 	pw->label_zoom = gtk_label_new("");
-	gtk_container_add(GTK_CONTAINER(frame), pw->label_zoom);
+	gq_gtk_container_add(GTK_WIDGET(frame), pw->label_zoom);
 	gtk_widget_show(pw->label_zoom);
 
 	// Add the "Find" button to the status bar area.
@@ -2262,7 +2247,7 @@ static void pan_copy_path_cb(GtkWidget *, gpointer data)
 	FileData *fd;
 
 	fd = pan_menu_click_fd(pw);
-	if (fd) file_util_copy_path_to_clipboard(fd, TRUE);
+	if (fd) file_util_copy_path_to_clipboard(fd, TRUE, ClipboardAction::COPY);
 }
 
 static void pan_copy_path_unquoted_cb(GtkWidget *, gpointer data)
@@ -2271,7 +2256,7 @@ static void pan_copy_path_unquoted_cb(GtkWidget *, gpointer data)
 	FileData *fd;
 
 	fd = pan_menu_click_fd(pw);
-	if (fd) file_util_copy_path_to_clipboard(fd, FALSE);
+	if (fd) file_util_copy_path_to_clipboard(fd, FALSE, ClipboardAction::COPY);
 }
 
 static void pan_exif_date_toggle_cb(GtkWidget *widget, gpointer data)
@@ -2406,9 +2391,9 @@ static GtkWidget *pan_popup_menu(PanWindow *pw)
 				G_CALLBACK(pan_move_cb), pw);
 	menu_item_add_sensitive(menu, _("_Rename..."), active,
 				G_CALLBACK(pan_rename_cb), pw);
-	menu_item_add_sensitive(menu, _("_Copy path to clipboard"), active,
+	menu_item_add_sensitive(menu, _("_Copy to clipboard"), active,
 				G_CALLBACK(pan_copy_path_cb), pw);
-	menu_item_add_sensitive(menu, _("_Copy path unquoted to clipboard"), active,
+	menu_item_add_sensitive(menu, _("_Copy to clipboard (unquoted)"), active,
 				G_CALLBACK(pan_copy_path_unquoted_cb), pw);
 
 	menu_item_add_divider(menu);
@@ -2546,15 +2531,15 @@ static void pan_window_dnd_init(PanWindow *pw)
 	widget = pw->imd->pr;
 
 	gtk_drag_source_set(widget, GDK_BUTTON2_MASK,
-			    dnd_file_drag_types, dnd_file_drag_types_count,
-			    static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
+	                    dnd_file_drag_types.data(), dnd_file_drag_types.size(),
+	                    static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
 	g_signal_connect(G_OBJECT(widget), "drag_data_get",
 			 G_CALLBACK(pan_window_set_dnd_data), pw);
 
 	gtk_drag_dest_set(widget,
-			  static_cast<GtkDestDefaults>(GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP),
-			  dnd_file_drop_types, dnd_file_drop_types_count,
-			  static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
+	                  static_cast<GtkDestDefaults>(GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP),
+	                  dnd_file_drop_types.data(), dnd_file_drop_types.size(),
+	                  static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
 	g_signal_connect(G_OBJECT(widget), "drag_data_received",
 			 G_CALLBACK(pan_window_get_dnd_data), pw);
 }
