@@ -27,6 +27,7 @@
 
 #include "debug.h"
 #include "main-defines.h"
+#include "misc.h"
 #include "options.h"
 #include "renderer-tiles.h"
 
@@ -74,13 +75,6 @@ enum {
 	PR_SCROLLER_DEAD_ZONE = 6
 };
 
-/* when scaling image to below this size, use nearest pixel for scaling
- * (below about 4, the other scale types become slow generating their conversion tables)
- */
-//~ enum {
-//~ PR_MIN_SCALE_SIZE = 8
-//~ };
-
 enum {
 	SIGNAL_ZOOM = 0,
 	SIGNAL_CLICKED,
@@ -164,27 +158,20 @@ static void pixbuf_renderer_init_wrapper(PixbufRenderer *pr, void *)
 
 GType pixbuf_renderer_get_type()
 {
-	static GType pixbuf_renderer_type = 0;
-
-	if (!pixbuf_renderer_type)
-		{
-		static const GTypeInfo pixbuf_renderer_info =
-			{
-			sizeof(PixbufRendererClass), /* class_size */
-			nullptr,		/* base_init */
-			nullptr,		/* base_finalize */
-			static_cast<GClassInitFunc>(pixbuf_renderer_class_init_wrapper),
-			nullptr,		/* class_finalize */
-			nullptr,		/* class_data */
-			sizeof(PixbufRenderer), /* instance_size */
-			0,		/* n_preallocs */
-			reinterpret_cast<GInstanceInitFunc>(pixbuf_renderer_init_wrapper), /* instance_init */
-			nullptr,		/* value_table */
-			};
-
-		pixbuf_renderer_type = g_type_register_static(GTK_TYPE_EVENT_BOX, "PixbufRenderer",
-							      &pixbuf_renderer_info, static_cast<GTypeFlags>(0));
-		}
+	static const GTypeInfo pixbuf_renderer_info = {
+	    sizeof(PixbufRendererClass), /* class_size */
+	    nullptr,		/* base_init */
+	    nullptr,		/* base_finalize */
+	    static_cast<GClassInitFunc>(pixbuf_renderer_class_init_wrapper),
+	    nullptr,		/* class_finalize */
+	    nullptr,		/* class_data */
+	    sizeof(PixbufRenderer), /* instance_size */
+	    0,		/* n_preallocs */
+	    reinterpret_cast<GInstanceInitFunc>(pixbuf_renderer_init_wrapper), /* instance_init */
+	    nullptr,		/* value_table */
+	};
+	static GType pixbuf_renderer_type = g_type_register_static(GTK_TYPE_EVENT_BOX, "PixbufRenderer",
+	                                                           &pixbuf_renderer_info, static_cast<GTypeFlags>(0));
 
 	return pixbuf_renderer_type;
 }
@@ -246,14 +233,14 @@ static void pixbuf_renderer_class_init(PixbufRendererClass *renderer_class)
 							     FALSE,
 							     static_cast<GParamFlags>(G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	g_object_class_install_property(gobject_class,
-					PROP_SCROLL_RESET,
-					g_param_spec_uint("scroll_reset",
-							  "New image scroll reset",
-							  nullptr,
-							  PR_SCROLL_RESET_TOPLEFT,
-							  PR_SCROLL_RESET_NOCHANGE,
-							  PR_SCROLL_RESET_TOPLEFT,
-							  static_cast<GParamFlags>(G_PARAM_READABLE | G_PARAM_WRITABLE)));
+	                                PROP_SCROLL_RESET,
+	                                g_param_spec_uint("scroll_reset",
+	                                                  "New image scroll reset",
+	                                                  nullptr,
+	                                                  ScrollReset::TOPLEFT,
+	                                                  ScrollReset::NOCHANGE,
+	                                                  ScrollReset::TOPLEFT,
+	                                                  static_cast<GParamFlags>(G_PARAM_READABLE | G_PARAM_WRITABLE)));
 
 	g_object_class_install_property(gobject_class,
 					PROP_DELAY_FLIP,
@@ -433,7 +420,7 @@ static void pixbuf_renderer_init(PixbufRenderer *pr)
 	pr->scale = 1.0;
 	pr->aspect_ratio = 1.0;
 
-	pr->scroll_reset = PR_SCROLL_RESET_TOPLEFT;
+	pr->scroll_reset = ScrollReset::TOPLEFT;
 
 	pr->scroller_id = 0;
 	pr->scroller_overlay = -1;
@@ -514,7 +501,7 @@ static void pixbuf_renderer_set_property(GObject *object, guint prop_id,
 			pr->zoom_expand = g_value_get_boolean(value);
 			break;
 		case PROP_SCROLL_RESET:
-			pr->scroll_reset = static_cast<PixbufRendererScrollResetType>(g_value_get_uint(value));
+			pr->scroll_reset = static_cast<ScrollReset>(g_value_get_uint(value));
 			break;
 		case PROP_DELAY_FLIP:
 			pr->delay_flip = g_value_get_boolean(value);
@@ -626,50 +613,6 @@ static void pixbuf_renderer_get_property(GObject *object, guint prop_id,
  *-------------------------------------------------------------------
  */
 
-static void widget_set_cursor(GtkWidget *widget, gint icon)
-{
-	GdkCursor *cursor;
-	GdkDisplay *display;
-
-	if (!gtk_widget_get_window(widget)) return;
-
-	display = gdk_display_get_default();
-
-	if (icon == -1)
-		{
-		cursor = nullptr;
-		}
-	else
-		{
-		cursor = gdk_cursor_new_for_display(display, static_cast<GdkCursorType>(icon));
-		}
-
-	gdk_window_set_cursor(gtk_widget_get_window(widget), cursor);
-
-	if (cursor) g_object_unref(G_OBJECT(cursor));
-}
-
-gboolean pr_clip_region(gint x, gint y, gint w, gint h,
-			       gint clip_x, gint clip_y, gint clip_w, gint clip_h,
-			       gint *rx, gint *ry, gint *rw, gint *rh)
-{
-	if (clip_x + clip_w <= x ||
-	    clip_x >= x + w ||
-	    clip_y + clip_h <= y ||
-	    clip_y >= y + h)
-		{
-		return FALSE;
-		}
-
-	*rx = MAX(x, clip_x);
-	*rw = MIN((x + w), (clip_x + clip_w)) - *rx;
-
-	*ry = MAX(y, clip_y);
-	*rh = MIN((y + h), (clip_y + clip_h)) - *ry;
-
-	return TRUE;
-}
-
 static gboolean pr_parent_window_sizable(PixbufRenderer *pr)
 {
 	GdkWindowState state;
@@ -731,16 +674,6 @@ void pixbuf_renderer_set_parent(PixbufRenderer *pr, GtkWindow *window)
 	pr->parent_window = GTK_WIDGET(window);
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-GtkWindow *pixbuf_renderer_get_parent_unused(PixbufRenderer *pr)
-{
-	g_return_val_if_fail(IS_PIXBUF_RENDERER(pr), NULL);
-
-	return GTK_WINDOW(pr->parent_window);
-}
-#pragma GCC diagnostic pop
-
 
 /*
  *-------------------------------------------------------------------
@@ -931,6 +864,9 @@ static void pr_scroller_stop(PixbufRenderer *pr)
  *-------------------------------------------------------------------
  */
 
+/**
+ * @brief Background color
+ */
 void pixbuf_renderer_set_color(PixbufRenderer *pr, GdkRGBA *color)
 {
 	g_return_if_fail(IS_PIXBUF_RENDERER(pr));
@@ -1075,10 +1011,11 @@ static SourceTile *pr_source_tile_request(PixbufRenderer *pr, gint x, gint y)
 		st->blank = FALSE;
 		}
 
-	pr->renderer->invalidate_region(pr->renderer, st->x * pr->scale, st->y * pr->scale,
-				  pr->source_tile_width * pr->scale, pr->source_tile_height * pr->scale);
-	if (pr->renderer2) pr->renderer2->invalidate_region(pr->renderer2, st->x * pr->scale, st->y * pr->scale,
-				  pr->source_tile_width * pr->scale, pr->source_tile_height * pr->scale);
+	GdkRectangle rect{st->x, st->y, pr->source_tile_width, pr->source_tile_height};
+	pr_scale_region(rect, pr->scale);
+
+	pr->renderer->invalidate_region(pr->renderer, rect);
+	if (pr->renderer2) pr->renderer2->invalidate_region(pr->renderer2, rect);
 	return st;
 }
 
@@ -1142,42 +1079,40 @@ GList *pr_source_tile_compute_region(PixbufRenderer *pr, gint x, gint y, gint w,
 
 static void pr_source_tile_changed(PixbufRenderer *pr, gint x, gint y, gint width, gint height)
 {
-	GList *work;
-
 	if (width < 1 || height < 1) return;
 
-	work = pr->source_tiles;
-	while (work)
+	const GdkRectangle request_rect{x, y, width, height};
+	GdkRectangle st_rect{0, 0, pr->source_tile_width, pr->source_tile_height};
+	GdkRectangle r;
+
+	for (GList *work = pr->source_tiles; work; work = work->next)
 		{
-		SourceTile *st;
-		gint rx;
-		gint ry;
-		gint rw;
-		gint rh;
+		auto *st = static_cast<SourceTile *>(work->data);
 
-		st = static_cast<SourceTile *>(work->data);
-		work = work->next;
+		st_rect.x = st->x;
+		st_rect.y = st->y;
 
-		if (pr_clip_region(st->x, st->y, pr->source_tile_width, pr->source_tile_height,
-				   x, y, width, height,
-				   &rx, &ry, &rw, &rh))
+		if (gdk_rectangle_intersect(&st_rect, &request_rect, &r))
 			{
 			GdkPixbuf *pixbuf;
 
-			pixbuf = gdk_pixbuf_new_subpixbuf(st->pixbuf, rx - st->x, ry - st->y, rw, rh);
+			pixbuf = gdk_pixbuf_new_subpixbuf(st->pixbuf, r.x - st->x, r.y - st->y, r.width, r.height);
 			if (pr->func_tile_request &&
-			    pr->func_tile_request(pr, rx, ry, rw, rh, pixbuf, pr->func_tile_data))
+			    pr->func_tile_request(pr, r.x, r.y, r.width, r.height, pixbuf, pr->func_tile_data))
 				{
-					pr->renderer->invalidate_region(pr->renderer, rx * pr->scale, ry * pr->scale,
-							      rw * pr->scale, rh * pr->scale);
-					if (pr->renderer2) pr->renderer2->invalidate_region(pr->renderer2, rx * pr->scale, ry * pr->scale,
-								rw * pr->scale, rh * pr->scale);
+				pr_scale_region(r, pr->scale);
+
+				pr->renderer->invalidate_region(pr->renderer, r);
+				if (pr->renderer2) pr->renderer2->invalidate_region(pr->renderer2, r);
 				}
 			g_object_unref(pixbuf);
 			}
 		}
 }
 
+/**
+ * @brief Display an on-request array of pixbuf tiles
+ */
 void pixbuf_renderer_set_tiles(PixbufRenderer *pr, gint width, gint height,
 			       gint tile_width, gint tile_height, gint cache_size,
 			       PixbufRendererTileRequestFunc func_request,
@@ -1381,13 +1316,13 @@ static void pr_update_pixel_signal(PixbufRenderer *pr)
 
 
 void pr_tile_coords_map_orientation(gint orientation,
-				     gdouble tile_x, gdouble tile_y, /* coordinates of the tile */
-				     gdouble image_w, gdouble image_h,
-				     gdouble tile_w, gdouble tile_h,
-				     gdouble *res_x, gdouble *res_y)
+                                    gdouble tile_x, gdouble tile_y, /* coordinates of the tile */
+                                    gdouble image_w, gdouble image_h,
+                                    gdouble tile_w, gdouble tile_h,
+                                    gdouble &res_x, gdouble &res_y)
 {
-	*res_x = tile_x;
-	*res_y = tile_y;
+	res_x = tile_x;
+	res_y = tile_y;
 	switch (orientation)
 		{
 		case EXIF_ORIENTATION_TOP_LEFT:
@@ -1395,34 +1330,34 @@ void pr_tile_coords_map_orientation(gint orientation,
 			break;
 		case EXIF_ORIENTATION_TOP_RIGHT:
 			/* mirrored */
-			*res_x = image_w - tile_x - tile_w;
+			res_x = image_w - tile_x - tile_w;
 			break;
 		case EXIF_ORIENTATION_BOTTOM_RIGHT:
 			/* upside down */
-			*res_x = image_w - tile_x - tile_w;
-			*res_y = image_h - tile_y - tile_h;
+			res_x = image_w - tile_x - tile_w;
+			res_y = image_h - tile_y - tile_h;
 			break;
 		case EXIF_ORIENTATION_BOTTOM_LEFT:
 			/* flipped */
-			*res_y = image_h - tile_y - tile_h;
+			res_y = image_h - tile_y - tile_h;
 			break;
 		case EXIF_ORIENTATION_LEFT_TOP:
-			*res_x = tile_y;
-			*res_y = tile_x;
+			res_x = tile_y;
+			res_y = tile_x;
 			break;
 		case EXIF_ORIENTATION_RIGHT_TOP:
 			/* rotated -90 (270) */
-			*res_x = tile_y;
-			*res_y = image_w - tile_x - tile_w;
+			res_x = tile_y;
+			res_y = image_w - tile_x - tile_w;
 			break;
 		case EXIF_ORIENTATION_RIGHT_BOTTOM:
-			*res_x = image_h - tile_y - tile_h;
-			*res_y = image_w - tile_x - tile_w;
+			res_x = image_h - tile_y - tile_h;
+			res_y = image_w - tile_x - tile_w;
 			break;
 		case EXIF_ORIENTATION_LEFT_BOTTOM:
 			/* rotated 90 */
-			*res_x = image_h - tile_y - tile_h;
-			*res_y = tile_x;
+			res_x = image_h - tile_y - tile_h;
+			res_y = tile_x;
 			break;
 		default:
 			/* The other values are out of range */
@@ -1430,17 +1365,11 @@ void pr_tile_coords_map_orientation(gint orientation,
 		}
 }
 
-void pr_tile_region_map_orientation(gint orientation,
-				     gint area_x, gint area_y, /* coordinates of the area inside tile */
-				     gint tile_w, gint tile_h,
-				     gint area_w, gint area_h,
-				     gint *res_x, gint *res_y,
-				     gint *res_w, gint *res_h)
+GdkRectangle pr_tile_region_map_orientation(gint orientation,
+                                            GdkRectangle area, /* coordinates of the area inside tile */
+                                            gint tile_w, gint tile_h)
 {
-	*res_x = area_x;
-	*res_y = area_y;
-	*res_w = area_w;
-	*res_h = area_h;
+	GdkRectangle res = area;
 
 	switch (orientation)
 		{
@@ -1449,60 +1378,56 @@ void pr_tile_region_map_orientation(gint orientation,
 			break;
 		case EXIF_ORIENTATION_TOP_RIGHT:
 			/* mirrored */
-			*res_x = tile_w - area_x - area_w;
+			res.x = tile_w - area.x - area.width;
 			break;
 		case EXIF_ORIENTATION_BOTTOM_RIGHT:
 			/* upside down */
-			*res_x = tile_w - area_x - area_w;
-			*res_y = tile_h - area_y - area_h;
+			res.x = tile_w - area.x - area.width;
+			res.y = tile_h - area.y - area.height;
 			break;
 		case EXIF_ORIENTATION_BOTTOM_LEFT:
 			/* flipped */
-			*res_y = tile_h - area_y - area_h;
+			res.y = tile_h - area.y - area.height;
 			break;
 		case EXIF_ORIENTATION_LEFT_TOP:
-			*res_x = area_y;
-			*res_y = area_x;
-			*res_w = area_h;
-			*res_h = area_w;
+			res.x = area.y;
+			res.y = area.x;
+			res.width = area.height;
+			res.height = area.width;
 			break;
 		case EXIF_ORIENTATION_RIGHT_TOP:
 			/* rotated -90 (270) */
-			*res_x = area_y;
-			*res_y = tile_w - area_x - area_w;
-			*res_w = area_h;
-			*res_h = area_w;
+			res.x = area.y;
+			res.y = tile_w - area.x - area.width;
+			res.width = area.height;
+			res.height = area.width;
 			break;
 		case EXIF_ORIENTATION_RIGHT_BOTTOM:
-			*res_x = tile_h - area_y - area_h;
-			*res_y = tile_w - area_x - area_w;
-			*res_w = area_h;
-			*res_h = area_w;
+			res.x = tile_h - area.y - area.height;
+			res.y = tile_w - area.x - area.width;
+			res.width = area.height;
+			res.height = area.width;
 			break;
 		case EXIF_ORIENTATION_LEFT_BOTTOM:
 			/* rotated 90 */
-			*res_x = tile_h - area_y - area_h;
-			*res_y = area_x;
-			*res_w = area_h;
-			*res_h = area_w;
+			res.x = tile_h - area.y - area.height;
+			res.y = area.x;
+			res.width = area.height;
+			res.height = area.width;
 			break;
 		default:
 			/* The other values are out of range */
 			break;
 		}
+
+	return res;
 }
 
-void pr_coords_map_orientation_reverse(gint orientation,
-				     gint area_x, gint area_y,
-				     gint tile_w, gint tile_h,
-				     gint area_w, gint area_h,
-				     gint *res_x, gint *res_y,
-				     gint *res_w, gint *res_h)
+GdkRectangle pr_coords_map_orientation_reverse(gint orientation,
+                                               GdkRectangle area,
+                                               gint tile_w, gint tile_h)
 {
-	*res_x = area_x;
-	*res_y = area_y;
-	*res_w = area_w;
-	*res_h = area_h;
+	GdkRectangle res = area;
 
 	switch (orientation)
 		{
@@ -1511,47 +1436,57 @@ void pr_coords_map_orientation_reverse(gint orientation,
 			break;
 		case EXIF_ORIENTATION_TOP_RIGHT:
 			/* mirrored */
-			*res_x = tile_w - area_x - area_w;
+			res.x = tile_w - area.x - area.width;
 			break;
 		case EXIF_ORIENTATION_BOTTOM_RIGHT:
 			/* upside down */
-			*res_x = tile_w - area_x - area_w;
-			*res_y = tile_h - area_y - area_h;
+			res.x = tile_w - area.x - area.width;
+			res.y = tile_h - area.y - area.height;
 			break;
 		case EXIF_ORIENTATION_BOTTOM_LEFT:
 			/* flipped */
-			*res_y = tile_h - area_y - area_h;
+			res.y = tile_h - area.y - area.height;
 			break;
 		case EXIF_ORIENTATION_LEFT_TOP:
-			*res_x = area_y;
-			*res_y = area_x;
-			*res_w = area_h;
-			*res_h = area_w;
+			res.x = area.y;
+			res.y = area.x;
+			res.width = area.height;
+			res.height = area.width;
 			break;
 		case EXIF_ORIENTATION_RIGHT_TOP:
 			/* rotated -90 (270) */
-			*res_x = tile_w - area_y - area_h;
-			*res_y = area_x;
-			*res_w = area_h;
-			*res_h = area_w;
+			res.x = tile_w - area.y - area.height;
+			res.y = area.x;
+			res.width = area.height;
+			res.height = area.width;
 			break;
 		case EXIF_ORIENTATION_RIGHT_BOTTOM:
-			*res_x = tile_w - area_y - area_h;
-			*res_y = tile_h - area_x - area_w;
-			*res_w = area_h;
-			*res_h = area_w;
+			res.x = tile_w - area.y - area.height;
+			res.y = tile_h - area.x - area.width;
+			res.width = area.height;
+			res.height = area.width;
 			break;
 		case EXIF_ORIENTATION_LEFT_BOTTOM:
 			/* rotated 90 */
-			*res_x = area_y;
-			*res_y = tile_h - area_x - area_w;
-			*res_w = area_h;
-			*res_h = area_w;
+			res.x = area.y;
+			res.y = tile_h - area.x - area.width;
+			res.width = area.height;
+			res.height = area.width;
 			break;
 		default:
 			/* The other values are out of range */
 			break;
 		}
+
+	return res;
+}
+
+void pr_scale_region(GdkRectangle &region, gdouble scale)
+{
+	region.x *= scale;
+	region.y *= scale;
+	region.width *= scale;
+	region.height *= scale;
 }
 
 
@@ -1805,17 +1740,17 @@ static void pr_zoom_sync(PixbufRenderer *pr, gdouble zoom,
 		{
 		switch (pr->scroll_reset)
 			{
-			case PR_SCROLL_RESET_NOCHANGE:
+			case ScrollReset::NOCHANGE:
 				/* maintain old scroll position */
 				pr->x_scroll = (static_cast<gdouble>(pr->image_width) * old_center_x * pr->scale) - pr->vis_width / 2.0;
 				pr->y_scroll = (static_cast<gdouble>(pr->image_height) * old_center_y * pr->scale * pr->aspect_ratio) - pr->vis_height / 2.0;
 				break;
-			case PR_SCROLL_RESET_CENTER:
+			case ScrollReset::CENTER:
 				/* center new image */
 				pr->x_scroll = (static_cast<gdouble>(pr->image_width) / 2.0 * pr->scale) - pr->vis_width / 2.0;
 				pr->y_scroll = (static_cast<gdouble>(pr->image_height) / 2.0 * pr->scale * pr->aspect_ratio) - pr->vis_height / 2.0;
 				break;
-			case PR_SCROLL_RESET_TOPLEFT:
+			case ScrollReset::TOPLEFT:
 			default:
 				/* reset to upper left */
 				pr->x_scroll = 0;
@@ -2291,6 +2226,8 @@ static void pr_create_anaglyph_color(GdkPixbuf *pixbuf, GdkPixbuf *right, gint x
 					dp[0] = sp[0];
 					dp[1] = sp[1];
 					break;
+				default:
+					break;
 				}
 			sp += COLOR_BYTES;
 			dp += COLOR_BYTES;
@@ -2345,6 +2282,8 @@ static void pr_create_anaglyph_gray(GdkPixbuf *pixbuf, GdkPixbuf *right, gint x,
 					dp[1] = g2;
 					dp[2] = g1;
 					break;
+				default:
+					break;
 				}
 			sp += COLOR_BYTES;
 			dp += COLOR_BYTES;
@@ -2390,6 +2329,8 @@ static void pr_create_anaglyph_dubois(GdkPixbuf *pixbuf, GdkPixbuf *right, gint
 		case YB:
 			memcpy(pr_dubois_matrix, pr_dubois_matrix_YB, sizeof pr_dubois_matrix);
 			break;
+		default:
+			break;
 		}
 
 	srs = gdk_pixbuf_get_rowstride(right);
@@ -2516,7 +2457,7 @@ static void pr_set_pixbuf(PixbufRenderer *pr, GdkPixbuf *pixbuf, gdouble zoom, P
 
 	if (pr->stereo_mode & PR_STEREO_TEMP_DISABLE)
 		{
-		gint disable = !pr->pixbuf || ! pr->stereo_data;
+		gint disable = !pr->stereo_data;
 		pr_stereo_temp_disable(pr, disable);
 		}
 
@@ -2526,6 +2467,9 @@ static void pr_set_pixbuf(PixbufRenderer *pr, GdkPixbuf *pixbuf, gdouble zoom, P
 	pr_zoom_sync(pr, zoom, static_cast<PrZoomFlags>(flags | PR_ZOOM_FORCE | PR_ZOOM_NEW), 0, 0);
 }
 
+/**
+ * @brief Display a pixbuf
+ */
 void pixbuf_renderer_set_pixbuf(PixbufRenderer *pr, GdkPixbuf *pixbuf, gdouble zoom)
 {
 	g_return_if_fail(IS_PIXBUF_RENDERER(pr));
@@ -2537,6 +2481,9 @@ void pixbuf_renderer_set_pixbuf(PixbufRenderer *pr, GdkPixbuf *pixbuf, gdouble z
 	pr_update_signal(pr);
 }
 
+/**
+ * @brief Same as pixbuf_renderer_set_pixbuf but waits with redrawing for pixbuf_renderer_area_changed
+ */
 void pixbuf_renderer_set_pixbuf_lazy(PixbufRenderer *pr, GdkPixbuf *pixbuf, gdouble zoom, gint orientation, StereoPixbufData stereo_data)
 {
 	g_return_if_fail(IS_PIXBUF_RENDERER(pr));
@@ -2567,15 +2514,9 @@ void pixbuf_renderer_set_orientation(PixbufRenderer *pr, gint orientation)
 	pr_zoom_sync(pr, pr->zoom, PR_ZOOM_FORCE, 0, 0);
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-gint pixbuf_renderer_get_orientation_unused(PixbufRenderer *pr)
-{
-	if (!pr) return 1;
-	return pr->orientation;
-}
-#pragma GCC diagnostic pop
-
+/**
+ * @brief Sets the format of stereo data in the input pixbuf
+ */
 void pixbuf_renderer_set_stereo_data(PixbufRenderer *pr, StereoPixbufData stereo_data)
 {
 	g_return_if_fail(IS_PIXBUF_RENDERER(pr));
@@ -2605,11 +2546,13 @@ void pixbuf_renderer_set_post_process_func(PixbufRenderer *pr, PixbufRendererPos
 
 }
 
-
+/**
+ * @brief Move image data from source to pr, source is then set to NULL image
+ */
 void pixbuf_renderer_move(PixbufRenderer *pr, PixbufRenderer *source)
 {
 	GObject *object;
-	PixbufRendererScrollResetType scroll_reset;
+	ScrollReset scroll_reset;
 
 	g_return_if_fail(IS_PIXBUF_RENDERER(pr));
 	g_return_if_fail(IS_PIXBUF_RENDERER(source));
@@ -2630,7 +2573,7 @@ void pixbuf_renderer_move(PixbufRenderer *pr, PixbufRenderer *source)
 	pr->y_mouse  = source->y_mouse;
 
 	scroll_reset = pr->scroll_reset;
-	pr->scroll_reset = PR_SCROLL_RESET_NOCHANGE;
+	pr->scroll_reset = ScrollReset::NOCHANGE;
 
 	pr->func_post_process = source->func_post_process;
 	pr->post_process_user_data = source->post_process_user_data;
@@ -2671,7 +2614,7 @@ void pixbuf_renderer_move(PixbufRenderer *pr, PixbufRenderer *source)
 void pixbuf_renderer_copy(PixbufRenderer *pr, PixbufRenderer *source)
 {
 	GObject *object;
-	PixbufRendererScrollResetType scroll_reset;
+	ScrollReset scroll_reset;
 
 	g_return_if_fail(IS_PIXBUF_RENDERER(pr));
 	g_return_if_fail(IS_PIXBUF_RENDERER(source));
@@ -2692,7 +2635,7 @@ void pixbuf_renderer_copy(PixbufRenderer *pr, PixbufRenderer *source)
 	pr->y_mouse  = source->y_mouse;
 
 	scroll_reset = pr->scroll_reset;
-	pr->scroll_reset = PR_SCROLL_RESET_NOCHANGE;
+	pr->scroll_reset = ScrollReset::NOCHANGE;
 
 	pr->orientation = source->orientation;
 	pr->stereo_data = source->stereo_data;
@@ -2723,6 +2666,9 @@ void pixbuf_renderer_copy(PixbufRenderer *pr, PixbufRenderer *source)
 	pr->scroll_reset = scroll_reset;
 }
 
+/**
+ * @brief Update region of existing image
+ */
 void pixbuf_renderer_area_changed(PixbufRenderer *pr, gint x, gint y, gint w, gint h)
 {
 	g_return_if_fail(IS_PIXBUF_RENDERER(pr));
@@ -2846,11 +2792,6 @@ void pixbuf_renderer_stereo_fixed_set(PixbufRenderer *pr, gint width, gint heigh
 	pr->stereo_fixed_y_right = y2;
 }
 
-gint pixbuf_renderer_stereo_get(PixbufRenderer *pr)
-{
-	return pr->stereo_mode;
-}
-
 static void pr_stereo_temp_disable(PixbufRenderer *pr, gboolean disable)
 {
 	if (pr->stereo_temp_disable == disable) return;
@@ -2870,18 +2811,17 @@ static void pr_stereo_temp_disable(PixbufRenderer *pr, gboolean disable)
 	pr_size_sync(pr, pr->window_width, pr->window_height); /* recalculate new viewport */
 }
 
+/**
+ * @brief x_pixel and y_pixel are the pixel coordinates see #pixbuf_renderer_get_mouse_position
+ */
 gboolean pixbuf_renderer_get_pixel_colors(PixbufRenderer *pr, gint x_pixel, gint y_pixel,
-                                          gint *r_mouse, gint *g_mouse, gint *b_mouse)
+                                          gint *r_mouse, gint *g_mouse, gint *b_mouse, gint *a_mouse)
 {
 	GdkPixbuf *pb = pr->pixbuf;
 	gint p_alpha;
 	gint prs;
 	guchar *p_pix;
 	guchar *pp;
-	gint map_x;
-	gint map_y;
-	gint map_w;
-	gint map_h;
 	size_t xoff;
 	size_t yoff;
 
@@ -2893,27 +2833,25 @@ gboolean pixbuf_renderer_get_pixel_colors(PixbufRenderer *pr, gint x_pixel, gint
 		*r_mouse = -1;
 		*g_mouse = -1;
 		*b_mouse = -1;
+		*a_mouse = -1;
 		return FALSE;
 		}
 
 	if (!pb) return FALSE;
 
-	pr_tile_region_map_orientation(pr->orientation,
-					x_pixel, y_pixel,
-					pr->image_width, pr->image_height,
-					1, 1, /*single pixel */
-					&map_x, &map_y,
-					&map_w, &map_h);
+	GdkRectangle map_rect = pr_tile_region_map_orientation(pr->orientation,
+	                                                       {x_pixel, y_pixel, 1, 1}, /*single pixel */
+	                                                       pr->image_width, pr->image_height);
 
-	if (map_x < 0 || map_x > gdk_pixbuf_get_width(pr->pixbuf) - 1) return  FALSE;
-	if (map_y < 0 || map_y > gdk_pixbuf_get_height(pr->pixbuf) - 1) return  FALSE;
+	if (map_rect.x < 0 || map_rect.x > gdk_pixbuf_get_width(pr->pixbuf) - 1) return FALSE;
+	if (map_rect.y < 0 || map_rect.y > gdk_pixbuf_get_height(pr->pixbuf) - 1) return FALSE;
 
 	p_alpha = gdk_pixbuf_get_has_alpha(pb);
 	prs = gdk_pixbuf_get_rowstride(pb);
 	p_pix = gdk_pixbuf_get_pixels(pb);
 
-	xoff = static_cast<size_t>(map_x) * (p_alpha ? 4 : 3);
-	yoff = static_cast<size_t>(map_y) * prs;
+	xoff = static_cast<size_t>(map_rect.x) * (p_alpha ? 4 : 3);
+	yoff = static_cast<size_t>(map_rect.y) * prs;
 	pp = p_pix + yoff + xoff;
 	*r_mouse = *pp;
 	pp++;
@@ -2921,6 +2859,12 @@ gboolean pixbuf_renderer_get_pixel_colors(PixbufRenderer *pr, gint x_pixel, gint
 	pp++;
 	*b_mouse = *pp;
 
+	if (p_alpha)
+		{
+		pp++;
+		*a_mouse = *pp;
+		}
+
 	return TRUE;
 }
 
@@ -2997,6 +2941,9 @@ gboolean pixbuf_renderer_get_scaled_size(PixbufRenderer *pr, gint *width, gint *
 	return TRUE;
 }
 
+/**
+ * @brief Region of image in pixel coordinates
+ */
 gboolean pixbuf_renderer_get_visible_rect(PixbufRenderer *pr, GdkRectangle *rect)
 {
 	g_return_val_if_fail(IS_PIXBUF_RENDERER(pr), FALSE);
@@ -3019,30 +2966,6 @@ gboolean pixbuf_renderer_get_visible_rect(PixbufRenderer *pr, GdkRectangle *rect
 	return TRUE;
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-gboolean pixbuf_renderer_get_virtual_rect_unused(PixbufRenderer *pr, GdkRectangle *rect)
-{
-	g_return_val_if_fail(IS_PIXBUF_RENDERER(pr), FALSE);
-	g_return_val_if_fail(rect != nullptr, FALSE);
-
-	if ((!pr->pixbuf && !pr->source_tiles_enabled))
-		{
-		rect->x = 0;
-		rect->y = 0;
-		rect->width = 0;
-		rect->height = 0;
-		return FALSE;
-		}
-
-	rect->x = pr->x_scroll;
-	rect->y = pr->y_scroll;
-	rect->width = pr->vis_width;
-	rect->height = pr->vis_height;
-	return TRUE;
-}
-#pragma GCC diagnostic pop
-
 void pixbuf_renderer_set_size_early(PixbufRenderer *, guint, guint)
 {
 #if 0
