@@ -23,11 +23,11 @@
 #define PIXBUF_UTIL_H
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
+#include <gdk/gdk.h>
 #include <glib.h>
-#include <gtk/gtk.h>
 #include <pango/pango.h>
 
-struct FileData;
+class FileData;
 
 gboolean pixbuf_to_file_as_png (GdkPixbuf *pixbuf, const gchar *filename);
 
@@ -37,7 +37,7 @@ gboolean register_theme_icon_as_stock(const gchar *key, const gchar *icon);
 GdkPixbuf *pixbuf_inline(const gchar *key);
 GdkPixbuf *pixbuf_fallback(FileData *fd, gint requested_width, gint requested_height);
 
-gboolean pixbuf_scale_aspect(gint req_w, gint req_h, gint old_w, gint old_h, gint *new_w, gint *new_h);
+gboolean pixbuf_scale_aspect(gint req_w, gint req_h, gint old_w, gint old_h, gint &new_w, gint &new_h);
 
 #define PIXBUF_INLINE_ARCHIVE               "gq-icon-archive-file"
 #define PIXBUF_INLINE_BROKEN                "gq-icon-broken"
@@ -83,8 +83,8 @@ GdkPixbuf *pixbuf_copy_mirror(GdkPixbuf *src, gboolean mirror, gboolean flip);
 GdkPixbuf* pixbuf_apply_orientation(GdkPixbuf *pixbuf, gint orientation);
 
 void pixbuf_draw_rect_fill(GdkPixbuf *pb,
-			   gint x, gint y, gint w, gint h,
-			   gint r, gint g, gint b, gint a);
+                           GdkRectangle rect,
+                           gint r, gint g, gint b, gint a);
 
 void pixbuf_set_rect_fill(GdkPixbuf *pb,
 			  gint x, gint y, gint w, gint h,
@@ -93,45 +93,39 @@ void pixbuf_set_rect_fill(GdkPixbuf *pb,
 void pixbuf_set_rect(GdkPixbuf *pb,
 		     gint x, gint y, gint w, gint h,
 		     gint r, gint g, gint b, gint a,
-		     gint left, gint right, gint top, gint bottom);
+		     gint left_width, gint right_width, gint top_width, gint bottom_width);
 
 void pixbuf_pixel_set(GdkPixbuf *pb, gint x, gint y, gint r, gint g, gint b, gint a);
 
 
-void pixbuf_draw_layout(GdkPixbuf *pixbuf, PangoLayout *layout, GtkWidget *widget,
-			gint x, gint y,
-			guint8 r, guint8 g, guint8 b, guint8 a);
+void pixbuf_draw_layout(GdkPixbuf *pixbuf, PangoLayout *layout,
+                        gint x, gint y,
+                        guint8 r, guint8 g, guint8 b, guint8 a);
 
+void pixbuf_draw_triangle(GdkPixbuf *pb, GdkRectangle clip,
+                          GdkPoint c1, GdkPoint c2, GdkPoint c3,
+                          guint8 r, guint8 g, guint8 b, guint8 a);
 
-void pixbuf_draw_triangle(GdkPixbuf *pb,
-			  gint clip_x, gint clip_y, gint clip_w, gint clip_h,
-			  gint x1, gint y1, gint x2, gint y2, gint x3, gint y3,
-			  guint8 r, guint8 g, guint8 b, guint8 a);
+void pixbuf_draw_line(GdkPixbuf *pb, GdkRectangle clip,
+                      gint x1, gint y1, gint x2, gint y2,
+                      guint8 r, guint8 g, guint8 b, guint8 a);
 
-void pixbuf_draw_line(GdkPixbuf *pb,
-		      gint clip_x, gint clip_y, gint clip_w, gint clip_h,
-		      gint x1, gint y1, gint x2, gint y2,
-		      guint8 r, guint8 g, guint8 b, guint8 a);
-
-void pixbuf_draw_shadow(GdkPixbuf *pb,
-			gint clip_x, gint clip_y, gint clip_w, gint clip_h,
-			gint x, gint y, gint w, gint h, gint border,
-			guint8 r, guint8 g, guint8 b, guint8 a);
+void pixbuf_draw_shadow(GdkPixbuf *pb, GdkRectangle clip,
+                        gint x, gint y, gint w, gint h, gint border,
+                        guint8 r, guint8 g, guint8 b, guint8 a);
 
 void pixbuf_desaturate_rect(GdkPixbuf *pb,
 			    gint x, gint y, gint w, gint h);
+
 void pixbuf_highlight_overunderexposed(GdkPixbuf *pb,
-			    gint x, gint y, gint w, gint h);
+				       gint x, gint y, gint w, gint h);
+
 void pixbuf_ignore_alpha_rect(GdkPixbuf *pb,
-                 gint x, gint y, gint w, gint h);
+			      gint x, gint y, gint w, gint h);
 
 /* clipping utils */
 
-gboolean util_clip_region(gint x, gint y, gint w, gint h,
-		          gint clip_x, gint clip_y, gint clip_w, gint clip_h,
-		          gint *rx, gint *ry, gint *rw, gint *rh);
-void util_clip_triangle(gint x1, gint y1, gint x2, gint y2, gint x3, gint y3,
-                        gint &rx, gint &ry, gint &rw, gint &rh);
+GdkRectangle util_triangle_bounding_box(GdkPoint c1, GdkPoint c2, GdkPoint c3);
 
 
 #endif
