@@ -21,6 +21,7 @@
 #include "print.h"
 
 #include <cstddef>
+#include <cstring>
 
 #include <cairo.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
@@ -54,18 +55,6 @@
 namespace
 {
 
-constexpr gint PRE_FORMATTED_COLUMNS = 4;
-
-} // namespace
-
-/* reverse order is important */
-enum TextPosition {
-	FOOTER_2,
-	FOOTER_1,
-	HEADER_2,
-	HEADER_1
-};
-
 struct PrintWindow
 {
 	GtkWidget *vbox;
@@ -83,7 +72,9 @@ struct PrintWindow
 	GSList *page_group;
 };
 
-static gint print_layout_page_count(PrintWindow *pw)
+constexpr gint PRE_FORMATTED_COLUMNS = 4;
+
+gint print_layout_page_count(PrintWindow *pw)
 {
 	gint images;
 
@@ -94,9 +85,9 @@ static gint print_layout_page_count(PrintWindow *pw)
 	return images;
 }
 
-static gboolean print_job_render_image(PrintWindow *pw);
+gboolean print_job_render_image(PrintWindow *pw);
 
-static void print_job_render_image_loader_done(ImageLoader *il, gpointer data)
+void print_job_render_image_loader_done(ImageLoader *il, gpointer data)
 {
 	auto pw = static_cast<PrintWindow *>(data);
 	GdkPixbuf *pixbuf;
@@ -117,7 +108,7 @@ static void print_job_render_image_loader_done(ImageLoader *il, gpointer data)
 		}
 }
 
-static gboolean print_job_render_image(PrintWindow *pw)
+gboolean print_job_render_image(PrintWindow *pw)
 {
 	FileData *fd = nullptr;
 
@@ -141,7 +132,7 @@ static gboolean print_job_render_image(PrintWindow *pw)
 }
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wunused-but-set-parameter"
-static void font_activated_cb(GtkFontChooser *widget, gchar *fontname, gpointer option)
+void font_activated_cb(GtkFontChooser *widget, gchar *fontname, gpointer option)
 {
 	option = g_strdup(fontname);
 
@@ -151,7 +142,7 @@ static void font_activated_cb(GtkFontChooser *widget, gchar *fontname, gpointer
 }
 #pragma GCC diagnostic pop
 
-static void font_response_cb(GtkDialog *dialog, int response_id, gpointer option)
+void font_response_cb(GtkDialog *dialog, int response_id, gpointer option)
 {
 	gchar *font;
 
@@ -166,7 +157,7 @@ static void font_response_cb(GtkDialog *dialog, int response_id, gpointer option
 	gq_gtk_widget_destroy(GTK_WIDGET(dialog));
 }
 
-static void print_set_font_cb(GtkWidget *widget, gpointer data)
+void print_set_font_cb(GtkWidget *widget, gpointer data)
 {
 	gpointer option;
 	GtkWidget *dialog;
@@ -190,7 +181,7 @@ static void print_set_font_cb(GtkWidget *widget, gpointer data)
 	gtk_widget_show(dialog);
 }
 
-static gint set_toggle(GSList *list, TextPosition pos)
+gint set_toggle(GSList *list, TextPosition pos)
 {
 	GtkToggleButton *current_sel;
 	GtkToggleButton *new_sel;
@@ -210,135 +201,75 @@ static gint set_toggle(GSList *list, TextPosition pos)
 	return new_pos;
 }
 
-static void image_text_position_h1_cb(GtkWidget *widget, gpointer data)
+void image_text_position_cb(GtkWidget *widget, gpointer data, TextPosition pos)
 {
-	auto pw = static_cast<PrintWindow *>(data);
-	gint new_set;
+	if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) return;
 
-	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
+	auto pw = static_cast<PrintWindow *>(data);
+	gint new_set = set_toggle(pw->page_group, pos);
+	if (new_set >= 0)
 		{
-		new_set = set_toggle(pw->page_group, HEADER_1);
-		if (new_set >= 0)
-			{
-			options->printer.page_text_position = new_set;
-			}
-		options->printer.image_text_position = HEADER_1;
+		options->printer.page_text_position = static_cast<TextPosition>(new_set);
 		}
+
+	options->printer.image_text_position = pos;
 }
 
-static void image_text_position_h2_cb(GtkWidget *widget, gpointer data)
+void image_text_position_h1_cb(GtkWidget *widget, gpointer data)
 {
-	auto pw = static_cast<PrintWindow *>(data);
-	gint new_set;
-
-	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
-		{
-		new_set = set_toggle(pw->page_group, HEADER_2);
-		if (new_set >= 0)
-			{
-			options->printer.page_text_position = new_set;
-			}
-		options->printer.image_text_position = HEADER_2;
-		}
+	image_text_position_cb(widget, data, HEADER_1);
 }
 
-static void image_text_position_f1_cb(GtkWidget *widget, gpointer data)
+void image_text_position_h2_cb(GtkWidget *widget, gpointer data)
 {
-	auto pw = static_cast<PrintWindow *>(data);
-	gint new_set;
-
-	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
-		{
-		new_set = set_toggle(pw->page_group, FOOTER_1);
-		if (new_set >= 0)
-			{
-			options->printer.page_text_position = new_set;
-			}
-		options->printer.image_text_position = FOOTER_1;
-		}
+	image_text_position_cb(widget, data, HEADER_2);
 }
 
-static void image_text_position_f2_cb(GtkWidget *widget, gpointer data)
+void image_text_position_f1_cb(GtkWidget *widget, gpointer data)
 {
-	auto pw = static_cast<PrintWindow *>(data);
-	gint new_set;
-
-	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
-		{
-		new_set = set_toggle(pw->page_group, FOOTER_2);
-		if (new_set >= 0)
-			{
-			options->printer.page_text_position = new_set;
-			}
-		options->printer.image_text_position = FOOTER_2;
-		}
+	image_text_position_cb(widget, data, FOOTER_1);
 }
 
-static void page_text_position_h1_cb(GtkWidget *widget, gpointer data)
+void image_text_position_f2_cb(GtkWidget *widget, gpointer data)
 {
-	auto pw = static_cast<PrintWindow *>(data);
-	gint new_set;
-
-	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
-		{
-		new_set = set_toggle(pw->image_group, HEADER_1);
-		if (new_set >= 0)
-			{
-			options->printer.image_text_position = new_set;
-			}
-		options->printer.page_text_position = HEADER_1;
-		}
+	image_text_position_cb(widget, data, FOOTER_2);
 }
 
-static void page_text_position_h2_cb(GtkWidget *widget, gpointer data)
+void page_text_position_cb(GtkWidget *widget, gpointer data, TextPosition pos)
 {
-	auto pw = static_cast<PrintWindow *>(data);
-	gint new_set;
+	if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) return;
 
-	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
+	auto pw = static_cast<PrintWindow *>(data);
+	gint new_set = set_toggle(pw->image_group, pos);
+	if (new_set >= 0)
 		{
-		new_set = set_toggle(pw->image_group, HEADER_2);
-		if (new_set >= 0)
-			{
-			options->printer.image_text_position = new_set;
-			}
-		options->printer.page_text_position = HEADER_2;
+		options->printer.image_text_position = static_cast<TextPosition>(new_set);
 		}
+
+	options->printer.page_text_position = pos;
 }
 
-static void page_text_position_f1_cb(GtkWidget *widget, gpointer data)
+void page_text_position_h1_cb(GtkWidget *widget, gpointer data)
 {
-	auto pw = static_cast<PrintWindow *>(data);
-	gint new_set;
-
-	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
-		{
-		new_set = set_toggle(pw->image_group, FOOTER_1);
-		if (new_set >= 0)
-			{
-			options->printer.image_text_position = new_set;
-			}
-		options->printer.page_text_position = FOOTER_1;
-		}
+	page_text_position_cb(widget, data, HEADER_1);
 }
 
-static void page_text_position_f2_cb(GtkWidget *widget, gpointer data)
+void page_text_position_h2_cb(GtkWidget *widget, gpointer data)
 {
-	auto pw = static_cast<PrintWindow *>(data);
-	gint new_set;
-
-	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
-		{
-		new_set = set_toggle(pw->image_group, FOOTER_2);
-		if (new_set >= 0)
-			{
-			options->printer.image_text_position = new_set;
-			}
-		options->printer.page_text_position = FOOTER_2;
-		}
+	page_text_position_cb(widget, data, HEADER_2);
 }
 
-static void set_print_image_text_string(gchar **template_string, const gchar *value)
+void page_text_position_f1_cb(GtkWidget *widget, gpointer data)
+{
+	page_text_position_cb(widget, data, FOOTER_1);
+}
+
+void page_text_position_f2_cb(GtkWidget *widget, gpointer data)
+{
+	page_text_position_cb(widget, data, FOOTER_2);
+}
+
+void set_print_image_text_string(gchar **template_string, const gchar *value)
 {
 	g_assert(template_string);
 
@@ -346,7 +277,7 @@ static void set_print_image_text_string(gchar **template_string, const gchar *va
 	*template_string = g_strdup(value);
 }
 
-static void image_text_template_view_changed_cb(GtkWidget *, gpointer data)
+void image_text_template_view_changed_cb(GtkWidget *, gpointer data)
 {
 	GtkWidget *pTextView;
 	GtkTextBuffer *pTextBuffer;
@@ -363,7 +294,7 @@ static void image_text_template_view_changed_cb(GtkWidget *, gpointer data)
 					  gtk_text_buffer_get_text(pTextBuffer, &iStart, &iEnd, TRUE));
 }
 
-static void print_text_menu(GtkWidget *box, PrintWindow *pw)
+void print_text_menu(GtkWidget *box, PrintWindow *pw)
 {
 	GtkWidget *group;
 	GtkWidget *hbox;
@@ -496,21 +427,15 @@ static void print_text_menu(GtkWidget *box, PrintWindow *pw)
 	gtk_widget_show(button);
 }
 
-static gboolean paginate_cb(GtkPrintOperation *, GtkPrintContext *, gpointer data)
+gboolean paginate_cb(GtkPrintOperation *, GtkPrintContext *, gpointer data)
 {
 	auto pw = static_cast<PrintWindow *>(data);
 
-	if (pw->job_render_finished)
-		{
-		return TRUE;
-		}
-
-	return FALSE;
+	return pw->job_render_finished;
 }
 
 gchar *form_image_text(const gchar *template_string, FileData *fd, PrintWindow *pw, gint page_nr, gint total)
 {
-	const gchar *name;
 	gchar *text = nullptr;
 	GHashTable *vars;
 	gchar *window_title;
@@ -519,8 +444,6 @@ gchar *form_image_text(const gchar *template_string, FileData *fd, PrintWindow *
 
 	if (!fd) return nullptr;
 
-	name = fd->name;
-
 	vars = g_hash_table_new_full(g_str_hash, g_str_equal, nullptr, g_free);
 
 	window_title = g_strdup(gtk_window_get_title(GTK_WINDOW(pw->parent)));
@@ -542,9 +465,9 @@ gchar *form_image_text(const gchar *template_string, FileData *fd, PrintWindow *
 
 	osd_template_insert(vars, "number", g_strdup_printf("%d", page_nr + 1), OSDT_NO_DUP);
 	osd_template_insert(vars, "total", g_strdup_printf("%d", total), OSDT_NO_DUP);
-	osd_template_insert(vars, "name", const_cast<gchar *>(name), OSDT_NONE);
-	osd_template_insert(vars, "date", fd ? (const_cast<gchar *>(text_from_time(fd->date))) : "", OSDT_NONE);
-	osd_template_insert(vars, "size", fd ? (text_from_size_abrev(fd->size)) : g_strdup(""), OSDT_FREE);
+	osd_template_insert(vars, "name", fd->name, OSDT_NONE);
+	osd_template_insert(vars, "date", text_from_time(fd->date), OSDT_NONE);
+	osd_template_insert(vars, "size", text_from_size_abrev(fd->size), OSDT_FREE);
 
 	if (fd->pixbuf)
 		{
@@ -572,7 +495,16 @@ gchar *form_image_text(const gchar *template_string, FileData *fd, PrintWindow *
 	return text;
 }
 
-static void draw_page(GtkPrintOperation *, GtkPrintContext *context, gint page_nr, gpointer data)
+gchar *print_get_page_text(const PrintWindow *pw)
+{
+	GtkTextIter start;
+	GtkTextIter end;
+	gtk_text_buffer_get_bounds(pw->page_text, &start, &end);
+
+	return gtk_text_buffer_get_text(pw->page_text, &start, &end, FALSE);
+}
+
+void draw_page(GtkPrintOperation *, GtkPrintContext *context, gint page_nr, gpointer data)
 {
 	auto pw = static_cast<PrintWindow *>(data);
 	FileData *fd;
@@ -587,30 +519,17 @@ static void draw_page(GtkPrintOperation *, GtkPrintContext *context, gint page_n
 	GdkPixbuf *rotated = nullptr;
 	PangoLayout *layout_image = nullptr;
 	PangoLayout *layout_page = nullptr;
-	PangoFontDescription *desc;
-	GString *image_text = g_string_new(nullptr);
-	GString *page_text = g_string_new(nullptr);
-	PangoRectangle ink_rect;
-	PangoRectangle logical_rect;
 	gdouble w;
 	gdouble h;
 	gdouble scale;
 	gdouble image_text_width;
-	gdouble image_text_height;
 	gdouble page_text_width;
-	gdouble page_text_height;
 	gint image_y;
-	gint incr_y;
 	gdouble pango_height;
 	gdouble pango_image_height;
 	gdouble pango_page_height;
-	GtkTextIter start;
-	GtkTextIter end;
-	gchar *tmp;
-	gint total;
 
 	fd = static_cast<FileData *>(g_list_nth_data(pw->source_selection, page_nr));
-	total = g_list_length(pw->source_selection);
 
 	pixbuf = static_cast<GdkPixbuf *>(g_list_nth_data(pw->print_pixbuf_queue, page_nr));
 	if (fd->exif_orientation != EXIF_ORIENTATION_TOP_LEFT)
@@ -622,68 +541,54 @@ static void draw_page(GtkPrintOperation *, GtkPrintContext *context, gint page_n
 	pixbuf_image_width = gdk_pixbuf_get_width(pixbuf);
 	pixbuf_image_height = gdk_pixbuf_get_height(pixbuf);
 
-	if (options->printer.show_image_text)
-		{
-		image_text = g_string_append(image_text, form_image_text(options->printer.template_string, fd, pw, page_nr, total));
-		}
-
-	if (options->printer.show_page_text)
-		{
-		gtk_text_buffer_get_bounds(GTK_TEXT_BUFFER(pw->page_text), &start, &end);
-
-		tmp = gtk_text_buffer_get_text(GTK_TEXT_BUFFER(pw->page_text), &start, &end, FALSE);
-		page_text = g_string_append(page_text, tmp);
-
-		g_free(tmp);
-		}
-
 	cr = gtk_print_context_get_cairo_context(context);
 	context_width = gtk_print_context_get_width(context);
 	context_height = gtk_print_context_get_height(context);
 
+	const auto create_layout = [cr](const gchar *text, const gchar *font, gdouble &text_width, gdouble &pango_height) -> PangoLayout *
+	{
+		if (!text) return nullptr;
+
+		size_t text_len = strlen(text);
+		if (text_len == 0) return nullptr;
+
+		PangoLayout *layout = pango_cairo_create_layout(cr);
+
+		pango_layout_set_text(layout, text, text_len);
+
+		g_autoptr(PangoFontDescription) desc = pango_font_description_from_string(font);
+		pango_layout_set_font_description(layout, desc);
+
+		PangoRectangle ink_rect;
+		PangoRectangle logical_rect;
+		pango_layout_get_extents(layout, &ink_rect, &logical_rect);
+		text_width = static_cast<gdouble>(logical_rect.width) / PANGO_SCALE;
+		pango_height = static_cast<gdouble>(logical_rect.height) / PANGO_SCALE + PRINT_TEXT_PADDING * 2;
+
+		pango_layout_set_alignment(layout, PANGO_ALIGN_CENTER);
+		pango_layout_set_text(layout, text, text_len);
+
+		return layout;
+	};
+
 	pango_image_height = 0;
 	pango_page_height = 0;
 	image_text_width = 0;
 	page_text_width = 0;
 
-	if (image_text->len > 0)
+	if (options->printer.show_image_text)
 		{
-		layout_image = pango_cairo_create_layout(cr);
+		gint total = g_list_length(pw->source_selection);
+		g_autofree gchar *image_text = form_image_text(options->printer.template_string, fd, pw, page_nr, total);
 
-		pango_layout_set_text(layout_image, image_text->str, -1);
-		desc = pango_font_description_from_string(options->printer.image_font);
-		pango_layout_set_font_description(layout_image, desc);
-
-		pango_layout_get_extents(layout_image, &ink_rect, &logical_rect);
-		image_text_width = (static_cast<gdouble>(logical_rect.width) / PANGO_SCALE) ;
-		image_text_height = (static_cast<gdouble>(logical_rect.height) / PANGO_SCALE);
-
-		pango_layout_set_alignment(layout_image, PANGO_ALIGN_CENTER);
-		pango_layout_set_text(layout_image, image_text->str, -1);
-
-		pango_image_height = image_text_height + PRINT_TEXT_PADDING * 2;
-
-		pango_font_description_free(desc);
+		layout_image = create_layout(image_text, options->printer.image_font, image_text_width, pango_image_height);
 		}
 
-	if (page_text->len > 0)
+	if (options->printer.show_page_text)
 		{
-		layout_page = pango_cairo_create_layout(cr);
+		g_autofree gchar *page_text = print_get_page_text(pw);
 
-		pango_layout_set_text(layout_page, page_text->str, -1);
-		desc = pango_font_description_from_string(options->printer.page_font);
-		pango_layout_set_font_description(layout_page, desc);
-
-		pango_layout_get_extents(layout_page, &ink_rect, &logical_rect);
-		page_text_width = (static_cast<gdouble>(logical_rect.width) / PANGO_SCALE) ;
-		page_text_height = (static_cast<gdouble>(logical_rect.height) / PANGO_SCALE);
-
-		pango_layout_set_alignment(layout_page, PANGO_ALIGN_CENTER);
-		pango_layout_set_text(layout_page, page_text->str, -1);
-
-		pango_page_height = page_text_height + PRINT_TEXT_PADDING * 2;
-
-		pango_font_description_free(desc);
+		layout_page = create_layout(page_text, options->printer.page_font, page_text_width, pango_page_height);
 		}
 
 	pango_height = pango_image_height + pango_page_height;
@@ -705,69 +610,58 @@ static void draw_page(GtkPrintOperation *, GtkPrintContext *context, gint page_n
 		width_offset = (context_width - (pixbuf_image_width * scale)) / 2;
 		}
 
-	incr_y = height_offset;
+	image_y = height_offset;
 
-	if (options->printer.page_text_position == HEADER_1 && page_text->len > 0)
+	if (layout_page)
 		{
+		gint incr_y = height_offset;
+
+		if (layout_image && options->printer.page_text_position < options->printer.image_text_position)
+			{
+			incr_y += pango_image_height;
+			}
+
+		if (options->printer.page_text_position < HEADER_2)
+			{
+			incr_y += h;
+			}
+		else
+			{
+			image_y += pango_page_height;
+			}
+
+		if (options->printer.page_text_position == FOOTER_1)
+			{
+			incr_y += PRINT_TEXT_PADDING;
+			}
+
 		cairo_move_to(cr, (w / 2) - (page_text_width / 2) + width_offset, incr_y);
 		pango_cairo_show_layout(cr, layout_page);
-
-		incr_y = incr_y + pango_page_height;
 		}
 
-	if (options->printer.image_text_position == HEADER_1 && image_text->len > 0)
+	if (layout_image)
 		{
-		cairo_move_to(cr, (w / 2) - (image_text_width / 2) + width_offset, incr_y + PRINT_TEXT_PADDING);
-		pango_cairo_show_layout(cr, layout_image);
+		gint incr_y = height_offset;
 
-		incr_y = incr_y + pango_image_height;
-		}
+		if (layout_page && options->printer.image_text_position <= options->printer.page_text_position)
+			{
+			incr_y += pango_page_height;
+			}
 
-	if (options->printer.page_text_position == HEADER_2 && page_text->len > 0)
-		{
-		cairo_move_to(cr, (w / 2) - (page_text_width / 2) + width_offset, incr_y);
-		pango_cairo_show_layout(cr, layout_page);
+		if (options->printer.image_text_position < HEADER_2)
+			{
+			incr_y += h;
+			}
+		else
+			{
+			image_y += pango_image_height;
+			}
 
-		incr_y = incr_y + pango_page_height;
-		}
+		if (options->printer.image_text_position == HEADER_1)
+			{
+			incr_y += PRINT_TEXT_PADDING;
+			}
 
-	if (options->printer.image_text_position == HEADER_2 && image_text->len > 0)
-		{
-		cairo_move_to(cr, (w / 2) - (image_text_width / 2) + width_offset, incr_y);
-		pango_cairo_show_layout(cr, layout_image);
-
-		incr_y = incr_y + pango_image_height;
-		}
-
-	image_y = incr_y;
-	incr_y = incr_y + h;
-
-	if (options->printer.page_text_position == FOOTER_1 && page_text->len > 0)
-		{
-		cairo_move_to(cr, (w / 2) - (page_text_width / 2) + width_offset, incr_y + PRINT_TEXT_PADDING);
-		pango_cairo_show_layout(cr, layout_page);
-
-		incr_y = incr_y + pango_page_height;
-		}
-
-	if (options->printer.image_text_position == FOOTER_1 && image_text->len > 0)
-		{
-		cairo_move_to(cr, (w / 2) - (image_text_width / 2) + width_offset, incr_y);
-		pango_cairo_show_layout(cr, layout_image);
-
-		incr_y = incr_y + pango_image_height;
-		}
-
-	if (options->printer.page_text_position == FOOTER_2 && page_text->len > 0)
-		{
-		cairo_move_to(cr, (w / 2) - (page_text_width / 2) + width_offset, incr_y);
-		pango_cairo_show_layout(cr, layout_page);
-
-		incr_y = incr_y + pango_page_height;
-		}
-
-	if (options->printer.image_text_position == FOOTER_2 && image_text->len > 0)
-		{
 		cairo_move_to(cr, (w / 2) - (image_text_width / 2) + width_offset, incr_y);
 		pango_cairo_show_layout(cr, layout_image);
 		}
@@ -778,21 +672,12 @@ static void draw_page(GtkPrintOperation *, GtkPrintContext *context, gint page_n
 	gdk_cairo_set_source_pixbuf(cr, pixbuf, width_offset / scale, image_y / scale);
 	cairo_fill(cr);
 
-	if (image_text->len > 0)
-		{
-		g_object_unref(layout_image);
-		g_string_free(image_text, TRUE);
-		}
-	if (page_text->len > 0)
-		{
-		g_object_unref(layout_page);
-		g_string_free(page_text, TRUE);
-		}
-
+	if (layout_image) g_object_unref(layout_image);
+	if (layout_page) g_object_unref(layout_page);
 	if (rotated) g_object_unref(rotated);
 }
 
-static void begin_print(GtkPrintOperation *operation, GtkPrintContext *, gpointer user_data)
+void begin_print(GtkPrintOperation *operation, GtkPrintContext *, gpointer user_data)
 {
 	auto pw = static_cast<PrintWindow *>(user_data);
 	gint page_count;
@@ -811,20 +696,7 @@ GObject *option_tab_cb(GtkPrintOperation *, gpointer user_data)
 	return G_OBJECT(pw->vbox);
 }
 
-static void print_pref_store(PrintWindow *pw)
-{
-	gchar *tmp;
-	GtkTextIter start;
-	GtkTextIter end;
-
-	gtk_text_buffer_get_bounds(GTK_TEXT_BUFFER(pw->page_text), &start, &end);
-	tmp = gtk_text_buffer_get_text(GTK_TEXT_BUFFER(pw->page_text), &start, &end, FALSE);
-	g_free(options->printer.page_text);
-	options->printer.page_text = g_strdup(tmp);
-	g_free(tmp);
-}
-
-static void end_print_cb(GtkPrintOperation *operation, GtkPrintContext *, gpointer data)
+void end_print_cb(GtkPrintOperation *operation, GtkPrintContext *, gpointer data)
 {
 	auto pw = static_cast<PrintWindow *>(data);
 	GList *work;
@@ -860,7 +732,8 @@ static void end_print_cb(GtkPrintOperation *operation, GtkPrintContext *, gpoint
 	g_free(path);
 	g_object_unref(page_setup);
 
-	print_pref_store(pw);
+	g_free(options->printer.page_text);
+	options->printer.page_text = print_get_page_text(pw);
 
 	work = pw->print_pixbuf_queue;
 	while (work)
@@ -877,11 +750,16 @@ static void end_print_cb(GtkPrintOperation *operation, GtkPrintContext *, gpoint
 	g_free(pw);
 }
 
-static void print_response_cb(GtkDialog *dialog, gint, gpointer)
+void print_response_cb(GtkDialog *dialog, gint, gpointer)
 {
 	gq_gtk_widget_destroy(GTK_WIDGET(dialog));
 }
 
+} // namespace
+
+/**
+ * @brief Do not free selection or list, the print window takes control of them
+ */
 void print_window_new(FileData *, GList *selection, GList *, GtkWidget *parent)
 {
 	GtkWidget *vbox;
