@@ -25,12 +25,16 @@
 #include <glib.h>
 #include <gtk/gtk.h>
 
-struct FileData;
+class FileData;
+
+/* reverse order is important */
+enum TextPosition : gint {
+	FOOTER_2,
+	FOOTER_1,
+	HEADER_2,
+	HEADER_1,
+};
 
-/**
- * @headerfile print_window_new
- * do not free selection or list, the print window takes control of them
- */
 void print_window_new(FileData *fd, GList *selection, GList *list, GtkWidget *parent);
 
 #endif
