@@ -23,6 +23,7 @@
 
 #include <cstdlib>
 #include <cstring>
+#include <stack>
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtk.h>
@@ -47,13 +48,67 @@
 #include "main-defines.h"
 #include "metadata.h"
 #include "options.h"
-#include "pixbuf-renderer.h"
 #include "secure-save.h"
 #include "slideshow.h"
 #include "typedefs.h"
 #include "ui-fileops.h"
 #include "ui-utildlg.h"
 
+namespace
+{
+
+struct GQParserData
+{
+	using StartFunc = void (*)(GQParserData *, const gchar *, const gchar **, const gchar **, gpointer);
+	using EndFunc = void (*)(gpointer);
+
+	struct ParseFunc
+	{
+		StartFunc start_func;
+		EndFunc end_func;
+		gpointer data;
+	};
+
+	void func_push(StartFunc start_func, EndFunc end_func, gpointer data)
+	{
+		ParseFunc parse_func{start_func, end_func, data};
+
+		parse_func_stack.push(parse_func);
+	}
+
+	void func_pop()
+	{
+		parse_func_stack.pop();
+	}
+
+	void func_set_data(gpointer data)
+	{
+		auto& func = parse_func_stack.top();
+		func.data = data;
+	}
+
+	void start_func(const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values)
+	{
+		auto& func = parse_func_stack.top();
+		if (!func.start_func) return;
+
+		func.start_func(this, element_name, attribute_names, attribute_values, func.data);
+	}
+
+	void end_func()
+	{
+		auto& func = parse_func_stack.top();
+		if (!func.end_func) return;
+
+		func.end_func(func.data);
+	}
+
+	std::stack<ParseFunc> parse_func_stack;
+	gboolean startup = FALSE; /* reading config for the first time - add commandline and defaults */
+};
+
+} // namespace
+
 /*
  *-----------------------------------------------------------------------------
  * line write/parse routines (public)
@@ -187,14 +242,6 @@ gboolean read_int_option(const gchar *option, const gchar *label, const gchar *v
 	return TRUE;
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-void write_ushort_option_unused(GString *str, gint, const gchar *label, guint16 n)
-{
-	g_string_append_printf(str, "%s = \"%uh\" ", label, n);
-}
-#pragma GCC diagnostic pop
-
 gboolean read_ushort_option(const gchar *option, const gchar *label, const gchar *value, guint16 *n)
 {
 	if (g_ascii_strcasecmp(option, label) != 0) return FALSE;
@@ -374,12 +421,14 @@ static void write_global_attributes(GString *outstr, gint indent)
 	WRITE_NL(); WRITE_UINT(*options, dnd_default_action);
 	WRITE_NL(); WRITE_BOOL(*options, place_dialogs_under_mouse);
 	WRITE_NL(); WRITE_INT(*options, clipboard_selection);
+	WRITE_NL(); WRITE_UINT(*options, rectangle_draw_aspect_ratio);
 
 	WRITE_NL(); WRITE_BOOL(*options, save_window_positions);
 	WRITE_NL(); WRITE_BOOL(*options, use_saved_window_positions_for_new_windows);
 	WRITE_NL(); WRITE_BOOL(*options, save_window_workspace);
 	WRITE_NL(); WRITE_BOOL(*options, tools_restore_state);
 	WRITE_NL(); WRITE_BOOL(*options, save_dialog_window_positions);
+	WRITE_NL(); WRITE_BOOL(*options, hide_window_decorations);
 	WRITE_NL(); WRITE_BOOL(*options, show_window_ids);
 	WRITE_NL(); WRITE_BOOL(*options, expand_menu_toolbar);
 	WRITE_NL(); WRITE_BOOL(*options, hamburger_menu);
@@ -400,6 +449,7 @@ static void write_global_attributes(GString *outstr, gint indent)
 	WRITE_NL(); WRITE_BOOL(*options, collections_duplicates);
 	WRITE_NL(); WRITE_BOOL(*options, collections_on_top);
 	WRITE_NL(); WRITE_BOOL(*options, hide_window_in_fullscreen);
+	WRITE_NL(); WRITE_BOOL(*options, hide_osd_in_fullscreen);
 
 	WRITE_NL(); WRITE_BOOL(*options, selectable_bars.menu_bar);
 	WRITE_NL(); WRITE_BOOL(*options, selectable_bars.status_bar);
@@ -880,12 +930,14 @@ static gboolean load_global_params(const gchar **attribute_names, const gchar **
 		if (READ_UINT_ENUM(*options, dnd_default_action)) continue;
 		if (READ_BOOL(*options, place_dialogs_under_mouse)) continue;
 		if (READ_INT(*options, clipboard_selection)) continue;
+		if (READ_UINT_ENUM(*options, rectangle_draw_aspect_ratio)) continue;
 
 		if (READ_BOOL(*options, save_window_positions)) continue;
 		if (READ_BOOL(*options, use_saved_window_positions_for_new_windows)) continue;
 		if (READ_BOOL(*options, save_window_workspace)) continue;
 		if (READ_BOOL(*options, tools_restore_state)) continue;
 		if (READ_BOOL(*options, save_dialog_window_positions)) continue;
+		if (READ_BOOL(*options, hide_window_decorations)) continue;
 		if (READ_BOOL(*options, show_window_ids)) continue;
 		if (READ_BOOL(*options, expand_menu_toolbar)) continue;
 		if (READ_BOOL(*options, hamburger_menu)) continue;
@@ -905,6 +957,7 @@ static gboolean load_global_params(const gchar **attribute_names, const gchar **
 		if (READ_BOOL(*options, collections_duplicates)) continue;
 		if (READ_BOOL(*options, collections_on_top)) continue;
 		if (READ_BOOL(*options, hide_window_in_fullscreen)) continue;
+		if (READ_BOOL(*options, hide_osd_in_fullscreen)) continue;
 
 		if (READ_BOOL(*options, selectable_bars.menu_bar)) continue;
 		if (READ_BOOL(*options, selectable_bars.status_bar)) continue;
@@ -926,7 +979,7 @@ static gboolean load_global_params(const gchar **attribute_names, const gchar **
 		if (READ_BOOL(*options, image.limit_autofit_size)) continue;
 		if (READ_INT(*options, image.max_autofit_size)) continue;
 		if (READ_INT(*options, image.max_enlargement_size)) continue;
-		if (READ_UINT_CLAMP(*options, image.scroll_reset_method, 0, PR_SCROLL_RESET_COUNT - 1)) continue;
+		if (READ_UINT_ENUM_CLAMP(*options, image.scroll_reset_method, 0, ScrollReset::COUNT - 1)) continue;
 		if (READ_INT(*options, image.tile_cache_max)) continue;
 		if (READ_INT(*options, image.image_cache_max)) continue;
 		if (READ_UINT_CLAMP(*options, image.zoom_quality, GDK_INTERP_NEAREST, GDK_INTERP_BILINEAR)) continue;
@@ -1057,8 +1110,8 @@ static gboolean load_global_params(const gchar **attribute_names, const gchar **
 		if (READ_CHAR(*options, printer.image_font)) continue;
 		if (READ_CHAR(*options, printer.page_font)) continue;
 		if (READ_CHAR(*options, printer.page_text)) continue;
-		if (READ_INT(*options, printer.image_text_position)) continue;
-		if (READ_INT(*options, printer.page_text_position)) continue;
+		if (READ_INT_ENUM(*options, printer.image_text_position)) continue;
+		if (READ_INT_ENUM(*options, printer.page_text_position)) continue;
 		if (READ_BOOL(*options, printer.show_image_text)) continue;
 		if (READ_BOOL(*options, printer.show_page_text)) continue;
 
@@ -1086,7 +1139,7 @@ static gboolean load_global_params(const gchar **attribute_names, const gchar **
 	return TRUE;
 }
 
-static void options_load_color_profiles(GQParserData *, GMarkupParseContext *, const gchar *, const gchar **attribute_names, const gchar **attribute_values, gpointer, GError **)
+static void options_load_color_profiles(const gchar **attribute_names, const gchar **attribute_values)
 {
 	while (*attribute_names)
 		{
@@ -1105,7 +1158,7 @@ static void options_load_color_profiles(GQParserData *, GMarkupParseContext *, c
 
 }
 
-static void options_load_profile(GQParserData *parser_data, GMarkupParseContext *, const gchar *, const gchar **attribute_names, const gchar **attribute_values, gpointer data, GError **)
+static void options_load_profile(GQParserData *parser_data, const gchar **attribute_names, const gchar **attribute_values, gpointer data)
 {
 	gint i = GPOINTER_TO_INT(data);
 	if (i < 0 || i >= COLOR_PROFILE_INPUTS) return;
@@ -1120,11 +1173,11 @@ static void options_load_profile(GQParserData *parser_data, GMarkupParseContext
 		log_printf("unknown attribute %s = %s\n", option, value);
 		}
 	i++;
-	options_parse_func_set_data(parser_data, GINT_TO_POINTER(i));
+	parser_data->func_set_data(GINT_TO_POINTER(i));
 
 }
 
-static void options_load_marks_tooltips(GQParserData *parser_data, GMarkupParseContext *, const gchar *, const gchar **attribute_names, const gchar **attribute_values, gpointer data, GError **)
+static void options_load_marks_tooltips(GQParserData *parser_data, const gchar **attribute_names, const gchar **attribute_values, gpointer data)
 {
 	gint i = GPOINTER_TO_INT(data);
 	if (i < 0 || i >= FILEDATA_MARKS_SIZE) return;
@@ -1137,32 +1190,30 @@ static void options_load_marks_tooltips(GQParserData *parser_data, GMarkupParseC
 		log_printf("unknown attribute %s = %s\n", option, value);
 		}
 	i++;
-	options_parse_func_set_data(parser_data, GINT_TO_POINTER(i));
+	parser_data->func_set_data(GINT_TO_POINTER(i));
 
 }
 
-static void options_load_disabled_plugins(GQParserData *parser_data, GMarkupParseContext *, const gchar *, const gchar **attribute_names, const gchar **attribute_values, gpointer data, GError **)
+static void options_load_disabled_plugins(GQParserData *parser_data, const gchar **attribute_names, const gchar **attribute_values, gpointer data)
 {
 	gint i = GPOINTER_TO_INT(data);
-	struct {
-		gchar *path;
-	} tmp;
 
 	while (*attribute_names)
 		{
 		const gchar *option = *attribute_names++;
 		const gchar *value = *attribute_values++;
-		tmp.path = nullptr;
-		if (READ_CHAR_FULL("path", tmp.path))
+
+		gchar *path = nullptr;
+		if (READ_CHAR_FULL("path", path))
 			{
-			options->disabled_plugins = g_list_append(options->disabled_plugins, g_strdup(tmp.path));
+			options->disabled_plugins = g_list_append(options->disabled_plugins, path);
 			continue;
 			}
 
 		log_printf("unknown attribute %s = %s\n", option, value);
 		}
 	i++;
-	options_parse_func_set_data(parser_data, GINT_TO_POINTER(i));
+	parser_data->func_set_data(GINT_TO_POINTER(i));
 }
 
 /*
@@ -1170,12 +1221,6 @@ static void options_load_disabled_plugins(GQParserData *parser_data, GMarkupPars
  * xml file structure (private)
  *-----------------------------------------------------------------------------
  */
-struct GQParserData
-{
-	GList *parse_func_stack;
-	gboolean startup; /* reading config for the first time - add commandline and defaults */
-};
-
 static const gchar *options_get_id(const gchar **attribute_names, const gchar **attribute_values)
 {
 	while (*attribute_names)
@@ -1190,38 +1235,38 @@ static const gchar *options_get_id(const gchar **attribute_names, const gchar **
 }
 
 
-void options_parse_leaf(GQParserData *parser_data, GMarkupParseContext *, const gchar *element_name, const gchar **, const gchar **, gpointer, GError **)
+static void options_parse_leaf(GQParserData *parser_data, const gchar *element_name, const gchar **, const gchar **, gpointer)
 {
 	log_printf("unexpected: %s\n", element_name);
-	options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
+	parser_data->func_push(options_parse_leaf, nullptr, nullptr);
 }
 
-static void options_parse_color_profiles(GQParserData *parser_data, GMarkupParseContext *context, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer data, GError **error)
+static void options_parse_color_profiles(GQParserData *parser_data, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer data)
 {
 	if (g_ascii_strcasecmp(element_name, "profile") == 0)
 		{
-		options_load_profile(parser_data, context, element_name, attribute_names, attribute_values, data, error);
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
+		options_load_profile(parser_data, attribute_names, attribute_values, data);
 		}
 	else
 		{
-		log_printf("unexpected in <profile>: <%s>\n", element_name);
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
+		log_printf("unexpected in <color_profiles>: <%s>\n", element_name);
 		}
+
+	parser_data->func_push(options_parse_leaf, nullptr, nullptr);
 }
 
-static void options_parse_marks_tooltips(GQParserData *parser_data, GMarkupParseContext *context, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer data, GError **error)
+static void options_parse_marks_tooltips(GQParserData *parser_data, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer data)
 {
 	if (g_ascii_strcasecmp(element_name, "tooltip") == 0)
 		{
-		options_load_marks_tooltips(parser_data, context, element_name, attribute_names, attribute_values, data, error);
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
+		options_load_marks_tooltips(parser_data, attribute_names, attribute_values, data);
 		}
 	else
 		{
-		log_printf("unexpected in <profile>: <%s>\n", element_name);
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
+		log_printf("unexpected in <marks_tooltips>: <%s>\n", element_name);
 		}
+
+	parser_data->func_push(options_parse_leaf, nullptr, nullptr);
 }
 
 static void class_filter_load_filter_type(const gchar **attribute_names, const gchar **attribute_values)
@@ -1286,131 +1331,132 @@ static void class_filter_load_filter_type(const gchar **attribute_names, const g
 		}
 }
 
-static void options_parse_class_filter(GQParserData *parser_data, GMarkupParseContext *, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer, GError **)
+static void options_parse_class_filter(GQParserData *parser_data, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer)
 {
 	if (g_ascii_strcasecmp(element_name, "filter_type") == 0)
 		{
 		class_filter_load_filter_type(attribute_names, attribute_values);
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
 		}
 	else
 		{
-		log_printf("unexpected in <profile>: <%s>\n", element_name);
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
+		log_printf("unexpected in <class_filter>: <%s>\n", element_name);
 		}
+
+	parser_data->func_push(options_parse_leaf, nullptr, nullptr);
 }
 
-static void options_parse_disabled_plugins(GQParserData *parser_data, GMarkupParseContext *context, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer data, GError **error)
+static void options_parse_disabled_plugins(GQParserData *parser_data, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer data)
 {
 	if (g_ascii_strcasecmp(element_name, "plugin") == 0)
 		{
-		options_load_disabled_plugins(parser_data, context, element_name, attribute_names, attribute_values, data, error);
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
+		options_load_disabled_plugins(parser_data, attribute_names, attribute_values, data);
 		}
 	else
 		{
-		log_printf("unexpected in <profile>: <%s>\n", element_name);
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
+		log_printf("unexpected in <disabled_plugins>: <%s>\n", element_name);
 		}
+
+	parser_data->func_push(options_parse_leaf, nullptr, nullptr);
 }
 
-static void options_parse_filter(GQParserData *parser_data, GMarkupParseContext *, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer, GError **)
+static void options_parse_filter(GQParserData *parser_data, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer)
 {
 	if (g_ascii_strcasecmp(element_name, "file_type") == 0)
 		{
 		filter_load_file_type(attribute_names, attribute_values);
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
 		}
 	else
 		{
 		log_printf("unexpected in <filter>: <%s>\n", element_name);
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
 		}
+
+	parser_data->func_push(options_parse_leaf, nullptr, nullptr);
 }
 
-static void options_parse_filter_end(GQParserData *parser_data, GMarkupParseContext *, const gchar *, gpointer, GError **)
+static void options_parse_filter_end(gpointer data)
 {
-	if (parser_data->startup) filter_add_defaults();
+	gboolean startup = GPOINTER_TO_INT(data);
+	if (startup) filter_add_defaults();
 	filter_rebuild();
 }
 
-static void options_parse_keyword_end(GQParserData *, GMarkupParseContext *, const gchar *, gpointer data, GError **)
+static void options_parse_keyword_end(gpointer data)
 {
 	auto iter_ptr = static_cast<GtkTreeIter *>(data);
 	gtk_tree_iter_free(iter_ptr);
 }
 
 
-static void options_parse_keyword(GQParserData *parser_data, GMarkupParseContext *, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer data, GError **)
+static void options_parse_keyword(GQParserData *parser_data, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer data)
 {
-	auto iter_ptr = static_cast<GtkTreeIter *>(data);
 	if (g_ascii_strcasecmp(element_name, "keyword") == 0)
 		{
+		auto iter_ptr = static_cast<GtkTreeIter *>(data);
 		GtkTreeIter *child = keyword_add_from_config(keyword_tree, iter_ptr, attribute_names, attribute_values);
-		options_parse_func_push(parser_data, options_parse_keyword, options_parse_keyword_end, child);
+		parser_data->func_push(options_parse_keyword, options_parse_keyword_end, child);
 		}
 	else
 		{
 		log_printf("unexpected in <keyword>: <%s>\n", element_name);
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
+		parser_data->func_push(options_parse_leaf, nullptr, nullptr);
 		}
 }
 
 
 
-static void options_parse_keyword_tree(GQParserData *parser_data, GMarkupParseContext *, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer, GError **)
+static void options_parse_keyword_tree(GQParserData *parser_data, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer)
 {
 	if (g_ascii_strcasecmp(element_name, "keyword") == 0)
 		{
 		GtkTreeIter *iter_ptr = keyword_add_from_config(keyword_tree, nullptr, attribute_names, attribute_values);
-		options_parse_func_push(parser_data, options_parse_keyword, options_parse_keyword_end, iter_ptr);
+		parser_data->func_push(options_parse_keyword, options_parse_keyword_end, iter_ptr);
 		}
 	else
 		{
 		log_printf("unexpected in <keyword_tree>: <%s>\n", element_name);
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
+		parser_data->func_push(options_parse_leaf, nullptr, nullptr);
 		}
 }
 
 
-static void options_parse_global(GQParserData *parser_data, GMarkupParseContext *context, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer data, GError **error)
+static void options_parse_global(GQParserData *parser_data, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer data)
 {
 	if (g_ascii_strcasecmp(element_name, "color_profiles") == 0)
 		{
-		options_load_color_profiles(parser_data, context, element_name, attribute_names, attribute_values, data, error);
-		options_parse_func_push(parser_data, options_parse_color_profiles, nullptr, GINT_TO_POINTER(0));
+		options_load_color_profiles(attribute_names, attribute_values);
+		parser_data->func_push(options_parse_color_profiles, nullptr, GINT_TO_POINTER(0));
 		}
 	else if (g_ascii_strcasecmp(element_name, "filter") == 0)
 		{
-		options_parse_func_push(parser_data, options_parse_filter, options_parse_filter_end, nullptr);
+		parser_data->func_push(options_parse_filter, options_parse_filter_end, GINT_TO_POINTER(parser_data->startup));
 		}
 	else if (g_ascii_strcasecmp(element_name, "marks_tooltips") == 0)
 		{
-		options_load_marks_tooltips(parser_data, context, element_name, attribute_names, attribute_values, data, error);
-		options_parse_func_push(parser_data, options_parse_marks_tooltips, nullptr, nullptr);
+		options_load_marks_tooltips(parser_data, attribute_names, attribute_values, data);
+		parser_data->func_push(options_parse_marks_tooltips, nullptr, nullptr);
 		}
 	else if (g_ascii_strcasecmp(element_name, "class_filter") == 0)
 		{
-		options_parse_func_push(parser_data, options_parse_class_filter, nullptr, nullptr);
+		parser_data->func_push(options_parse_class_filter, nullptr, nullptr);
 		}
 	else if (g_ascii_strcasecmp(element_name, "keyword_tree") == 0)
 		{
 		if (!keyword_tree) keyword_tree_new();
-		options_parse_func_push(parser_data, options_parse_keyword_tree, nullptr, nullptr);
+		parser_data->func_push(options_parse_keyword_tree, nullptr, nullptr);
 		}
 	else if (g_ascii_strcasecmp(element_name, "disabled_plugins") == 0)
 		{
-		options_load_disabled_plugins(parser_data, context, element_name, attribute_names, attribute_values, data, error);
-		options_parse_func_push(parser_data, options_parse_disabled_plugins, nullptr, nullptr);
+		options_load_disabled_plugins(parser_data, attribute_names, attribute_values, data);
+		parser_data->func_push(options_parse_disabled_plugins, nullptr, nullptr);
 		}
 	else
 		{
 		log_printf("unexpected in <global>: <%s>\n", element_name);
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
+		parser_data->func_push(options_parse_leaf, nullptr, nullptr);
 		}
 }
 
-static void options_parse_global_end(GQParserData *, GMarkupParseContext *, const gchar *, gpointer, GError **)
+static void options_parse_global_end(gpointer)
 {
 #if !HAVE_EXIV2
 	/* some options do not work without exiv2 */
@@ -1421,38 +1467,37 @@ static void options_parse_global_end(GQParserData *, GMarkupParseContext *, cons
 #endif
 }
 
-static void options_parse_pane_exif(GQParserData *parser_data, GMarkupParseContext *, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer data, GError **)
+static void options_parse_pane_exif(GQParserData *parser_data, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer data)
 {
-	auto pane = static_cast<GtkWidget *>(data);
 	if (g_ascii_strcasecmp(element_name, "entry") == 0)
 		{
+		auto pane = static_cast<GtkWidget *>(data);
 		bar_pane_exif_entry_add_from_config(pane, attribute_names, attribute_values);
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
 		}
 	else
 		{
 		log_printf("unexpected in <pane_exif>: <%s>\n", element_name);
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
 		}
+
+	parser_data->func_push(options_parse_leaf, nullptr, nullptr);
 }
 
-static void options_parse_pane_keywords(GQParserData *parser_data, GMarkupParseContext *, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer data, GError **)
+static void options_parse_pane_keywords(GQParserData *parser_data, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer data)
 {
-	auto pane = static_cast<GtkWidget *>(data);
-
 	if (g_ascii_strcasecmp(element_name, "expanded") == 0)
 		{
+		auto pane = static_cast<GtkWidget *>(data);
 		bar_pane_keywords_entry_add_from_config(pane, attribute_names, attribute_values);
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
 		}
 	else
 		{
 		log_printf("unexpected in <pane_keywords>: <%s>\n", element_name);
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
 		}
+
+	parser_data->func_push(options_parse_leaf, nullptr, nullptr);
 }
 
-static void options_parse_bar(GQParserData *parser_data, GMarkupParseContext *, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer data, GError **)
+static void options_parse_bar(GQParserData *parser_data, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer data)
 {
 	auto bar = static_cast<GtkWidget *>(data);
 	if (g_ascii_strcasecmp(element_name, "pane_comment") == 0)
@@ -1467,7 +1512,7 @@ static void options_parse_bar(GQParserData *parser_data, GMarkupParseContext *,
 			pane = bar_pane_comment_new_from_config(attribute_names, attribute_values);
 			bar_add(bar, pane);
 			}
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
+		parser_data->func_push(options_parse_leaf, nullptr, nullptr);
 		}
 #if HAVE_LIBCHAMPLAIN && HAVE_LIBCHAMPLAIN_GTK
 	else if (g_ascii_strcasecmp(element_name, "pane_gps") == 0)
@@ -1485,7 +1530,7 @@ static void options_parse_bar(GQParserData *parser_data, GMarkupParseContext *,
 				pane = bar_pane_gps_new_from_config(attribute_names, attribute_values);
 				bar_add(bar, pane);
 				}
-			options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
+			parser_data->func_push(options_parse_leaf, nullptr, nullptr);
 			}
 		}
 #endif
@@ -1501,7 +1546,7 @@ static void options_parse_bar(GQParserData *parser_data, GMarkupParseContext *,
 			pane = bar_pane_exif_new_from_config(attribute_names, attribute_values);
 			bar_add(bar, pane);
 			}
-		options_parse_func_push(parser_data, options_parse_pane_exif, nullptr, pane);
+		parser_data->func_push(options_parse_pane_exif, nullptr, pane);
 		}
 	else if (g_ascii_strcasecmp(element_name, "pane_histogram") == 0)
 		{
@@ -1515,7 +1560,7 @@ static void options_parse_bar(GQParserData *parser_data, GMarkupParseContext *,
 			pane = bar_pane_histogram_new_from_config(attribute_names, attribute_values);
 			bar_add(bar, pane);
 			}
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
+		parser_data->func_push(options_parse_leaf, nullptr, nullptr);
 		}
 	else if (g_ascii_strcasecmp(element_name, "pane_rating") == 0)
 		{
@@ -1529,7 +1574,7 @@ static void options_parse_bar(GQParserData *parser_data, GMarkupParseContext *,
 			pane = bar_pane_rating_new_from_config(attribute_names, attribute_values);
 			bar_add(bar, pane);
 			}
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
+		parser_data->func_push(options_parse_leaf, nullptr, nullptr);
 		}
 	else if (g_ascii_strcasecmp(element_name, "pane_keywords") == 0)
 		{
@@ -1543,75 +1588,73 @@ static void options_parse_bar(GQParserData *parser_data, GMarkupParseContext *,
 			pane = bar_pane_keywords_new_from_config(attribute_names, attribute_values);
 			bar_add(bar, pane);
 			}
-		options_parse_func_push(parser_data, options_parse_pane_keywords, nullptr, pane);
+		parser_data->func_push(options_parse_pane_keywords, nullptr, pane);
 		}
 	else if (g_ascii_strcasecmp(element_name, "clear") == 0)
 		{
 		bar_clear(bar);
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
+		parser_data->func_push(options_parse_leaf, nullptr, nullptr);
 		}
 	else
 		{
 		log_printf("unexpected in <bar>: <%s>\n", element_name);
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
+		parser_data->func_push(options_parse_leaf, nullptr, nullptr);
 		}
 }
 
-static void options_parse_toolbar(GQParserData *parser_data, GMarkupParseContext *, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer data, GError **)
+static void options_parse_toolbar(GQParserData *parser_data, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer data)
 {
 	auto lw = static_cast<LayoutWindow *>(data);
 	if (g_ascii_strcasecmp(element_name, "toolitem") == 0)
 		{
 		layout_toolbar_add_from_config(lw, TOOLBAR_MAIN, attribute_names, attribute_values);
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
 		}
 	else if (g_ascii_strcasecmp(element_name, "clear") == 0)
 		{
 		layout_toolbar_clear(lw, TOOLBAR_MAIN);
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
 		}
 	else
 		{
 		log_printf("unexpected in <toolbar>: <%s>\n", element_name);
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
 		}
+
+	parser_data->func_push(options_parse_leaf, nullptr, nullptr);
 }
 
-static void options_parse_statusbar(GQParserData *parser_data, GMarkupParseContext *, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer data, GError **)
+static void options_parse_statusbar(GQParserData *parser_data, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer data)
 {
 	auto lw = static_cast<LayoutWindow *>(data);
 	if (g_ascii_strcasecmp(element_name, "toolitem") == 0)
 		{
 		layout_toolbar_add_from_config(lw, TOOLBAR_STATUS, attribute_names, attribute_values);
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
 		}
 	else if (g_ascii_strcasecmp(element_name, "clear") == 0)
 		{
 		layout_toolbar_clear(lw, TOOLBAR_STATUS);
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
 		}
 	else
 		{
 		log_printf("unexpected in <statusbar>: <%s>\n", element_name);
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
 		}
+
+	parser_data->func_push(options_parse_leaf, nullptr, nullptr);
 }
 
-static void options_parse_dialogs(GQParserData *parser_data, GMarkupParseContext *, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer, GError **)
+static void options_parse_dialogs(GQParserData *parser_data, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer)
 {
 	if (g_ascii_strcasecmp(element_name, "window") == 0)
 		{
 		generic_dialog_windows_load_config(attribute_names, attribute_values);
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
 		}
 	else
 		{
 		log_printf("unexpected in <dialogs>: <%s>\n", element_name);
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
 		}
+
+	parser_data->func_push(options_parse_leaf, nullptr, nullptr);
 }
 
-static void options_parse_layout(GQParserData *parser_data, GMarkupParseContext *, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer data, GError **)
+static void options_parse_layout(GQParserData *parser_data, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer data)
 {
 	auto lw = static_cast<LayoutWindow *>(data);
 	if (g_ascii_strcasecmp(element_name, "bar") == 0)
@@ -1626,50 +1669,59 @@ static void options_parse_layout(GQParserData *parser_data, GMarkupParseContext
 			bar_update_from_config(lw->bar, attribute_names, attribute_values, lw, FALSE);
 			}
 
-		options_parse_func_push(parser_data, options_parse_bar, nullptr, lw->bar);
+		parser_data->func_push(options_parse_bar, nullptr, lw->bar);
 		}
 	else if (g_ascii_strcasecmp(element_name, "bar_sort") == 0)
 		{
-		bar_sort_cold_start(lw, attribute_names, attribute_values);
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
+		if (g_list_length(layout_window_list) == 1)
+			{
+			bar_sort_cold_start(lw, attribute_names, attribute_values);
+			}
+		else
+			{
+			GtkWidget *bar = bar_sort_new_from_config(lw, attribute_names, attribute_values);
+			layout_bar_sort_set(lw, bar);
+			gtk_widget_show(lw->bar_sort);
+			}
+		parser_data->func_push(options_parse_leaf, nullptr, nullptr);
 		}
 	else if (g_ascii_strcasecmp(element_name, "toolbar") == 0)
 		{
-		options_parse_func_push(parser_data, options_parse_toolbar, nullptr, lw);
+		parser_data->func_push(options_parse_toolbar, nullptr, lw);
 		}
 	else if (g_ascii_strcasecmp(element_name, "statusbar") == 0)
 		{
-		options_parse_func_push(parser_data, options_parse_statusbar, nullptr, lw);
+		parser_data->func_push(options_parse_statusbar, nullptr, lw);
 		}
 	else if (g_ascii_strcasecmp(element_name, "dialogs") == 0)
 		{
-		options_parse_func_push(parser_data, options_parse_dialogs, nullptr, nullptr);
+		parser_data->func_push(options_parse_dialogs, nullptr, nullptr);
 		}
 	else
 		{
 		log_printf("unexpected in <layout>: <%s>\n", element_name);
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
+		parser_data->func_push(options_parse_leaf, nullptr, nullptr);
 		}
 }
 
-static void options_parse_layout_end(GQParserData *, GMarkupParseContext *, const gchar *, gpointer data, GError **)
+static void options_parse_layout_end(gpointer data)
 {
 	auto lw = static_cast<LayoutWindow *>(data);
 	layout_util_sync(lw);
 }
 
-static void options_parse_toplevel(GQParserData *parser_data, GMarkupParseContext *, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer, GError **)
+static void options_parse_toplevel(GQParserData *parser_data, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer)
 {
 	if (g_ascii_strcasecmp(element_name, "gq") == 0)
 		{
 		/* optional top-level node */
-		options_parse_func_push(parser_data, options_parse_toplevel, nullptr, nullptr);
+		parser_data->func_push(options_parse_toplevel, nullptr, nullptr);
 		return;
 		}
 	if (g_ascii_strcasecmp(element_name, "global") == 0)
 		{
 		load_global_params(attribute_names, attribute_values);
-		options_parse_func_push(parser_data, options_parse_global, options_parse_global_end, nullptr);
+		parser_data->func_push(options_parse_global, options_parse_global_end, nullptr);
 		return;
 		}
 
@@ -1685,94 +1737,47 @@ static void options_parse_toplevel(GQParserData *parser_data, GMarkupParseContex
 			{
 			lw = layout_new_from_config(attribute_names, attribute_values, parser_data->startup);
 			}
-		options_parse_func_push(parser_data, options_parse_layout, options_parse_layout_end, lw);
+		parser_data->func_push(options_parse_layout, options_parse_layout_end, lw);
 		}
 	else
 		{
 		log_printf("unexpected in <toplevel>: <%s>\n", element_name);
-		options_parse_func_push(parser_data, options_parse_leaf, nullptr, nullptr);
+		parser_data->func_push(options_parse_leaf, nullptr, nullptr);
 		}
 }
 
 
-
-
-
 /*
  *-----------------------------------------------------------------------------
  * parser
  *-----------------------------------------------------------------------------
  */
 
-
-struct GQParserFuncData
-{
-	GQParserStartFunc start_func;
-	GQParserEndFunc end_func;
-	gpointer data;
-};
-
-void options_parse_func_push(GQParserData *parser_data, GQParserStartFunc start_func, GQParserEndFunc end_func, gpointer data)
-{
-	auto func_data = g_new0(GQParserFuncData, 1);
-	func_data->start_func = start_func;
-	func_data->end_func = end_func;
-	func_data->data = data;
-
-	parser_data->parse_func_stack = g_list_prepend(parser_data->parse_func_stack, func_data);
-}
-
-void options_parse_func_pop(GQParserData *parser_data)
-{
-	g_free(parser_data->parse_func_stack->data);
-	parser_data->parse_func_stack = g_list_delete_link(parser_data->parse_func_stack, parser_data->parse_func_stack);
-}
-
-void options_parse_func_set_data(GQParserData *parser_data, gpointer data)
-{
-	auto func = static_cast<GQParserFuncData *>(parser_data->parse_func_stack->data);
-	func->data = data;
-}
-
-
-static void start_element(GMarkupParseContext *context,
-			  const gchar *element_name,
-			  const gchar **attribute_names,
-			  const gchar **attribute_values,
-			  gpointer user_data,
-			  GError **error)
+static void start_element(GMarkupParseContext *,
+                          const gchar *element_name,
+                          const gchar **attribute_names,
+                          const gchar **attribute_values,
+                          gpointer user_data,
+                          GError **)
 {
 	auto parser_data = static_cast<GQParserData *>(user_data);
-	auto func = static_cast<GQParserFuncData *>(parser_data->parse_func_stack->data);
 	DEBUG_2("start %s", element_name);
 
-	if (func->start_func)
-		func->start_func(parser_data, context, element_name, attribute_names, attribute_values, func->data, error);
+	parser_data->start_func(element_name, attribute_names, attribute_values);
 }
 
-static void end_element(GMarkupParseContext *context,
-			  const gchar *element_name,
-			  gpointer user_data,
-			  GError **error)
+static void end_element(GMarkupParseContext *,
+                        const gchar *element_name,
+                        gpointer user_data,
+                        GError **)
 {
 	auto parser_data = static_cast<GQParserData *>(user_data);
-	auto func = static_cast<GQParserFuncData *>(parser_data->parse_func_stack->data);
 	DEBUG_2("end %s", element_name);
 
-	if (func->end_func)
-		func->end_func(parser_data, context, element_name, func->data, error);
-
-	options_parse_func_pop(parser_data);
+	parser_data->end_func();
+	parser_data->func_pop();
 }
 
-static GMarkupParser parser = {
-	start_element,
-	end_element,
-	nullptr,
-	nullptr,
-	nullptr
-};
-
 /*
  *-----------------------------------------------------------------------------
  * load configuration (public)
@@ -1781,15 +1786,23 @@ static GMarkupParser parser = {
 
 gboolean load_config_from_buf(const gchar *buf, gsize size, gboolean startup)
 {
+	static GMarkupParser parser = {
+	    start_element,
+	    end_element,
+	    nullptr,
+	    nullptr,
+	    nullptr
+	};
+
 	GMarkupParseContext *context;
 	gboolean ret = TRUE;
 
-	auto parser_data = g_new0(GQParserData, 1);
+	GQParserData parser_data;
 
-	parser_data->startup = startup;
-	options_parse_func_push(parser_data, options_parse_toplevel, nullptr, nullptr);
+	parser_data.startup = startup;
+	parser_data.func_push(options_parse_toplevel, nullptr, nullptr);
 
-	context = g_markup_parse_context_new(&parser, static_cast<GMarkupParseFlags>(0), parser_data, nullptr);
+	context = g_markup_parse_context_new(&parser, static_cast<GMarkupParseFlags>(0), &parser_data, nullptr);
 
 	if (g_markup_parse_context_parse(context, buf, size, nullptr) == FALSE)
 		{
@@ -1797,8 +1810,6 @@ gboolean load_config_from_buf(const gchar *buf, gsize size, gboolean startup)
 		DEBUG_1("Parse failed");
 		}
 
-	g_free(parser_data);
-
 	g_markup_parse_context_free(context);
 	return ret;
 }
@@ -1806,18 +1817,14 @@ gboolean load_config_from_buf(const gchar *buf, gsize size, gboolean startup)
 gboolean load_config_from_file(const gchar *utf8_path, gboolean startup)
 {
 	gsize size;
-	gchar *buf;
-	gboolean ret = TRUE;
+	g_autofree gchar *buf = nullptr;
 
 	if (g_file_get_contents(utf8_path, &buf, &size, nullptr) == FALSE)
 		{
 		return FALSE;
 		}
-	ret = load_config_from_buf(buf, size, startup);
-	g_free(buf);
-	return ret;
+
+	return load_config_from_buf(buf, size, startup);
 }
 
-
-
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
