@@ -26,16 +26,18 @@
 #include <sys/un.h>
 #include <unistd.h>
 
+#include <algorithm>
 #include <cerrno>
 #include <csignal>
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
-
-#include <config.h>
+#include <vector>
 
 #include <gtk/gtk.h>
 
+#include <config.h>
+
 #include "cache-maint.h"
 #include "collect-io.h"
 #include "collect.h"
@@ -155,7 +157,7 @@ static gboolean remote_server_client_cb(GIOChannel *source, GIOCondition conditi
 				{
 				buffer[termpos] = '\0';
 
-				if (strlen(buffer) > 0)
+				if (buffer[0] != '\0')
 					{
 					if (rc->read_func) rc->read_func(rc, buffer, source, rc->read_data);
 					g_io_channel_write_chars(source, "<gq_end_of_command>", -1, nullptr, nullptr); /* empty line finishes the command */
@@ -179,7 +181,7 @@ static gboolean remote_server_client_cb(GIOChannel *source, GIOCondition conditi
 		rc->clients = g_list_remove(rc->clients, client);
 
 		DEBUG_1("HUP detected, closing client.");
-		DEBUG_1("client count %d", g_list_length(rc->clients));
+		DEBUG_1("client count %u", g_list_length(rc->clients));
 
 		g_source_remove(client->channel_id);
 		close(client->fd);
@@ -211,7 +213,7 @@ static void remote_server_client_add(RemoteConnection *rc, gint fd)
 	g_io_channel_unref(channel);
 
 	rc->clients = g_list_append(rc->clients, client);
-	DEBUG_1("client count %d", g_list_length(rc->clients));
+	DEBUG_1("client count %u", g_list_length(rc->clients));
 }
 
 static void remote_server_clients_close(RemoteConnection *rc)
@@ -462,8 +464,7 @@ void remote_close(RemoteConnection *rc)
 		unlink(rc->path);
 		}
 
-	if (rc->read_data)
-		g_free(rc->read_data);
+	g_free(rc->read_data);
 
 	close(rc->fd);
 
@@ -771,11 +772,12 @@ static void gr_pixel_info(const gchar *, GIOChannel *channel, gpointer)
 	gint r_mouse;
 	gint g_mouse;
 	gint b_mouse;
+	gint a_mouse;
 	PixbufRenderer *pr;
 
 	if (!layout_valid(&lw_id)) return;
 
-	pr = reinterpret_cast<PixbufRenderer*>(lw_id->image->pr);
+	pr = PIXBUF_RENDERER(lw_id->image->pr);
 
 	if (pr)
 		{
@@ -786,12 +788,20 @@ static void gr_pixel_info(const gchar *, GIOChannel *channel, gpointer)
 
 		if (x_pixel >= 0 && y_pixel >= 0)
 			{
-			pixbuf_renderer_get_pixel_colors(pr, x_pixel, y_pixel,
-							 &r_mouse, &g_mouse, &b_mouse);
+			pixbuf_renderer_get_pixel_colors(pr, x_pixel, y_pixel, &r_mouse, &g_mouse, &b_mouse, &a_mouse);
 
-			pixel_info = g_strdup_printf(_("[%d,%d]: RGB(%3d,%3d,%3d)"),
-						 x_pixel, y_pixel,
-						 r_mouse, g_mouse, b_mouse);
+			if (gdk_pixbuf_get_has_alpha(pr->pixbuf))
+				{
+				pixel_info = g_strdup_printf(_("[%d,%d]: RGBA(%3d,%3d,%3d,%3d)"),
+				x_pixel, y_pixel,
+				r_mouse, g_mouse, b_mouse, a_mouse);
+				}
+			else
+				{
+				pixel_info = g_strdup_printf(_("[%d,%d]: RGB(%3d,%3d,%3d)"),
+				x_pixel, y_pixel,
+				r_mouse, g_mouse, b_mouse);
+				}
 
 			g_io_channel_write_chars(channel, pixel_info, -1, nullptr, nullptr);
 			g_io_channel_write_chars(channel, "<gq_end_of_command>", -1, nullptr, nullptr);
@@ -811,32 +821,28 @@ static void gr_pixel_info(const gchar *, GIOChannel *channel, gpointer)
 
 static void gr_rectangle(const gchar *, GIOChannel *channel, gpointer)
 {
-	gchar *rectangle_info;
-	PixbufRenderer *pr;
+	if (!options->draw_rectangle) return;
+	if (!layout_valid(&lw_id)) return;
+
+	auto *pr = PIXBUF_RENDERER(lw_id->image->pr);
+	if (!pr) return;
+
 	gint x1;
 	gint y1;
 	gint x2;
 	gint y2;
+	image_get_rectangle(x1, y1, x2, y2);
 
-	if (!options->draw_rectangle) return;
-	if (!layout_valid(&lw_id)) return;
+	gchar *rectangle_info = g_strdup_printf(_("%dx%d+%d+%d"),
+	                                        std::abs(x1 - x2),
+	                                        std::abs(y1 - y2),
+	                                        std::min(x1, x2),
+	                                        std::min(y1, y2));
 
-	pr = reinterpret_cast<PixbufRenderer*>(lw_id->image->pr);
+	g_io_channel_write_chars(channel, rectangle_info, -1, nullptr, nullptr);
+	g_io_channel_write_chars(channel, "<gq_end_of_command>", -1, nullptr, nullptr);
 
-	if (pr)
-		{
-		image_get_rectangle(&x1, &y1, &x2, &y2);
-		rectangle_info = g_strdup_printf(_("%dx%d+%d+%d"),
-					(x2 > x1) ? x2 - x1 : x1 - x2,
-					(y2 > y1) ? y2 - y1 : y1 - y2,
-					(x2 > x1) ? x1 : x2,
-					(y2 > y1) ? y1 : y2);
-
-		g_io_channel_write_chars(channel, rectangle_info, -1, nullptr, nullptr);
-		g_io_channel_write_chars(channel, "<gq_end_of_command>", -1, nullptr, nullptr);
-
-		g_free(rectangle_info);
-		}
+	g_free(rectangle_info);
 }
 
 static void gr_render_intent(const gchar *, GIOChannel *channel, gpointer)
@@ -1119,11 +1125,11 @@ static void gr_selection_remove(const gchar *text, GIOChannel *, gpointer)
 		{
 		if (fd_to_deselect)
 			{
-			log_printf("remote sent \"--selection-remove:\" but current image is not selected");
+			log_printf("remote sent \"--selection-remove=\" but current image is not selected");
 			}
 		else
 			{
-			log_printf("remote sent \"--selection-remove:%s\" but that filename is not selected",
+			log_printf("remote sent \"--selection-remove=%s\" but that filename is not selected",
 				   filename);
 			}
 		}
@@ -1258,20 +1264,20 @@ static void gr_filelist_recurse(const gchar *text, GIOChannel *channel, gpointer
 static void gr_file_tell(const gchar *, GIOChannel *channel, gpointer)
 {
 	gchar *out_string;
-	gchar *collection_name = nullptr;
 
 	if (!layout_valid(&lw_id)) return;
 
-	if (image_get_path(lw_id->image))
+	const gchar *filename = image_get_path(lw_id->image);
+	if (filename)
 		{
 		if (lw_id->image->collection && lw_id->image->collection->name)
 			{
-			collection_name = remove_extension_from_path(lw_id->image->collection->name);
-			out_string = g_strconcat(image_get_path(lw_id->image), "    Collection: ", collection_name, NULL);
+			g_autofree gchar *collection_name = remove_extension_from_path(lw_id->image->collection->name);
+			out_string = g_strconcat(filename, "    Collection: ", collection_name, NULL);
 			}
 		else
 			{
-			out_string = g_strconcat(image_get_path(lw_id->image), NULL);
+			out_string = g_strconcat(filename, NULL);
 			}
 		}
 	else
@@ -1282,32 +1288,26 @@ static void gr_file_tell(const gchar *, GIOChannel *channel, gpointer)
 	g_io_channel_write_chars(channel, out_string, -1, nullptr, nullptr);
 	g_io_channel_write_chars(channel, "<gq_end_of_command>", -1, nullptr, nullptr);
 
-	g_free(collection_name);
 	g_free(out_string);
 }
 
 static void gr_file_info(const gchar *, GIOChannel *channel, gpointer)
 {
-	gchar *filename;
 	FileData *fd;
-	gchar *country_name;
-	gchar *country_code;
-	gchar *timezone;
-	gchar *local_time;
 	GString *out_string;
 	FileFormatClass format_class;
 
 	if (!layout_valid(&lw_id)) return;
 
-	if (image_get_path(lw_id->image))
+	const gchar *filename = image_get_path(lw_id->image);
+	if (filename)
 		{
-		filename = g_strdup(image_get_path(lw_id->image));
 		fd = file_data_new_group(filename);
 		out_string = g_string_new(nullptr);
 
 		if (fd->pixbuf)
 			{
-			format_class = filter_file_get_class(image_get_path(lw_id->image));
+			format_class = filter_file_get_class(filename);
 			}
 		else
 			{
@@ -1323,32 +1323,28 @@ static void gr_file_info(const gchar *, GIOChannel *channel, gpointer)
 
 		if (fd->exif)
 			{
-			country_name = exif_get_data_as_text(fd->exif, "formatted.countryname");
+			g_autofree gchar *country_name = exif_get_data_as_text(fd->exif, "formatted.countryname");
 			if (country_name)
 				{
 				g_string_append_printf(out_string, _("Country name: %s\n"), country_name);
-				g_free(country_name);
 				}
 
-			country_code = exif_get_data_as_text(fd->exif, "formatted.countrycode");
-			if (country_name)
+			g_autofree gchar *country_code = exif_get_data_as_text(fd->exif, "formatted.countrycode");
+			if (country_code)
 				{
 				g_string_append_printf(out_string, _("Country code: %s\n"), country_code);
-				g_free(country_code);
 				}
 
-			timezone = exif_get_data_as_text(fd->exif, "formatted.timezone");
+			g_autofree gchar *timezone = exif_get_data_as_text(fd->exif, "formatted.timezone");
 			if (timezone)
 				{
 				g_string_append_printf(out_string, _("Timezone: %s\n"), timezone);
-				g_free(timezone);
 				}
 
-			local_time = exif_get_data_as_text(fd->exif, "formatted.localtime");
+			g_autofree gchar *local_time = exif_get_data_as_text(fd->exif, "formatted.localtime");
 			if (local_time)
 				{
 				g_string_append_printf(out_string, ("Local time: %s\n"), local_time);
-				g_free(local_time);
 				}
 			}
 
@@ -1357,7 +1353,6 @@ static void gr_file_info(const gchar *, GIOChannel *channel, gpointer)
 
 		g_string_free(out_string, TRUE);
 		file_data_unref(fd);
-		g_free(filename);
 		}
 }
 
@@ -1419,7 +1414,7 @@ static void gr_config_load(const gchar *text, GIOChannel *, gpointer)
 		}
 	else
 		{
-		log_printf("remote sent filename that does not exist:\"%s\"\n", filename);
+		log_printf("remote sent filename that does not exist:\"%s\"\n", filename ? filename : "(null)");
 		layout_set_path(nullptr, homedir());
 		}
 
@@ -1597,10 +1592,10 @@ static void gr_action(const gchar *text, GIOChannel *, gpointer)
 		}
 	else
 		{
-		action = gtk_action_group_get_action(lw_id->action_group, text);
+		action = gq_gtk_action_group_get_action(lw_id->action_group, text);
 		if (action)
 			{
-			gtk_action_activate(action);
+			gq_gtk_action_activate(action);
 			}
 		else
 			{
@@ -1611,42 +1606,33 @@ static void gr_action(const gchar *text, GIOChannel *, gpointer)
 
 static void gr_action_list(const gchar *, GIOChannel *channel, gpointer)
 {
-	ActionItem *action_item;
 	gint max_length = 0;
-	GList *list = nullptr;
-	GString *out_string = g_string_new(nullptr);
 
 	if (!layout_valid(&lw_id))
 		{
 		return;
 		}
 
-	list = get_action_items();
+	std::vector<ActionItem> list = get_action_items();
 
 	/* Get the length required for padding */
-	for (GList *work = list; work; work = work->next)
+	for (const ActionItem &action_item : list)
 		{
-		action_item = static_cast<ActionItem *>(work->data);
-		const auto length = g_utf8_strlen(action_item->name, -1);
+		const auto length = g_utf8_strlen(action_item.name, -1);
 		max_length = MAX(length, max_length);
 		}
 
 	/* Pad the action names to the same column for readable output */
-	for (GList *work = list; work; work = work->next)
+	g_autoptr(GString) out_string = g_string_new(nullptr);
+	for (const ActionItem &action_item : list)
 		{
-		action_item = static_cast<ActionItem *>(work->data);
-
-		g_string_append_printf(out_string, "%-*s", max_length + 4, action_item->name);
-		out_string = g_string_append(out_string, action_item->label);
+		g_string_append_printf(out_string, "%-*s", max_length + 4, action_item.name);
+		out_string = g_string_append(out_string, action_item.label);
 		out_string = g_string_append(out_string, "\n");
 		}
 
-	action_items_free(list);
-
 	g_io_channel_write_chars(channel, out_string->str, -1, nullptr, nullptr);
 	g_io_channel_write_chars(channel, "<gq_end_of_command>", -1, nullptr, nullptr);
-
-	g_string_free(out_string, TRUE);
 }
 
 static void gr_raise(const gchar *, GIOChannel *, gpointer)
@@ -1718,77 +1704,75 @@ struct RemoteCommandEntry {
 };
 
 static RemoteCommandEntry remote_commands[] = {
-	/* short, long                  callback,               extra, prefer, parameter, description */
-	{ nullptr, "--action:",          gr_action,            TRUE,  FALSE, N_("<ACTION>"), N_("execute keyboard action (See Help/Reference/Remote Keyboard Actions)") },
-	{ nullptr, "--action-list",          gr_action_list,    FALSE,  FALSE, nullptr, N_("list available keyboard actions (some are redundant)") },
-	{ "-b", "--back",               gr_image_prev,          FALSE, FALSE, nullptr, N_("previous image") },
-	{ nullptr, "--close-window",       gr_close_window,        FALSE, FALSE, nullptr, N_("close window") },
-	{ nullptr, "--config-load:",       gr_config_load,         TRUE,  FALSE, N_("<FILE>|layout ID"), N_("load configuration from FILE") },
-	{ "-cm","--cache-metadata",      gr_cache_metadata,               FALSE, FALSE, nullptr, N_("clean the metadata cache") },
-	{ "-cr:", "--cache-render:",    gr_cache_render,        TRUE, FALSE, N_("<folder>  "), N_(" render thumbnails") },
-	{ "-crr:", "--cache-render-recurse:", gr_cache_render_recurse, TRUE, FALSE, N_("<folder> "), N_("render thumbnails recursively") },
-	{ "-crs:", "--cache-render-shared:", gr_cache_render_standard, TRUE, FALSE, N_("<folder> "), N_(" render thumbnails (see Help)") },
-	{ "-crsr:", "--cache-render-shared-recurse:", gr_cache_render_standard_recurse, TRUE, FALSE, N_("<folder>"), N_(" render thumbnails recursively (see Help)") },
-	{ "-cs:", "--cache-shared:",    gr_cache_shared,        TRUE, FALSE, N_("clear|clean"), N_("clear or clean shared thumbnail cache") },
-	{ "-ct:", "--cache-thumbs:",    gr_cache_thumb,         TRUE, FALSE, N_("clear|clean"), N_("clear or clean thumbnail cache") },
-	{ "-d", "--delay=",             gr_slideshow_delay,     TRUE,  FALSE, N_("<[H:][M:][N][.M]>"), N_("set slide show delay to Hrs Mins N.M seconds") },
-	{ nullptr, "--first",              gr_image_first,         FALSE, FALSE, nullptr, N_("first image") },
-	{ "-f", "--fullscreen",         gr_fullscreen_toggle,   FALSE, TRUE,  nullptr, N_("toggle full screen") },
-	{ nullptr, "--file:",              gr_file_load,           TRUE,  FALSE, N_("<FILE>|<URL>"), N_("open FILE or URL, bring Geeqie window to the top") },
-	{ nullptr, "file:",                gr_file_load,           TRUE,  FALSE, N_("<FILE>|<URL>"), N_("open FILE or URL, bring Geeqie window to the top") },
-	{ nullptr, "--File:",              gr_file_load_no_raise,  TRUE,  FALSE, N_("<FILE>|<URL>"), N_("open FILE or URL, do not bring Geeqie window to the top") },
-	{ nullptr, "File:",                gr_file_load_no_raise,  TRUE,  FALSE, N_("<FILE>|<URL>"), N_("open FILE or URL, do not bring Geeqie window to the top") },
-	{ "-fs","--fullscreen-start",   gr_fullscreen_start,    FALSE, FALSE, nullptr, N_("start full screen") },
-	{ "-fS","--fullscreen-stop",    gr_fullscreen_stop,     FALSE, FALSE, nullptr, N_("stop full screen") },
-	{ nullptr, "--geometry=",          gr_geometry,            TRUE, FALSE, N_("<GEOMETRY>"), N_("set window geometry") },
-	{ nullptr, "--get-collection:",    gr_collection,          TRUE,  FALSE, N_("<COLLECTION>"), N_("get collection content") },
-	{ nullptr, "--get-collection-list", gr_collection_list,    FALSE, FALSE, nullptr, N_("get collection list") },
-	{ nullptr, "--get-destination:",  	gr_get_destination,     TRUE,  FALSE, N_("<FILE>"), N_("get destination path of FILE (See Plugins Configuration)") },
-	{ nullptr, "--get-file-info",      gr_file_info,           FALSE, FALSE, nullptr, N_("get file info") },
-	{ nullptr, "--get-filelist:",      gr_filelist,            TRUE,  FALSE, N_("[<FOLDER>]"), N_("get list of files and class") },
-	{ nullptr, "--get-filelist-recurse:", gr_filelist_recurse, TRUE,  FALSE, N_("[<FOLDER>]"), N_("get list of files and class recursive") },
-	{ nullptr, "--get-rectangle",      gr_rectangle,           FALSE, FALSE, nullptr, N_("get rectangle co-ordinates") },
-	{ nullptr, "--get-render-intent",  gr_render_intent,       FALSE, FALSE, nullptr, N_("get render intent") },
-	{ nullptr, "--get-selection",      gr_get_selection,       FALSE, FALSE, nullptr, N_("get list of selected files") },
-	{ nullptr, "--get-sidecars:",      gr_get_sidecars,        TRUE,  FALSE, N_("<FILE>"), N_("get list of sidecars of FILE") },
-	{ nullptr, "--get-window-list",    gr_window_list,         FALSE, FALSE, nullptr, N_("get window list") },
-	{ nullptr, "--id:",                gr_lw_id,               TRUE, FALSE, N_("<ID>"), N_("window id for following commands") },
-	{ nullptr, "--last",               gr_image_last,          FALSE, FALSE, nullptr, N_("last image") },
-	{ nullptr, "--list-add:",          gr_list_add,            TRUE,  FALSE, N_("<FILE>"), N_("add FILE to command line collection list") },
-	{ nullptr, "--list-clear",         gr_list_clear,          FALSE, FALSE, nullptr, N_("clear command line collection list") },
+	/* short,     long                    callback,                extra, prefer,  parameter,               description */
+	{ nullptr, "--action=",               gr_action,               TRUE,  FALSE, N_("<ACTION>"),          N_("execute keyboard action (See Help/Reference/Remote Keyboard Actions)") },
+	{ nullptr, "--action-list",           gr_action_list,          FALSE, FALSE, nullptr,                 N_("list available keyboard actions (some are redundant)") },
+	{ "-b",    "--back",                  gr_image_prev,           FALSE, FALSE, nullptr,                 N_("previous image") },
+	{ nullptr, "--close-window",          gr_close_window,         FALSE, FALSE, nullptr,                 N_("close window") },
+	{ nullptr, "--config-load=",          gr_config_load,          TRUE,  FALSE, N_("<FILE>|layout ID"),  N_("load configuration from FILE") },
+	{ nullptr, "--cache-metadata",        gr_cache_metadata,       FALSE, FALSE, nullptr,                 N_("clean the metadata cache") },
+	{ nullptr, "--cache-render=",         gr_cache_render,         TRUE,  FALSE, N_("<folder>  "),        N_(" render thumbnails") },
+	{ nullptr, "--cache-render-recurse=", gr_cache_render_recurse, TRUE,  FALSE, N_("<folder> "),         N_("render thumbnails recursively") },
+	{ nullptr, "--cache-render-shared=",  gr_cache_render_standard,TRUE,  FALSE, N_("<folder> "),         N_(" render thumbnails (see Help)") },
+	{ nullptr, "--cache-render-shared-recurse=", gr_cache_render_standard_recurse, TRUE, FALSE, N_("<folder>"), N_(" render thumbnails recursively (see Help)") },
+	{ nullptr, "--cache-shared=",         gr_cache_shared,         TRUE,  FALSE, N_("clean|clear"),       N_("clean or clear shared thumbnail cache") },
+	{ nullptr, "--cache-thumbs=",         gr_cache_thumb,          TRUE,  FALSE, N_("clean|clear"),       N_("clean or clear thumbnail cache") },
+	{ "-d",    "--delay=",                gr_slideshow_delay,      TRUE,  FALSE, N_("<[H:][M:][N][.M]>"), N_("set slide show delay to Hrs Mins N.M seconds") },
+	{ nullptr, "--first",                 gr_image_first,          FALSE, FALSE, nullptr,                 N_("first image") },
+	{ "-f",    "--fullscreen",            gr_fullscreen_toggle,    FALSE, TRUE,  nullptr,                 N_("toggle full screen") },
+	{ nullptr, "--file=",                 gr_file_load,            TRUE,  FALSE, N_("<FILE>|<URL>"),      N_("open FILE or URL, bring Geeqie window to the top") },
+	{ nullptr, "file=",                   gr_file_load,            TRUE,  FALSE, N_("<FILE>|<URL>"),      N_("open FILE or URL, bring Geeqie window to the top") },
+	{ nullptr, "--File=",                 gr_file_load_no_raise,   TRUE,  FALSE, N_("<FILE>|<URL>"),      N_("open FILE or URL, do not bring Geeqie window to the top") },
+	{ nullptr, "File=",                   gr_file_load_no_raise,   TRUE,  FALSE, N_("<FILE>|<URL>"),      N_("open FILE or URL, do not bring Geeqie window to the top") },
+	{ nullptr, "--fullscreen-start",      gr_fullscreen_start,     FALSE, FALSE, nullptr,                 N_("start full screen") },
+	{ nullptr, "--fullscreen-stop",       gr_fullscreen_stop,      FALSE, FALSE, nullptr,                 N_("stop full screen") },
+	{ nullptr, "--geometry=",             gr_geometry,             TRUE,  FALSE, N_("<GEOMETRY>"),        N_("set window geometry") },
+	{ nullptr, "--get-collection=",       gr_collection,           TRUE,  FALSE, N_("<COLLECTION>"),      N_("get collection content") },
+	{ nullptr, "--get-collection-list",   gr_collection_list,      FALSE, FALSE, nullptr,                 N_("get collection list") },
+	{ nullptr, "--get-destination=",      gr_get_destination,      TRUE,  FALSE, N_("<FILE>"),            N_("get destination path of FILE (See Plugins Configuration)") },
+	{ nullptr, "--get-file-info",         gr_file_info,            FALSE, FALSE, nullptr,                 N_("get file info") },
+	{ nullptr, "--get-filelist=",         gr_filelist,             TRUE,  FALSE, N_("[<FOLDER>]"),        N_("get list of files and class") },
+	{ nullptr, "--get-filelist-recurse=", gr_filelist_recurse,     TRUE,  FALSE, N_("[<FOLDER>]"),        N_("get list of files and class recursive") },
+	{ nullptr, "--get-rectangle",         gr_rectangle,            FALSE, FALSE, nullptr,                 N_("get rectangle co-ordinates") },
+	{ nullptr, "--get-render-intent",     gr_render_intent,        FALSE, FALSE, nullptr,                 N_("get render intent") },
+	{ nullptr, "--get-selection",         gr_get_selection,        FALSE, FALSE, nullptr,                 N_("get list of selected files") },
+	{ nullptr, "--get-sidecars=",         gr_get_sidecars,         TRUE,  FALSE, N_("<FILE>"),            N_("get list of sidecars of FILE") },
+	{ nullptr, "--get-window-list",       gr_window_list,          FALSE, FALSE, nullptr,                 N_("get window list") },
+	{ nullptr, "--id=",                   gr_lw_id,                TRUE,  FALSE, N_("<ID>"),              N_("window id for following commands") },
+	{ nullptr, "--last",                  gr_image_last,           FALSE, FALSE, nullptr,                 N_("last image") },
+	{ nullptr, "--list-add=",             gr_list_add,             TRUE,  FALSE, N_("<FILE>"),            N_("add FILE to command line collection list") },
+	{ nullptr, "--list-clear",            gr_list_clear,           FALSE, FALSE, nullptr,                 N_("clear command line collection list") },
 #if HAVE_LUA
-	{ nullptr, "--lua:",               gr_lua,                 TRUE, FALSE, N_("<FILE>,<lua script>"), N_("run lua script on FILE") },
+	{ nullptr, "--lua=",                  gr_lua,                  TRUE,  FALSE, N_("<FILE>,<lua script>"), N_("run lua script on FILE") },
 #endif
-	{ nullptr, "--new-window",         gr_new_window,          FALSE, FALSE, nullptr, N_("new window") },
-	{ "-n", "--next",               gr_image_next,          FALSE, FALSE, nullptr, N_("next image") },
-	{ nullptr, "--pixel-info",         gr_pixel_info,          FALSE, FALSE, nullptr, N_("print pixel info of mouse pointer on current image") },
-	{ nullptr, "--print0",             gr_print0,              TRUE, FALSE, nullptr, N_("terminate returned data with null character instead of newline") },
-	{ nullptr, "--PWD:",               gr_pwd,                 TRUE, FALSE, N_("<PWD>"), N_("use PWD as working directory for following commands") },
-	{ "-q", "--quit",               gr_quit,                FALSE, FALSE, nullptr, N_("quit") },
-	{ nullptr, "--raise",              gr_raise,               FALSE, FALSE, nullptr, N_("bring the Geeqie window to the top") },
-	{ nullptr, "raise",                gr_raise,               FALSE, FALSE, nullptr, N_("bring the Geeqie window to the top") },
-	{ nullptr, "--selection-add:",     gr_selection_add,       TRUE,  FALSE, N_("[<FILE>]"), N_("adds the current file (or the specified file) to the current selection") },
-	{ nullptr, "--selection-clear",    gr_selection_clear,     FALSE, FALSE, nullptr, N_("clears the current selection") },
-	{ nullptr, "--selection-remove:",  gr_selection_remove,    TRUE,  FALSE, N_("[<FILE>]"), N_("removes the current file (or the specified file) from the current selection") },
-	{ "-s", "--slideshow",          gr_slideshow_toggle,    FALSE, TRUE,  nullptr, N_("toggle slide show") },
-	{ nullptr, "--slideshow-recurse:", gr_slideshow_start_rec, TRUE,  FALSE, N_("<FOLDER>"), N_("start recursive slide show in FOLDER") },
-	{ "-ss","--slideshow-start",    gr_slideshow_start,     FALSE, FALSE, nullptr, N_("start slide show") },
-	{ "-sS","--slideshow-stop",     gr_slideshow_stop,      FALSE, FALSE, nullptr, N_("stop slide show") },
-	{ nullptr, "--tell",               gr_file_tell,           FALSE, FALSE, nullptr, N_("print filename [and Collection] of current image") },
-	{ "+t", "--tools-show",         gr_tools_show,          FALSE, TRUE,  nullptr, N_("show tools") },
-	{ "-t", "--tools-hide",	        gr_tools_hide,          FALSE, TRUE,  nullptr, N_("hide tools") },
-	{ nullptr, "--view:",              gr_file_view,           TRUE,  FALSE, N_("<FILE>"), N_("open FILE in new window") },
-	{ nullptr, "view:",                gr_file_view,           TRUE,  FALSE, N_("<FILE>"), N_("open FILE in new window") },
+	{ nullptr, "--new-window",            gr_new_window,           FALSE, FALSE, nullptr,                 N_("new window") },
+	{ "-n",    "--next",                  gr_image_next,           FALSE, FALSE, nullptr,                 N_("next image") },
+	{ nullptr, "--pixel-info",            gr_pixel_info,           FALSE, FALSE, nullptr,                 N_("print pixel info of mouse pointer on current image") },
+	{ nullptr, "--print0",                gr_print0,               TRUE,  FALSE, nullptr,                 N_("terminate returned data with null character instead of newline") },
+	{ nullptr, "--PWD=",                  gr_pwd,                  TRUE,  FALSE, N_("<PWD>"),             N_("use PWD as working directory for following commands") },
+	{ "-q",    "--quit",                  gr_quit,                 FALSE, FALSE, nullptr,                 N_("quit") },
+	{ nullptr, "--raise",                 gr_raise,                FALSE, FALSE, nullptr,                 N_("bring the Geeqie window to the top") },
+	{ nullptr, "raise",                   gr_raise,                FALSE, FALSE, nullptr,                 N_("bring the Geeqie window to the top") },
+	{ nullptr, "--selection-add=",        gr_selection_add,        TRUE,  FALSE, N_("[<FILE>]"),          N_("adds the current file (or the specified file) to the current selection") },
+	{ nullptr, "--selection-clear",       gr_selection_clear,      FALSE, FALSE, nullptr,                 N_("clears the current selection") },
+	{ nullptr, "--selection-remove=",     gr_selection_remove,     TRUE,  FALSE, N_("[<FILE>]"),          N_("removes the current file (or the specified file) from the current selection") },
+	{ "-s",    "--slideshow",             gr_slideshow_toggle,     FALSE, TRUE,  nullptr,                 N_("toggle slide show") },
+	{ nullptr, "--slideshow-recurse=",    gr_slideshow_start_rec,  TRUE,  FALSE, N_("<FOLDER>"),          N_("start recursive slide show in FOLDER") },
+	{ nullptr, "--slideshow-start",       gr_slideshow_start,      FALSE, FALSE, nullptr,                 N_("start slide show") },
+	{ nullptr, "--slideshow-stop",        gr_slideshow_stop,       FALSE, FALSE, nullptr,                 N_("stop slide show") },
+	{ nullptr, "--tell",                  gr_file_tell,            FALSE, FALSE, nullptr,                 N_("print filename [and Collection] of current image") },
+	{ "-T",    "--tools-show",            gr_tools_show,           FALSE, TRUE,  nullptr,                 N_("show tools") },
+	{ "-t",    "--tools-hide",	          gr_tools_hide,           FALSE, TRUE,  nullptr,                 N_("hide tools") },
+	{ nullptr, "--view=",                 gr_file_view,            TRUE,  FALSE, N_("<FILE>"),            N_("open FILE in new window") },
+	{ nullptr, "view=",                   gr_file_view,            TRUE,  FALSE, N_("<FILE>"),            N_("open FILE in new window") },
 	{ nullptr, nullptr, nullptr, FALSE, FALSE, nullptr, nullptr }
 };
 
 static RemoteCommandEntry *remote_command_find(const gchar *text, const gchar **offset)
 {
-	gboolean match = FALSE;
-	gint i;
+	gint i = 0;
 
-	i = 0;
-	while (!match && remote_commands[i].func != nullptr)
+	while (remote_commands[i].func != nullptr)
 		{
 		if (remote_commands[i].needs_extra)
 			{
@@ -1822,6 +1806,15 @@ static RemoteCommandEntry *remote_command_find(const gchar *text, const gchar **
 	return nullptr;
 }
 
+gboolean is_remote_command(const gchar *text)
+{
+	RemoteCommandEntry *entry = nullptr;
+
+	entry = remote_command_find(text, nullptr);
+
+	return entry ? TRUE : FALSE;
+}
+
 static void remote_cb(RemoteConnection *, const gchar *text, GIOChannel *channel, gpointer data)
 {
 	RemoteCommandEntry *entry;
@@ -1863,7 +1856,7 @@ void remote_help()
 			}
 		i++;
 		}
-	printf_term(FALSE, _("\n\n  All other command line parameters are used as plain files if they exist.\n\n  The name of a collection, with or without either path or extension (.gqv) may be used.\n"));
+	printf_term(FALSE, "%s", _("\n\n  All other command line parameters are used as plain files if they exist.\n\n  The name of a collection, with or without either path or extension (.gqv) may be used.\n"));
 }
 
 GList *remote_build_list(GList *list, gint argc, gchar *argv[], GList **errors)
@@ -1940,7 +1933,7 @@ void remote_control(const gchar *arg_exec, GList *remote_list, const gchar *path
 					g_string_append(command, " ");
 					g_string_append(command, text);
 					}
-				if (entry->opt_l && strcmp(entry->opt_l, "file:") == 0)
+				if (entry->opt_l && strcmp(entry->opt_l, "file=") == 0)
 					{
 					blank = TRUE;
 					}
@@ -1987,7 +1980,7 @@ void remote_control(const gchar *arg_exec, GList *remote_list, const gchar *path
 			entry = remote_command_find(text, nullptr);
 			if (entry &&
 			    entry->opt_l &&
-			    strcmp(entry->opt_l, "file:") == 0) use_path = FALSE;
+			    strcmp(entry->opt_l, "file=") == 0) use_path = FALSE;
 
 			remote_client_send(rc, text);
 
@@ -1996,12 +1989,12 @@ void remote_control(const gchar *arg_exec, GList *remote_list, const gchar *path
 
 		if (cmd_list && cmd_list->next)
 			{
-			prefix = "--list-add:";
+			prefix = "--list-add=";
 			remote_client_send(rc, "--list-clear");
 			}
 		else
 			{
-			prefix = "file:";
+			prefix = "file=";
 			}
 
 		work = cmd_list;
@@ -2021,7 +2014,7 @@ void remote_control(const gchar *arg_exec, GList *remote_list, const gchar *path
 			{
 			gchar *text;
 
-			text = g_strdup_printf("file:%s", path);
+			text = g_strdup_printf("file=%s", path);
 			remote_client_send(rc, text);
 			g_free(text);
 
@@ -2037,7 +2030,7 @@ void remote_control(const gchar *arg_exec, GList *remote_list, const gchar *path
 			name = static_cast<const gchar *>(work->data);
 			work = work->next;
 
-			text = g_strdup_printf("file:%s", name);
+			text = g_strdup_printf("file=%s", name);
 			remote_client_send(rc, text);
 			g_free(text);
 
