@@ -21,7 +21,11 @@
 
 #include "similar.h"
 
+#include <algorithm>
+#include <cmath>
 #include <cstdlib>
+#include <functional>
+#include <vector>
 
 #include "options.h"
 
@@ -51,6 +55,86 @@
  * find images that have been re-saved to other formats, dimensions, or compression.
  */
 
+namespace
+{
+
+using ImageSimilarityCheckAbort = std::function<bool(gdouble)>;
+
+void image_sim_channel_equal(guint8 *pix, gsize len)
+{
+	struct IndexedPix
+	{
+		gsize index;
+		guint8 pix;
+	};
+
+	std::vector<IndexedPix> buf;
+	buf.reserve(len);
+
+	for (gsize i = 0; i < len; i++)
+		{
+		buf.push_back({i, pix[i]});
+		}
+
+	std::sort(buf.begin(), buf.end(), [](const IndexedPix &a, const IndexedPix &b){ return a.pix < b.pix; });
+
+	for (gsize i = 0; i < len; i++)
+		{
+		gint n = buf[i].index;
+
+		pix[n] = static_cast<guint8>(255 * i / len);
+		}
+}
+
+/*
+ * 4 rotations (0, 90, 180, 270) combined with two mirrors (0, H)
+ * generate all possible isometric transformations
+ * = 8 tests
+ * = change dir of x, change dir of y, exchange x and y = 2^3 = 8
+ */
+gdouble image_sim_data_compare_transfo(const ImageSimilarityData *a, const ImageSimilarityData *b, gchar transfo, const ImageSimilarityCheckAbort &check_abort)
+{
+	if (!a || !b || !a->filled || !b->filled) return 0.0;
+
+	gint sim = 0.0;
+	gint i2;
+	gint *i;
+	gint j2;
+	gint *j;
+
+	if (transfo & 1) { i = &j2; j = &i2; } else { i = &i2; j = &j2; }
+	for (gint j1 = 0; j1 < 32; j1++)
+		{
+		if (transfo & 2) *j = 31-j1; else *j = j1;
+		for (gint i1 = 0; i1 < 32; i1++)
+			{
+			if (transfo & 4) *i = 31-i1; else *i = i1;
+			sim += abs(a->avg_r[i1*32+j1] - b->avg_r[i2*32+j2]);
+			sim += abs(a->avg_g[i1*32+j1] - b->avg_g[i2*32+j2]);
+			sim += abs(a->avg_b[i1*32+j1] - b->avg_b[i2*32+j2]);
+			/* check for abort, if so return 0.0 */
+			if (check_abort(sim)) return 0.0;
+			}
+		}
+
+	return 1.0 - (static_cast<gdouble>(sim) / (255.0 * 1024.0 * 3.0));
+}
+
+gdouble image_sim_data_compare(const ImageSimilarityData *a, const ImageSimilarityData *b, const ImageSimilarityCheckAbort &check_abort)
+{
+	gchar max_t = (options->rot_invariant_sim ? 8 : 1);
+	gdouble max_score = 0;
+
+	for (gchar t = 0; t < max_t; t++)
+	{
+		max_score = std::max(image_sim_data_compare_transfo(a, b, t, check_abort), max_score);
+	}
+
+	return max_score;
+}
+
+} // namespace
+
 ImageSimilarityData *image_sim_new()
 {
 	auto sd = g_new0(ImageSimilarityData, 1);
@@ -63,47 +147,6 @@ void image_sim_free(ImageSimilarityData *sd)
 	g_free(sd);
 }
 
-static gint image_sim_channel_eq_sort_cb(gconstpointer a, gconstpointer b)
-{
-	auto pa = static_cast<const gint *>(a);
-	auto pb = static_cast<const gint *>(b);
-	if (pa[1] < pb[1]) return -1;
-	if (pa[1] > pb[1]) return 1;
-	return 0;
-}
-
-static void image_sim_channel_equal(guint8 *pix, gint len)
-{
-	gint *buf;
-	gint i;
-	gint p;
-
-	buf = g_new0(gint, len * 2);
-
-	p = 0;
-	for (i = 0; i < len; i++)
-		{
-		buf[p] = i;
-		p++;
-		buf[p] = static_cast<gint>(pix[i]);
-		p++;
-		}
-
-	qsort(buf, len, sizeof(gint) * 2, image_sim_channel_eq_sort_cb);
-
-	p = 0;
-	for (i = 0; i < len; i++)
-		{
-		gint n;
-
-		n = buf[p];
-		p += 2;
-		pix[n] = static_cast<guint8>(255 * i / len);
-		}
-
-	g_free(buf);
-}
-
 static void image_sim_channel_norm(guint8 *pix, gint len)
 {
 	guint8 l;
@@ -162,13 +205,6 @@ void image_sim_alternate_processing(ImageSimilarityData *sd)
 		}
 }
 
-gint mround(gdouble x)
-{
-	gint ipart = x;
-	gdouble fpart = x-ipart;
-	return (fpart < 0.5 ? ipart : ipart+1);
-}
-
 void image_sim_fill_data(ImageSimilarityData *sd, GdkPixbuf *pixbuf)
 {
 	gint w;
@@ -218,11 +254,10 @@ void image_sim_fill_data(ImageSimilarityData *sd, GdkPixbuf *pixbuf)
 
 	j = 0;
 
-	h_left = h;
 	for (ys = 0; ys < 32; ys++)
 		{
 		if (y_small) j = static_cast<gdouble>(h) / 32 * ys;
-		else y_inc = mround(static_cast<gdouble>(h_left)/(32-ys));
+		else y_inc = std::lround(static_cast<gdouble>(h_left)/(32-ys));
 		i = 0;
 
 		w_left = w;
@@ -237,7 +272,7 @@ void image_sim_fill_data(ImageSimilarityData *sd, GdkPixbuf *pixbuf)
 			guchar *xpos;
 
 			if (x_small) i = static_cast<gdouble>(w) / 32 * xs;
-			else x_inc = mround(static_cast<gdouble>(w_left)/(32-xs));
+			else x_inc = std::lround(static_cast<gdouble>(w_left)/(32-xs));
 			xy_inc = x_inc * y_inc;
 			r = g = b = 0;
 			xpos = pix + (i * p_step);
@@ -284,7 +319,7 @@ ImageSimilarityData *image_sim_new_from_pixbuf(GdkPixbuf *pixbuf)
 	return sd;
 }
 
-static gdouble alternate_image_sim_compare_fast(ImageSimilarityData *a, ImageSimilarityData *b, gdouble min)
+static gdouble alternate_image_sim_compare_fast(const ImageSimilarityData *a, const ImageSimilarityData *b, gdouble min)
 {
 	gint sim;
 	gint i;
@@ -293,7 +328,6 @@ static gdouble alternate_image_sim_compare_fast(ImageSimilarityData *a, ImageSim
 
 	if (!a || !b || !a->filled || !b->filled) return 0.0;
 
-	min = 1.0 - min;
 	sim = 0.0;
 	ld = 0;
 
@@ -321,95 +355,9 @@ static gdouble alternate_image_sim_compare_fast(ImageSimilarityData *a, ImageSim
 	return (1.0 - ((gdouble)sim / (255.0 * 1024.0 * 4.0)) );
 }
 
-gdouble image_sim_compare_transfo(ImageSimilarityData *a, ImageSimilarityData *b, gchar transfo)
-{
-	gint sim;
-	gint i1;
-	gint i2;
-	gint *i;
-	gint j1;
-	gint j2;
-	gint *j;
-
-	if (!a || !b || !a->filled || !b->filled) return 0.0;
-
-	sim = 0.0;
-
-	if (transfo & 1) { i = &j2; j = &i2; } else { i = &i2; j = &j2; }
-	for (j1 = 0; j1 < 32; j1++)
-		{
-		if (transfo & 2) *j = 31-j1; else *j = j1;
-		for (i1 = 0; i1 < 32; i1++)
-			{
-			if (transfo & 4) *i = 31-i1; else *i = i1;
-			sim += abs(a->avg_r[i1*32+j1] - b->avg_r[i2*32+j2]);
-			sim += abs(a->avg_g[i1*32+j1] - b->avg_g[i2*32+j2]);
-			sim += abs(a->avg_b[i1*32+j1] - b->avg_b[i2*32+j2]);
-			}
-		}
-
-	return 1.0 - (static_cast<gdouble>(sim) / (255.0 * 1024.0 * 3.0));
-}
-
 gdouble image_sim_compare(ImageSimilarityData *a, ImageSimilarityData *b)
 {
-	gint max_t = (options->rot_invariant_sim ? 8 : 1);
-
-	gint t;
-	gdouble score;
-	gdouble max_score = 0;
-
-	for(t = 0; t < max_t; t++)
-	{
-		score = image_sim_compare_transfo(a, b, t);
-		if (score > max_score) max_score = score;
-	}
-	return max_score;
-}
-
-
-/*
-4 rotations (0, 90, 180, 270) combined with two mirrors (0, H)
-generate all possible isometric transformations
-= 8 tests
-= change dir of x, change dir of y, exchange x and y = 2^3 = 8
-*/
-gdouble image_sim_compare_fast_transfo(ImageSimilarityData *a, ImageSimilarityData *b, gdouble min, gchar transfo)
-{
-	gint sim;
-	gint i1;
-	gint i2;
-	gint *i;
-	gint j1;
-	gint j2;
-	gint *j;
-
-	if (options->alternate_similarity_algorithm.enabled)
-		{
-		return alternate_image_sim_compare_fast(a, b, min);
-		}
-
-	if (!a || !b || !a->filled || !b->filled) return 0.0;
-
-	min = 1.0 - min;
-	sim = 0.0;
-
-	if (transfo & 1) { i = &j2; j = &i2; } else { i = &i2; j = &j2; }
-	for (j1 = 0; j1 < 32; j1++)
-		{
-		if (transfo & 2) *j = 31-j1; else *j = j1;
-		for (i1 = 0; i1 < 32; i1++)
-			{
-			if (transfo & 4) *i = 31-i1; else *i = i1;
-			sim += abs(a->avg_r[i1*32+j1] - b->avg_r[i2*32+j2]);
-			sim += abs(a->avg_g[i1*32+j1] - b->avg_g[i2*32+j2]);
-			sim += abs(a->avg_b[i1*32+j1] - b->avg_b[i2*32+j2]);
-			}
-		/* check for abort, if so return 0.0 */
-		if (static_cast<gdouble>(sim) / (255.0 * 1024.0 * 3.0) > min) return 0.0;
-		}
-
-	return (1.0 - (static_cast<gdouble>(sim) / (255.0 * 1024.0 * 3.0)) );
+	return image_sim_data_compare(a, b, [](gdouble){ return false; });
 }
 
 /* this uses a cutoff point so that it can abort early when it gets to
@@ -417,17 +365,13 @@ gdouble image_sim_compare_fast_transfo(ImageSimilarityData *a, ImageSimilarityDa
  */
 gdouble image_sim_compare_fast(ImageSimilarityData *a, ImageSimilarityData *b, gdouble min)
 {
-	gint max_t = (options->rot_invariant_sim ? 8 : 1);
+	min = 1.0 - min;
 
-	gint t;
-	gdouble score;
-	gdouble max_score = 0;
+	if (options->alternate_similarity_algorithm.enabled)
+		{
+		return alternate_image_sim_compare_fast(a, b, min);
+		}
 
-	for(t = 0; t < max_t; t++)
-	{
-		score = image_sim_compare_fast_transfo(a, b, min, t);
-		if (score > max_score) max_score = score;
-	}
-	return max_score;
+	return image_sim_data_compare(a, b, [min](gdouble sim){ return (sim / (255.0 * 1024.0 * 3.0)) > min; });
 }
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
