@@ -26,6 +26,7 @@
 #include <cstdlib>
 #include <cstring>
 #include <ctime>
+#include <string>
 
 #include <glib-object.h>
 
@@ -353,38 +354,34 @@ static void thumb_loader_std_save(ThumbLoaderStd *tl, GdkPixbuf *pixbuf)
 		{
 		const gchar *mark_uri;
 		gchar *mark_app;
-		gchar *mark_mtime;
 		gchar *pathl;
 		gboolean success;
 
 		mark_uri = (tl->cache_local) ? tl->local_uri :tl->thumb_uri;
 
 		mark_app = g_strdup_printf("%s %s", GQ_APPNAME, VERSION);
-		mark_mtime = g_strdup_printf("%llu", static_cast<unsigned long long>(tl->source_mtime));
+		const std::string mark_mtime = std::to_string(static_cast<unsigned long long>(tl->source_mtime));
 		pathl = path_from_utf8(tmp_path);
 		success = gdk_pixbuf_save(pixbuf, pathl, "png", nullptr,
-					  THUMB_MARKER_URI, mark_uri,
-					  THUMB_MARKER_MTIME, mark_mtime,
-					  THUMB_MARKER_APP, mark_app,
-					  NULL);
+		                          THUMB_MARKER_URI, mark_uri,
+		                          THUMB_MARKER_MTIME, mark_mtime.c_str(),
+		                          THUMB_MARKER_APP, mark_app,
+		                          NULL);
 		if (success)
 			{
-			chmod(pathl, (tl->cache_local) ? tl->source_mode : S_IRUSR & S_IWUSR);
+			chmod(pathl, (tl->cache_local) ? tl->source_mode : S_IRUSR | S_IWUSR);
 			success = rename_file(tmp_path, tl->thumb_path);
 			}
 
 		g_free(pathl);
-
-		g_free(mark_mtime);
 		g_free(mark_app);
-
 		g_free(tmp_path);
+
 		if (!success)
 			{
 			DEBUG_1("thumb save failed: %s", tl->fd->path);
 			DEBUG_1("            thumb: %s", tl->thumb_path);
 			}
-
 		}
 
 	g_object_unref(G_OBJECT(pixbuf));
@@ -560,7 +557,7 @@ static GdkPixbuf *thumb_loader_std_finish(ThumbLoaderStd *tl, GdkPixbuf *pixbuf,
 				struct stat st;
 
 				if (pixbuf_scale_aspect(cache_w, cache_h, sw, sh,
-								  &thumb_w, &thumb_h))
+				                        thumb_w, thumb_h))
 					{
 					pixbuf_thumb = gdk_pixbuf_scale_simple(pixbuf, thumb_w, thumb_h,
 									       static_cast<GdkInterpType>(options->thumbnails.quality));
@@ -581,8 +578,7 @@ static GdkPixbuf *thumb_loader_std_finish(ThumbLoaderStd *tl, GdkPixbuf *pixbuf,
 					}
 				}
 			}
-		else if (tl->cache_hit &&
-			 tl->cache_local && !tl->thumb_path_local)
+		else if (tl->cache_local && !tl->thumb_path_local)
 			{
 			/* A local cache save was requested, but a valid thumb is in $HOME,
 			 * so specifically save as a local thumbnail.
@@ -616,7 +612,7 @@ static GdkPixbuf *thumb_loader_std_finish(ThumbLoaderStd *tl, GdkPixbuf *pixbuf,
 			}
 
 		if (pixbuf_scale_aspect(tl->requested_width, tl->requested_height, sw, sh,
-						  &thumb_w, &thumb_h))
+		                        thumb_w, thumb_h))
 			{
 			result = gdk_pixbuf_scale_simple(pixbuf, thumb_w, thumb_h,
 							 static_cast<GdkInterpType>(options->thumbnails.quality));
@@ -792,7 +788,6 @@ void thumb_loader_std_set_cache(ThumbLoaderStd *tl, gboolean enable_cache, gbool
 
 gboolean thumb_loader_std_start(ThumbLoaderStd *tl, FileData *fd)
 {
-	static gchar *thumb_cache = nullptr;
 	struct stat st;
 
 	if (!tl || !fd) return FALSE;
@@ -810,10 +805,7 @@ gboolean thumb_loader_std_start(ThumbLoaderStd *tl, FileData *fd)
 	tl->source_size = st.st_size;
 	tl->source_mode = st.st_mode;
 
-	if (!thumb_cache)
-		{
-		thumb_cache = g_strdup(get_thumbnails_standard_cache_dir());
-		}
+	static const gchar *thumb_cache = get_thumbnails_standard_cache_dir();
 
 	if (strncmp(tl->fd->path, thumb_cache, strlen(thumb_cache)) != 0)
 		{
@@ -1007,6 +999,11 @@ static gboolean thumb_loader_std_thumb_file_validate_idle_cb(gpointer data)
 	return G_SOURCE_REMOVE;
 }
 
+/**
+ * @brief Validates a non local thumbnail file,
+ * calling func_valid with the information when app is idle
+ * for thumbnail's without a file: uri, validates against allowed_age in days
+ */
 ThumbLoaderStd *thumb_loader_std_thumb_file_validate(const gchar *thumb_path, gint allowed_days,
 						     void (*func_valid)(const gchar *path, gboolean valid, gpointer data),
 						     gpointer data)
