@@ -27,7 +27,7 @@
 
 #include "typedefs.h"
 
-struct FileData;
+class FileData;
 struct ImageLoader;
 
 struct ThumbLoader
