@@ -22,6 +22,8 @@
 #ifndef TYPEDEFS_H
 #define TYPEDEFS_H
 
+#include <cstdio>
+
 #include <glib.h>
 
 enum ZoomMode {
@@ -211,6 +213,15 @@ enum StereoPixbufData {
 	STEREO_PIXBUF_NONE     = 3
 };
 
+enum ScrollReset : guint {
+	TOPLEFT  = 0,
+	CENTER   = 1,
+	NOCHANGE = 2,
+	COUNT /**< Keep it last */
+};
+
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(FILE, fclose)
+
 using FileUtilDoneFunc = void (*)(gboolean, const gchar *, gpointer);
 
 #define FILEDATA_MARKS_SIZE 10
