@@ -21,9 +21,9 @@
 
 #include "ui-bookmark.h"
 
+#include <array>
 #include <cstdlib>
 #include <cstring>
-#include <memory>
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gdk/gdk.h>
@@ -58,12 +58,18 @@
 namespace
 {
 
-constexpr gint bookmark_drop_types_n = 3;
-constexpr gint bookmark_drag_types_n = 2;
+struct BookButtonData
+{
+	GtkWidget *button;
+	GtkWidget *image;
+	GtkWidget *label;
 
-} // namespace
-
-struct BookButtonData;
+	gchar *key;
+	gchar *name;
+	gchar *path;
+	gchar *icon;
+	gchar *parent;
+};
 
 struct BookMarkData
 {
@@ -81,19 +87,6 @@ struct BookMarkData
 	BookButtonData *active_button;
 };
 
-struct BookButtonData
-{
-	GtkWidget *button;
-	GtkWidget *image;
-	GtkWidget *label;
-
-	gchar *key;
-	gchar *name;
-	gchar *path;
-	gchar *icon;
-	gchar *parent;
-};
-
 struct BookPropData
 {
 	GtkWidget *name_entry;
@@ -109,20 +102,30 @@ enum {
 	TARGET_TEXT_PLAIN
 };
 
-static GtkTargetEntry bookmark_drop_types[] = {
+constexpr std::array<GtkTargetEntry, 3> bookmark_drop_types{{
 	{ const_cast<gchar *>("text/uri-list"), 0, TARGET_URI_LIST },
 	{ const_cast<gchar *>("x-url/http"),    0, TARGET_X_URL },
 	{ const_cast<gchar *>("_NETSCAPE_URL"), 0, TARGET_X_URL }
-};
+}};
 
-static GtkTargetEntry bookmark_drag_types[] = {
+constexpr std::array<GtkTargetEntry, 2> bookmark_drag_types{{
 	{ const_cast<gchar *>("text/uri-list"), 0, TARGET_URI_LIST },
 	{ const_cast<gchar *>("text/plain"),    0, TARGET_TEXT_PLAIN }
-};
+}};
 
-static GList *bookmark_widget_list = nullptr;
-static GList *bookmark_default_list = nullptr;
+GList *bookmark_widget_list = nullptr;
+GList *bookmark_default_list = nullptr;
 
+const gchar *bookmark_icon(const gchar *path)
+{
+	if (!isfile(path)) return nullptr;
+
+	g_autofree gchar *real_path = realpath(path, nullptr);
+
+	return strstr(real_path, get_collections_dir()) ? PIXBUF_INLINE_COLLECTION : GQ_ICON_FILE;
+}
+
+} // namespace
 
 static void bookmark_populate_all(const gchar *key);
 
@@ -256,8 +259,8 @@ static void bookmark_edit_ok_cb(GenericDialog *, gpointer data)
 		history_list_add_to_key(p->bb->parent, new_string, 0);
 		}
 
-	if (path && strlen(path) > 0) tab_completion_append_to_history(p->path_entry, path);
-	if (icon && strlen(icon) > 0) tab_completion_append_to_history(p->icon_entry, icon);
+	if (path && path[0] != '\0') tab_completion_append_to_history(p->path_entry, path);
+	if (icon && icon[0] != '\0') tab_completion_append_to_history(p->icon_entry, icon);
 
 	g_free(path);
 	g_free(new_string);
@@ -457,6 +460,8 @@ static gboolean bookmark_keypress_cb(GtkWidget *button, GdkEventKey *event, gpoi
 				return TRUE;
 				}
 			break;
+		default:
+			break;
 		}
 
 	return FALSE;
@@ -695,8 +700,8 @@ static void bookmark_populate(BookMarkData *bm)
 					 G_CALLBACK(bookmark_keypress_cb), bm);
 
 			gtk_drag_source_set(b->button, GDK_BUTTON1_MASK,
-					    bookmark_drag_types, bookmark_drag_types_n,
-					    static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
+			                    bookmark_drag_types.data(), bookmark_drag_types.size(),
+			                    static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
 			g_signal_connect(G_OBJECT(b->button), "drag_data_get",
 					 G_CALLBACK(bookmark_drag_set_data), bm);
 			g_signal_connect(G_OBJECT(b->button), "drag_begin",
@@ -739,8 +744,6 @@ static void bookmark_dnd_get_data(GtkWidget *, GdkDragContext *,
 	auto bm = static_cast<BookMarkData *>(data);
 	GList *list = nullptr;
 	GList *errors = nullptr;
-	GList *work;
-	gchar *real_path;
 	gchar **uris;
 
 	if (!bm->editable) return;
@@ -756,33 +759,14 @@ static void bookmark_dnd_get_data(GtkWidget *, GdkDragContext *,
 			}
 		g_strfreev(uris);
 
-		work = list;
-		while (work)
+		for (GList *work = list; work; work = work->next)
 			{
 			auto path = static_cast<gchar *>(work->data);
-			gchar *buf;
-
-			work = work->next;
 
 			if (bm->only_directories && !isdir(path)) continue;
 
-			real_path = realpath(path, nullptr);
-
-			if (strstr(real_path, get_collections_dir()) && isfile(path))
-				{
-				buf = bookmark_string(filename_from_path(path), path, "gq-icon-collection");
-				}
-			else if (isfile(path))
-				{
-				buf = bookmark_string(filename_from_path(path), path, GQ_ICON_FILE);
-				}
-			else
-				{
-				buf = bookmark_string(filename_from_path(path), path, nullptr);
-				}
+			g_autofree gchar *buf = bookmark_string(filename_from_path(path), path, bookmark_icon(path));
 			history_list_add_to_key(bm->key, buf, 0);
-			g_free(buf);
-			g_free(real_path);
 			}
 
 		g_list_free_full(list, g_free);
@@ -791,7 +775,7 @@ static void bookmark_dnd_get_data(GtkWidget *, GdkDragContext *,
 		}
 }
 
-static void bookmark_list_destroy(GtkWidget *, gpointer data)
+static void bookmark_list_destroy(gpointer data)
 {
 	auto bm = static_cast<BookMarkData *>(data);
 
@@ -838,16 +822,14 @@ GtkWidget *bookmark_list_new(const gchar *key,
 
 	bookmark_populate(bm);
 
-	g_signal_connect(G_OBJECT(bm->box), "destroy",
-			 G_CALLBACK(bookmark_list_destroy), bm);
-	g_object_set_data(G_OBJECT(bm->box), BOOKMARK_DATA_KEY, bm);
+	g_object_set_data_full(G_OBJECT(bm->box), BOOKMARK_DATA_KEY, bm, bookmark_list_destroy);
 	g_object_set_data(G_OBJECT(scrolled), BOOKMARK_DATA_KEY, bm);
 	bm->widget = scrolled;
 
 	gtk_drag_dest_set(scrolled,
-			  static_cast<GtkDestDefaults>(GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP),
-			  bookmark_drop_types, bookmark_drop_types_n,
-			  static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
+	                  static_cast<GtkDestDefaults>(GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP),
+	                  bookmark_drop_types.data(), bookmark_drop_types.size(),
+	                  static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
 	g_signal_connect(G_OBJECT(scrolled), "drag_data_received",
 			 G_CALLBACK(bookmark_dnd_get_data), bm);
 
@@ -906,36 +888,19 @@ void bookmark_list_set_only_directories(GtkWidget *list, gboolean only_directori
 void bookmark_list_add(GtkWidget *list, const gchar *name, const gchar *path)
 {
 	BookMarkData *bm;
-	gchar *real_path;
 
 	bm = static_cast<BookMarkData *>(g_object_get_data(G_OBJECT(list), BOOKMARK_DATA_KEY));
 	if (!bm) return;
 
-	std::unique_ptr<gchar, decltype(&g_free)> buf(bookmark_string(name, path, nullptr), g_free);
-	real_path = realpath(path, nullptr);
-
-	if (strstr(real_path, get_collections_dir()) && isfile(path))
-		{
-		buf.reset(bookmark_string(name, path, "gq-icon-collection"));
-		}
-	else
-		{
-		if (isfile(path))
-			{
-			buf.reset(bookmark_string(name, path, GQ_ICON_FILE));
-			}
-		else
-			{
-			buf.reset(bookmark_string(name, path, nullptr));
-			}
-		}
-
-	history_list_add_to_key(bm->key, buf.get(), 0);
-	g_free(real_path);
+	g_autofree gchar *buf = bookmark_string(name, path, bookmark_icon(path));
+	history_list_add_to_key(bm->key, buf, 0);
 
 	bookmark_populate_all(bm->key);
 }
 
+/**
+ * @brief Allows apps to set up the defaults
+ */
 void bookmark_add_default(const gchar *name, const gchar *path)
 {
 	if (!name || !path) return;
@@ -957,7 +922,7 @@ struct HistoryComboData
 	gint history_levels;
 };
 
-static void history_combo_destroy(GtkWidget *, gpointer data)
+static void history_combo_destroy(gpointer data)
 {
 	auto hc = static_cast<HistoryComboData *>(data);
 
@@ -981,10 +946,8 @@ GtkWidget *history_combo_new(GtkWidget **entry, const gchar *text,
 
 	hc->entry = gtk_bin_get_child(GTK_BIN(hc->combo));
 
-	g_object_set_data(G_OBJECT(hc->combo), "history_combo_data", hc);
+	g_object_set_data_full(G_OBJECT(hc->combo), "history_combo_data", hc, history_combo_destroy);
 	g_object_set_data(G_OBJECT(hc->entry), "history_combo_data", hc);
-	g_signal_connect(G_OBJECT(hc->combo), "destroy",
-			 G_CALLBACK(history_combo_destroy), hc);
 
 	work = history_list_get_by_key(hc->history_key);
 	while (work)
@@ -1031,7 +994,7 @@ void history_combo_append_history(GtkWidget *widget, const gchar *text)
 		new_text = g_strdup(gq_gtk_entry_get_text(GTK_ENTRY(hc->entry)));
 		}
 
-	if (new_text && strlen(new_text) > 0)
+	if (new_text && new_text[0] != '\0')
 		{
 		GtkTreeModel *store;
 		GList *work;
