@@ -19,22 +19,35 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#include <cstdio>
-#include <cstdlib>
-#include <cstring>
-#include <memory>
+#include "ui-fileops.h"
 
+#include <fcntl.h>
+#include <sys/mman.h>
+#include <sys/stat.h>
 #include <unistd.h>
 #include <utime.h>
 
-#include "main.h"
-#include "ui-fileops.h"
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
 
-#include "md5-util.h"
+#include <glib-object.h>
+#include <gtk/gtk.h>
+
+#include <config.h>
+
+#include "compat.h"
+#include "debug.h"
 #include "filefilter.h"
+#include "intl.h"
 #include "layout.h"
-#include "utilops.h"
+#include "main-defines.h"
+#include "md5-util.h"
+#include "options.h"
 #include "secure-save.h"
+#include "typedefs.h"
+#include "ui-utildlg.h"
+#include "utilops.h"
 
 /*
  *-----------------------------------------------------------------------------
@@ -42,23 +55,25 @@
  *-----------------------------------------------------------------------------
  */
 
+namespace
+{
 
+using FileDescriptor = int;
+G_DEFINE_AUTO_CLEANUP_FREE_FUNC(FileDescriptor, close, -1) // NOLINT(readability-non-const-parameter)
+
+} // namespace
 
 void print_term(gboolean err, const gchar *text_utf8)
 {
-	gchar *text_l;
+	gchar *text_l = g_locale_from_utf8(text_utf8, -1, nullptr, nullptr, nullptr);
+	const gchar *text = text_l ? text_l : text_utf8;
+
+	fputs(text, err ? stderr : stdout);
 
-	text_l = g_locale_from_utf8(text_utf8, -1, nullptr, nullptr, nullptr);
-	if (err)
-		{
-		fputs((text_l) ? text_l : text_utf8, stderr);
-		}
-	else
-		{
-		fputs((text_l) ? text_l : text_utf8, stdout);
-		}
 	if(command_line && command_line->ssi)
-		secure_fputs(command_line->ssi, (text_l) ? text_l : text_utf8);
+		{
+		secure_fputs(command_line->ssi, text);
+		}
 	g_free(text_l);
 }
 
@@ -177,134 +192,80 @@ gchar *path_from_utf8(const gchar *utf8)
 /* first we try the HOME environment var, if that doesn't work, we try g_get_homedir(). */
 const gchar *homedir()
 {
-	static gchar *home = nullptr;
+	static gchar *home = []()
+	{
+		gchar *home = path_to_utf8(getenv("HOME"));
+		if (home) return home;
 
-	if (!home)
-		home = path_to_utf8(getenv("HOME"));
-
-	if (!home)
-		home = path_to_utf8(g_get_home_dir());
+		return path_to_utf8(g_get_home_dir());
+	}();
 
 	DEBUG_1("Home directory: %s", home);
 
 	return home;
 }
 
-static gchar *xdg_dir_get(const gchar *key, const gchar *fallback)
-{
-	gchar *dir = getenv(key);
-
-	if (!dir || dir[0] == '\0')
-		{
-    		return g_build_filename(homedir(), fallback, NULL);
-    		}
-
-	DEBUG_1("Got xdg %s: %s", key, dir);
-
-	return path_to_utf8(dir);
-}
-
 const gchar *xdg_data_home_get()
 {
-	static const gchar *xdg_data_home = nullptr;
-
-	if (xdg_data_home) return xdg_data_home;
-
-	xdg_data_home = xdg_dir_get("XDG_DATA_HOME", ".local/share");
+	static const gchar *xdg_data_home = path_to_utf8(g_get_user_data_dir());
 
 	return xdg_data_home;
 }
 
 const gchar *xdg_config_home_get()
 {
-	static const gchar *xdg_config_home = nullptr;
-
-	if (xdg_config_home) return xdg_config_home;
-
-	xdg_config_home = xdg_dir_get("XDG_CONFIG_HOME", ".config");
+	static const gchar *xdg_config_home = path_to_utf8(g_get_user_config_dir());
 
 	return xdg_config_home;
 }
 
 const gchar *xdg_cache_home_get()
 {
-	static const gchar *xdg_cache_home = nullptr;
-
-	if (xdg_cache_home) return xdg_cache_home;
-
-	xdg_cache_home = xdg_dir_get("XDG_CACHE_HOME", ".cache");
+	static const gchar *xdg_cache_home = path_to_utf8(g_get_user_cache_dir());
 
 	return xdg_cache_home;
 }
 
 const gchar *get_rc_dir()
 {
-	static gchar *rc_dir = nullptr;
-
-	if (rc_dir) return rc_dir;
-
-	if (USE_XDG)
-		{
-		rc_dir = g_build_filename(xdg_config_home_get(), GQ_APPNAME_LC, NULL);
-		}
-	else
-		{
-		rc_dir = g_build_filename(homedir(), GQ_RC_DIR, NULL);
-		}
+#if USE_XDG
+	static gchar *rc_dir = g_build_filename(xdg_config_home_get(), GQ_APPNAME_LC, NULL);
+#else
+	static gchar *rc_dir = g_build_filename(homedir(), GQ_RC_DIR, NULL);
+#endif
 
 	return rc_dir;
 }
 
 const gchar *get_collections_dir()
 {
-	static gchar *collections_dir = nullptr;
-
-	if (collections_dir) return collections_dir;
-
-	if (USE_XDG)
-		{
-		collections_dir = g_build_filename(xdg_data_home_get(), GQ_APPNAME_LC, GQ_COLLECTIONS_DIR, NULL);
-		}
-	else
-		{
-		collections_dir = g_build_filename(get_rc_dir(), GQ_COLLECTIONS_DIR, NULL);
-		}
+#if USE_XDG
+	static gchar *collections_dir = g_build_filename(xdg_data_home_get(), GQ_APPNAME_LC, GQ_COLLECTIONS_DIR, NULL);
+#else
+	static gchar *collections_dir = g_build_filename(get_rc_dir(), GQ_COLLECTIONS_DIR, NULL);
+#endif
 
 	return collections_dir;
 }
 
 const gchar *get_trash_dir()
 {
-	static gchar *trash_dir = nullptr;
-
-	if (trash_dir) return trash_dir;
-
-	if (USE_XDG)
-		{
-		trash_dir = g_build_filename(xdg_data_home_get(), GQ_APPNAME_LC, GQ_TRASH_DIR, NULL);
-		}
-	else
-		{
-		trash_dir = g_build_filename(get_rc_dir(), GQ_TRASH_DIR, NULL);
-	}
+#if USE_XDG
+	static gchar *trash_dir = g_build_filename(xdg_data_home_get(), GQ_APPNAME_LC, GQ_TRASH_DIR, NULL);
+#else
+	static gchar *trash_dir = g_build_filename(get_rc_dir(), GQ_TRASH_DIR, NULL);
+#endif
 
 	return trash_dir;
 }
 
 const gchar *get_window_layouts_dir()
 {
-	static gchar *window_layouts_dir = nullptr;
-
-	if (window_layouts_dir) return window_layouts_dir;
-
-	if (USE_XDG)
-		{
-		window_layouts_dir = g_build_filename(xdg_config_home_get(), GQ_APPNAME_LC, GQ_WINDOW_LAYOUTS_DIR, NULL);
-		}
-	else
-		{
-		window_layouts_dir = g_build_filename(get_rc_dir(), GQ_WINDOW_LAYOUTS_DIR, NULL);
-		}
+#if USE_XDG
+	static gchar *window_layouts_dir = g_build_filename(xdg_config_home_get(), GQ_APPNAME_LC, GQ_WINDOW_LAYOUTS_DIR, NULL);
+#else
+	static gchar *window_layouts_dir = g_build_filename(get_rc_dir(), GQ_WINDOW_LAYOUTS_DIR, NULL);
+#endif
 
 	return window_layouts_dir;
 }
@@ -432,28 +393,6 @@ gboolean unlink_file(const gchar *s)
 	return ret;
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-gboolean symlink_utf8_unused(const gchar *source, const gchar *target)
-{
-	gchar *sl;
-	gchar *tl;
-	gboolean ret;
-
-	if (!source || !target) return FALSE;
-
-	sl = path_from_utf8(source);
-	tl = path_from_utf8(target);
-
-	ret = (symlink(sl, tl) == 0);
-
-	g_free(sl);
-	g_free(tl);
-
-	return ret;
-}
-#pragma GCC diagnostic pop
-
 gboolean mkdir_utf8(const gchar *s, gint mode)
 {
 	gchar *sl;
@@ -484,7 +423,8 @@ gboolean rmdir_utf8(const gchar *s)
 gboolean copy_file_attributes(const gchar *s, const gchar *t, gint perms, gint mtime)
 {
 	struct stat st;
-	gchar *sl, *tl;
+	gchar *sl;
+	gchar *tl;
 	gboolean ret = FALSE;
 
 	if (!s || !t) return FALSE;
@@ -502,16 +442,18 @@ gboolean copy_file_attributes(const gchar *s, const gchar *t, gint perms, gint m
 
 		if (perms)
 			{
-			ret = chown(tl, st.st_uid, st.st_gid);
 			/* Ignores chown errors, while still doing chown
 			   (so root still can copy files preserving ownership) */
-			ret = TRUE;
-			if (chmod(tl, st.st_mode) < 0) {
-                            struct stat st2;
-                            if (stat(tl, &st2) != 0 || st2.st_mode != st.st_mode) {
-                                ret = FALSE;
-                            }
-                        }
+			chown(tl, st.st_uid, st.st_gid);
+
+			if (chmod(tl, st.st_mode) < 0)
+				{
+				struct stat st2;
+				if (stat(tl, &st2) != 0 || st2.st_mode != st.st_mode)
+					{
+					ret = FALSE;
+					}
+				}
 			}
 
 		tb.actime = st.st_atime;
@@ -537,21 +479,16 @@ static gboolean hard_linked(const gchar *a, const gchar *b)
 		sta.st_ino == stb.st_ino);
 }
 
-static void fclose_safe(FILE *fp)
-{
-	if (fp) fclose(fp);
-}
-
 gboolean copy_file(const gchar *s, const gchar *t)
 {
 	gchar buf[16384];
 	size_t b;
 	gint fd = -1;
 
-	std::unique_ptr<gchar, decltype(&g_free)> sl{path_from_utf8(s), g_free};
-	std::unique_ptr<gchar, decltype(&g_free)> tl{path_from_utf8(t), g_free};
+	g_autofree gchar *sl = path_from_utf8(s);
+	g_autofree gchar *tl = path_from_utf8(t);
 
-	if (hard_linked(sl.get(), tl.get()))
+	if (hard_linked(sl, tl))
 		{
 		return TRUE;
 		}
@@ -569,14 +506,9 @@ gboolean copy_file(const gchar *s, const gchar *t)
 			return FALSE;
 			}
 
-		gchar *link_target;
-		ssize_t i;
-
-		link_target = static_cast<gchar *>(g_malloc(st.st_size + 1));
-		i = readlink(sl, link_target, st.st_size);
-		if (i<0)
+		g_autofree auto *link_target = static_cast<gchar *>(g_malloc(st.st_size + 1));
+		if (readlink(sl, link_target, st.st_size) < 0)
 			{
-			g_free(link_target);
 			return FALSE;  // try a "normal" copy
 			}
 		link_target[st.st_size] = '\0';
@@ -594,37 +526,28 @@ gboolean copy_file(const gchar *s, const gchar *t)
 			g_free(link_target);
 			link_target = absolute;
 
-			gchar *realPath;
-			realPath = realpath(link_target, nullptr);
-
-			if (realPath != nullptr) // successfully resolved into an absolute path
+			gchar *realPath = realpath(link_target, nullptr);
+			if (!realPath) // could not get absolute path, got some error instead
 				{
-				g_free(link_target);
-				link_target = g_strdup(realPath);
-				g_free(realPath);
-				}
-			else                 // could not get absolute path, got some error instead
-				{
-				g_free(link_target);
 				return FALSE;  // so try a "normal" copy
 				}
+
+			g_free(link_target);
+			link_target = realPath; // successfully resolved into an absolute path
 			}
 
 		if (stat_utf8(tl, &st)) unlink(tl); // first try to remove directory entry in destination directory if such entry exists
 
-		gint success = (symlink(link_target, tl) == 0);
-		g_free(link_target);
-
-		return success;
+		return symlink(link_target, tl) == 0;
 		};
 
-	if (copy_symlink(sl.get(), tl.get()))
+	if (copy_symlink(sl, tl))
 		{
 		return TRUE;
 		}
 
 	// if symlink did not succeed, continue on to try a copy procedure
-	std::unique_ptr<FILE, decltype(&fclose_safe)> fi{fopen(sl.get(), "rb"), fclose_safe};
+	g_autoptr(FILE) fi = fopen(sl, "rb");
 	if (!fi)
 		{
 		return FALSE;
@@ -632,44 +555,47 @@ gboolean copy_file(const gchar *s, const gchar *t)
 
 	/* First we write to a temporary file, then we rename it on success,
 	   and attributes from original file are copied */
-	std::unique_ptr<gchar, decltype(&g_free)> randname{g_strconcat(tl.get(), ".tmp_XXXXXX", NULL), g_free};
+	g_autofree gchar *randname = g_strconcat(tl, ".tmp_XXXXXX", NULL);
 	if (!randname)
 		{
 		return FALSE;
 		}
 
-	fd = g_mkstemp(randname.get());
+	fd = g_mkstemp(randname);
 	if (fd == -1)
 		{
 		return FALSE;
 		}
 
-	std::unique_ptr<FILE, decltype(&fclose_safe)> fo{fdopen(fd, "wb"), fclose_safe};
-	if (!fo) {
+	g_autoptr(FILE) fo = fdopen(fd, "wb");
+	if (!fo)
+		{
 		close(fd);
 		return FALSE;
-	}
+		}
 
-	while ((b = fread(buf, sizeof(gchar), sizeof(buf), fi.get())) && b != 0)
+	while ((b = fread(buf, sizeof(gchar), sizeof(buf), fi)) != 0)
 		{
-		if (fwrite(buf, sizeof(gchar), b, fo.get()) != b)
+		if (fwrite(buf, sizeof(gchar), b, fo) != b)
 			{
-			unlink(randname.get());
+			unlink(randname);
 			return FALSE;
 			}
 		}
 
-	if (rename(randname.get(), tl.get()) < 0) {
-		unlink(randname.get());
+	if (rename(randname, tl) < 0)
+		{
+		unlink(randname);
 		return FALSE;
-	}
+		}
 
 	return copy_file_attributes(s, t, TRUE, TRUE);
 }
 
 gboolean move_file(const gchar *s, const gchar *t)
 {
-	gchar *sl, *tl;
+	gchar *sl;
+	gchar *tl;
 	gboolean ret = TRUE;
 
 	if (!s || !t) return FALSE;
@@ -701,7 +627,8 @@ gboolean move_file(const gchar *s, const gchar *t)
 
 gboolean rename_file(const gchar *s, const gchar *t)
 {
-	gchar *sl, *tl;
+	gchar *sl;
+	gchar *tl;
 	gboolean ret;
 
 	if (!s || !t) return FALSE;
@@ -727,11 +654,6 @@ gchar *get_current_dir()
 	return path8;
 }
 
-void string_list_free(GList *list)
-{
-	g_list_free_full(list, g_free);
-}
-
 GList *string_list_copy(const GList *list)
 {
 	GList *new_list = nullptr;
@@ -782,38 +704,6 @@ gchar *unique_filename(const gchar *path, const gchar *ext, const gchar *divider
 	return unique;
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-gchar *unique_filename_simple_unused(const gchar *path)
-{
-	gchar *unique;
-	const gchar *name;
-	const gchar *ext;
-
-	if (!path) return NULL;
-
-	name = filename_from_path(path);
-	if (!name) return NULL;
-
-	ext = registered_extension_from_path(name);
-
-	if (!ext)
-		{
-		unique = unique_filename(path, NULL, "_", TRUE);
-		}
-	else
-		{
-		gchar *base;
-
-		base = remove_extension_from_path(path);
-		unique = unique_filename(base, ext, "_", TRUE);
-		g_free(base);
-		}
-
-	return unique;
-}
-#pragma GCC diagnostic pop
-
 const gchar *filename_from_path(const gchar *path)
 {
 	const gchar *base;
@@ -866,9 +756,13 @@ gchar *remove_extension_from_path(const gchar *path)
 	return g_strndup(path, strlen(path) - (reg_ext == nullptr ? 0 : strlen(reg_ext)));
 }
 
+/**
+ * @brief Warning note: this modifies path string!
+ */
 void parse_out_relatives(gchar *path)
 {
-	gint s, t;
+	gint s;
+	gint t;
 
 	if (!path) return;
 
@@ -887,7 +781,8 @@ void parse_out_relatives(gchar *path)
 				s = p;
 				continue;
 				}
-			else if (path[p] == '.' && (path[p+1] == G_DIR_SEPARATOR || path[p+1] == '\0'))
+
+			if (path[p] == '.' && (path[p+1] == G_DIR_SEPARATOR || path[p+1] == '\0'))
 				{
 				/* /../ or /.., remove previous part, ie. /a/b/../ becomes /a/ */
 				s = p + 1;
@@ -909,34 +804,21 @@ void parse_out_relatives(gchar *path)
 
 gboolean file_in_path(const gchar *name)
 {
-	gchar *path;
-	gchar *namel;
-	gint p, l;
-	gboolean ret = FALSE;
-
 	if (!name) return FALSE;
-	path = g_strdup(getenv("PATH"));
+
+	g_autofree gchar *path = g_strdup(getenv("PATH"));
 	if (!path) return FALSE;
-	namel = path_from_utf8(name);
 
-	p = 0;
-	l = strlen(path);
-	while (p < l && !ret)
+	g_auto(GStrv) paths = g_strsplit(path, ":", 0);
+	g_autofree gchar *namel = path_from_utf8(name);
+
+	for (gint i = 0; paths[i]; i++)
 		{
-		gchar *f;
-		gint e = p;
-		while (path[e] != ':' && path[e] != '\0') e++;
-		path[e] = '\0';
-		e++;
-		f = g_build_filename(path + p, namel, NULL);
-		if (isfile(f)) ret = TRUE;
-		g_free(f);
-		p = e;
+		g_autofree gchar *f = g_build_filename(paths[i], namel, NULL);
+		if (isfile(f)) return TRUE;
 		}
-	g_free(namel);
-	g_free(path);
 
-	return ret;
+	return FALSE;
 }
 
 gboolean recursive_mkdir_if_not_exists(const gchar *path, mode_t mode)
@@ -994,12 +876,15 @@ gboolean md5_get_digest_from_file_utf8(const gchar *path, guchar digest[16])
 	return success;
 }
 
-
+/**
+ * @brief Generate md5 string from file,
+ * on failure returns newly allocated copy of error_text, error_text may be NULL
+ */
 gchar *md5_text_from_file_utf8(const gchar *path, const gchar *error_text)
 {
-	std::unique_ptr<gchar, decltype(&g_free)> pathl{path_from_utf8(path), g_free};
+	g_autofree gchar *pathl = path_from_utf8(path);
 
-	auto md5_text = md5_get_string_from_file(pathl.get());
+	auto md5_text = md5_get_string_from_file(pathl);
 
 	return md5_text ? md5_text : g_strdup(error_text);
 }
@@ -1064,72 +949,55 @@ static void download_web_file_cancel_button_cb(GenericDialog *, gpointer data)
 
 gboolean download_web_file(const gchar *text, gboolean minimized, gpointer data)
 {
-	gchar *scheme;
-	auto lw = static_cast<LayoutWindow *>(data);
-	gchar *tmp_dir;
-	GError *error = nullptr;
-	WebData *web;
-	gchar *base;
-	gboolean ret = FALSE;
-	gchar *message;
-	FileFormatClass format_class;
-
-	scheme = g_uri_parse_scheme(text);
-	if (g_strcmp0("http", scheme) == 0 || g_strcmp0("https", scheme) == 0)
+	g_autofree gchar *scheme = g_uri_parse_scheme(text);
+	if (g_strcmp0("http", scheme) != 0 && g_strcmp0("https", scheme) != 0)
 		{
-		format_class = filter_file_get_class(text);
-
-		if (format_class == FORMAT_CLASS_IMAGE || format_class == FORMAT_CLASS_RAWIMAGE || format_class == FORMAT_CLASS_VIDEO || format_class == FORMAT_CLASS_DOCUMENT)
-			{
-			tmp_dir = g_dir_make_tmp("geeqie_XXXXXX", &error);
-			if (error)
-				{
-				log_printf("Error: could not create temporary file n%s\n", error->message);
-				g_error_free(error);
-				error = nullptr;
-				ret = TRUE;
-				}
-			else
-				{
-				web = g_new0(WebData, 1);
-				web->lw = lw;
-
-				web->web_file = g_file_new_for_uri(text);
-
-				base = g_strdup(g_file_get_basename(web->web_file));
-				web->tmp_g_file = g_file_new_for_path(g_build_filename(tmp_dir, base, NULL));
-
-				web->gd = generic_dialog_new(_("Download web file"), "download_web_file", nullptr, TRUE, download_web_file_cancel_button_cb, web);
-
-				message = g_strconcat(_("Downloading "), base, NULL);
-				generic_dialog_add_message(web->gd, GQ_ICON_DIALOG_INFO, message, nullptr, FALSE);
-
-				web->progress = gtk_progress_bar_new();
-				gq_gtk_box_pack_start(GTK_BOX(web->gd->vbox), web->progress, FALSE, FALSE, 0);
-				gtk_widget_show(web->progress);
-				if (minimized)
-					{
-					gtk_window_iconify(GTK_WINDOW(web->gd->dialog));
-					}
-
-				gtk_widget_show(web->gd->dialog);
-				web->cancellable = g_cancellable_new();
-				g_file_copy_async(web->web_file, web->tmp_g_file, G_FILE_COPY_OVERWRITE, G_PRIORITY_LOW, web->cancellable, web_file_progress_cb, web, web_file_async_ready_cb, web);
-
-				g_free(base);
-				g_free(message);
-				ret = TRUE;
-				}
-			}
-		}
-	else
-		{
-		ret = FALSE;
+		return FALSE;
 		}
 
-	g_free(scheme);
-	return ret;
+	FileFormatClass format_class = filter_file_get_class(text);
+	if (format_class != FORMAT_CLASS_IMAGE &&
+	    format_class != FORMAT_CLASS_RAWIMAGE &&
+	    format_class != FORMAT_CLASS_VIDEO &&
+	    format_class != FORMAT_CLASS_DOCUMENT)
+		{
+		return FALSE;
+		}
 
+	g_autoptr(GError) error = nullptr;
+	g_autofree gchar *tmp_dir = g_dir_make_tmp("geeqie_XXXXXX", &error);
+	if (error)
+		{
+		log_printf("Error: could not create temporary file n%s\n", error->message);
+		return FALSE;
+		}
+
+	auto *web = g_new0(WebData, 1);
+	web->lw = static_cast<LayoutWindow *>(data);
+
+	web->web_file = g_file_new_for_uri(text);
+
+	g_autofree gchar *base = g_file_get_basename(web->web_file);
+	web->tmp_g_file = g_file_new_for_path(g_build_filename(tmp_dir, base, NULL));
+
+	web->gd = generic_dialog_new(_("Download web file"), "download_web_file", nullptr, TRUE, download_web_file_cancel_button_cb, web);
+
+	g_autofree gchar *message = g_strconcat(_("Downloading "), base, NULL);
+	generic_dialog_add_message(web->gd, GQ_ICON_DIALOG_INFO, message, nullptr, FALSE);
+
+	web->progress = gtk_progress_bar_new();
+	gq_gtk_box_pack_start(GTK_BOX(web->gd->vbox), web->progress, FALSE, FALSE, 0);
+	gtk_widget_show(web->progress);
+	if (minimized)
+		{
+		gtk_window_iconify(GTK_WINDOW(web->gd->dialog));
+		}
+
+	gtk_widget_show(web->gd->dialog);
+	web->cancellable = g_cancellable_new();
+	g_file_copy_async(web->web_file, web->tmp_g_file, G_FILE_COPY_OVERWRITE, G_PRIORITY_LOW, web->cancellable, web_file_progress_cb, web, web_file_async_ready_cb, web);
+
+	return TRUE;
 }
 
 gboolean rmdir_recursive(GFile *file, GCancellable *cancellable, GError **error)
@@ -1167,4 +1035,19 @@ gint scale_factor()
 	layout_valid(&lw);
 	return gtk_widget_get_scale_factor(lw->window);
 }
+
+guchar *map_file(const gchar *path, gsize &map_len)
+{
+	g_auto(FileDescriptor) fd = open(path, O_RDONLY | O_NONBLOCK);
+	if (fd == -1) return nullptr;
+
+	struct stat st;
+	if (fstat(fd, &st) == -1) return nullptr;
+
+	auto *map_data = static_cast<guchar *>(mmap(nullptr, st.st_size, PROT_READ, MAP_PRIVATE, fd, 0));
+	if (map_data == MAP_FAILED) return nullptr;
+
+	map_len = st.st_size;
+	return map_data;
+}
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
