@@ -38,11 +38,13 @@
 void print_term(gboolean err, const gchar *text_utf8);
 
 #define printf_term(err, ...) \
-	do { \
+	G_STMT_START \
+		{ \
 		gchar *msg = g_strdup_printf(__VA_ARGS__); \
 		print_term(err, msg); \
 		g_free(msg); \
-	} while (0)
+		} \
+	G_STMT_END
 
 #if GQ_DEBUG_PATH_UTF8
 #define path_to_utf8(path) path_to_utf8_debug(path, __FILE__, __LINE__)
@@ -84,12 +86,6 @@ gboolean move_file(const gchar *s, const gchar *t);
 gboolean rename_file(const gchar *s, const gchar *t);
 gchar *get_current_dir();
 
-/**
- * @headerfile string_list_free
- * return True on success, it is up to you to free
- * the lists with string_list_free()
- */
-void string_list_free(GList *list);
 GList *string_list_copy(const GList *list);
 
 gchar *unique_filename(const gchar *path, const gchar *ext, const gchar *divider, gboolean pad);
@@ -101,22 +97,12 @@ gchar *remove_extension_from_path(const gchar *path);
 
 gboolean file_extension_match(const gchar *path, const gchar *ext);
 
-/**
- * @headerfile parse_out_relatives
- * warning note: this modifies path string!
- */
 void parse_out_relatives(gchar *path);
 
 gboolean file_in_path(const gchar *name);
 
 gboolean recursive_mkdir_if_not_exists(const gchar *path, mode_t mode);
 
-
-/**
- * @headerfile md5_text_from_file_utf8
- * generate md5 string from file,
- * on failure returns newly allocated copy of error_text, error_text may be NULL
- */
 gchar *md5_text_from_file_utf8(const gchar *path, const gchar *error_text);
 gboolean md5_get_digest_from_file_utf8(const gchar *path, guchar digest[16]);
 
@@ -124,5 +110,7 @@ gboolean download_web_file(const gchar *text, gboolean minimized, gpointer data)
 gboolean rmdir_recursive(GFile *file, GCancellable *cancellable, GError **error);
 gint scale_factor();
 
+guchar *map_file(const gchar *path, gsize &map_len);
+
 #endif
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
