@@ -25,6 +25,7 @@
 
 #include <pango/pango.h>
 
+#include "compat.h"
 #include "layout.h"
 
 /*
@@ -89,8 +90,8 @@ static gint actions_sort_cb(gconstpointer a, gconstpointer b)
 	const gchar *accel_path_b;
 	GtkAccelKey key_b;
 
-	accel_path_a = gtk_action_get_accel_path(GTK_ACTION(a));
-	accel_path_b = gtk_action_get_accel_path(GTK_ACTION(b));
+	accel_path_a = gq_gtk_action_get_accel_path(GTK_ACTION(a));
+	accel_path_b = gq_gtk_action_get_accel_path(GTK_ACTION(b));
 
 	if (accel_path_a && gtk_accel_map_lookup_entry(accel_path_a, &key_a) && accel_path_b && gtk_accel_map_lookup_entry(accel_path_b, &key_b))
 		{
@@ -114,8 +115,6 @@ static void menu_item_add_main_window_accelerator(GtkWidget *menu, GtkAccelGroup
 {
 	gchar *menu_label;
 	gchar *menu_label_text;
-	gchar *action_label;
-	gchar *action_label_text;
 	LayoutWindow *lw;
 	GList *groups;
 	GList *actions;
@@ -130,21 +129,23 @@ static void menu_item_add_main_window_accelerator(GtkWidget *menu, GtkAccelGroup
 	lw = static_cast<LayoutWindow *>(layout_window_list->data); /* get the actions from the first window, it should not matter, they should be the same in all windows */
 
 	g_assert(lw && lw->ui_manager);
-	groups = gtk_ui_manager_get_action_groups(lw->ui_manager);
+	groups = gq_gtk_ui_manager_get_action_groups(lw->ui_manager);
 
 	while (groups)
 		{
-		actions = gtk_action_group_list_actions(GTK_ACTION_GROUP(groups->data));
+		actions = gq_gtk_action_group_list_actions(GTK_ACTION_GROUP(groups->data));
 		actions = g_list_sort(actions, actions_sort_cb);
 
 		while (actions)
 			{
 			action = GTK_ACTION(actions->data);
-			accel_path = gtk_action_get_accel_path(action);
+			accel_path = gq_gtk_action_get_accel_path(action);
 			if (accel_path && gtk_accel_map_lookup_entry(accel_path, &key))
 				{
+				g_autofree gchar *action_label = nullptr;
 				g_object_get(action, "label", &action_label, NULL);
 
+				g_autofree gchar *action_label_text = nullptr;
 				pango_parse_markup(action_label, -1, '_', nullptr, &action_label_text, nullptr, nullptr);
 
 				if (g_strcmp0(action_label_text, menu_label_text) == 0)
@@ -156,8 +157,6 @@ static void menu_item_add_main_window_accelerator(GtkWidget *menu, GtkAccelGroup
 						break;
 						}
 					}
-				g_free(action_label_text);
-				g_free(action_label);
 				}
 			actions = actions->next;
 			}
@@ -387,6 +386,9 @@ void menu_item_add_divider(GtkWidget *menu)
 	gtk_widget_show(item);
 }
 
+/**
+ * @brief Use to avoid mnemonics, for example filenames
+ */
 GtkWidget *menu_item_add_simple(GtkWidget *menu, const gchar *label,
 				GCallback func, gpointer data)
 {
@@ -415,67 +417,10 @@ GtkWidget *popup_menu_short_lived()
 	menu = gtk_menu_new();
 
 	/* take ownership of menu */
-#ifdef GTK_OBJECT_FLOATING
-	/* GTK+ < 2.10 */
-	g_object_ref(G_OBJECT(menu));
-	gtk_object_sink(GTK_OBJECT(menu));
-#else
-	/* GTK+ >= 2.10 */
 	g_object_ref_sink(G_OBJECT(menu));
-#endif
 
 	g_signal_connect(G_OBJECT(menu), "selection_done",
 			 G_CALLBACK(popup_menu_short_lived_cb), menu);
 	return menu;
 }
-
-// @todo Used only in vd_menu_position_cb_unused(). Remove?
-gboolean popup_menu_position_clamp(GtkMenu *menu, gint *x, gint *y, gint height)
-{
-	gboolean adjusted = FALSE;
-	gint w;
-	gint h;
-	gint xw;
-	gint xh;
-	GtkRequisition requisition;
-
-	G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
-	gtk_widget_get_requisition(GTK_WIDGET(menu), &requisition);
-	w = requisition.width;
-	h = requisition.height;
-	xw = gdk_screen_width();
-	xh = gdk_screen_height();
-	G_GNUC_END_IGNORE_DEPRECATIONS;
-
-	if (*x + w > xw)
-		{
-		*x = xw - w;
-		adjusted = TRUE;
-		}
-	if (*y + h > xh)
-		{
-		if (height)
-			{
-			*y = MAX(0, *y - h - height);
-			}
-		else
-			{
-			*y = xh - h;
-			}
-		adjusted = TRUE;
-		};
-
-	if (*x < 0)
-		{
-		*x = 0;
-		adjusted = TRUE;
-		}
-	if (*y < 0)
-		{
-		*y = 0;
-		adjusted = TRUE;
-		}
-
-	return adjusted;
-}
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
