@@ -23,8 +23,11 @@
 
 #include <langinfo.h>
 
+#include <algorithm>
 #include <cstdlib>
 #include <cstring>
+#include <string>
+#include <utility>
 
 #include <pango/pango.h>
 
@@ -49,16 +52,7 @@
 GtkWidget *pref_box_new(GtkWidget *parent_box, gboolean fill,
 			GtkOrientation orientation, gboolean padding)
 {
-	GtkWidget *box;
-
-	if (orientation == GTK_ORIENTATION_HORIZONTAL)
-		{
-		box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, padding);
-		}
-	else
-		{
-		box = gtk_box_new(GTK_ORIENTATION_VERTICAL, padding);
-		}
+	GtkWidget *box = gtk_box_new(orientation, padding);
 
 	gq_gtk_box_pack_start(GTK_BOX(parent_box), box, fill, fill, 0);
 	gtk_widget_show(box);
@@ -151,14 +145,7 @@ GtkWidget *pref_frame_new(GtkWidget *parent_box, gboolean fill,
 	gq_gtk_box_pack_start(GTK_BOX(parent_box), frame, fill, fill, 0);
 	gtk_widget_show(frame);
 
-	if (orientation == GTK_ORIENTATION_HORIZONTAL)
-		{
-		box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, padding);
-		}
-	else
-		{
-		box = gtk_box_new(GTK_ORIENTATION_VERTICAL, padding);
-		}
+	box = gtk_box_new(orientation, padding);
 	gq_gtk_container_add(GTK_WIDGET(frame), box);
 	gtk_container_set_border_width(GTK_CONTAINER(box), PREF_PAD_BORDER);
 	gtk_widget_show(box);
@@ -301,15 +288,6 @@ GtkWidget *pref_checkbox_new(GtkWidget *parent_box, const gchar *text, gboolean
 	return real_pref_checkbox_new(parent_box, text, FALSE, active, func, data);
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-GtkWidget *pref_checkbox_new_mnemonic_unused(GtkWidget *parent_box, const gchar *text, gboolean active,
-				      GCallback func, gpointer data)
-{
-	return real_pref_checkbox_new(parent_box, text, TRUE, active, func, data);
-}
-#pragma GCC diagnostic pop
-
 static void pref_checkbox_int_cb(GtkWidget *widget, gpointer data)
 {
 	auto result = static_cast<gboolean *>(data);
@@ -344,31 +322,13 @@ void pref_checkbox_link_sensitivity(GtkWidget *button, GtkWidget *widget)
 	pref_checkbox_link_sensitivity_cb(button, widget);
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-static void pref_checkbox_link_sensitivity_swap_cb_unused(GtkWidget *button, gpointer data)
-{
-	auto *widget = static_cast<GtkWidget *>(data);
-
-	gtk_widget_set_sensitive(widget, !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button)));
-}
-
-void pref_checkbox_link_sensitivity_swap_unused(GtkWidget *button, GtkWidget *widget)
-{
-	g_signal_connect(G_OBJECT(button), "toggled",
-			 G_CALLBACK(pref_checkbox_link_sensitivity_swap_cb_unused), widget);
-
-	pref_checkbox_link_sensitivity_swap_cb_unused(button, widget);
-}
-#pragma GCC diagnostic pop
-
 static GtkWidget *real_pref_radiobutton_new(GtkWidget *parent_box, GtkWidget *sibling,
 					    const gchar *text, gboolean mnemonic_text, gboolean active,
 					    GCallback func, gpointer data)
 {
 	GtkWidget *button;
 #if HAVE_GTK4
-	GtkToggleButton *group;;
+	GtkToggleButton *group;
 #else
 	GSList *group;
 #endif
@@ -421,43 +381,6 @@ GtkWidget *pref_radiobutton_new(GtkWidget *parent_box, GtkWidget *sibling,
 	return real_pref_radiobutton_new(parent_box, sibling, text, FALSE, active, func, data);
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-GtkWidget *pref_radiobutton_new_mnemonic_unused(GtkWidget *parent_box, GtkWidget *sibling,
-					 const gchar *text, gboolean active,
-					 GCallback func, gpointer data)
-{
-	return real_pref_radiobutton_new(parent_box, sibling, text, TRUE, active, func, data);
-}
-
-#define PREF_RADIO_VALUE_KEY "pref_radio_value"
-
-static void pref_radiobutton_int_cb_unused(GtkWidget *widget, gpointer data)
-{
-	auto *result = static_cast<gboolean *>(data);
-
-	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
-		{
-		*result = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(widget), PREF_RADIO_VALUE_KEY));
-		}
-}
-
-GtkWidget *pref_radiobutton_new_int_unused(GtkWidget *parent_box, GtkWidget *sibling,
-				    const gchar *text, gboolean active,
-				    gboolean *result, gboolean value,
-				    GCallback, gpointer)
-{
-	GtkWidget *button;
-
-	button = pref_radiobutton_new(parent_box, sibling, text, active,
-				      G_CALLBACK(pref_radiobutton_int_cb_unused), result);
-	g_object_set_data(G_OBJECT(button), PREF_RADIO_VALUE_KEY, GINT_TO_POINTER(value));
-	if (active) *result = value;
-
-	return button;
-}
-#pragma GCC diagnostic pop
-
 static GtkWidget *real_pref_spin_new(GtkWidget *parent_box, const gchar *text, const gchar *suffix,
 				     gboolean mnemonic_text,
 				     gdouble min, gdouble max, gdouble step, gint digits,
@@ -514,18 +437,6 @@ GtkWidget *pref_spin_new(GtkWidget *parent_box, const gchar *text, const gchar *
 				  min, max, step, digits, value, func, data);
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-GtkWidget *pref_spin_new_mnemonic_unused(GtkWidget *parent_box, const gchar *text, const gchar *suffix,
-				  gdouble min, gdouble max, gdouble step, gint digits,
-				  gdouble value,
-				  GCallback func, gpointer data)
-{
-	return real_pref_spin_new(parent_box, text, suffix, TRUE,
-				  min, max, step, digits, value, func, data);
-}
-#pragma GCC diagnostic pop
-
 static void pref_spin_int_cb(GtkWidget *widget, gpointer data)
 {
 	auto var = static_cast<gint *>(data);
@@ -753,32 +664,6 @@ GtkWidget *pref_toolbar_button(GtkWidget *toolbar,
 	return item;
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-void pref_toolbar_button_set_icon_unused(GtkWidget *button, GtkWidget *widget, const gchar *icon_name)
-{
-	if (widget)
-		{
-		gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(button), widget);
-		}
-	else if (icon_name)
-		{
-		gtk_tool_button_set_icon_name(GTK_TOOL_BUTTON(button), icon_name);
-		}
-}
-
-GtkWidget *pref_toolbar_spacer_unused(GtkWidget *toolbar)
-{
-	GtkWidget *item;
-
-	item = GTK_WIDGET(gtk_separator_tool_item_new());
-	gq_gtk_container_add(GTK_WIDGET(toolbar), item);
-	gtk_widget_show(item);
-
-	return item;
-}
-#pragma GCC diagnostic pop
-
 
 /*
  *-----------------------------------------------------------------------------
@@ -835,23 +720,12 @@ static gboolean date_selection_popup_release_cb(GtkWidget *, GdkEventButton *, g
 static gboolean date_selection_popup_press_cb(GtkWidget *, GdkEventButton *event, gpointer data)
 {
 	auto ds = static_cast<DateSelection *>(data);
-	gint x;
-	gint y;
-	gint w;
-	gint h;
-	gint xr;
-	gint yr;
-	GdkWindow *window;
+	GdkWindow *window = gtk_widget_get_window(ds->window);
 
-	xr = static_cast<gint>(event->x_root);
-	yr = static_cast<gint>(event->y_root);
+	auto xr = static_cast<gint>(event->x_root);
+	auto yr = static_cast<gint>(event->y_root);
 
-	window = gtk_widget_get_window(ds->window);
-	gdk_window_get_origin(window, &x, &y);
-	w = gdk_window_get_width(window);
-	h = gdk_window_get_height(window);
-
-	if (xr < x || yr < y || xr > x + w || yr > y + h)
+	if (!window_received_event(window, {xr, yr}))
 		{
 		g_signal_connect(G_OBJECT(ds->window), "button_release_event",
 				 G_CALLBACK(date_selection_popup_release_cb), ds);
@@ -1148,29 +1022,6 @@ void date_selection_time_set(GtkWidget *widget, time_t t)
 	date_selection_set(widget, lt->tm_mday, lt->tm_mon + 1, lt->tm_year + 1900);
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-time_t date_selection_time_get_unused(GtkWidget *widget)
-{
-	struct tm lt;
-	gint day = 0;
-	gint month = 0;
-	gint year = 0;
-
-	date_selection_get(widget);
-
-	lt.tm_sec = 0;
-	lt.tm_min = 0;
-	lt.tm_hour = 0;
-	lt.tm_mday = day;
-	lt.tm_mon = month - 1;
-	lt.tm_year = year - 1900;
-	lt.tm_isdst = 0;
-
-	return mktime(&lt);
-}
-#pragma GCC diagnostic pop
-
 /*
  *-----------------------------------------------------------------------------
  * storing data in a history list with key,data pairs
@@ -1178,8 +1029,6 @@ time_t date_selection_time_get_unused(GtkWidget *widget)
  */
 
 #define PREF_LIST_MARKER_INT "[INT]:"
-#define PREF_LIST_MARKER_DOUBLE "[DOUBLE]:"
-#define PREF_LIST_MARKER_STRING "[STRING]:"
 
 static GList *pref_list_find(const gchar *group, const gchar *token)
 {
@@ -1219,7 +1068,7 @@ static gboolean pref_list_get(const gchar *group, const gchar *key, const gchar
 	if (work)
 		{
 		*result = static_cast<const gchar *>(work->data) + strlen(token);
-		if (strlen(*result) == 0) *result = nullptr;
+		if (*result[0] == '\0') *result = nullptr;
 		ret = TRUE;
 		}
 	else
@@ -1272,11 +1121,7 @@ static void pref_list_set(const gchar *group, const gchar *key, const gchar *mar
 
 void pref_list_int_set(const gchar *group, const gchar *key, gint value)
 {
-	gchar *text;
-
-	text = g_strdup_printf("%d", value);
-	pref_list_set(group, key, PREF_LIST_MARKER_INT, text);
-	g_free(text);
+	pref_list_set(group, key, PREF_LIST_MARKER_INT, std::to_string(value).c_str());
 }
 
 gboolean pref_list_int_get(const gchar *group, const gchar *key, gint *result)
@@ -1299,47 +1144,6 @@ gboolean pref_list_int_get(const gchar *group, const gchar *key, gint *result)
 	return FALSE;
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-void pref_list_double_set_unused(const gchar *group, const gchar *key, gdouble value)
-{
-	gchar text[G_ASCII_DTOSTR_BUF_SIZE];
-
-	g_ascii_dtostr(text, sizeof(text), value);
-	pref_list_set(group, key, PREF_LIST_MARKER_DOUBLE, text);
-}
-
-gboolean pref_list_double_get_unused(const gchar *group, const gchar *key, gdouble *result)
-{
-	const gchar *text;
-
-	if (!group || !key)
-		{
-		*result = 0;
-		return FALSE;
-		}
-
-	if (pref_list_get(group, key, PREF_LIST_MARKER_DOUBLE, &text) && text)
-		{
-		*result = g_ascii_strtod(text, nullptr);
-		return TRUE;
-		}
-
-	*result = 0;
-	return FALSE;
-}
-
-void pref_list_string_set_unused(const gchar *group, const gchar *key, const gchar *value)
-{
-	pref_list_set(group, key, PREF_LIST_MARKER_STRING, value);
-}
-
-gboolean pref_list_string_get_unused(const gchar *group, const gchar *key, const gchar **result)
-{
-	return pref_list_get(group, key, PREF_LIST_MARKER_STRING, result);
-}
-#pragma GCC diagnostic pop
-
 void pref_color_button_set_cb(GtkWidget *widget, gpointer data)
 {
 	auto color = static_cast<GdkRGBA *>(data);
@@ -1445,149 +1249,150 @@ gchar *text_widget_text_pull_selected(GtkWidget *text_widget)
 	
 }
 
-static gint simple_sort_cb(gconstpointer a, gconstpointer b)
+ActionItem::ActionItem(const gchar *name, const gchar *label, const gchar *icon_name)
+    : name(g_strdup(name))
+    , label(g_strdup(label))
+    , icon_name(g_strdup(icon_name))
+{}
+
+ActionItem::ActionItem(const ActionItem &other)
+    : name(g_strdup(other.name))
+    , label(g_strdup(other.label))
+    , icon_name(g_strdup(other.icon_name))
+{}
+
+ActionItem::ActionItem(ActionItem &&other) noexcept
+    : name(std::exchange(other.name, nullptr))
+    , label(std::exchange(other.label, nullptr))
+    , icon_name(std::exchange(other.icon_name, nullptr))
+{}
+
+ActionItem::~ActionItem()
 {
-	const ActionItem *a_action;
-	const ActionItem *b_action;
-
-	a_action = static_cast<const ActionItem *>(a);
-	b_action = static_cast<const ActionItem *>(b);
-
-	return g_strcmp0(a_action->name, b_action->name);
+	g_free(name);
+	g_free(label);
+	g_free(icon_name);
 }
 
-void free_action_items_cb(gpointer data)
+ActionItem &ActionItem::operator=(const ActionItem &other)
 {
-	ActionItem *action_item;
+	if (this != &other)
+		{
+		g_free(name);
+		name = g_strdup(other.name);
 
-	action_item = static_cast<ActionItem *>(data);
-	g_free((gchar *)action_item->icon_name);
-	g_free((gchar *)action_item->name);
-	g_free((gchar *)action_item->label);
-	g_free(action_item);
+		g_free(label);
+		label = g_strdup(other.label);
+
+		g_free(icon_name);
+		icon_name = g_strdup(other.icon_name);
+		}
+
+	return *this;
 }
 
-void action_items_free(GList *list)
+ActionItem &ActionItem::operator=(ActionItem &&other) noexcept
 {
-	g_list_free_full(list, free_action_items_cb);
+	if (this != &other)
+		{
+		g_free(name);
+		name = std::exchange(other.name, nullptr);
+
+		g_free(label);
+		label = std::exchange(other.label, nullptr);
+
+		g_free(icon_name);
+		icon_name = std::exchange(other.icon_name, nullptr);
+		}
+
+	return *this;
+}
+
+bool ActionItem::has_label(const gchar *label) const
+{
+	return g_strcmp0(this->label, label) == 0;
+}
+
+static gchar *get_action_label(GtkAction *action, const gchar *action_name)
+{
+	g_autofree gchar *tooltip = nullptr;
+	g_autofree gchar *label = nullptr;
+	g_object_get(action, "tooltip", &tooltip, "label", &label, NULL);
+
+	/* .desktop items need the program name, Geeqie menu items need the tooltip */
+	if (g_strstr_len(action_name, -1, ".desktop") == nullptr &&
+	    /* Tooltips with newlines affect output format */
+	    tooltip && (g_strstr_len(tooltip, -1, "\n") == nullptr))
+		{
+		return g_strdup(tooltip);
+		}
+
+	return g_strdup(label);
 }
 
 /**
  * @brief Get a list of menu actions
  * @param
- * @returns GList ActionItem
- *
- * Free returned list with action_items_free(list)
+ * @returns std::vector<ActionItem>
  *
  * The list generated is used in the --remote --action-list command and
  * programmable mouse buttons 8 and 9.
  */
-GList* get_action_items()
+std::vector<ActionItem> get_action_items()
 {
-	ActionItem *action_item;
-	const gchar *accel_path;
-	gboolean duplicate;
-	gchar *action_name;
-	gchar *label;
-	gchar *tooltip;
-	GList *actions;
-	GList *groups;
-	GList *list_duplicates = nullptr;
-	GList *list_unique = nullptr;
-	GList *work1;
-	GList *work2;
-	GtkAction *action;
 	LayoutWindow *lw = nullptr;
 
 	if (!layout_valid(&lw))
 		{
-		return nullptr;
+		return {};
 		}
 
-	groups = gtk_ui_manager_get_action_groups(lw->ui_manager);
-	while (groups)
-		{
-		actions = gtk_action_group_list_actions(GTK_ACTION_GROUP(groups->data));
-		while (actions)
-			{
-			action = GTK_ACTION(actions->data);
-			accel_path = gtk_action_get_accel_path(action);
+	std::vector<ActionItem> list_duplicates;
 
+	for (GList *groups = gq_gtk_ui_manager_get_action_groups(lw->ui_manager); groups; groups = groups->next)
+		{
+		GtkActionGroup *action_group = GTK_ACTION_GROUP(groups->data);
+		for (GList *actions = gq_gtk_action_group_list_actions(action_group); actions; actions = actions->next)
+			{
+			GtkAction *action = GTK_ACTION(actions->data);
+
+			const gchar *accel_path = gq_gtk_action_get_accel_path(action);
 			if (accel_path && gtk_accel_map_lookup_entry(accel_path, nullptr))
 				{
-				g_object_get(action, "tooltip", &tooltip, "label", &label, NULL);
-
-				action_name = g_path_get_basename(accel_path);
+				g_autofree gchar *action_name = g_path_get_basename(accel_path);
 
 				/* Menu actions are irrelevant */
 				if (g_strstr_len(action_name, -1, "Menu") == nullptr)
 					{
-					action_item = g_new0(ActionItem, 1);
+					g_autofree gchar *action_label = get_action_label(action, action_name);
 
-					/* .desktop items need the program name, Geeqie menu items need the tooltip */
-					if (g_strstr_len(action_name, -1, ".desktop") == nullptr)
-						{
-
-						/* Tooltips with newlines affect output format */
-						if (tooltip && (g_strstr_len(tooltip, -1, "\n") == nullptr) )
-							{
-							action_item->label = g_strdup(tooltip);
-							}
-						else
-							{
-							action_item->label = g_strdup(label);
-							}
-						}
-					else
-						{
-						action_item->label = g_strdup(label);
-						}
-
-					action_item->name = action_name;
-					action_item->icon_name = g_strdup(gtk_action_get_stock_id(action));
-
-					list_duplicates = g_list_prepend(list_duplicates, action_item);
+					list_duplicates.emplace_back(action_name, action_label, gq_gtk_action_get_stock_id(action));
 					}
 				}
-			actions = actions->next;
 			}
-
-		groups = groups->next;
 		}
 
 	/* Use the shortest name i.e. ignore -Alt versions. Sort makes the shortest first in the list */
-	list_duplicates = g_list_sort(list_duplicates, simple_sort_cb);
+	const auto action_item_compare_names = [](const ActionItem &a, const ActionItem &b)
+	{
+		return g_strcmp0(a.name, b.name) < 0;
+	};
+	std::sort(list_duplicates.begin(), list_duplicates.end(), action_item_compare_names);
 
 	/* Ignore duplicate entries */
-	work1 = list_duplicates;
-	while (work1)
+	std::vector<ActionItem> list_unique;
+	for (const ActionItem &action_item : list_duplicates)
 		{
-		duplicate = FALSE;
-		work2 = list_unique;
-		/* The first entry must be unique, list_unique is null so control bypasses the while */
-		while (work2)
+		const auto action_item_has_label = [label = action_item.label](const ActionItem &action_item)
+		{
+			return action_item.has_label(label);
+		};
+		if (std::none_of(list_unique.cbegin(), list_unique.cend(), action_item_has_label))
 			{
-			if (g_strcmp0(static_cast<ActionItem *>(work2->data)->label, static_cast<ActionItem *>(work1->data)->label) == 0)
-				{
-				duplicate = TRUE;
-				break;
-				}
-			work2 = work2->next;
+			list_unique.push_back(action_item);
 			}
-
-		if (!duplicate)
-			{
-			action_item = g_new0(ActionItem, 1);
-			action_item->name = g_strdup(static_cast<ActionItem *>(work1->data)->name);
-			action_item->label = g_strdup(static_cast<ActionItem *>(work1->data)->label);
-			action_item->icon_name = g_strdup(static_cast<ActionItem *>(work1->data)->icon_name);
-			list_unique = g_list_append(list_unique, action_item);
-			}
-		work1 = work1->next;
 		}
 
-	g_list_free_full(list_duplicates, free_action_items_cb);
-
 	return list_unique;
 }
 
@@ -1609,10 +1414,10 @@ gboolean defined_mouse_buttons(GtkWidget *, GdkEventButton *event, gpointer data
 					}
 				else
 					{
-					action = gtk_action_group_get_action(lw->action_group, options->mouse_button_8);
+					action = gq_gtk_action_group_get_action(lw->action_group, options->mouse_button_8);
 					if (action)
 						{
-						gtk_action_activate(action);
+						gq_gtk_action_activate(action);
 						}
 					ret = TRUE;
 					}
@@ -1627,11 +1432,10 @@ gboolean defined_mouse_buttons(GtkWidget *, GdkEventButton *event, gpointer data
 					}
 				else
 					{
-					action = gtk_action_group_get_action(lw->action_group, options->mouse_button_9);
-					ret = TRUE;
+					action = gq_gtk_action_group_get_action(lw->action_group, options->mouse_button_9);
 					if (action)
 						{
-						gtk_action_activate(action);
+						gq_gtk_action_activate(action);
 						}
 					ret = TRUE;
 					}
@@ -1646,13 +1450,59 @@ gboolean defined_mouse_buttons(GtkWidget *, GdkEventButton *event, gpointer data
 
 GdkPixbuf *gq_gtk_icon_theme_load_icon_copy(GtkIconTheme *icon_theme, const gchar *icon_name, gint size, GtkIconLookupFlags flags)
 {
-	GError *error = nullptr;
-	GdkPixbuf *icon = gtk_icon_theme_load_icon(icon_theme, icon_name, size, flags, &error);
+	g_autoptr(GError) error = nullptr;
+	g_autoptr(GdkPixbuf) icon = gtk_icon_theme_load_icon(icon_theme, icon_name, size, flags, &error);
 	if (error) return nullptr;
 
-	GdkPixbuf *pixbuf = gdk_pixbuf_copy(icon);
-	g_object_unref(icon);
-	return pixbuf;
+	return gdk_pixbuf_copy(icon);
+}
+
+gboolean window_get_pointer_position(GdkWindow *window, GdkPoint &pos)
+{
+	GdkSeat *seat = gdk_display_get_default_seat(gdk_window_get_display(window));
+	GdkDevice *device = gdk_seat_get_pointer(seat);
+
+	gdk_window_get_device_position(window, device, &pos.x, &pos.y, nullptr);
+	gint width = gdk_window_get_width(window);
+	gint height = gdk_window_get_height(window);
+
+	return 0 <= pos.x && pos.x < width &&
+	       0 <= pos.y && pos.y < height;
+}
+
+GdkRectangle window_get_position_geometry(GdkWindow *window)
+{
+	GdkRectangle rect;
+
+	gdk_window_get_position(window, &rect.x, &rect.y);
+	rect.width = gdk_window_get_width(window);
+	rect.height = gdk_window_get_height(window);
+
+	return rect;
+}
+
+GdkRectangle window_get_root_origin_geometry(GdkWindow *window)
+{
+	GdkRectangle rect;
+
+	gdk_window_get_root_origin(window, &rect.x, &rect.y);
+	rect.width = gdk_window_get_width(window);
+	rect.height = gdk_window_get_height(window);
+
+	return rect;
+}
+
+gboolean window_received_event(GdkWindow *window, GdkPoint event)
+{
+	gint x;
+	gint y;
+	gdk_window_get_origin(window, &x, &y);
+
+	gint widht = gdk_window_get_width(window);
+	gint height = gdk_window_get_height(window);
+
+	return x <= event.x && event.x <= x + widht &&
+	       y <= event.y && event.y <= y + height;
 }
 
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
