@@ -22,11 +22,14 @@
 #ifndef UI_MISC_H
 #define UI_MISC_H
 
+#include <ctime>
+#include <vector>
 
-#include <sys/time.h>
-#include <sys/types.h>
-#include <time.h>
-
+#include <gdk-pixbuf/gdk-pixbuf.h>
+#include <gdk/gdk.h>
+#include <glib-object.h>
+#include <glib.h>
+#include <gtk/gtk.h>
 
 /* these values are per GNOME HIG */
 
@@ -115,35 +118,20 @@ GtkWidget *pref_button_new(GtkWidget *parent_box, const gchar *icon_name,
 
 GtkWidget *pref_checkbox_new(GtkWidget *parent_box, const gchar *text, gboolean active,
 			     GCallback func, gpointer data);
-GtkWidget *pref_checkbox_new_mnemonic(GtkWidget *parent_box, const gchar *text, gboolean active,
-				      GCallback func, gpointer data);
 
 GtkWidget *pref_checkbox_new_int(GtkWidget *parent_box, const gchar *text, gboolean active,
 				 gboolean *result);
 
 void pref_checkbox_link_sensitivity(GtkWidget *button, GtkWidget *widget);
-void pref_checkbox_link_sensitivity_swap(GtkWidget *button, GtkWidget *widget);
 
 GtkWidget *pref_radiobutton_new(GtkWidget *parent_box, GtkWidget *sibling,
 				const gchar *text, gboolean active,
 				GCallback func, gpointer data);
-GtkWidget *pref_radiobutton_new_mnemonic(GtkWidget *parent_box, GtkWidget *sibling,
-					 const gchar *text, gboolean active,
-					 GCallback func, gpointer data);
-
-GtkWidget *pref_radiobutton_new_int(GtkWidget *parent_box, GtkWidget *sibling,
-				    const gchar *text, gboolean active,
-				    gboolean *result, gboolean value,
-				    GCallback func, gpointer data);
 
 GtkWidget *pref_spin_new(GtkWidget *parent_box, const gchar *text, const gchar *suffix,
 			 gdouble min, gdouble max, gdouble step, gint digits,
 			 gdouble value,
 			 GCallback func, gpointer data);
-GtkWidget *pref_spin_new_mnemonic(GtkWidget *parent_box, const gchar *text, const gchar *suffix,
-				  gdouble min, gdouble max, gdouble step, gint digits,
-				  gdouble value,
-				  GCallback func, gpointer data);
 
 GtkWidget *pref_spin_new_int(GtkWidget *parent_box, const gchar *text, const gchar *suffix,
 			     gint min, gint max, gint step,
@@ -184,8 +172,6 @@ GtkWidget *pref_toolbar_button(GtkWidget *toolbar,
 			       const gchar *icon_name, const gchar *label, gboolean toggle,
 			       const gchar *description,
 			       GCallback func, gpointer data);
-void pref_toolbar_button_set_icon(GtkWidget *button, GtkWidget *widget, const gchar *stock_id);
-GtkWidget *pref_toolbar_spacer(GtkWidget *toolbar);
 
 
 GtkWidget *date_selection_new();
@@ -194,7 +180,6 @@ void date_selection_set(GtkWidget *widget, gint day, gint month, gint year);
 GDateTime *date_selection_get(GtkWidget *widget);
 
 void date_selection_time_set(GtkWidget *widget, time_t t);
-time_t date_selection_time_get(GtkWidget *widget);
 
 
 enum SizerPositionType {
@@ -212,12 +197,6 @@ void sizer_set_limits(GtkWidget *sizer,
 void pref_list_int_set(const gchar *group, const gchar *key, gint value);
 gboolean pref_list_int_get(const gchar *group, const gchar *key, gint *result);
 
-void pref_list_double_set(const gchar *group, const gchar *key, gdouble value);
-gboolean pref_list_double_get(const gchar *group, const gchar *key, gdouble *result);
-
-void pref_list_string_set(const gchar *group, const gchar *key, const gchar *value);
-gboolean pref_list_string_get(const gchar *group, const gchar *key, const gchar **result);
-
 
 void pref_color_button_set_cb(GtkWidget *widget, gpointer data);
 GtkWidget *pref_color_button_new(GtkWidget *parent_box,
@@ -229,17 +208,30 @@ gchar *text_widget_text_pull_selected(GtkWidget *text_widget);
 
 struct ActionItem
 {
-	const gchar *name; /* GtkActionEntry terminology */
-	const gchar *label;
-	const gchar *icon_name;
+	ActionItem(const gchar *name, const gchar *label, const gchar *icon_name);
+	ActionItem(const ActionItem &other);
+	ActionItem(ActionItem &&other) noexcept;
+	~ActionItem();
+	ActionItem &operator=(const ActionItem &other);
+	ActionItem &operator=(ActionItem &&other) noexcept;
+
+	bool has_label(const gchar *label) const;
+
+	gchar *name = nullptr; /* GtkActionEntry terminology */
+	gchar *label = nullptr;
+	gchar *icon_name = nullptr;
 };
 
-GList* get_action_items();
-void action_items_free(GList *list);
+std::vector<ActionItem> get_action_items();
 
 gboolean defined_mouse_buttons(GtkWidget *widget, GdkEventButton *event, gpointer data);
 
 // Copy pixbuf returned by gtk_icon_theme_load_icon() to avoid GTK+ keeping the old icon theme loaded
 GdkPixbuf *gq_gtk_icon_theme_load_icon_copy(GtkIconTheme *icon_theme, const gchar *icon_name, gint size, GtkIconLookupFlags flags);
+
+gboolean window_get_pointer_position(GdkWindow *window, GdkPoint &pos);
+GdkRectangle window_get_position_geometry(GdkWindow *window);
+GdkRectangle window_get_root_origin_geometry(GdkWindow *window);
+gboolean window_received_event(GdkWindow *window, GdkPoint event);
 #endif
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
