@@ -21,6 +21,7 @@
 
 #include "ui-pathsel.h"
 
+#include <array>
 #include <cstring>
 
 #include <dirent.h>
@@ -49,17 +50,6 @@
 namespace
 {
 
-constexpr gint dest_drag_types_n = 2;
-
-} // namespace
-
-enum {
-	DEST_WIDTH = 250,
-	DEST_HEIGHT = 210
-};
-
-#define PATH_SEL_USE_HEADINGS FALSE
-
 enum {
 	FILTER_COLUMN_NAME = 0,
 	FILTER_COLUMN_FILTER
@@ -96,6 +86,22 @@ struct DestDel_Data
 	gchar *path;
 };
 
+enum {
+	TARGET_URI_LIST,
+	TARGET_TEXT_PLAIN
+};
+
+constexpr std::array<GtkTargetEntry, 2> dest_drag_types{{
+	{ const_cast<gchar *>("text/uri-list"), 0, TARGET_URI_LIST },
+	{ const_cast<gchar *>("text/plain"),    0, TARGET_TEXT_PLAIN }
+}};
+
+constexpr gint DEST_WIDTH = 250;
+constexpr gint DEST_HEIGHT = 210;
+
+constexpr gboolean PATH_SEL_USE_HEADINGS = FALSE;
+
+} // namespace
 
 static void dest_view_delete_dlg_cancel(GenericDialog *gd, gpointer data);
 
@@ -128,12 +134,13 @@ static gboolean dest_check_filter(const gchar *filter, const gchar *file)
 	const gchar *f_ptr = filter;
 	const gchar *strt_ptr;
 	gint i;
-	gint l;
-
-	l = strlen(file);
 
 	if (filter[0] == '*') return TRUE;
-	while (f_ptr < filter + strlen(filter))
+
+	const gchar *filter_end = filter + strlen(filter);
+	const gint l = strlen(file);
+
+	while (f_ptr < filter_end)
 		{
 		strt_ptr = f_ptr;
 		i=0;
@@ -155,9 +162,9 @@ static gboolean dest_check_filter(const gchar *filter, const gchar *file)
 #define CASE_SORT strcmp
 #endif
 
-static gint dest_sort_cb(gpointer a, gpointer b)
+static gint dest_sort_cb(gconstpointer a, gconstpointer b)
 {
-	return CASE_SORT((gchar *)a, (gchar *)b);
+	return CASE_SORT(static_cast<const gchar *>(a), static_cast<const gchar *>(b));
 }
 
 static gboolean is_hidden(const gchar *name)
@@ -216,8 +223,8 @@ static void dest_populate(Dest_Data *dd, const gchar *path)
 	closedir(dp);
 	g_free(pathl);
 
-	path_list = g_list_sort(path_list, reinterpret_cast<GCompareFunc>(dest_sort_cb));
-	file_list = g_list_sort(file_list, reinterpret_cast<GCompareFunc>(dest_sort_cb));
+	path_list = g_list_sort(path_list, dest_sort_cb);
+	file_list = g_list_sort(file_list, dest_sort_cb);
 
 	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(dd->d_view)));
 	gtk_list_store_clear(store);
@@ -309,10 +316,10 @@ static void dest_change_dir(Dest_Data *dd, const gchar *path, gboolean retain_na
 
 	if (old_name)
 		{
-		gchar *basename = g_path_get_basename(full_path);
+		const size_t full_path_len = strlen(full_path);
+		g_autofree gchar *basename = g_path_get_basename(full_path);
 
-		gtk_editable_select_region(GTK_EDITABLE(dd->entry), strlen(full_path) - strlen(basename), strlen(full_path));
-		g_free(basename);
+		gtk_editable_select_region(GTK_EDITABLE(dd->entry), full_path_len - strlen(basename), full_path_len);
 		}
 
 	g_free(full_path);
@@ -324,16 +331,6 @@ static void dest_change_dir(Dest_Data *dd, const gchar *path, gboolean retain_na
  *-----------------------------------------------------------------------------
  */
 
-enum {
-	TARGET_URI_LIST,
-	TARGET_TEXT_PLAIN
-};
-
-static GtkTargetEntry dest_drag_types[] = {
-	{ const_cast<gchar *>("text/uri-list"), 0, TARGET_URI_LIST },
-	{ const_cast<gchar *>("text/plain"),    0, TARGET_TEXT_PLAIN }
-};
-
 static void dest_dnd_set_data(GtkWidget *view, GdkDragContext *,
 				  GtkSelectionData *selection_data,
 				  guint, guint, gpointer)
@@ -367,16 +364,16 @@ static void dest_dnd_set_data(GtkWidget *view, GdkDragContext *,
 static void dest_dnd_init(Dest_Data *dd)
 {
 	gtk_tree_view_enable_model_drag_source(GTK_TREE_VIEW(dd->d_view), GDK_BUTTON1_MASK,
-					       dest_drag_types, dest_drag_types_n,
-					       static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK | GDK_ACTION_ASK));
+	                                       dest_drag_types.data(), dest_drag_types.size(),
+	                                       static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK | GDK_ACTION_ASK));
 	g_signal_connect(G_OBJECT(dd->d_view), "drag_data_get",
 			 G_CALLBACK(dest_dnd_set_data), dd);
 
 	if (dd->f_view)
 		{
 		gtk_tree_view_enable_model_drag_source(GTK_TREE_VIEW(dd->f_view), GDK_BUTTON1_MASK,
-						       dest_drag_types, dest_drag_types_n,
-						       static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK | GDK_ACTION_ASK));
+		                                       dest_drag_types.data(), dest_drag_types.size(),
+		                                       static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK | GDK_ACTION_ASK));
 		g_signal_connect(G_OBJECT(dd->f_view), "drag_data_get",
 				 G_CALLBACK(dest_dnd_set_data), dd);
 		}
@@ -445,7 +442,7 @@ static gint dest_view_rename_cb(TreeEditData *ted, const gchar *old_name, const
 		gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, new_name, 1, new_path, -1);
 
 		text = gq_gtk_entry_get_text(GTK_ENTRY(dd->entry));
-		if (text && old_path && strcmp(text, old_path) == 0)
+		if (text && strcmp(text, old_path) == 0)
 			{
 			gq_gtk_entry_set_text(GTK_ENTRY(dd->entry), new_path);
 			}
@@ -710,6 +707,8 @@ static gboolean dest_keypress_cb(GtkWidget *view, GdkEventKey *event, gpointer d
 				return TRUE;
 				}
 			break;
+		default:
+			break;
 		}
 
 	return FALSE;
@@ -970,7 +969,7 @@ static void dest_filter_changed_cb(GtkEditable *, gpointer data)
 
 	g_free(dd->filter);
 	dd->filter = nullptr;
-	if (strlen(buf) > 0) dd->filter = g_strdup(buf);
+	if (buf[0] != '\0') dd->filter = g_strdup(buf);
 
 	path = g_strdup(dd->path);
 	dest_populate(dd, path);
@@ -1033,7 +1032,7 @@ GtkWidget *path_selection_new_with_files(GtkWidget *entry, const gchar *path,
 	gtk_widget_show(dd->hidden_button);
 
 	gq_gtk_box_pack_start(GTK_BOX(table), hbox1, FALSE, FALSE, 0);
-	gtk_widget_show_all(hbox1);
+	gq_gtk_widget_show_all(hbox1);
 
 	hbox2 = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, PREF_PAD_GAP);
 	if (filter)
@@ -1219,38 +1218,6 @@ GtkWidget *path_selection_new_with_files(GtkWidget *entry, const gchar *path,
 	return table;
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-GtkWidget *path_selection_new_unused(const gchar *path, GtkWidget *entry)
-{
-	return path_selection_new_with_files(entry, path, nullptr, nullptr);
-}
-
-void path_selection_sync_to_entry_unused(GtkWidget *entry)
-{
-	auto *dd = static_cast<Dest_Data *>(g_object_get_data(G_OBJECT(entry), "destination_data"));
-	const gchar *path;
-
-	if (!dd) return;
-
-	path = gq_gtk_entry_get_text(GTK_ENTRY(entry));
-
-	if (isdir(path) && (!dd->path || strcmp(path, dd->path) != 0))
-		{
-		dest_populate(dd, path);
-		}
-	else
-		{
-		gchar *buf = remove_level_from_path(path);
-		if (isdir(buf) && (!dd->path || strcmp(buf, dd->path) != 0))
-			{
-			dest_populate(dd, buf);
-			}
-		g_free(buf);
-		}
-}
-#pragma GCC diagnostic pop
-
 void path_selection_add_select_func(GtkWidget *entry,
 				    void (*func)(const gchar *, gpointer), gpointer data)
 {
@@ -1261,23 +1228,4 @@ void path_selection_add_select_func(GtkWidget *entry,
 	dd->select_func = func;
 	dd->select_data = data;
 }
-
-void path_selection_add_filter(GtkWidget *entry, const gchar *filter, const gchar *description, gboolean set)
-{
-	auto dd = static_cast<Dest_Data *>(g_object_get_data(G_OBJECT(entry), "destination_data"));
-
-	if (!dd) return;
-	if (!filter) return;
-
-	dest_filter_add(dd, filter, description, set);
-}
-
-void path_selection_clear_filter(GtkWidget *entry)
-{
-	auto dd = static_cast<Dest_Data *>(g_object_get_data(G_OBJECT(entry), "destination_data"));
-
-	if (!dd) return;
-
-	dest_filter_clear(dd);
-}
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
