@@ -146,7 +146,7 @@ static void tab_completion_read_dir(TabCompData *td, const gchar *path)
 	g_free(pathl);
 }
 
-static void tab_completion_destroy(GtkWidget *, gpointer data)
+static void tab_completion_destroy(gpointer data)
 {
 	auto td = static_cast<TabCompData *>(data);
 
@@ -201,7 +201,7 @@ static void tab_completion_emit_tab_signal(TabCompData *td)
 }
 
 #ifdef TAB_COMPLETION_ENABLE_POPUP_MENU
-void tab_completion_iter_menu_items(GtkWidget *widget, gpointer data)
+static void tab_completion_iter_menu_items(GtkWidget *widget, gpointer data)
 {
 	auto td = static_cast<TabCompData *>(data);
 	GtkWidget *child;
@@ -425,7 +425,7 @@ static gboolean tab_completion_do(TabCompData *td)
 	if (ptr > entry_dir) ptr--;
 	ptr[0] = '\0';
 
-	if (strlen(entry_dir) == 0)
+	if (entry_dir[0] == '\0')
 		{
 		g_free(entry_dir);
 		entry_dir = g_strdup(G_DIR_SEPARATOR_S); /** @FIXME win32 */
@@ -476,7 +476,6 @@ static gboolean tab_completion_do(TabCompData *td)
 			while (!done)
 				{
 				list = poss;
-				if (!list) done = TRUE;
 				while (list && !done)
 					{
 					auto file = static_cast<gchar *>(list->data);
@@ -651,8 +650,6 @@ GtkWidget *tab_completion_new_with_history(GtkWidget **entry, const gchar *text,
 	td->history_key = g_strdup(history_key);
 	td->history_levels = max_levels;
 
-	work = history_list_get_by_key(td->history_key);
-
 	work = history_list_get_by_key(history_key);
 	while (work)
 		{
@@ -763,12 +760,10 @@ void tab_completion_add_to_entry(GtkWidget *entry, void (*enter_func)(const gcha
 	td->filter = g_strdup(filter);
 	td->filter_desc = g_strdup(filter_desc);
 
-	g_object_set_data(G_OBJECT(td->entry), "tab_completion_data", td);
+	g_object_set_data_full(G_OBJECT(entry), "tab_completion_data", td, tab_completion_destroy);
 
 	g_signal_connect(G_OBJECT(entry), "key_press_event",
 			 G_CALLBACK(tab_completion_key_pressed), td);
-	g_signal_connect(G_OBJECT(entry), "destroy",
-			 G_CALLBACK(tab_completion_destroy), td);
 }
 
 void tab_completion_add_tab_func(GtkWidget *entry, void (*tab_func)(const gchar *, gpointer), gpointer data)
@@ -862,7 +857,7 @@ static void tab_completion_select_show(TabCompData *td)
 		}
 
 	path = gq_gtk_entry_get_text(GTK_ENTRY(td->entry));
-	if (strlen(path) == 0) path = nullptr;
+	if (path[0] == '\0') path = nullptr;
 	if (td->fd_folders_only)
 		{
 		file_dialog_add_path_widgets(td->fd, nullptr, path, td->history_key, nullptr, nullptr);
