@@ -21,7 +21,6 @@
 
 #include "ui-utildlg.h"
 
-#include <cstdint>
 #include <cstdio>
 #include <cstring>
 #include <ctime>
@@ -46,6 +45,12 @@
 #include "ui-tabcomp.h"
 #include "window.h"
 
+namespace
+{
+
+constexpr gint max_buffer_size = 16384;
+} // namespace
+
 /*
  *-----------------------------------------------------------------------------
  * generic dialog
@@ -54,17 +59,14 @@
 
 struct DialogWindow
 {
-	gint x;
-	gint y;
-	gint w;
-	gint h;
+	GdkRectangle rect;
 	gchar *title;
 	gchar *role;
 };
 
 static GList *dialog_windows = nullptr;
 
-static void generic_dialog_save_window(const gchar *title, const gchar *role, gint x, gint y, gint h, gint w)
+static void generic_dialog_save_window(const gchar *title, const gchar *role, GdkRectangle rect)
 {
 	GList *work;
 
@@ -74,27 +76,21 @@ static void generic_dialog_save_window(const gchar *title, const gchar *role, gi
 		auto dw = static_cast<DialogWindow *>(work->data);
 		if (g_strcmp0(dw->title ,title) == 0 && g_strcmp0(dw->role, role) == 0)
 			{
-			dw->x = x;
-			dw->y = y;
-			dw->w = w;
-			dw->h = h;
+			dw->rect = rect;
 			return;
 			}
 		work = work->next;
 		}
 
 	auto dw = g_new0(DialogWindow, 1);
+	dw->rect = rect;
 	dw->title = g_strdup(title);
 	dw->role = g_strdup(role);
-	dw->x = x;
-	dw->y = y;
-	dw->w = w;
-	dw->h = h;
 
 	dialog_windows = g_list_append(dialog_windows, dw);
 }
 
-static gboolean generic_dialog_find_window(const gchar *title, const gchar *role, gint *x, gint *y, gint *h, gint *w)
+static gboolean generic_dialog_find_window(const gchar *title, const gchar *role, GdkRectangle &rect)
 {
 	GList *work;
 
@@ -105,10 +101,7 @@ static gboolean generic_dialog_find_window(const gchar *title, const gchar *role
 
 		if (g_strcmp0(dw->title,title) == 0 && g_strcmp0(dw->role, role) == 0)
 			{
-			*x = dw->x;
-			*y = dw->y;
-			*w = dw->w;
-			*h = dw->h;
+			rect = dw->rect;
 			return TRUE;
 			}
 		work = work->next;
@@ -121,14 +114,6 @@ void generic_dialog_close(GenericDialog *gd)
 	gchar *ident_string;
 	gchar *full_title;
 	gchar *actual_title;
-	gint x;
-	gint y;
-	gint h;
-	gint w;
-
-	gdk_window_get_root_origin(gtk_widget_get_window (gd->dialog), &x, &y);
-	w = gdk_window_get_width(gtk_widget_get_window (gd->dialog));
-	h = gdk_window_get_height(gtk_widget_get_window (gd->dialog));
 
 	/* The window title is modified in window.cc: window_new()
 	 * by appending the string " - Geeqie"
@@ -137,7 +122,9 @@ void generic_dialog_close(GenericDialog *gd)
 	full_title = g_strdup(gtk_window_get_title(GTK_WINDOW(gd->dialog)));
 	actual_title = strndup(full_title, g_strrstr(full_title, ident_string) - full_title);
 
-	generic_dialog_save_window(actual_title, gtk_window_get_role(GTK_WINDOW(gd->dialog)), x, y, w, h);
+	GdkRectangle rect = window_get_root_origin_geometry(gtk_widget_get_window(gd->dialog));
+
+	generic_dialog_save_window(actual_title, gtk_window_get_role(GTK_WINDOW(gd->dialog)), rect);
 
 	gq_gtk_widget_destroy(gd->dialog);
 	g_free(gd);
@@ -330,39 +317,32 @@ GtkWidget *generic_dialog_add_message(GenericDialog *gd, const gchar *icon_name,
 
 void generic_dialog_windows_load_config(const gchar **attribute_names, const gchar **attribute_values)
 {
-	auto dw =  g_new0(DialogWindow, 1);
-	gchar *title = nullptr;
-	gchar *role = nullptr;
-	gint x = 0;
-	gint y = 0;
-	gint w = 0;
-	gint h = 0;
+	auto dw = g_new0(DialogWindow, 1);
 
 	while (*attribute_names)
 		{
 		const gchar *option = *attribute_names++;
 		const gchar *value = *attribute_values++;
-		if (READ_CHAR_FULL("title", title)) continue;
-		if (READ_CHAR_FULL("role", role)) continue;
-		if (READ_INT_FULL("x", x)) continue;
-		if (READ_INT_FULL("y", y)) continue;
-		if (READ_INT_FULL("w", w)) continue;
-		if (READ_INT_FULL("h", h)) continue;
+		if (READ_CHAR(*dw, title)) continue;
+		if (READ_CHAR(*dw, role)) continue;
+		if (READ_INT(dw->rect, x)) continue;
+		if (READ_INT(dw->rect, y)) continue;
+		if (READ_INT_FULL("w", dw->rect.width)) continue;
+		if (READ_INT_FULL("h", dw->rect.height)) continue;
 
 		log_printf("unknown attribute %s = %s\n", option, value);
 		}
 
-	if (title && title[0] != 0)
+	if (dw->title && dw->title[0] != 0)
 		{
-		dw->title = g_strdup(title);
-		dw->role = g_strdup(role);
-		dw->x = x;
-		dw->y = y;
-		dw->w = w;
-		dw->h = h;
-
 		dialog_windows = g_list_append(dialog_windows, dw);
 		}
+	else
+		{
+		g_free(dw->title);
+		g_free(dw->role);
+		g_free(dw);
+		}
 }
 
 void generic_dialog_windows_write_config(GString *outstr, gint indent)
@@ -381,10 +361,10 @@ void generic_dialog_windows_write_config(GString *outstr, gint indent)
 			WRITE_NL(); WRITE_STRING("<window ");
 			write_char_option(outstr, indent + 1, "title", dw->title);
 			write_char_option(outstr, indent + 1, "role", dw->role);
-			WRITE_INT(*dw, x);
-			WRITE_INT(*dw, y);
-			WRITE_INT(*dw, w);
-			WRITE_INT(*dw, h);
+			WRITE_INT(dw->rect, x);
+			WRITE_INT(dw->rect, y);
+			write_int_option(outstr, indent, "w", dw->rect.width);
+			write_int_option(outstr, indent, "h", dw->rect.height);
 			WRITE_STRING("/>");
 			work = work->next;
 			}
@@ -400,10 +380,6 @@ static void generic_dialog_setup(GenericDialog *gd,
 				 void (*cancel_cb)(GenericDialog *, gpointer), gpointer data)
 {
 	GtkWidget *vbox;
-	gint x;
-	gint y;
-	gint w;
-	gint h;
 	GtkWidget *scrolled;
 
 	gd->auto_close = auto_close;
@@ -416,10 +392,11 @@ static void generic_dialog_setup(GenericDialog *gd,
 
 	if (options->save_dialog_window_positions)
 		{
-		if (generic_dialog_find_window(title, role, &x, &y, &w, &h))
+		GdkRectangle rect;
+		if (generic_dialog_find_window(title, role, rect))
 			{
-			gtk_window_set_default_size(GTK_WINDOW(gd->dialog), w, h);
-			gq_gtk_window_move(GTK_WINDOW(gd->dialog), x, y);
+			gtk_window_set_default_size(GTK_WINDOW(gd->dialog), rect.width, rect.height);
+			gq_gtk_window_move(GTK_WINDOW(gd->dialog), rect.x, rect.y);
 			}
 		}
 
@@ -488,6 +465,9 @@ static void generic_dialog_setup(GenericDialog *gd,
 	gd->default_cb = nullptr;
 }
 
+/**
+ * @brief When parent is not NULL, the dialog is set as a transient of the window containing parent
+ */
 GenericDialog *generic_dialog_new(const gchar *title,
 				  const gchar *role,
 				  GtkWidget *parent, gboolean auto_close,
@@ -606,20 +586,15 @@ static void show_notification_message(AppImageData *appimage_data)
 	gtk_widget_show(appimage_data->window);
 }
 
-void appimage_notification_func(gpointer data, gpointer)
+static void appimage_notification_func(gpointer data, gpointer)
 {
-	auto appimage_data = static_cast<AppImageData *>(data);
-	gboolean internet_available = FALSE;
-	gchar **version_split;
-	GFile *file_github;
-	GFileInfo *file_info;
+	FILE *pipe;
 	GNetworkMonitor *net_mon;
 	GSocketConnectable *geeqie_github;
-	struct tm current_version_date;
-	int64_t file_github_date;
-	unsigned int year;
-	unsigned int month;
-	unsigned int day;
+	auto appimage_data = static_cast<AppImageData *>(data);
+	char buffer[max_buffer_size];
+	char result[max_buffer_size];
+	gboolean internet_available = FALSE;
 
 	/* If this is a release version, do not check for updates */
 	if (g_strrstr(VERSION, "git"))
@@ -635,31 +610,39 @@ void appimage_notification_func(gpointer data, gpointer)
 
 		if (internet_available)
 			{
-			/* VERSION looks like: 2.0.1+git20220116-c791cbee */
-			version_split = g_strsplit_set(VERSION, "+-", -1);
+			pipe = popen("curl --max-time 2 --silent https://api.github.com/repos/BestImageViewer/geeqie/releases/tags/continuous", "r");
 
-			file_github = g_file_new_for_uri("https://github.com/BestImageViewer/geeqie/releases/download/continuous/Geeqie-latest-x86_64.AppImage");
-			file_info = g_file_query_info(file_github, "time::modified", G_FILE_QUERY_INFO_NONE, nullptr, nullptr);
-
-			file_github_date = g_file_info_get_attribute_uint64(file_info, "time::modified");
-
-			sscanf(version_split[1] + 3, "%4u%2u%2u", &year, &month, &day);
-			current_version_date.tm_year  = year - 1900;
-			current_version_date.tm_mon   = month - 1;
-			current_version_date.tm_mday  = day;
-			current_version_date.tm_hour  = 0;
-			current_version_date.tm_min   = 0;
-			current_version_date.tm_sec   = 0;
-			current_version_date.tm_isdst = 0;
-
-			if (file_github_date > mktime(&current_version_date))
+			if (pipe == nullptr)
 				{
-				show_notification_message(appimage_data);
+				log_printf("Failed to get date from GitHub");
 				}
+			else
+				{
+				while (fgets(buffer, max_buffer_size, pipe) != nullptr)
+					{
+					strcat(result, buffer);
+					}
+				pclose(pipe);
 
-			g_object_unref(file_github);
-			g_object_unref(file_info);
-			g_strfreev(version_split);
+				/* GitHub date looks like: "published_at": "2024-04-17T08:50:08Z" */
+				gchar *start_date = g_strstr_len(result, -1, "published_at");
+				start_date += 16; // skip 'published_at": "' part
+				start_date[10] = '\0'; // drop everything after YYYY-mm-dd part
+
+				std::tm github_version_date{};
+				strptime(start_date, "%Y-%m-%d", &github_version_date);
+
+				/* VERSION looks like: 2.0.1+git20220116-c791cbee */
+				g_auto(GStrv) version_split = g_strsplit_set(VERSION, "+-", -1);
+
+				std::tm current_version_date{};
+				strptime(version_split[1] + 3, "%Y-%m-%d", &current_version_date);
+
+				if (mktime(&github_version_date) > mktime(&current_version_date))
+					{
+					show_notification_message(appimage_data);
+					}
+				}
 			}
 		}
 }
@@ -728,6 +711,12 @@ static void file_dialog_entry_enter_cb(const gchar *, gpointer data)
 	if (gd->default_cb) gd->default_cb(gd, gd->data);
 }
 
+/**
+ * @brief Default_path is default base directory, and is only used if no history
+ * exists for history_key (HOME is used if default_path is NULL).
+ * path can be a full path or only a file name. If name only, appended to
+ * the default_path or the last history (see default_path)
+ */
 void file_dialog_add_path_widgets(FileDialog *fdlg, const gchar *default_path, const gchar *path,
 				  const gchar *history_key, const gchar *filter, const gchar *filter_desc)
 {
@@ -781,21 +770,6 @@ void file_dialog_add_path_widgets(FileDialog *fdlg, const gchar *default_path, c
 			 G_CALLBACK(file_dialog_entry_cb), fdlg);
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-void file_dialog_add_filter_unused(FileDialog *fdlg, const gchar *filter, const gchar *filter_desc, gboolean set)
-{
-	if (!fdlg->entry) return;
-	path_selection_add_filter(fdlg->entry, filter, filter_desc, set);
-}
-
-void file_dialog_clear_filter_unused(FileDialog *fdlg)
-{
-	if (!fdlg->entry) return;
-	path_selection_clear_filter(fdlg->entry);
-}
-#pragma GCC diagnostic pop
-
 void file_dialog_sync_history(FileDialog *fdlg, gboolean dir_only)
 {
 	if (!fdlg->dest_path) return;
