@@ -23,6 +23,7 @@
 
 #include <unistd.h>
 
+#include <array>
 #include <cstring>
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
@@ -54,11 +55,6 @@
 namespace
 {
 
-constexpr gint DIALOG_WIDTH = 750;
-
-/* thumbnail spec has a max depth of 4 (.thumb??/fail/appname/??.png) */
-constexpr gint UTILITY_DELETE_MAX_DEPTH = 5;
-
 struct PixmapErrors
 {
 	GdkPixbuf *error;
@@ -66,6 +62,36 @@ struct PixmapErrors
 	GdkPixbuf *apply;
 };
 
+struct ClipboardData
+{
+	GList *path_list; /**< g_strdup(fd->path) */
+	gboolean quoted;
+	ClipboardAction action;
+};
+
+enum ClipboardDestination {
+	CLIPBOARD_TEXT_PLAIN	= 0,
+	CLIPBOARD_TEXT_URI_LIST	= 1,
+	CLIPBOARD_X_SPECIAL_GNOME_COPIED_FILES	= 2,
+	CLIPBOARD_UTF8_STRING	= 3
+};
+
+constexpr std::array<GtkTargetEntry, 4> target_types
+{{
+	{const_cast<gchar *>("text/plain"), 0, CLIPBOARD_TEXT_PLAIN},
+	{const_cast<gchar *>("text/uri-list"), 0, CLIPBOARD_TEXT_URI_LIST},
+	{const_cast<gchar *>("x-special/gnome-copied-files"), 0, CLIPBOARD_X_SPECIAL_GNOME_COPIED_FILES},
+	{const_cast<gchar *>("UTF8_STRING"), 0, CLIPBOARD_UTF8_STRING},
+}};
+
+constexpr gint DIALOG_DEF_IMAGE_DIM_X = 150;
+constexpr gint DIALOG_DEF_IMAGE_DIM_Y = 100;
+
+constexpr gint DIALOG_WIDTH = 750;
+
+/* thumbnail spec has a max depth of 4 (.thumb??/fail/appname/??.png) */
+constexpr gint UTILITY_DELETE_MAX_DEPTH = 5;
+
 GdkPixbuf *file_util_get_error_icon(FileData *fd, GList *list, GtkWidget *)
 {
 	static PixmapErrors pe = []() -> PixmapErrors
@@ -101,39 +127,12 @@ GdkPixbuf *file_util_get_error_icon(FileData *fd, GList *list, GtkWidget *)
 
 } // namespace
 
-enum ClipboardDestination {
-	CLIPBOARD_TEXT_PLAIN	= 0,
-	CLIPBOARD_TEXT_URI_LIST	= 1,
-	CLIPBOARD_X_SPECIAL_GNOME_COPIED_FILES	= 2,
-	CLIPBOARD_UTF8_STRING	= 3
-};
-
-static GtkTargetEntry target_types[] =
-{
-	{const_cast<gchar *>("text/plain"), 0, CLIPBOARD_TEXT_PLAIN},
-	{const_cast<gchar *>("text/uri-list"), 0, CLIPBOARD_TEXT_URI_LIST},
-	{const_cast<gchar *>("x-special/gnome-copied-files"), 0, CLIPBOARD_X_SPECIAL_GNOME_COPIED_FILES},
-	{const_cast<gchar *>("UTF8_STRING"), 0, CLIPBOARD_UTF8_STRING},
-};
-static gint target_types_n = 4;
-
-struct ClipboardData
-{
-	GList *path_list; /**< g_strdup(fd->path) */
-	gboolean quoted;
-};
-
 /*
  *--------------------------------------------------------------------------
  * Adds 1 or 2 images (if 2, side by side) to a GenericDialog
  *--------------------------------------------------------------------------
  */
 
-enum {
-	DIALOG_DEF_IMAGE_DIM_X = 150,
-	DIALOG_DEF_IMAGE_DIM_Y = 100
-};
-
 static void generic_dialog_add_image(GenericDialog *gd, GtkWidget *box,
 				     FileData *fd1, const gchar *header1,
 				     gboolean second_image,
@@ -967,7 +966,7 @@ static gint file_util_perform_ci_dir_cb(gpointer, EditorFlags flags, GList *, gp
 	return EDITOR_CB_CONTINUE; /* does not matter, there was just single directory */
 }
 
-void file_util_perform_ci(UtilityData *ud)
+static void file_util_perform_ci(UtilityData *ud)
 {
 	switch (ud->type)
 		{
@@ -1062,10 +1061,10 @@ static void file_util_check_abort_cb(GenericDialog *, gpointer data)
 	file_util_dialog_run(ud);
 }
 
-void file_util_check_ci(UtilityData *ud)
+static void file_util_check_ci(UtilityData *ud)
 {
 	gint error = CHANGE_OK;
-	gchar *desc = nullptr;
+	g_autofree gchar *desc = nullptr;
 
 	if (ud->type != UTILITY_TYPE_CREATE_FOLDER &&
 	    ud->type != UTILITY_TYPE_RENAME_FOLDER)
@@ -1088,13 +1087,10 @@ void file_util_check_ci(UtilityData *ud)
 		}
 	else
 		{
-		if (ud->type == UTILITY_TYPE_CREATE_FOLDER || ud->type == UTILITY_TYPE_RENAME_FOLDER)
+		if (isdir(ud->dest_path) || isfile(ud->dest_path))
 			{
-			if (isdir(ud->dest_path) || isfile(ud->dest_path))
-				{
-				error = CHANGE_DEST_EXISTS;
-				desc = g_strdup_printf(_("%s already exists"), ud->dest_path);
-				}
+			error = CHANGE_DEST_EXISTS;
+			desc = g_strdup_printf(_("%s already exists"), ud->dest_path);
 			}
 		}
 
@@ -1105,34 +1101,23 @@ void file_util_check_ci(UtilityData *ud)
 		return;
 		}
 
+	GenericDialog *d = file_util_gen_dlg(ud->messages.title, "dlg_confirm",
+	                                     ud->parent, TRUE,
+	                                     file_util_check_abort_cb, ud);
 	if (!(error & CHANGE_ERROR_MASK))
 		{
 		/* just a warning */
-		GenericDialog *d;
-
-		d = file_util_gen_dlg(ud->messages.title, "dlg_confirm",
-					ud->parent, TRUE,
-					file_util_check_abort_cb, ud);
-
 		generic_dialog_add_message(d, GQ_ICON_DIALOG_WARNING, _("Really continue?"), desc, TRUE);
 
 		generic_dialog_add_button(d, GQ_ICON_GO_NEXT, _("Co_ntinue"),
 					  file_util_check_resume_cb, TRUE);
-		gtk_widget_show(d->dialog);
 		}
 	else
 		{
 		/* fatal error */
-		GenericDialog *d;
-
-		d = file_util_gen_dlg(ud->messages.title, "dlg_confirm",
-					ud->parent, TRUE,
-					file_util_check_abort_cb, ud);
 		generic_dialog_add_message(d, GQ_ICON_DIALOG_WARNING, _("This operation can't continue:"), desc, TRUE);
-
-		gtk_widget_show(d->dialog);
 		}
-	g_free(desc);
+	gtk_widget_show(d->dialog);
 }
 
 
@@ -2162,7 +2147,7 @@ static void file_util_write_metadata_details_dialog(UtilityData *ud, FileData *f
 		}
 	else
 		{
-		message2 = g_strdup_printf(_("The following metadata tags will be written to the image file itself."));
+		message2 = g_strdup_printf("%s", _("The following metadata tags will be written to the image file itself."));
 		}
 
 	box = generic_dialog_add_message(gd, icon_name, _("Overview of changed metadata"), message1, TRUE);
@@ -2266,16 +2251,33 @@ static void file_util_delete_full(FileData *source_fd, GList *flist, GtkWidget *
 	ud->done_func = done_func;
 
 	ud->details_func = file_util_details_dialog;
-	if(options->file_ops.safe_delete_enable)
+
+	if (g_list_length(flist) > 1)
 		{
-		message = _("This will move the following files to the Trash bin");
+		if(options->file_ops.safe_delete_enable)
+			{
+			message = g_strdup_printf("%s%d%s", _("⚠ This will move the following    "), g_list_length(flist), _("    files to the Trash bin"));
+			}
+		else
+			{
+			message = g_strdup_printf("%s%d%s",_("⚠ This will permanently delete the following    "), g_list_length(flist), _("    files"));
+			}
+		ud->messages.question = _("Delete files?");
 		}
 	else
 		{
-		message = _("This will permanently delete the following files");
+		if(options->file_ops.safe_delete_enable)
+			{
+			message = _("This will move the following file to the Trash bin");
+			}
+		else
+			{
+			message = _("This will permanently delete the following file");
+			}
+		ud->messages.question = _("Delete file?");
 		}
+
 	ud->messages.title = _("Delete");
-	ud->messages.question = _("Delete files?");
 	ud->messages.desc_flist = message;
 	ud->messages.desc_source_fd = "";
 	ud->messages.fail = _("File deletion failed");
@@ -2587,8 +2589,6 @@ static gboolean file_util_delete_dir_empty_path(UtilityData *ud, FileData *fd, g
 	GList *flist;
 	GList *work;
 
-	gboolean ok = TRUE;
-
 	DEBUG_1("deltree into: %s", fd->path);
 
 	level++;
@@ -2605,15 +2605,12 @@ static gboolean file_util_delete_dir_empty_path(UtilityData *ud, FileData *fd, g
 		return 0;
 		}
 
+	gboolean ok = file_data_sc_add_ci_delete(fd);
 	if (ok)
 		{
-		ok = file_data_sc_add_ci_delete(fd);
-		if (ok)
-			{
-			ud->content_list = g_list_prepend(ud->content_list, fd);
-			}
-		// ud->fail_fd = fd
+		ud->content_list = g_list_prepend(ud->content_list, fd);
 		}
+	// ud->fail_fd = fd
 
 	work = dlist;
 	while (work && ok)
@@ -2647,7 +2644,6 @@ static gboolean file_util_delete_dir_empty_path(UtilityData *ud, FileData *fd, g
 	filelist_free(dlist);
 	filelist_free(flist);
 
-
 	DEBUG_1("deltree done: %s", fd->path);
 
 	return ok;
@@ -3131,14 +3127,6 @@ void file_util_start_editor_from_filelist(const gchar *key, GList *list, const g
 	file_util_start_editor_full(key, nullptr, list, nullptr, working_directory, parent, UTILITY_PHASE_ENTERING);
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-void file_util_start_filter_from_file_unused(const gchar *key, FileData *fd, const gchar *dest_path, GtkWidget *parent)
-{
-	file_util_start_editor_full(key, fd, nullptr, dest_path, nullptr, parent, UTILITY_PHASE_ENTERING);
-}
-#pragma GCC diagnostic pop
-
 void file_util_start_filter_from_filelist(const gchar *key, GList *list, const gchar *dest_path, GtkWidget *parent)
 {
 	file_util_start_editor_full(key, nullptr, list, dest_path, nullptr, parent, UTILITY_PHASE_ENTERING);
@@ -3188,7 +3176,15 @@ static void clipboard_get_func(GtkClipboard *clipboard, GtkSelectionData *select
 
 	if (clipboard == gtk_clipboard_get(GDK_SELECTION_CLIPBOARD) && info == CLIPBOARD_X_SPECIAL_GNOME_COPIED_FILES)
 		{
-		g_string_append(path_list_str, "copy");
+		switch (cbd->action)
+			{
+			case ClipboardAction::COPY:
+				g_string_append(path_list_str, "copy");
+				break;
+			case ClipboardAction::CUT:
+				g_string_append(path_list_str, "cut");
+				break;
+			}
 
 		while (work)
 			{
@@ -3247,31 +3243,38 @@ static void clipboard_clear_func(GtkClipboard *, gpointer data)
 	g_free(cbd);
 }
 
-void file_util_copy_path_to_clipboard(FileData *fd, gboolean quoted)
+static gboolean path_list_to_clipboard(GList *path_list, gboolean quoted, ClipboardAction action, GdkAtom selection)
 {
-	ClipboardData *cbd;
+	auto *cbd = g_new0(ClipboardData, 1);
+	cbd->path_list = path_list;
+	cbd->quoted = quoted;
+	cbd->action = action;
 
+	return gtk_clipboard_set_with_data(gtk_clipboard_get(selection), target_types.data(), target_types.size(), clipboard_get_func, clipboard_clear_func, cbd);
+}
+
+/**
+ * @brief
+ * @param fd
+ * @param quoted
+ * @param action
+ */
+void file_util_copy_path_to_clipboard(FileData *fd, gboolean quoted, ClipboardAction action)
+{
 	if (!fd || !*fd->path) return;
 
-
 	if (options->clipboard_selection == CLIPBOARD_PRIMARY || options->clipboard_selection == CLIPBOARD_BOTH)
 		{
-		cbd = g_new0(ClipboardData, 1);
-		cbd->path_list = nullptr;
-		cbd->quoted = quoted;
-		cbd->path_list = g_list_append(cbd->path_list, g_strdup(fd->path));
+		GList *path_list = g_list_append(nullptr, g_strdup(fd->path));
 
-		gtk_clipboard_set_with_data(gtk_clipboard_get(GDK_SELECTION_PRIMARY), target_types, target_types_n, clipboard_get_func, clipboard_clear_func, cbd);
+		path_list_to_clipboard(path_list, quoted, action, GDK_SELECTION_PRIMARY);
 		}
 
 	if (options->clipboard_selection == CLIPBOARD_CLIPBOARD || options->clipboard_selection == CLIPBOARD_BOTH)
 		{
-		cbd = g_new0(ClipboardData, 1);
-		cbd->path_list = nullptr;
-		cbd->quoted = quoted;
-		cbd->path_list = g_list_append(cbd->path_list, g_strdup(fd->path));
+		GList *path_list = g_list_append(nullptr, g_strdup(fd->path));
 
-		gtk_clipboard_set_with_data(gtk_clipboard_get(GDK_SELECTION_CLIPBOARD), target_types, target_types_n, clipboard_get_func, clipboard_clear_func, cbd);
+		path_list_to_clipboard(path_list, quoted, action, GDK_SELECTION_CLIPBOARD);
 		}
 }
 
@@ -3279,53 +3282,35 @@ void file_util_copy_path_to_clipboard(FileData *fd, gboolean quoted)
  * @brief
  * @param fd_list List of fd
  * @param quoted
- *
- *
+ * @param action
  */
-void file_util_copy_path_list_to_clipboard(GList *fd_list, gboolean quoted)
+void file_util_path_list_to_clipboard(GList *fd_list, gboolean quoted, ClipboardAction action)
 {
-	ClipboardData *cbd;
-	FileData *fd;
-	GList *work;
+	// FIXME Is it safe to use FileList::to_path_list()?
+	static const auto get_path_list = [](GList *fd_list)
+	{
+		GList *path_list = nullptr;
 
-	if (options->clipboard_selection == CLIPBOARD_PRIMARY || options->clipboard_selection == CLIPBOARD_BOTH)
-		{
-		cbd = g_new0(ClipboardData, 1);
-		cbd->path_list = nullptr;
-		cbd->quoted = quoted;
-		work = fd_list;
-
-		while (work)
+		for (GList *work = fd_list; work; work = work->next)
 			{
-			fd = static_cast<FileData *>(work->data);
-			work = work->next;
+			auto *fd = static_cast<FileData *>(work->data);
 
 			if (!fd || !*fd->path) continue;
 
-			cbd->path_list = g_list_append(cbd->path_list, g_strdup(fd->path));
+			path_list = g_list_append(path_list, g_strdup(fd->path));
 			}
 
-			gtk_clipboard_set_with_data(gtk_clipboard_get(GDK_SELECTION_PRIMARY), target_types, target_types_n, clipboard_get_func, clipboard_clear_func, cbd);
+		return path_list;
+	};
+
+	if (options->clipboard_selection == CLIPBOARD_PRIMARY || options->clipboard_selection == CLIPBOARD_BOTH)
+		{
+		path_list_to_clipboard(get_path_list(fd_list), quoted, action, GDK_SELECTION_PRIMARY);
 		}
 
 	if (options->clipboard_selection == CLIPBOARD_CLIPBOARD || options->clipboard_selection == CLIPBOARD_BOTH)
 		{
-		cbd = g_new0(ClipboardData, 1);
-		cbd->path_list = nullptr;
-		cbd->quoted = quoted;
-		work = fd_list;
-
-		while (work)
-			{
-			fd = static_cast<FileData *>(work->data);
-			work = work->next;
-
-			if (!fd || !*fd->path) continue;
-
-			cbd->path_list = g_list_append(cbd->path_list, g_strdup(fd->path));
-			}
-
-			gtk_clipboard_set_with_data(gtk_clipboard_get(GDK_SELECTION_CLIPBOARD), target_types, target_types_n, clipboard_get_func, clipboard_clear_func, cbd);
+		path_list_to_clipboard(get_path_list(fd_list), quoted, action, GDK_SELECTION_CLIPBOARD);
 		}
 
 	filelist_free(fd_list);
