@@ -23,8 +23,10 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
+#include <array>
 #include <cstring>
 
+#include <gio/gio.h>
 #include <glib-object.h>
 
 #include <config.h>
@@ -53,41 +55,37 @@
 namespace
 {
 
-/** @FIXME Emblems should be attached to icons via e.g.:
- * GIcon *....
- * icon = g_themed_icon_new("folder");
- * emblem_icon = g_themed_icon_new("emblem_symbolic_link");
- * emblem = g_emblem_new(emblem_icon);
- * emblemed = g_emblemed_icon_new(icon, emblem);
- * gtk_icon_info_load_icon(icon_info, NULL)
- * But there does not seem to be a way to get GtkIconInfo from a GIcon
- */
+constexpr std::array<GtkTargetEntry, 1> vd_dnd_drop_types{{
+	{ const_cast<gchar *>("text/uri-list"), 0, TARGET_URI_LIST }
+}};
+
 GdkPixbuf *create_folder_icon_with_emblem(GtkIconTheme *icon_theme, const gchar *emblem, const gchar *fallback_icon, gint size)
 {
-	if (!gtk_icon_theme_has_icon(icon_theme, emblem))
+	GdkPixbuf *pixbuf = nullptr;
+	GtkIconInfo *info;
+
+	GIcon *icon_folder = g_themed_icon_new(GQ_ICON_DIRECTORY);
+	GIcon *icon_emblem = g_themed_icon_new(emblem);
+	GEmblem *emblem_new = g_emblem_new(icon_emblem);
+	GIcon *emblemed_icon = g_emblemed_icon_new(icon_folder, emblem_new);
+
+	info = gtk_icon_theme_lookup_by_gicon(icon_theme, emblemed_icon, size, GTK_ICON_LOOKUP_USE_BUILTIN);
+
+	if (info)
 		{
-		return gq_gtk_icon_theme_load_icon_copy(icon_theme, fallback_icon, size, GTK_ICON_LOOKUP_USE_BUILTIN);
+		pixbuf = gtk_icon_info_load_icon(info, nullptr);
 		}
 
-	GError *error = nullptr;
-	GdkPixbuf *icon = gtk_icon_theme_load_icon(icon_theme, emblem, size, GTK_ICON_LOOKUP_USE_BUILTIN, &error);
-	GdkPixbuf *pixbuf;
-	if (error)
+	if (pixbuf == nullptr)
 		{
-		log_printf("Error: %s\n", error->message);
-		g_error_free(error);
 		pixbuf = gq_gtk_icon_theme_load_icon_copy(icon_theme, fallback_icon, size, GTK_ICON_LOOKUP_USE_BUILTIN);
 		}
-	else
-		{
-		GdkPixbuf *directory_pixbuf = gtk_icon_theme_load_icon(icon_theme, GQ_ICON_DIRECTORY, size, GTK_ICON_LOOKUP_USE_BUILTIN, nullptr);
-		pixbuf = gdk_pixbuf_copy(directory_pixbuf);
-		g_object_unref(directory_pixbuf);
-		
-		gint scale = gdk_pixbuf_get_width(icon) / 2;
-		gdk_pixbuf_composite(icon, pixbuf, scale, scale, scale, scale, scale, scale, 0.5, 0.5, GDK_INTERP_HYPER, 255);
-		}
-	g_object_unref(icon);
+
+	g_object_unref(emblem_new);
+	g_object_unref(emblemed_icon);
+	g_object_unref(icon_emblem);
+	g_object_unref(icon_folder);
+	g_object_unref(info);
 
 	return pixbuf;
 }
@@ -119,12 +117,35 @@ void folder_icons_free(PixmapFolders *pf)
 {
 	if (!pf) return;
 
-	g_object_unref(pf->close);
-	g_object_unref(pf->open);
-	g_object_unref(pf->deny);
-	g_object_unref(pf->parent);
-	g_object_unref(pf->link);
-	g_object_unref(pf->read_only);
+	if (pf->close)
+		{
+		g_object_unref(pf->close);
+		}
+
+	if (pf->open)
+		{
+		g_object_unref(pf->open);
+		}
+
+	if (pf->deny)
+		{
+		g_object_unref(pf->deny);
+		}
+
+	if (pf->parent)
+		{
+		g_object_unref(pf->parent);
+		}
+
+	if (pf->link)
+		{
+		g_object_unref(pf->link);
+		}
+
+	if (pf->read_only)
+		{
+		g_object_unref(pf->read_only);
+		}
 
 	g_free(pf);
 }
@@ -152,11 +173,11 @@ static void vd_destroy_cb(GtkWidget *widget, gpointer data)
 	case DIRVIEW_TREE: vdtree_destroy_cb(widget, data); break;
 	}
 
-	if (vd->pf) folder_icons_free(vd->pf);
-	if (vd->drop_list) filelist_free(vd->drop_list);
+	folder_icons_free(vd->pf);
+	filelist_free(vd->drop_list);
 
-	if (vd->dir_fd) file_data_unref(vd->dir_fd);
-	if (vd->info) g_free(vd->info);
+	file_data_unref(vd->dir_fd);
+	g_free(vd->info);
 
 	g_free(vd);
 }
@@ -179,7 +200,7 @@ ViewDir *vd_new(LayoutWindow *lw)
 		case DIRVIEW_TREE: vd = vdtree_new(vd, lw->dir_fd); break;
 		}
 
-	gtk_container_add(GTK_CONTAINER(vd->widget), vd->view);
+	gq_gtk_container_add(GTK_WIDGET(vd->widget), vd->view);
 
 	vd_dnd_init(vd);
 
@@ -211,14 +232,6 @@ void vd_set_select_func(ViewDir *vd,
 	vd->select_data = data;
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-void vd_set_layout_unused(ViewDir *vd, LayoutWindow *layout)
-{
-	vd->layout = layout;
-}
-#pragma GCC diagnostic pop
-
 gboolean vd_set_fd(ViewDir *vd, FileData *dir_fd)
 {
 	gboolean ret = FALSE;
@@ -245,26 +258,10 @@ void vd_refresh(ViewDir *vd)
 	}
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-const gchar *vd_row_get_path_unused(ViewDir *vd, gint row)
-{
-	const gchar *ret = nullptr;
-
-	switch (vd->type)
-	{
-	case DIRVIEW_LIST: ret = vdlist_row_get_path(vd, row); break;
-	case DIRVIEW_TREE: ret = vdtree_row_get_path(vd, row); break;
-	}
-
-	return ret;
-}
-#pragma GCC diagnostic pop
-
 /* the calling stack is this:
    vd_select_row -> select_func -> layout_set_fd -> vd_set_fd
 */
-void vd_select_row(ViewDir *vd, FileData *fd)
+static void vd_select_row(ViewDir *vd, FileData *fd)
 {
 	if (fd && vd->select_func)
 		{
@@ -285,7 +282,7 @@ gboolean vd_find_row(ViewDir *vd, FileData *fd, GtkTreeIter *iter)
 	return ret;
 }
 
-FileData *vd_get_fd_from_tree_path(ViewDir *vd, GtkTreeView *tview, GtkTreePath *tpath)
+static FileData *vd_get_fd_from_tree_path(ViewDir *vd, GtkTreeView *tview, GtkTreePath *tpath)
 {
 	GtkTreeIter iter;
 	FileData *fd = nullptr;
@@ -453,11 +450,6 @@ static void vd_drop_menu_filter_cb(GtkWidget *widget, gpointer data)
 	file_util_start_filter_from_filelist(key, list, path, vd->widget);
 }
 
-static void vd_drop_menu_edit_item_free(gpointer data)
-{
-	g_free(data);
-}
-
 GtkWidget *vd_drop_menu(ViewDir *vd, gint active)
 {
 	GtkWidget *menu;
@@ -476,13 +468,11 @@ GtkWidget *vd_drop_menu(ViewDir *vd, gint active)
 		{
 		GtkWidget *item;
 		auto editor = static_cast<const EditorDescription *>(work->data);
-		gchar *key;
 		work = work->next;
 
 		if (!editor_is_filter(editor->key)) continue;
-		key = g_strdup(editor->key);
 		item = menu_item_add_sensitive(menu, editor->name, active, G_CALLBACK(vd_drop_menu_filter_cb), vd);
-		g_object_set_data_full(G_OBJECT(item), "filter_key", key, vd_drop_menu_edit_item_free);
+		g_object_set_data_full(G_OBJECT(item), "filter_key", g_strdup(editor->key), g_free);
 		}
 
 	g_list_free(editors_list);
@@ -593,7 +583,7 @@ static void vd_pop_menu_copy_path_cb(GtkWidget *, gpointer data)
 
 	if (!vd->click_fd) return;
 
-	file_util_copy_path_to_clipboard(vd->click_fd, TRUE);
+	file_util_copy_path_to_clipboard(vd->click_fd, TRUE, ClipboardAction::COPY);
 }
 
 static void vd_pop_menu_copy_path_unquoted_cb(GtkWidget *, gpointer data)
@@ -602,7 +592,16 @@ static void vd_pop_menu_copy_path_unquoted_cb(GtkWidget *, gpointer data)
 
 	if (!vd->click_fd) return;
 
-	file_util_copy_path_to_clipboard(vd->click_fd, FALSE);
+	file_util_copy_path_to_clipboard(vd->click_fd, FALSE, ClipboardAction::COPY);
+}
+
+static void vd_pop_menu_cut_path_cb(GtkWidget *, gpointer data)
+{
+	auto vd = static_cast<ViewDir *>(data);
+
+	if (!vd->click_fd) return;
+
+	file_util_copy_path_to_clipboard(vd->click_fd, FALSE, ClipboardAction::CUT);
 }
 
 static void vd_pop_submenu_dir_view_as_cb(GtkWidget *widget, gpointer data)
@@ -705,8 +704,7 @@ static void vd_pop_menu_sort_ascend_cb(GtkWidget *widget, gpointer data)
 
 	ascend = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget));
 	layout_views_set_sort_dir(vd->layout, vd->layout->options.dir_view_list_sort.method, ascend, vd->layout->options.dir_view_list_sort.case_sensitive);
-
-	if (vd->layout) layout_refresh(vd->layout);
+	layout_refresh(vd->layout);
 }
 
 static void vd_pop_menu_sort_case_cb(GtkWidget *widget, gpointer data)
@@ -720,8 +718,7 @@ static void vd_pop_menu_sort_case_cb(GtkWidget *widget, gpointer data)
 
 	case_sensitive = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget));
 	layout_views_set_sort_dir(vd->layout, vd->layout->options.dir_view_list_sort.method, vd->layout->options.dir_view_list_sort.ascend, case_sensitive);
-
-	if (vd->layout) layout_refresh(vd->layout);
+	layout_refresh(vd->layout);
 }
 
 static void vd_pop_menu_sort_cb(GtkWidget *widget, gpointer data)
@@ -739,8 +736,7 @@ static void vd_pop_menu_sort_cb(GtkWidget *widget, gpointer data)
 	if (type == SORT_NAME || type == SORT_NUMBER || type == SORT_TIME)
 		{
 		layout_views_set_sort_dir(vd->layout, type, vd->layout->options.dir_view_list_sort.ascend, vd->layout->options.dir_view_list_sort.case_sensitive);
-
-		if (vd->layout) layout_refresh(vd->layout);
+		layout_refresh(vd->layout);
 		}
 }
 
@@ -772,11 +768,10 @@ GtkWidget *vd_pop_menu(ViewDir *vd, FileData *fd)
 			{
 			if (fd)
 				{
-				gchar *parent;
-				new_folder_active = (fd && access_file(fd->path, W_OK | X_OK));
-				parent = remove_level_from_path(fd->path);
+				new_folder_active = access_file(fd->path, W_OK | X_OK);
+
+				g_autofree gchar *parent = remove_level_from_path(fd->path);
 				rename_delete_active = access_file(parent, W_OK | X_OK);
-				g_free(parent);
 				};
 			}
 			break;
@@ -810,12 +805,15 @@ GtkWidget *vd_pop_menu(ViewDir *vd, FileData *fd)
 	menu_item_add_sensitive(menu, _("_Rename..."), rename_delete_active,
 				G_CALLBACK(vd_pop_menu_rename_cb), vd);
 
-	menu_item_add(menu, _("_Copy path"),
+	menu_item_add(menu, _("_Copy to clipboard"),
 		      G_CALLBACK(vd_pop_menu_copy_path_cb), vd);
 
-	menu_item_add(menu, _("_Copy path unquoted"),
+	menu_item_add(menu, _("_Copy to clipboard (unquoted)"),
 		      G_CALLBACK(vd_pop_menu_copy_path_unquoted_cb), vd);
 
+	menu_item_add(menu, _("_Cut to clipboard"),
+		      G_CALLBACK(vd_pop_menu_cut_path_cb), vd);
+
 	menu_item_add_icon_sensitive(menu, _("_Delete..."), GQ_ICON_DELETE, rename_delete_active,
 				      G_CALLBACK(vd_pop_menu_delete_cb), vd);
 	menu_item_add_divider(menu);
@@ -865,19 +863,14 @@ void vd_new_folder(ViewDir *vd, FileData *dir_fd)
  *-----------------------------------------------------------------------------
  */
 
-static GtkTargetEntry vd_dnd_drop_types[] = {
-	{ const_cast<gchar *>("text/uri-list"), 0, TARGET_URI_LIST }
-};
-static gint vd_dnd_drop_types_count = 1;
-
 static void vd_dest_set(ViewDir *vd, gint enable)
 {
 	if (enable)
 		{
 		gtk_drag_dest_set(vd->view,
-				  static_cast<GtkDestDefaults>(GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP),
-				  vd_dnd_drop_types, vd_dnd_drop_types_count,
-				  static_cast<GdkDragAction>(GDK_ACTION_MOVE | GDK_ACTION_COPY));
+		                  static_cast<GtkDestDefaults>(GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP),
+		                  vd_dnd_drop_types.data(), vd_dnd_drop_types.size(),
+		                  static_cast<GdkDragAction>(GDK_ACTION_MOVE | GDK_ACTION_COPY));
 		}
 	else
 		{
@@ -902,6 +895,8 @@ static void vd_dnd_get(GtkWidget *, GdkDragContext *,
 			uri_selection_data_set_uris_from_filelist(selection_data, list);
 			g_list_free(list);
 			break;
+		default:
+			break;
 		}
 }
 
@@ -1037,27 +1032,15 @@ void vd_dnd_drop_scroll_cancel(ViewDir *vd)
 static gboolean vd_auto_scroll_idle_cb(gpointer data)
 {
 	auto vd = static_cast<ViewDir *>(data);
-	GdkSeat *seat;
-	GdkDevice *device;
 
 	if (vd->drop_fd)
 		{
-		GdkWindow *window;
-		gint x;
-		gint y;
-		gint w;
-		gint h;
+		GdkWindow *window = gtk_widget_get_window(vd->view);
 
-		window = gtk_widget_get_window(vd->view);
-		seat = gdk_display_get_default_seat(gdk_window_get_display(window));
-		device = gdk_seat_get_pointer(seat);
-		gdk_window_get_device_position(window, device, &x, &y, nullptr);
-
-		w = gdk_window_get_width(window);
-		h = gdk_window_get_height(window);
-		if (x >= 0 && x < w && y >= 0 && y < h)
+		GdkPoint pos;
+		if (window_get_pointer_position(window, pos))
 			{
-			vd_dnd_drop_update(vd, x, y);
+			vd_dnd_drop_update(vd, pos.x, pos.y);
 			}
 		}
 
@@ -1116,8 +1099,8 @@ static void vd_dnd_drop_leave(GtkWidget *, GdkDragContext *, guint, gpointer dat
 void vd_dnd_init(ViewDir *vd)
 {
 	gtk_drag_source_set(vd->view, static_cast<GdkModifierType>(GDK_BUTTON1_MASK | GDK_BUTTON2_MASK),
-			    dnd_file_drag_types, dnd_file_drag_types_count,
-			    static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_ASK));
+	                    dnd_file_drag_types.data(), dnd_file_drag_types.size(),
+	                    static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_ASK));
 	g_signal_connect(G_OBJECT(vd->view), "drag_data_get",
 			 G_CALLBACK(vd_dnd_get), vd);
 	g_signal_connect(G_OBJECT(vd->view), "drag_begin",
@@ -1140,27 +1123,6 @@ void vd_dnd_init(ViewDir *vd)
  *----------------------------------------------------------------------------
  */
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-void vd_menu_position_cb_unused(GtkMenu *menu, gint *x, gint *y, gboolean *, gpointer data)
-{
-	auto *vd = static_cast<ViewDir *>(data);
-	GtkTreeModel *store;
-	GtkTreeIter iter;
-	GtkTreePath *tpath;
-	gint cw;
-	gint ch;
-
-	if (!vd_find_row(vd, vd->click_fd, &iter)) return;
-	store = gtk_tree_view_get_model(GTK_TREE_VIEW(vd->view));
-	tpath = gtk_tree_model_get_path(store, &iter);
-	tree_view_get_cell_clamped(GTK_TREE_VIEW(vd->view), tpath, 0, TRUE, x, y, &cw, &ch);
-	gtk_tree_path_free(tpath);
-	*y += ch;
-	popup_menu_position_clamp(menu, x, y, 0);
-}
-#pragma GCC diagnostic pop
-
 void vd_activate_cb(GtkTreeView *tview, GtkTreePath *tpath, GtkTreeViewColumn *, gpointer data)
 {
 	auto vd = static_cast<ViewDir *>(data);
