@@ -32,17 +32,15 @@
 #include "debug.h"
 #include "dnd.h"
 #include "filedata.h"
-#include "img-view.h"
 #include "intl.h"
 #include "layout-image.h"
 #include "main-defines.h"
-#include "metadata.h"
 #include "misc.h"
 #include "options.h"
 #include "ui-fileops.h"
+#include "ui-menu.h"
 #include "ui-misc.h"
 #include "ui-tree-edit.h"
-#include "uri-utils.h"
 #include "utilops.h"
 #include "view-file.h"
 
@@ -62,7 +60,7 @@ constexpr gint THUMB_BORDER_PADDING = 2;
 constexpr gint VFICON_TIP_DELAY = 500;
 
 enum {
-	FILE_COLUMN_POINTER = VIEW_FILE_COLUMN_POINTER,
+	FILE_COLUMN_POINTER = 0,
 	FILE_COLUMN_COUNT
 };
 
@@ -92,82 +90,40 @@ GList *vficon_selection_get_one(ViewFile *, FileData *fd)
 	return g_list_prepend(filelist_copy(fd->sidecar_files), file_data_ref(fd));
 }
 
-GList *vficon_pop_menu_file_list(ViewFile *vf)
+void vficon_pop_menu_rename_cb(ViewFile *vf)
 {
-	if (!VFICON(vf)->click_fd) return nullptr;
-
-	if (VFICON(vf)->click_fd->selected & SELECTION_SELECTED)
-		{
-		return vf_selection_get_list(vf);
-		}
-
-	return vficon_selection_get_one(vf, VFICON(vf)->click_fd);
-}
-
-void vficon_pop_menu_view_cb(GtkWidget *, gpointer data)
-{
-	auto vf = static_cast<ViewFile *>(data);
-
-	if (!VFICON(vf)->click_fd) return;
-
-	if (VFICON(vf)->click_fd->selected & SELECTION_SELECTED)
-		{
-		GList *list;
-
-		list = vf_selection_get_list(vf);
-		view_window_new_from_list(list);
-		filelist_free(list);
-		}
-	else
-		{
-		view_window_new(VFICON(vf)->click_fd);
-		}
-}
-
-void vficon_pop_menu_rename_cb(GtkWidget *, gpointer data)
-{
-	auto vf = static_cast<ViewFile *>(data);
-
 	file_util_rename(nullptr, vf_pop_menu_file_list(vf), vf->listview);
 }
 
-void vficon_pop_menu_show_names_cb(GtkWidget *, gpointer data)
+static void vficon_pop_menu_show_names_cb(GtkWidget *, gpointer data)
 {
 	auto vf = static_cast<ViewFile *>(data);
 
 	vficon_toggle_filenames(vf);
 }
 
-static void vficon_toggle_star_rating(ViewFile *vf)
+void vficon_pop_menu_add_items(ViewFile *vf, GtkWidget *menu)
+{
+	menu_item_add_check(menu, _("Show filename _text"), VFICON(vf)->show_text,
+	                    G_CALLBACK(vficon_pop_menu_show_names_cb), vf);
+}
+
+void vficon_pop_menu_show_star_rating_cb(ViewFile *vf)
 {
 	GtkAllocation allocation;
 
-	options->show_star_rating = !options->show_star_rating;
-
 	gtk_widget_get_allocation(vf->listview, &allocation);
 	vficon_populate_at_new_size(vf, allocation.width, allocation.height, TRUE);
 }
 
-void vficon_pop_menu_show_star_rating_cb(GtkWidget *, gpointer data)
+void vficon_pop_menu_refresh_cb(ViewFile *vf)
 {
-	auto vf = static_cast<ViewFile *>(data);
-
-	vficon_toggle_star_rating(vf);
+	vficon_refresh(vf);
 }
 
-void vficon_pop_menu_refresh_cb(GtkWidget *, gpointer data)
+void vficon_popup_destroy_cb(ViewFile *vf)
 {
-	auto vf = static_cast<ViewFile *>(data);
-
-	vf_refresh(vf);
-}
-
-void vficon_popup_destroy_cb(GtkWidget *, gpointer data)
-{
-	auto vf = static_cast<ViewFile *>(data);
-	vficon_selection_remove(vf, VFICON(vf)->click_fd, SELECTION_PRELIGHT, nullptr);
-	VFICON(vf)->click_fd = nullptr;
-	vf->popup = nullptr;
+	vficon_selection_remove(vf, vf->click_fd, SELECTION_PRELIGHT, nullptr);
 }
 
 /*
@@ -289,7 +245,7 @@ static FileData *vficon_find_data(ViewFile *vf, gint row, gint col, GtkTreeIter
 	return nullptr;
 }
 
-static FileData *vficon_find_data_by_coord(ViewFile *vf, gint x, gint y, GtkTreeIter *iter)
+FileData *vficon_find_data_by_coord(ViewFile *vf, gint x, gint y, GtkTreeIter *iter)
 {
 	GtkTreePath *tpath;
 	GtkTreeViewColumn *column;
@@ -487,102 +443,35 @@ static void tip_update(ViewFile *vf, FileData *fd)
  *-------------------------------------------------------------------
  */
 
-static void vficon_dnd_get(GtkWidget *, GdkDragContext *,
-			   GtkSelectionData *selection_data, guint,
-			   guint, gpointer data)
+void vficon_dnd_begin(ViewFile *vf, GtkWidget *widget, GdkDragContext *context)
 {
-	auto vf = static_cast<ViewFile *>(data);
-	GList *list = nullptr;
-
-	if (!VFICON(vf)->click_fd) return;
-
-	if (VFICON(vf)->click_fd->selected & SELECTION_SELECTED)
-		{
-		list = vf_selection_get_list(vf);
-		}
-	else
-		{
-		list = g_list_append(nullptr, file_data_ref(VFICON(vf)->click_fd));
-		}
-
-	if (!list) return;
-	uri_selection_data_set_uris_from_filelist(selection_data, list);
-	filelist_free(list);
-}
-
-static void vficon_drag_data_received(GtkWidget *, GdkDragContext *,
-				      int x, int y, GtkSelectionData *selection,
-				      guint info, guint, gpointer data)
-{
-	auto vf = static_cast<ViewFile *>(data);
-
-	if (info == TARGET_TEXT_PLAIN) {
-		FileData *fd = vficon_find_data_by_coord(vf, x, y, nullptr);
-
-		if (fd) {
-			/* Add keywords to file */
-			auto str = reinterpret_cast<gchar *>(gtk_selection_data_get_text(selection));
-			GList *kw_list = string_to_keywords_list(str);
-
-			metadata_append_list(fd, KEYWORD_KEY, kw_list);
-			g_list_free_full(kw_list, g_free);
-			g_free(str);
-		}
-	}
-}
-
-static void vficon_dnd_begin(GtkWidget *widget, GdkDragContext *context, gpointer data)
-{
-	auto vf = static_cast<ViewFile *>(data);
-
 	tip_unschedule(vf);
 
-	if (VFICON(vf)->click_fd && VFICON(vf)->click_fd->thumb_pixbuf)
+	if (vf->click_fd && vf->click_fd->thumb_pixbuf)
 		{
 		gint items;
 
-		if (VFICON(vf)->click_fd->selected & SELECTION_SELECTED)
+		if (vf->click_fd->selected & SELECTION_SELECTED)
 			items = g_list_length(VFICON(vf)->selection);
 		else
 			items = 1;
 
-		dnd_set_drag_icon(widget, context, VFICON(vf)->click_fd->thumb_pixbuf, items);
+		dnd_set_drag_icon(widget, context, vf->click_fd->thumb_pixbuf, items);
 		}
 }
 
-static void vficon_dnd_end(GtkWidget *, GdkDragContext *context, gpointer data)
+void vficon_dnd_end(ViewFile *vf, GdkDragContext *context)
 {
-	auto vf = static_cast<ViewFile *>(data);
-
-	vficon_selection_remove(vf, VFICON(vf)->click_fd, SELECTION_PRELIGHT, nullptr);
+	vficon_selection_remove(vf, vf->click_fd, SELECTION_PRELIGHT, nullptr);
 
 	if (gdk_drag_context_get_selected_action(context) == GDK_ACTION_MOVE)
 		{
-		vf_refresh(vf);
+		vficon_refresh(vf);
 		}
 
 	tip_unschedule(vf);
 }
 
-void vficon_dnd_init(ViewFile *vf)
-{
-	gtk_drag_source_set(vf->listview, static_cast<GdkModifierType>(GDK_BUTTON1_MASK | GDK_BUTTON2_MASK),
-			    dnd_file_drag_types, dnd_file_drag_types_count,
-			    static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
-	gtk_drag_dest_set(vf->listview, GTK_DEST_DEFAULT_ALL,
-			    dnd_file_drag_types, dnd_file_drag_types_count,
-			    static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
-
-	g_signal_connect(G_OBJECT(vf->listview), "drag_data_get",
-			 G_CALLBACK(vficon_dnd_get), vf);
-	g_signal_connect(G_OBJECT(vf->listview), "drag_begin",
-			 G_CALLBACK(vficon_dnd_begin), vf);
-	g_signal_connect(G_OBJECT(vf->listview), "drag_end",
-			 G_CALLBACK(vficon_dnd_end), vf);
-	g_signal_connect(G_OBJECT(vf->listview), "drag_data_received",
-			 G_CALLBACK(vficon_drag_data_received), vf);
-}
-
 /*
  *-------------------------------------------------------------------
  * cell updates
@@ -638,13 +527,6 @@ void vficon_marks_set(ViewFile *vf, gint)
 	vficon_populate_at_new_size(vf, allocation.width, allocation.height, TRUE);
 }
 
-void vficon_star_rating_set(ViewFile *vf, gint)
-{
-	GtkAllocation allocation;
-	gtk_widget_get_allocation(vf->listview, &allocation);
-	vficon_populate_at_new_size(vf, allocation.width, allocation.height, TRUE);
-}
-
 /*
  *-------------------------------------------------------------------
  * selections
@@ -826,17 +708,10 @@ static void vficon_select_region_util(ViewFile *vf, FileData *start, FileData *e
 		}
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-gboolean vficon_index_is_selected_unused(ViewFile *vf, gint row)
+gboolean vficon_is_selected(ViewFile *, FileData *fd)
 {
-	auto *fd = static_cast<FileData *>(g_list_nth_data(vf->list, row));
-
-	if (!fd) return FALSE;
-
 	return (fd->selected & SELECTION_SELECTED);
 }
-#pragma GCC diagnostic pop
 
 guint vficon_selection_count(ViewFile *vf, gint64 *bytes)
 {
@@ -892,6 +767,16 @@ GList *vficon_selection_get_list_by_index(ViewFile *vf)
 	return g_list_reverse(list);
 }
 
+void vficon_selection_foreach(ViewFile *vf, const ViewFile::SelectionCallback &func)
+{
+	for (GList *work = VFICON(vf)->selection; work; work = work->next)
+		{
+		auto *fd_n = static_cast<FileData *>(work->data);
+
+		func(fd_n);
+		}
+}
+
 void vficon_select_by_fd(ViewFile *vf, FileData *fd)
 {
 	if (!fd) return;
@@ -899,7 +784,7 @@ void vficon_select_by_fd(ViewFile *vf, FileData *fd)
 
 	if (!(fd->selected & SELECTION_SELECTED))
 		{
-		vf_select_none(vf);
+		vficon_select_none(vf);
 		vficon_select(vf, fd);
 		}
 
@@ -928,65 +813,33 @@ void vficon_select_list(ViewFile *vf, GList *list)
 
 void vficon_mark_to_selection(ViewFile *vf, gint mark, MarkToSelectionMode mode)
 {
-	GList *work;
-	gint n = mark - 1;
-
 	g_assert(mark >= 1 && mark <= FILEDATA_MARKS_SIZE);
 
-	work = vf->list;
-	while (work)
+	for (GList *work = vf->list; work; work = work->next)
 		{
 		auto fd = static_cast<FileData *>(work->data);
-		gboolean mark_val;
 		gboolean selected;
 
 		g_assert(fd->magick == FD_MAGICK);
 
-		mark_val = file_data_get_mark(fd, n);
-		selected = fd->selected & SELECTION_SELECTED;
-
-		switch (mode)
-			{
-			case MTS_MODE_SET: selected = mark_val;
-				break;
-			case MTS_MODE_OR: selected = mark_val || selected;
-				break;
-			case MTS_MODE_AND: selected = mark_val && selected;
-				break;
-			case MTS_MODE_MINUS: selected = !mark_val && selected;
-				break;
-			}
+		selected = file_data_mark_to_selection(fd, mark, mode, fd->selected & SELECTION_SELECTED);
 
 		vficon_select_util(vf, fd, selected);
-
-		work = work->next;
 		}
 }
 
 void vficon_selection_to_mark(ViewFile *vf, gint mark, SelectionToMarkMode mode)
 {
 	GList *slist;
-	GList *work;
-	gint n = mark -1;
 
 	g_assert(mark >= 1 && mark <= FILEDATA_MARKS_SIZE);
 
-	slist = vf_selection_get_list(vf);
-	work = slist;
-	while (work)
+	slist = vficon_selection_get_list(vf);
+	for (GList *work = slist; work; work = work->next)
 		{
 		auto fd = static_cast<FileData *>(work->data);
 
-		switch (mode)
-			{
-			case STM_MODE_SET: file_data_set_mark(fd, n, 1);
-				break;
-			case STM_MODE_RESET: file_data_set_mark(fd, n, 0);
-				break;
-			case STM_MODE_TOGGLE: file_data_set_mark(fd, n, !file_data_get_mark(fd, n));
-				break;
-			}
-		work = work->next;
+		file_data_selection_to_mark(fd, mark, mode);
 		}
 	filelist_free(slist);
 }
@@ -1191,9 +1044,8 @@ static gint page_height(ViewFile *vf)
  *-------------------------------------------------------------------
  */
 
-gboolean vficon_press_key_cb(GtkWidget *widget, GdkEventKey *event, gpointer data)
+gboolean vficon_press_key_cb(ViewFile *vf, GtkWidget *widget, GdkEventKey *event)
 {
-	auto vf = static_cast<ViewFile *>(data);
 	gint focus_row = 0;
 	gint focus_col = 0;
 	FileData *fd;
@@ -1232,7 +1084,7 @@ gboolean vficon_press_key_cb(GtkWidget *widget, GdkEventKey *event, gpointer dat
 			fd = vficon_find_data(vf, VFICON(vf)->focus_row, VFICON(vf)->focus_column, nullptr);
 			if (fd)
 				{
-				VFICON(vf)->click_fd = fd;
+				vf->click_fd = fd;
 				if (event->state & GDK_CONTROL_MASK)
 					{
 					gint selected;
@@ -1250,7 +1102,7 @@ gboolean vficon_press_key_cb(GtkWidget *widget, GdkEventKey *event, gpointer dat
 					}
 				else
 					{
-					vf_select_none(vf);
+					vficon_select_none(vf);
 					vficon_select(vf, fd);
 					vficon_send_layout_select(vf, fd);
 					}
@@ -1258,9 +1110,9 @@ gboolean vficon_press_key_cb(GtkWidget *widget, GdkEventKey *event, gpointer dat
 			break;
 		case GDK_KEY_Menu:
 			fd = vficon_find_data(vf, VFICON(vf)->focus_row, VFICON(vf)->focus_column, nullptr);
-			VFICON(vf)->click_fd = fd;
+			vf->click_fd = fd;
 
-			vficon_selection_add(vf, VFICON(vf)->click_fd, SELECTION_PRELIGHT, nullptr);
+			vficon_selection_add(vf, vf->click_fd, SELECTION_PRELIGHT, nullptr);
 			tip_unschedule(vf);
 
 			vf->popup = vf_pop_menu(vf);
@@ -1290,19 +1142,19 @@ gboolean vficon_press_key_cb(GtkWidget *widget, GdkEventKey *event, gpointer dat
 					}
 				else
 					{
-					vficon_select_region_util(vf, VFICON(vf)->click_fd, old_fd, FALSE);
+					vficon_select_region_util(vf, vf->click_fd, old_fd, FALSE);
 					}
-				vficon_select_region_util(vf, VFICON(vf)->click_fd, new_fd, TRUE);
+				vficon_select_region_util(vf, vf->click_fd, new_fd, TRUE);
 				vficon_send_layout_select(vf, new_fd);
 				}
 			else if (event->state & GDK_CONTROL_MASK)
 				{
-				VFICON(vf)->click_fd = new_fd;
+				vf->click_fd = new_fd;
 				}
 			else
 				{
-				VFICON(vf)->click_fd = new_fd;
-				vf_select_none(vf);
+				vf->click_fd = new_fd;
+				vficon_select_none(vf);
 				vficon_select(vf, new_fd);
 				vficon_send_layout_select(vf, new_fd);
 				}
@@ -1334,9 +1186,8 @@ static gboolean vficon_motion_cb(GtkWidget *, GdkEventMotion *event, gpointer da
 	return FALSE;
 }
 
-gboolean vficon_press_cb(GtkWidget *, GdkEventButton *bevent, gpointer data)
+gboolean vficon_press_cb(ViewFile *vf, GtkWidget *, GdkEventButton *bevent)
 {
-	auto vf = static_cast<ViewFile *>(data);
 	GtkTreeIter iter;
 	FileData *fd;
 
@@ -1346,8 +1197,8 @@ gboolean vficon_press_cb(GtkWidget *, GdkEventButton *bevent, gpointer data)
 
 	if (fd)
 		{
-		VFICON(vf)->click_fd = fd;
-		vficon_selection_add(vf, VFICON(vf)->click_fd, SELECTION_PRELIGHT, &iter);
+		vf->click_fd = fd;
+		vficon_selection_add(vf, vf->click_fd, SELECTION_PRELIGHT, &iter);
 
 		switch (bevent->button)
 			{
@@ -1359,13 +1210,13 @@ gboolean vficon_press_cb(GtkWidget *, GdkEventButton *bevent, gpointer data)
 
 				if (bevent->type == GDK_2BUTTON_PRESS && vf->layout)
 					{
-					if (VFICON(vf)->click_fd->format_class == FORMAT_CLASS_COLLECTION)
+					if (vf->click_fd->format_class == FORMAT_CLASS_COLLECTION)
 						{
-						collection_window_new(VFICON(vf)->click_fd->path);
+						collection_window_new(vf->click_fd->path);
 						}
 					else
 						{
-						vficon_selection_remove(vf, VFICON(vf)->click_fd, SELECTION_PRELIGHT, &iter);
+						vficon_selection_remove(vf, vf->click_fd, SELECTION_PRELIGHT, &iter);
 						layout_image_full_screen_start(vf->layout);
 						}
 					}
@@ -1382,9 +1233,8 @@ gboolean vficon_press_cb(GtkWidget *, GdkEventButton *bevent, gpointer data)
 	return FALSE;
 }
 
-gboolean vficon_release_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
+gboolean vficon_release_cb(ViewFile *vf, GtkWidget *widget, GdkEventButton *bevent)
 {
-	auto vf = static_cast<ViewFile *>(data);
 	GtkTreeIter iter;
 	FileData *fd = nullptr;
 	gboolean was_selected;
@@ -1401,12 +1251,12 @@ gboolean vficon_release_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer d
 		fd = vficon_find_data_by_coord(vf, static_cast<gint>(bevent->x), static_cast<gint>(bevent->y), &iter);
 		}
 
-	if (VFICON(vf)->click_fd)
+	if (vf->click_fd)
 		{
-		vficon_selection_remove(vf, VFICON(vf)->click_fd, SELECTION_PRELIGHT, nullptr);
+		vficon_selection_remove(vf, vf->click_fd, SELECTION_PRELIGHT, nullptr);
 		}
 
-	if (!fd || VFICON(vf)->click_fd != fd) return TRUE;
+	if (!fd || vf->click_fd != fd) return TRUE;
 
 	was_selected = !!(fd->selected & SELECTION_SELECTED);
 
@@ -1432,7 +1282,7 @@ gboolean vficon_release_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer d
 				}
 			else
 				{
-				vf_select_none(vf);
+				vficon_select_none(vf);
 
 				if ((bevent->state & GDK_SHIFT_MASK) && VFICON(vf)->prev_selection)
 					{
@@ -1696,7 +1546,7 @@ void vficon_sort_set(ViewFile *vf, SortType type, gboolean ascend, gboolean case
 
 	if (!vf->list) return;
 
-	vf_refresh(vf);
+	vficon_refresh(vf);
 }
 
 /*
@@ -1705,29 +1555,25 @@ void vficon_sort_set(ViewFile *vf, SortType type, gboolean ascend, gboolean case
  *-----------------------------------------------------------------------------
  */
 
-void vficon_thumb_progress_count(GList *list, gint *count, gint *done)
+void vficon_thumb_progress_count(const GList *list, gint &count, gint &done)
 {
-	GList *work = list;
-	while (work)
+	for (const GList *work = list; work; work = work->next)
 		{
 		auto fd = static_cast<FileData *>(work->data);
-		work = work->next;
 
-		if (fd->thumb_pixbuf) (*done)++;
-		(*count)++;
+		if (fd->thumb_pixbuf) done++;
+		count++;
 		}
 }
 
-void vficon_read_metadata_progress_count(GList *list, gint *count, gint *done)
+void vficon_read_metadata_progress_count(const GList *list, gint &count, gint &done)
 {
-	GList *work = list;
-	while (work)
+	for (const GList *work = list; work; work = work->next)
 		{
 		auto fd = static_cast<FileData *>(work->data);
-		work = work->next;
 
-		if (fd->metadata_in_idle_loaded) (*done)++;
-		(*count)++;
+		if (fd->metadata_in_idle_loaded) done++;
+		count++;
 		}
 }
 
@@ -1879,23 +1725,11 @@ FileData *vficon_star_next_fd(ViewFile *vf)
  *-----------------------------------------------------------------------------
  */
 
-gint vficon_index_by_fd(ViewFile *vf, FileData *in_fd)
+gint vficon_index_by_fd(const ViewFile *vf, const FileData *fd)
 {
-	gint p = 0;
-	GList *work;
+	if (!fd) return -1;
 
-	if (!in_fd) return -1;
-
-	work = vf->list;
-	while (work)
-		{
-		auto fd = static_cast<FileData *>(work->data);
-		if (fd == in_fd) return p;
-		work = work->next;
-		p++;
-		}
-
-	return -1;
+	return g_list_index(vf->list, fd);
 }
 
 /*
@@ -1915,6 +1749,8 @@ static gboolean vficon_refresh_real(ViewFile *vf, gboolean keep_position)
 	GList *old_selected = nullptr;
 	GtkTreePath *end_path = nullptr;
 	GtkTreePath *start_path = nullptr;
+	GtkTreeIter iter;
+	GtkTreeModel *store;
 
 	gtk_tree_view_get_visible_range(GTK_TREE_VIEW(vf->listview), &start_path, &end_path);
 
@@ -1990,7 +1826,7 @@ static gboolean vficon_refresh_real(ViewFile *vf, gboolean keep_position)
 			GList *to_delete = work;
 			work = work->next;
 			if (fd == VFICON(vf)->prev_selection) VFICON(vf)->prev_selection = nullptr;
-			if (fd == VFICON(vf)->click_fd) VFICON(vf)->click_fd = nullptr;
+			if (fd == vf->click_fd) vf->click_fd = nullptr;
 			file_data_unref(fd);
 			vf->list = g_list_delete_link(vf->list, to_delete);
 			}
@@ -2026,18 +1862,16 @@ static gboolean vficon_refresh_real(ViewFile *vf, gboolean keep_position)
 		GList *reversed_old_selected;
 
 		reversed_old_selected = g_list_reverse(old_selected);
-		while (reversed_old_selected)
+		for (old_selected = reversed_old_selected; old_selected; old_selected = old_selected->next)
 			{
-			GList *tmp;
-			tmp = g_list_find(VFICON(vf)->selection, reversed_old_selected->data);
+			GList *tmp = g_list_find(VFICON(vf)->selection, old_selected->data);
 			if (tmp)
 				{
 				VFICON(vf)->selection = g_list_remove_link(VFICON(vf)->selection, tmp);
 				VFICON(vf)->selection = g_list_concat(tmp, VFICON(vf)->selection);
 				}
-			reversed_old_selected = reversed_old_selected->next;
 			}
-		g_list_free(old_selected);
+		g_list_free(reversed_old_selected);
 		}
 
 	filelist_free(new_filelist);
@@ -2051,7 +1885,9 @@ static gboolean vficon_refresh_real(ViewFile *vf, gboolean keep_position)
 		}
 	file_data_unref(first_selected);
 
-	if (start_path)
+	store = gtk_tree_view_get_model(GTK_TREE_VIEW(vf->listview));
+
+	if (gtk_tree_model_get_iter_first(store, &iter) && start_path)
 		{
 		gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(vf->listview), start_path, nullptr, FALSE, 0.0, 0.0);
 		}
@@ -2077,97 +1913,79 @@ static void vficon_cell_data_cb(GtkTreeViewColumn *, GtkCellRenderer *cell,
 				GtkTreeModel *tree_model, GtkTreeIter *iter, gpointer data)
 {
 	auto cd = static_cast<ColumnData *>(data);
-	FileData *fd;
-	gchar *star_rating;
 	GList *list;
-	ViewFile *vf = cd->vf;
 
 	if (!GQV_IS_CELL_RENDERER_ICON(cell)) return;
 
 	gtk_tree_model_get(tree_model, iter, FILE_COLUMN_POINTER, &list, -1);
 
-	fd = static_cast<FileData *>(g_list_nth_data(list, cd->number));
-
+	auto *fd = static_cast<FileData *>(g_list_nth_data(list, cd->number));
 	if (fd)
 		{
-		const gchar *link;
-		gchar *name_sidecars = nullptr;
-		GdkRGBA color_bg;
-		GdkRGBA color_fg;
-		GtkStateType state = GTK_STATE_NORMAL;
-		GtkStyle *style;
+		ViewFile *vf = cd->vf;
 
 		g_assert(fd->magick == FD_MAGICK);
 
-		if (options->show_star_rating && fd->rating != STAR_RATING_NOT_READ)
+		GString *name_sidecars = g_string_new(nullptr);
+
+		if (VFICON(vf)->show_text)
 			{
-			star_rating = convert_rating_to_stars(fd->rating);
-			}
-		else
-			{
-			star_rating = nullptr;
+			if (islink(fd->path))
+				{
+				name_sidecars = g_string_append(name_sidecars, GQ_LINK_STR);
+				}
+
+			name_sidecars = g_string_append(name_sidecars, fd->name);
+
+			if (fd->sidecar_files)
+				{
+				gchar *sidecars = file_data_sc_list_to_string(fd);
+				g_string_append_printf(name_sidecars, " %s", sidecars);
+				g_free(sidecars);
+				}
+			else if (fd->disable_grouping)
+				{
+				name_sidecars = g_string_append(name_sidecars, _(" [NO GROUPING]"));
+				}
 			}
 
-		link = islink(fd->path) ? GQ_LINK_STR : "";
-		if (fd->sidecar_files)
+		if (options->show_star_rating)
 			{
-			gchar *sidecars = file_data_sc_list_to_string(fd);
-			if (options->show_star_rating && VFICON(vf)->show_text)
+			if (name_sidecars->len > 0)
 				{
-				name_sidecars = g_strdup_printf("%s%s %s\n%s", link, fd->name, sidecars, star_rating);
+				name_sidecars = g_string_append_c(name_sidecars, '\n');
 				}
-			else if (options->show_star_rating)
-				{
-				name_sidecars = g_strdup_printf("%s", star_rating);
-				}
-			else if (VFICON(vf)->show_text)
-				{
-				name_sidecars = g_strdup_printf("%s%s %s", link, fd->name, sidecars);
-				}
-			g_free(sidecars);
-			}
-		else
-			{
-			const gchar *disabled_grouping = fd->disable_grouping ? _(" [NO GROUPING]") : "";
-			if (options->show_star_rating && VFICON(vf)->show_text)
-				{
-				name_sidecars = g_strdup_printf("%s%s%s\n%s", link, fd->name, disabled_grouping, star_rating);
-				}
-			else if (options->show_star_rating)
-				{
-				name_sidecars = g_strdup_printf("%s", star_rating);
-				}
-			else if (VFICON(vf)->show_text)
-				{
-				name_sidecars = g_strdup_printf("%s%s%s", link, fd->name, disabled_grouping);
-				}
-			}
-		g_free(star_rating);
 
-		style = gtk_widget_get_style(vf->listview);
-		if (fd->selected & SELECTION_SELECTED)
-			{
-			state = GTK_STATE_SELECTED;
+			gchar *star_rating = (fd->rating != STAR_RATING_NOT_READ) ? convert_rating_to_stars(fd->rating) : nullptr;
+			name_sidecars = g_string_append(name_sidecars, star_rating);
+			g_free(star_rating);
 			}
 
+		GtkStyle *style = gtk_widget_get_style(vf->listview);
+		GtkStateType state = (fd->selected & SELECTION_SELECTED) ? GTK_STATE_SELECTED : GTK_STATE_NORMAL;
+
+		GdkRGBA color_fg;
 		convert_gdkcolor_to_gdkrgba(&style->text[state], &color_fg);
-		convert_gdkcolor_to_gdkrgba(&style->base[state], &color_bg);
 
+		GdkRGBA color_bg;
+		convert_gdkcolor_to_gdkrgba(&style->base[state], &color_bg);
 		if (fd->selected & SELECTION_PRELIGHT)
 			{
 			shift_color(&color_bg, -1, 0);
 			}
 
-		g_object_set(cell, "pixbuf", fd->thumb_pixbuf,
-					"text", name_sidecars,
-					"marks", file_data_get_marks(fd),
-					"show_marks", vf->marks_enabled,
-					"cell-background-rgba", &color_bg,
-					"cell-background-set", TRUE,
-					"foreground-rgba", &color_fg,
-					"foreground-set", TRUE,
-					"has-focus", (VFICON(vf)->focus_fd == fd), NULL);
-		g_free(name_sidecars);
+		g_object_set(cell,
+		             "pixbuf", fd->thumb_pixbuf,
+		             "text", name_sidecars->str,
+		             "marks", file_data_get_marks(fd),
+		             "show_marks", vf->marks_enabled,
+		             "cell-background-rgba", &color_bg,
+		             "cell-background-set", TRUE,
+		             "foreground-rgba", &color_fg,
+		             "foreground-set", TRUE,
+		             "has-focus", VFICON(vf)->focus_fd == fd,
+		             NULL);
+		g_string_free(name_sidecars, TRUE);
 		}
 	else
 		{
@@ -2241,10 +2059,8 @@ gboolean vficon_set_fd(ViewFile *vf, FileData *dir_fd)
 	return ret;
 }
 
-void vficon_destroy_cb(GtkWidget *, gpointer data)
+void vficon_destroy_cb(ViewFile *vf)
 {
-	auto vf = static_cast<ViewFile *>(data);
-
 	vf_refresh_idle_cancel(vf);
 
 	file_data_unregister_notify_func(vf_notify_cb, vf);
@@ -2258,7 +2074,7 @@ void vficon_destroy_cb(GtkWidget *, gpointer data)
 	g_list_free(VFICON(vf)->selection);
 }
 
-ViewFile *vficon_new(ViewFile *vf, FileData *)
+ViewFile *vficon_new(ViewFile *vf)
 {
 	GtkListStore *store;
 	GtkTreeSelection *selection;
