@@ -31,7 +31,6 @@
 #include "debug.h"
 #include "dnd.h"
 #include "filedata.h"
-#include "img-view.h"
 #include "intl.h"
 #include "layout-image.h"
 #include "layout.h"
@@ -40,15 +39,15 @@
 #include "misc.h"
 #include "options.h"
 #include "ui-fileops.h"
+#include "ui-menu.h"
 #include "ui-misc.h"
 #include "ui-tree-edit.h"
-#include "uri-utils.h"
 #include "utilops.h"
 #include "view-file.h"
 
 /* Index to tree store */
 enum {
-	FILE_COLUMN_POINTER = VIEW_FILE_COLUMN_POINTER,
+	FILE_COLUMN_POINTER = 0,
 	FILE_COLUMN_VERSION,
 	FILE_COLUMN_THUMB,
 	FILE_COLUMN_FORMATTED,
@@ -131,7 +130,7 @@ static gint vflist_find_row(const ViewFile *vf, const FileData *fd, GtkTreeIter
 	return -1;
 }
 
-static FileData *vflist_find_data_by_coord(ViewFile *vf, gint x, gint y, GtkTreeIter *)
+FileData *vflist_find_data_by_coord(ViewFile *vf, gint x, gint y, GtkTreeIter *)
 {
 	GtkTreePath *tpath;
 	GtkTreeViewColumn *column;
@@ -169,11 +168,11 @@ static gboolean vflist_store_clear_cb(GtkTreeModel *model, GtkTreePath *, GtkTre
 static void vflist_store_clear(ViewFile *vf, gboolean unlock_files)
 {
 	GtkTreeModel *store;
-	GList *files = nullptr;
 
 	if (unlock_files && vf->marks_enabled)
 		{
 		// unlock locked files in this directory
+		GList *files = nullptr;
 		filelist_read(vf->dir_fd, &files, nullptr);
 		GList *work = files;
 		while (work)
@@ -183,9 +182,9 @@ static void vflist_store_clear(ViewFile *vf, gboolean unlock_files)
 			file_data_unlock(fd);
 			file_data_unref(fd);  // undo the ref that got added in filelist_read
 			}
+		g_list_free(files);
 		}
 
-	g_list_free(files);
 	store = gtk_tree_view_get_model(GTK_TREE_VIEW(vf->listview));
 	gtk_tree_model_foreach(store, vflist_store_clear_cb, nullptr);
 	gtk_tree_store_clear(GTK_TREE_STORE(store));
@@ -220,101 +219,34 @@ static void vflist_move_cursor(ViewFile *vf, GtkTreeIter *iter)
  *-----------------------------------------------------------------------------
  */
 
-static void vflist_dnd_get(GtkWidget *, GdkDragContext *,
-			   GtkSelectionData *selection_data, guint,
-			   guint, gpointer data)
+void vflist_dnd_begin(ViewFile *vf, GtkWidget *widget, GdkDragContext *context)
 {
-	auto vf = static_cast<ViewFile *>(data);
-	GList *list = nullptr;
-
-	if (!VFLIST(vf)->click_fd) return;
-
-	if (vflist_row_is_selected(vf, VFLIST(vf)->click_fd))
-		{
-		list = vf_selection_get_list(vf);
-		}
-	else
-		{
-		list = g_list_append(nullptr, file_data_ref(VFLIST(vf)->click_fd));
-		}
-
-	if (!list) return;
-	uri_selection_data_set_uris_from_filelist(selection_data, list);
-	filelist_free(list);
-}
-
-static void vflist_dnd_begin(GtkWidget *widget, GdkDragContext *context, gpointer data)
-{
-	auto vf = static_cast<ViewFile *>(data);
-
-	vflist_color_set(vf, VFLIST(vf)->click_fd, TRUE);
+	vflist_color_set(vf, vf->click_fd, TRUE);
 
 	if (VFLIST(vf)->thumbs_enabled &&
-	    VFLIST(vf)->click_fd && VFLIST(vf)->click_fd->thumb_pixbuf)
+	    vf->click_fd && vf->click_fd->thumb_pixbuf)
 		{
 		guint items;
 
-		if (vflist_row_is_selected(vf, VFLIST(vf)->click_fd))
-			items = vf_selection_count(vf, nullptr);
+		if (vflist_row_is_selected(vf, vf->click_fd))
+			items = vflist_selection_count(vf, nullptr);
 		else
 			items = 1;
 
-		dnd_set_drag_icon(widget, context, VFLIST(vf)->click_fd->thumb_pixbuf, items);
+		dnd_set_drag_icon(widget, context, vf->click_fd->thumb_pixbuf, items);
 		}
 }
 
-static void vflist_dnd_end(GtkWidget *, GdkDragContext *context, gpointer data)
+void vflist_dnd_end(ViewFile *vf, GdkDragContext *context)
 {
-	auto vf = static_cast<ViewFile *>(data);
-
-	vflist_color_set(vf, VFLIST(vf)->click_fd, FALSE);
+	vflist_color_set(vf, vf->click_fd, FALSE);
 
 	if (gdk_drag_context_get_selected_action(context) == GDK_ACTION_MOVE)
 		{
-		vf_refresh(vf);
+		vflist_refresh(vf);
 		}
 }
 
-static void vflist_drag_data_received(GtkWidget *, GdkDragContext *,
-				      int x, int y, GtkSelectionData *selection,
-				      guint info, guint, gpointer data)
-{
-	auto vf = static_cast<ViewFile *>(data);
-
-	if (info == TARGET_TEXT_PLAIN) {
-		FileData *fd = vflist_find_data_by_coord(vf, x, y, nullptr);
-
-		if (fd) {
-			/* Add keywords to file */
-			auto str = reinterpret_cast<gchar *>(gtk_selection_data_get_text(selection));
-			GList *kw_list = string_to_keywords_list(str);
-
-			metadata_append_list(fd, KEYWORD_KEY, kw_list);
-			g_list_free_full(kw_list, g_free);
-			g_free(str);
-		}
-	}
-}
-
-void vflist_dnd_init(ViewFile *vf)
-{
-	gtk_drag_source_set(vf->listview, static_cast<GdkModifierType>(GDK_BUTTON1_MASK | GDK_BUTTON2_MASK),
-			    dnd_file_drag_types, dnd_file_drag_types_count,
-			    static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
-	gtk_drag_dest_set(vf->listview, GTK_DEST_DEFAULT_ALL,
-			    dnd_file_drag_types, dnd_file_drag_types_count,
-			    static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
-
-	g_signal_connect(G_OBJECT(vf->listview), "drag_data_get",
-			 G_CALLBACK(vflist_dnd_get), vf);
-	g_signal_connect(G_OBJECT(vf->listview), "drag_begin",
-			 G_CALLBACK(vflist_dnd_begin), vf);
-	g_signal_connect(G_OBJECT(vf->listview), "drag_end",
-			 G_CALLBACK(vflist_dnd_end), vf);
-	g_signal_connect(G_OBJECT(vf->listview), "drag_data_received",
-			 G_CALLBACK(vflist_drag_data_received), vf);
-}
-
 /*
  *-----------------------------------------------------------------------------
  * pop-up menu
@@ -349,44 +281,13 @@ GList *vflist_selection_get_one(ViewFile *vf, FileData *fd)
 	return g_list_prepend(list, file_data_ref(fd));
 }
 
-GList *vflist_pop_menu_file_list(ViewFile *vf)
+void vflist_pop_menu_rename_cb(ViewFile *vf)
 {
-	if (!VFLIST(vf)->click_fd) return nullptr;
-
-	if (vflist_row_is_selected(vf, VFLIST(vf)->click_fd))
-		{
-		return vf_selection_get_list(vf);
-		}
-	return vflist_selection_get_one(vf, VFLIST(vf)->click_fd);
-}
-
-
-void vflist_pop_menu_view_cb(GtkWidget *, gpointer data)
-{
-	auto vf = static_cast<ViewFile *>(data);
-
-	if (vflist_row_is_selected(vf, VFLIST(vf)->click_fd))
-		{
-		GList *list;
-
-		list = vf_selection_get_list(vf);
-		view_window_new_from_list(list);
-		filelist_free(list);
-		}
-	else
-		{
-		view_window_new(VFLIST(vf)->click_fd);
-		}
-}
-
-void vflist_pop_menu_rename_cb(GtkWidget *, gpointer data)
-{
-	auto vf = static_cast<ViewFile *>(data);
 	GList *list;
 
 	list = vf_pop_menu_file_list(vf);
 	if (options->file_ops.enable_in_place_rename &&
-	    list && !list->next && VFLIST(vf)->click_fd)
+	    list && !list->next && vf->click_fd)
 		{
 		GtkTreeModel *store;
 		GtkTreeIter iter;
@@ -394,14 +295,14 @@ void vflist_pop_menu_rename_cb(GtkWidget *, gpointer data)
 		filelist_free(list);
 
 		store = gtk_tree_view_get_model(GTK_TREE_VIEW(vf->listview));
-		if (vflist_find_row(vf, VFLIST(vf)->click_fd, &iter) >= 0)
+		if (vflist_find_row(vf, vf->click_fd, &iter) >= 0)
 			{
 			GtkTreePath *tpath;
 
 			tpath = gtk_tree_model_get_path(store, &iter);
 			tree_edit_by_path(GTK_TREE_VIEW(vf->listview), tpath,
-					  FILE_VIEW_COLUMN_FORMATTED, VFLIST(vf)->click_fd->name,
-					  vflist_row_rename_cb, vf);
+			                  FILE_VIEW_COLUMN_FORMATTED, vf->click_fd->name,
+			                  vflist_row_rename_cb, vf);
 			gtk_tree_path_free(tpath);
 			}
 		return;
@@ -410,11 +311,11 @@ void vflist_pop_menu_rename_cb(GtkWidget *, gpointer data)
 	file_util_rename(nullptr, list, vf->listview);
 }
 
-void vflist_pop_menu_thumbs_cb(GtkWidget *, gpointer data)
+static void vflist_pop_menu_thumbs_cb(GtkWidget *, gpointer data)
 {
 	auto vf = static_cast<ViewFile *>(data);
 
-	vflist_color_set(vf, VFLIST(vf)->click_fd, FALSE);
+	vflist_color_set(vf, vf->click_fd, FALSE);
 	if (vf->layout)
 		{
 		layout_thumb_set(vf->layout, !VFLIST(vf)->thumbs_enabled);
@@ -425,7 +326,13 @@ void vflist_pop_menu_thumbs_cb(GtkWidget *, gpointer data)
 		}
 }
 
-void vflist_star_rating_set(ViewFile *vf, gboolean enable)
+void vflist_pop_menu_add_items(ViewFile *vf, GtkWidget *menu)
+{
+	menu_item_add_check(menu, _("Show _thumbnails"), VFLIST(vf)->thumbs_enabled,
+	                    G_CALLBACK(vflist_pop_menu_thumbs_cb), vf);
+}
+
+static void vflist_star_rating_set(ViewFile *vf, gboolean enable)
 {
 	GList *columns;
 	GList *work;
@@ -461,33 +368,24 @@ void vflist_star_rating_set(ViewFile *vf, gboolean enable)
 	g_list_free(columns);
 }
 
-void vflist_pop_menu_show_star_rating_cb(GtkWidget *, gpointer data)
+void vflist_pop_menu_show_star_rating_cb(ViewFile *vf)
 {
-	auto vf = static_cast<ViewFile *>(data);
-
-	options->show_star_rating = !options->show_star_rating;
-
 	vflist_populate_view(vf, TRUE);
 
-	vflist_color_set(vf, VFLIST(vf)->click_fd, FALSE);
+	vflist_color_set(vf, vf->click_fd, FALSE);
 	vflist_star_rating_set(vf, options->show_star_rating);
 }
 
-void vflist_pop_menu_refresh_cb(GtkWidget *, gpointer data)
+void vflist_pop_menu_refresh_cb(ViewFile *vf)
 {
-	auto vf = static_cast<ViewFile *>(data);
-
-	vflist_color_set(vf, VFLIST(vf)->click_fd, FALSE);
-	vf_refresh(vf);
+	vflist_color_set(vf, vf->click_fd, FALSE);
+	vflist_refresh(vf);
 	gtk_tree_view_columns_autosize(GTK_TREE_VIEW(vf->listview));
 }
 
-void vflist_popup_destroy_cb(GtkWidget *, gpointer data)
+void vflist_popup_destroy_cb(ViewFile *vf)
 {
-	auto vf = static_cast<ViewFile *>(data);
-	vflist_color_set(vf, VFLIST(vf)->click_fd, FALSE);
-	VFLIST(vf)->click_fd = nullptr;
-	vf->popup = nullptr;
+	vflist_color_set(vf, vf->click_fd, FALSE);
 }
 
 
@@ -499,36 +397,31 @@ void vflist_popup_destroy_cb(GtkWidget *, gpointer data)
 
 static gboolean vflist_row_rename_cb(TreeEditData *, const gchar *old_name, const gchar *new_name, gpointer data)
 {
-	auto vf = static_cast<ViewFile *>(data);
-	gchar *new_path;
-
 	if (!new_name || !new_name[0]) return FALSE;
 
-	new_path = g_build_filename(vf->dir_fd->path, new_name, NULL);
+	auto vf = static_cast<ViewFile *>(data);
 
 	if (strchr(new_name, G_DIR_SEPARATOR) != nullptr)
 		{
-		gchar *text = g_strdup_printf(_("Invalid file name:\n%s"), new_name);
+		g_autofree gchar *text = g_strdup_printf(_("Invalid file name:\n%s"), new_name);
 		file_util_warning_dialog(_("Error renaming file"), text, GQ_ICON_DIALOG_ERROR, vf->listview);
-		g_free(text);
 		}
 	else
 		{
-		gchar *old_path = g_build_filename(vf->dir_fd->path, old_name, NULL);
+		g_autofree gchar *old_path = g_build_filename(vf->dir_fd->path, old_name, NULL);
 		FileData *fd = file_data_new_group(old_path); /* get the fd from cache */
-		file_util_rename_simple(fd, new_path, vf->listview);
-		file_data_unref(fd);
-		g_free(old_path);
-		}
 
-	g_free(new_path);
+		g_autofree gchar *new_path = g_build_filename(vf->dir_fd->path, new_name, NULL);
+		file_util_rename_simple(fd, new_path, vf->listview);
+
+		file_data_unref(fd);
+		}
 
 	return FALSE;
 }
 
-gboolean vflist_press_key_cb(GtkWidget *widget, GdkEventKey *event, gpointer data)
+gboolean vflist_press_key_cb(ViewFile *vf, GtkWidget *widget, GdkEventKey *event)
 {
-	auto vf = static_cast<ViewFile *>(data);
 	GtkTreePath *tpath;
 
 	if (event->keyval != GDK_KEY_Menu) return FALSE;
@@ -541,12 +434,12 @@ gboolean vflist_press_key_cb(GtkWidget *widget, GdkEventKey *event, gpointer dat
 
 		store = gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
 		gtk_tree_model_get_iter(store, &iter, tpath);
-		gtk_tree_model_get(store, &iter, FILE_COLUMN_POINTER, &VFLIST(vf)->click_fd, -1);
+		gtk_tree_model_get(store, &iter, FILE_COLUMN_POINTER, &vf->click_fd, -1);
 		gtk_tree_path_free(tpath);
 		}
 	else
 		{
-		VFLIST(vf)->click_fd = nullptr;
+		vf->click_fd = nullptr;
 		}
 
 	vf->popup = vf_pop_menu(vf);
@@ -555,9 +448,8 @@ gboolean vflist_press_key_cb(GtkWidget *widget, GdkEventKey *event, gpointer dat
 	return TRUE;
 }
 
-gboolean vflist_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
+gboolean vflist_press_cb(ViewFile *vf, GtkWidget *widget, GdkEventButton *bevent)
 {
-	auto vf = static_cast<ViewFile *>(data);
 	GtkTreePath *tpath;
 	GtkTreeIter iter;
 	FileData *fd = nullptr;
@@ -585,7 +477,7 @@ gboolean vflist_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer dat
 		gtk_tree_path_free(tpath);
 		}
 
-	VFLIST(vf)->click_fd = fd;
+	vf->click_fd = fd;
 
 	if (bevent->button == MOUSE_BUTTON_RIGHT)
 		{
@@ -626,9 +518,9 @@ gboolean vflist_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer dat
 
 	if (bevent->button == MOUSE_BUTTON_LEFT && bevent->type == GDK_2BUTTON_PRESS)
 		{
-		if (VFLIST(vf)->click_fd->format_class == FORMAT_CLASS_COLLECTION)
+		if (vf->click_fd->format_class == FORMAT_CLASS_COLLECTION)
 			{
-			collection_window_new(VFLIST(vf)->click_fd->path);
+			collection_window_new(vf->click_fd->path);
 			}
 		else
 			{
@@ -639,9 +531,8 @@ gboolean vflist_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer dat
 	return FALSE;
 }
 
-gboolean vflist_release_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
+gboolean vflist_release_cb(ViewFile *vf, GtkWidget *widget, GdkEventButton *bevent)
 {
-	auto vf = static_cast<ViewFile *>(data);
 	GtkTreePath *tpath;
 	GtkTreeIter iter;
 	FileData *fd = nullptr;
@@ -653,7 +544,7 @@ gboolean vflist_release_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer d
 
 	if (bevent->button == MOUSE_BUTTON_MIDDLE)
 		{
-		vflist_color_set(vf, VFLIST(vf)->click_fd, FALSE);
+		vflist_color_set(vf, vf->click_fd, FALSE);
 		}
 
 	if (bevent->button != MOUSE_BUTTON_LEFT && bevent->button != MOUSE_BUTTON_MIDDLE)
@@ -675,7 +566,7 @@ gboolean vflist_release_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer d
 
 	if (bevent->button == MOUSE_BUTTON_MIDDLE)
 		{
-		if (fd && VFLIST(vf)->click_fd == fd)
+		if (fd && vf->click_fd == fd)
 			{
 			GtkTreeSelection *selection;
 
@@ -692,7 +583,7 @@ gboolean vflist_release_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer d
 		return TRUE;
 		}
 
-	if (fd && VFLIST(vf)->click_fd == fd &&
+	if (fd && vf->click_fd == fd &&
 	    !(bevent->state & GDK_SHIFT_MASK ) &&
 	    !(bevent->state & GDK_CONTROL_MASK ) &&
 	    vflist_row_is_selected(vf, fd))
@@ -722,7 +613,7 @@ static void vflist_select_image(ViewFile *vf, FileData *sel_fd)
 	row = g_list_index(vf->list, sel_fd);
 	/** @FIXME sidecar data */
 
-	if (sel_fd && options->image.enable_read_ahead && row >= 0)
+	if (options->image.enable_read_ahead && row >= 0)
 		{
 		if (row > g_list_index(vf->list, cur_fd) &&
 		    static_cast<guint>(row + 1) < vf_count(vf, nullptr))
@@ -815,28 +706,24 @@ static void vflist_collapse_cb(GtkTreeView *, GtkTreeIter *iter, GtkTreePath *,
  *-----------------------------------------------------------------------------
  */
 
-
-static gchar* vflist_get_formatted(ViewFile *vf, const gchar *name, const gchar *sidecars, const gchar *size, const gchar *time, gboolean expanded, gboolean with_stars, const gchar *star_rating)
- {
+static gchar* vflist_get_formatted(ViewFile *vf, const gchar *name, const gchar *sidecars, const gchar *size, const gchar *time, gboolean expanded, const gchar *star_rating)
+{
 	gboolean multiline = vflist_is_multiline(vf);
-	gchar *text;
+	GString *text = g_string_new(nullptr);
+
+	g_string_printf(text, "%s %s", name, expanded ? "" : sidecars);
 
 	if (multiline)
 		{
-		if (with_stars)
+		g_string_append_printf(text, "\n%s\n%s", size, time);
+
+		if (star_rating)
 			{
-					text = g_strdup_printf("%s %s\n%s\n%s\n%s", name, expanded ? "" : sidecars, size, time, star_rating);
-			}
-		else
-			{
-			text = g_strdup_printf("%s %s\n%s\n%s", name, expanded ? "" : sidecars, size, time);
+			g_string_append_printf(text, "\n%s", star_rating);
 			}
 		}
-	else
-		{
-		text = g_strdup_printf("%s %s", name, expanded ? "" : sidecars);
-		}
-	return text;
+
+	return g_string_free(text, FALSE);
 }
 
 static void vflist_set_expanded(ViewFile *vf, GtkTreeIter *iter, gboolean expanded)
@@ -859,8 +746,8 @@ static void vflist_set_expanded(ViewFile *vf, GtkTreeIter *iter, gboolean expand
 					FILE_COLUMN_STAR_RATING, &star_rating,
 					-1);
 
-	formatted = vflist_get_formatted(vf, name, sidecars, size, time, expanded, FALSE, nullptr);
-	formatted_with_stars = vflist_get_formatted(vf, name, sidecars, size, time, expanded, TRUE, star_rating);
+	formatted = vflist_get_formatted(vf, name, sidecars, size, time, expanded, nullptr);
+	formatted_with_stars = vflist_get_formatted(vf, name, sidecars, size, time, expanded, star_rating);
 
 	gtk_tree_store_set(store, iter, FILE_COLUMN_FORMATTED, formatted,
 					FILE_COLUMN_EXPANDED, expanded,
@@ -909,8 +796,8 @@ static void vflist_setup_iter(ViewFile *vf, GtkTreeStore *store, GtkTreeIter *it
 	name = g_strdup_printf("%s%s%s", link, fd->name, disabled_grouping);
 	size = text_from_size(fd->size);
 
-	formatted = vflist_get_formatted(vf, name, sidecars, size, time, expanded, FALSE, nullptr);
-	formatted_with_stars = vflist_get_formatted(vf, name, sidecars, size, time, expanded, TRUE, star_rating);
+	formatted = vflist_get_formatted(vf, name, sidecars, size, time, expanded, nullptr);
+	formatted_with_stars = vflist_get_formatted(vf, name, sidecars, size, time, expanded, star_rating);
 
 	gtk_tree_store_set(store, iter, FILE_COLUMN_POINTER, fd,
 					FILE_COLUMN_VERSION, fd->version,
@@ -1135,39 +1022,35 @@ void vflist_sort_set(ViewFile *vf, SortType type, gboolean ascend, gboolean case
  */
 
 
-void vflist_thumb_progress_count(GList *list, gint *count, gint *done)
+void vflist_thumb_progress_count(const GList *list, gint &count, gint &done)
 {
-	GList *work = list;
-	while (work)
+	for (const GList *work = list; work; work = work->next)
 		{
 		auto fd = static_cast<FileData *>(work->data);
-		work = work->next;
 
-		if (fd->thumb_pixbuf) (*done)++;
+		if (fd->thumb_pixbuf) done++;
 
 		if (fd->sidecar_files)
 			{
 			vflist_thumb_progress_count(fd->sidecar_files, count, done);
 			}
-		(*count)++;
+		count++;
 		}
 }
 
-void vflist_read_metadata_progress_count(GList *list, gint *count, gint *done)
+void vflist_read_metadata_progress_count(const GList *list, gint &count, gint &done)
 {
-	GList *work = list;
-	while (work)
+	for (const GList *work = list; work; work = work->next)
 		{
 		auto fd = static_cast<FileData *>(work->data);
-		work = work->next;
 
-		if (fd->metadata_in_idle_loaded) (*done)++;
+		if (fd->metadata_in_idle_loaded) done++;
 
 		if (fd->sidecar_files)
 			{
 			vflist_read_metadata_progress_count(fd->sidecar_files, count, done);
 			}
-		(*count)++;
+		count++;
 		}
 }
 
@@ -1267,7 +1150,7 @@ void vflist_set_star_fd(ViewFile *vf, FileData *fd)
 					FILE_COLUMN_EXPANDED, &expanded,
 					-1);
 
-	formatted_with_stars = vflist_get_formatted(vf, name, sidecars, size, time, expanded, TRUE, star_rating);
+	formatted_with_stars = vflist_get_formatted(vf, name, sidecars, size, time, expanded, star_rating);
 
 	gtk_tree_store_set(store, &iter, FILE_COLUMN_FORMATTED_WITH_STARS, formatted_with_stars,
 					FILE_COLUMN_EXPANDED, expanded,
@@ -1361,31 +1244,21 @@ FileData *vflist_star_next_fd(ViewFile *vf)
  *-----------------------------------------------------------------------------
  */
 
-gint vflist_index_by_fd(ViewFile *vf, FileData *fd)
+gint vflist_index_by_fd(const ViewFile *vf, const FileData *fd)
 {
 	gint p = 0;
-	GList *work;
-	GList *work2;
 
-	work = vf->list;
-	while (work)
+	for (const GList *work = vf->list; work; work = work->next)
 		{
 		auto list_fd = static_cast<FileData *>(work->data);
 		if (list_fd == fd) return p;
 
-		work2 = list_fd->sidecar_files;
-		while (work2)
-			{
-			/** @FIXME return the same index also for sidecars
-			   it is sufficient for next/prev navigation but it should be rewritten
-			   without using indexes at all
-			*/
-			auto sidecar_fd = static_cast<FileData *>(work2->data);
-			if (sidecar_fd == fd) return p;
-			work2 = work2->next;
-			}
+		/** @FIXME return the same index also for sidecars
+		   it is sufficient for next/prev navigation but it should be rewritten
+		   without using indexes at all
+		*/
+		if (g_list_find(list_fd->sidecar_files, fd)) return p;
 
-		work = work->next;
 		p++;
 		}
 
@@ -1425,16 +1298,10 @@ static gboolean vflist_row_is_selected(ViewFile *vf, FileData *fd)
 	return found;
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-gboolean vflist_index_is_selected_unused(ViewFile *vf, gint row)
+gboolean vflist_is_selected(ViewFile *vf, FileData *fd)
 {
-	FileData *fd;
-
-	fd = vf_index_get_data(vf, row);
 	return vflist_row_is_selected(vf, fd);
 }
-#pragma GCC diagnostic pop
 
 guint vflist_selection_count(ViewFile *vf, gint64 *bytes)
 {
@@ -1534,6 +1401,24 @@ GList *vflist_selection_get_list_by_index(ViewFile *vf)
 	return g_list_reverse(list);
 }
 
+void vflist_selection_foreach(ViewFile *vf, const ViewFile::SelectionCallback &func)
+{
+	GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(vf->listview));
+	GtkTreeModel *store;
+	GtkTreeIter iter;
+	FileData *fd_n;
+
+	for (GList *work = gtk_tree_selection_get_selected_rows(selection, &store); work; work = work->next)
+		{
+		auto *tpath = static_cast<GtkTreePath *>(work->data);
+
+		gtk_tree_model_get_iter(store, &iter, tpath);
+		gtk_tree_model_get(store, &iter, FILE_COLUMN_POINTER, &fd_n, -1);
+
+		func(fd_n);
+		}
+}
+
 void vflist_select_all(ViewFile *vf)
 {
 	GtkTreeSelection *selection;
@@ -1692,7 +1577,6 @@ void vflist_mark_to_selection(ViewFile *vf, gint mark, MarkToSelectionMode mode)
 	GtkTreeIter iter;
 	GtkTreeSelection *selection;
 	gboolean valid;
-	gint n = mark - 1;
 
 	g_assert(mark >= 1 && mark <= FILEDATA_MARKS_SIZE);
 
@@ -1703,24 +1587,10 @@ void vflist_mark_to_selection(ViewFile *vf, gint mark, MarkToSelectionMode mode)
 	while (valid)
 		{
 		FileData *fd;
-		gboolean mark_val;
 		gboolean selected;
 		gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, FILE_COLUMN_POINTER, &fd, -1);
 
-		mark_val = file_data_get_mark(fd, n);
-		selected = gtk_tree_selection_iter_is_selected(selection, &iter);
-
-		switch (mode)
-			{
-			case MTS_MODE_SET: selected = mark_val;
-				break;
-			case MTS_MODE_OR: selected = mark_val || selected;
-				break;
-			case MTS_MODE_AND: selected = mark_val && selected;
-				break;
-			case MTS_MODE_MINUS: selected = !mark_val && selected;
-				break;
-			}
+		selected = file_data_mark_to_selection(fd, mark, mode, gtk_tree_selection_iter_is_selected(selection, &iter));
 
 		if (selected)
 			gtk_tree_selection_select_iter(selection, &iter);
@@ -1736,15 +1606,12 @@ void vflist_selection_to_mark(ViewFile *vf, gint mark, SelectionToMarkMode mode)
 	GtkTreeModel *store;
 	GtkTreeSelection *selection;
 	GList *slist;
-	GList *work;
-	gint n = mark - 1;
 
 	g_assert(mark >= 1 && mark <= FILEDATA_MARKS_SIZE);
 
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(vf->listview));
 	slist = gtk_tree_selection_get_selected_rows(selection, &store);
-	work = slist;
-	while (work)
+	for (GList *work = slist; work; work = work->next)
 		{
 		auto tpath = static_cast<GtkTreePath *>(work->data);
 		FileData *fd;
@@ -1757,15 +1624,7 @@ void vflist_selection_to_mark(ViewFile *vf, gint mark, SelectionToMarkMode mode)
 		   complete re-read of the directory - try to do only minimal update instead */
 		file_data_unregister_notify_func(vf_notify_cb, vf); /* we don't need the notification */
 
-		switch (mode)
-			{
-			case STM_MODE_SET: file_data_set_mark(fd, n, 1);
-				break;
-			case STM_MODE_RESET: file_data_set_mark(fd, n, 0);
-				break;
-			case STM_MODE_TOGGLE: file_data_set_mark(fd, n, !file_data_get_mark(fd, n));
-				break;
-			}
+		file_data_selection_to_mark(fd, mark, mode);
 
 		if (!file_data_filter_marks(fd, vf_marks_get_filter(vf))) /* file no longer matches the filter -> remove it */
 			{
@@ -1779,10 +1638,7 @@ void vflist_selection_to_mark(ViewFile *vf, gint mark, SelectionToMarkMode mode)
 			vflist_setup_iter_recursive(vf, GTK_TREE_STORE(store), &iter, fd->sidecar_files, nullptr, FALSE);
 			}
 
-
 		file_data_register_notify_func(vf_notify_cb, vf, NOTIFY_PRIORITY_MEDIUM);
-
-		work = work->next;
 		}
 	g_list_free_full(slist, reinterpret_cast<GDestroyNotify>(gtk_tree_path_free));
 }
@@ -2095,15 +1951,13 @@ gboolean vflist_set_fd(ViewFile *vf, FileData *dir_fd)
 	filelist_free(vf->list);
 	vf->list = nullptr;
 
-	ret = vf_refresh(vf);
+	ret = vflist_refresh(vf);
 	gtk_tree_view_columns_autosize(GTK_TREE_VIEW(vf->listview));
 	return ret;
 }
 
-void vflist_destroy_cb(GtkWidget *, gpointer data)
+void vflist_destroy_cb(ViewFile *vf)
 {
-	auto vf = static_cast<ViewFile *>(data);
-
 	file_data_unregister_notify_func(vf_notify_cb, vf);
 
 	vflist_select_idle_cancel(vf);
@@ -2114,7 +1968,7 @@ void vflist_destroy_cb(GtkWidget *, gpointer data)
 	filelist_free(vf->list);
 }
 
-ViewFile *vflist_new(ViewFile *vf, FileData *)
+ViewFile *vflist_new(ViewFile *vf)
 {
 	GtkTreeStore *store;
 	GtkTreeSelection *selection;
@@ -2202,7 +2056,7 @@ void vflist_thumb_set(ViewFile *vf, gboolean enable)
 
 	VFLIST(vf)->thumbs_enabled = enable;
 
-	/* vflist_populate_view is better than vf_refresh:
+	/* vflist_populate_view is better than vflist_refresh:
 	   - no need to re-read the directory
 	   - force update because the formatted string has changed
 	*/
