@@ -20,20 +20,25 @@
 
 #include "view-file.h"
 
+#include <array>
+
 #include <gdk/gdk.h>
 #include <glib-object.h>
 
 #include "archives.h"
 #include "compat.h"
 #include "debug.h"
+#include "dnd.h"
 #include "dupe.h"
 #include "filedata.h"
 #include "history-list.h"
+#include "img-view.h"
 #include "intl.h"
 #include "layout.h"
 #include "main-defines.h"
 #include "main.h"
 #include "menu.h"
+#include "metadata.h"
 #include "misc.h"
 #include "options.h"
 #include "thumb.h"
@@ -41,6 +46,7 @@
 #include "ui-menu.h"
 #include "ui-misc.h"
 #include "ui-utildlg.h"
+#include "uri-utils.h"
 #include "utilops.h"
 #include "view-file/view-file-icon.h"
 #include "view-file/view-file-list.h"
@@ -137,8 +143,8 @@ static gboolean vf_press_key_cb(GtkWidget *widget, GdkEventKey *event, gpointer
 
 	switch (vf->type)
 	{
-	case FILEVIEW_LIST: ret = vflist_press_key_cb(widget, event, data); break;
-	case FILEVIEW_ICON: ret = vficon_press_key_cb(widget, event, data); break;
+	case FILEVIEW_LIST: ret = vflist_press_key_cb(vf, widget, event); break;
+	case FILEVIEW_ICON: ret = vficon_press_key_cb(vf, widget, event); break;
 	default: ret = FALSE;
 	}
 
@@ -158,8 +164,8 @@ static gboolean vf_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer
 
 	switch (vf->type)
 	{
-	case FILEVIEW_LIST: ret = vflist_press_cb(widget, bevent, data); break;
-	case FILEVIEW_ICON: ret = vficon_press_cb(widget, bevent, data); break;
+	case FILEVIEW_LIST: ret = vflist_press_cb(vf, widget, bevent); break;
+	case FILEVIEW_ICON: ret = vficon_press_cb(vf, widget, bevent); break;
 	default: ret = FALSE;
 	}
 
@@ -173,8 +179,8 @@ static gboolean vf_release_cb(GtkWidget *widget, GdkEventButton *bevent, gpointe
 
 	switch (vf->type)
 	{
-	case FILEVIEW_LIST: ret = vflist_release_cb(widget, bevent, data); break;
-	case FILEVIEW_ICON: ret = vficon_release_cb(widget, bevent, data); break;
+	case FILEVIEW_LIST: ret = vflist_release_cb(vf, widget, bevent); break;
+	case FILEVIEW_ICON: ret = vficon_release_cb(vf, widget, bevent); break;
 	default: ret = FALSE;
 	}
 
@@ -230,6 +236,17 @@ GList *vf_selection_get_list_by_index(ViewFile *vf)
 	return ret;
 }
 
+void vf_selection_foreach(ViewFile *vf, const ViewFile::SelectionCallback &func)
+{
+	if (!vf) return;
+
+	switch (vf->type)
+	{
+	case FILEVIEW_LIST: vflist_selection_foreach(vf, func); break;
+	case FILEVIEW_ICON: vficon_selection_foreach(vf, func); break;
+	}
+}
+
 void vf_select_all(ViewFile *vf)
 {
 	switch (vf->type)
@@ -299,14 +316,113 @@ void vf_selection_to_mark(ViewFile *vf, gint mark, SelectionToMarkMode mode)
  *-----------------------------------------------------------------------------
  */
 
-
-static void vf_dnd_init(ViewFile *vf)
+static gboolean vf_is_selected(ViewFile *vf, FileData *fd)
 {
 	switch (vf->type)
 	{
-	case FILEVIEW_LIST: vflist_dnd_init(vf); break;
-	case FILEVIEW_ICON: vficon_dnd_init(vf); break;
+	case FILEVIEW_LIST: return vflist_is_selected(vf, fd);
+	case FILEVIEW_ICON: return vficon_is_selected(vf, fd);
 	}
+
+	return FALSE;
+}
+
+static void vf_dnd_get(GtkWidget *, GdkDragContext *,
+                       GtkSelectionData *selection_data, guint,
+                       guint, gpointer data)
+{
+	auto *vf = static_cast<ViewFile *>(data);
+
+	if (!vf->click_fd) return;
+
+	GList *list = nullptr;
+
+	if (vf_is_selected(vf, vf->click_fd))
+		{
+		list = vf_selection_get_list(vf);
+		}
+	else
+		{
+		list = g_list_append(nullptr, file_data_ref(vf->click_fd));
+		}
+
+	if (!list) return;
+
+	uri_selection_data_set_uris_from_filelist(selection_data, list);
+	filelist_free(list);
+}
+
+static void vf_dnd_begin(GtkWidget *widget, GdkDragContext *context, gpointer data)
+{
+	auto *vf = static_cast<ViewFile *>(data);
+
+	switch (vf->type)
+	{
+	case FILEVIEW_LIST: vflist_dnd_begin(vf, widget, context); break;
+	case FILEVIEW_ICON: vficon_dnd_begin(vf, widget, context); break;
+	}
+}
+
+static void vf_dnd_end(GtkWidget *, GdkDragContext *context, gpointer data)
+{
+	auto *vf = static_cast<ViewFile *>(data);
+
+	switch (vf->type)
+	{
+	case FILEVIEW_LIST: vflist_dnd_end(vf, context); break;
+	case FILEVIEW_ICON: vficon_dnd_end(vf, context); break;
+	}
+}
+
+static FileData *vf_find_data_by_coord(ViewFile *vf, gint x, gint y, GtkTreeIter *iter)
+{
+	switch (vf->type)
+	{
+	case FILEVIEW_LIST: return vflist_find_data_by_coord(vf, x, y, iter);
+	case FILEVIEW_ICON: return vficon_find_data_by_coord(vf, x, y, iter);
+	}
+
+	return nullptr;
+}
+
+static void vf_drag_data_received(GtkWidget *, GdkDragContext *,
+                                  int x, int y, GtkSelectionData *selection,
+                                  guint info, guint, gpointer data)
+{
+	if (info != TARGET_TEXT_PLAIN) return;
+
+	auto *vf = static_cast<ViewFile *>(data);
+
+	FileData *fd = vf_find_data_by_coord(vf, x, y, nullptr);
+	if (!fd) return;
+
+	/* Add keywords to file */
+	auto str = reinterpret_cast<gchar *>(gtk_selection_data_get_text(selection));
+	GList *kw_list = string_to_keywords_list(str);
+
+	metadata_append_list(fd, KEYWORD_KEY, kw_list);
+
+	g_list_free_full(kw_list, g_free);
+	g_free(str);
+}
+
+static void vf_dnd_init(ViewFile *vf)
+{
+	gtk_drag_source_set(vf->listview, static_cast<GdkModifierType>(GDK_BUTTON1_MASK | GDK_BUTTON2_MASK),
+	                    dnd_file_drag_types.data(), dnd_file_drag_types.size(),
+	                    static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
+	gtk_drag_dest_set(vf->listview, GTK_DEST_DEFAULT_ALL,
+	                  dnd_file_drag_types.data(), dnd_file_drag_types.size(),
+	                  static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
+
+	g_signal_connect(G_OBJECT(vf->listview), "drag_data_get",
+	                 G_CALLBACK(vf_dnd_get), vf);
+	g_signal_connect(G_OBJECT(vf->listview), "drag_begin",
+	                 G_CALLBACK(vf_dnd_begin), vf);
+	g_signal_connect(G_OBJECT(vf->listview), "drag_end",
+	                 G_CALLBACK(vf_dnd_end), vf);
+	g_signal_connect(G_OBJECT(vf->listview), "drag_data_received",
+	                 G_CALLBACK(vf_drag_data_received), vf);
 }
 
 /*
@@ -317,16 +433,14 @@ static void vf_dnd_init(ViewFile *vf)
 
 GList *vf_pop_menu_file_list(ViewFile *vf)
 {
-	GList *ret;
+	if (!vf->click_fd) return nullptr;
 
-	switch (vf->type)
-	{
-	case FILEVIEW_LIST: ret = vflist_pop_menu_file_list(vf); break;
-	case FILEVIEW_ICON: ret = vficon_pop_menu_file_list(vf); break;
-	default: ret = nullptr;
-	}
+	if (vf_is_selected(vf, vf->click_fd))
+		{
+		return vf_selection_get_list(vf);
+		}
 
-	return ret;
+	return vf_selection_get_one(vf, vf->click_fd);
 }
 
 GList *vf_selection_get_one(ViewFile *vf, FileData *fd)
@@ -355,35 +469,33 @@ static void vf_pop_menu_edit_cb(GtkWidget *widget, gpointer data)
 	file_util_start_editor_from_filelist(key, vf_pop_menu_file_list(vf), vf->dir_fd->path, vf->listview);
 }
 
-static void vf_pop_menu_view_cb(GtkWidget *widget, gpointer data)
+static void vf_pop_menu_view_cb(GtkWidget *, gpointer data)
 {
 	auto vf = static_cast<ViewFile *>(data);
 
-	switch (vf->type)
-	{
-	case FILEVIEW_LIST: vflist_pop_menu_view_cb(widget, data); break;
-	case FILEVIEW_ICON: vficon_pop_menu_view_cb(widget, data); break;
-	}
+	if (!vf->click_fd) return;
+
+	if (vf_is_selected(vf, vf->click_fd))
+		{
+		GList *list;
+
+		list = vf_selection_get_list(vf);
+		view_window_new_from_list(list);
+		filelist_free(list);
+		}
+	else
+		{
+		view_window_new(vf->click_fd);
+		}
 }
 
 static void vf_pop_menu_open_archive_cb(GtkWidget *, gpointer data)
 {
 	auto vf = static_cast<ViewFile *>(data);
 	LayoutWindow *lw_new;
-	FileData *fd = nullptr;
 	gchar *dest_dir;
 
-	switch (vf->type)
-	{
-	case FILEVIEW_LIST:
-		fd = (VFLIST(vf)->click_fd);
-		break;
-	case FILEVIEW_ICON:
-		fd = (VFICON(vf)->click_fd);
-		break;
-	}
-
-	dest_dir = open_archive(fd);
+	dest_dir = open_archive(vf->click_fd);
 	if (dest_dir)
 		{
 		lw_new = layout_new_from_default();
@@ -410,14 +522,14 @@ static void vf_pop_menu_move_cb(GtkWidget *, gpointer data)
 	file_util_move(nullptr, vf_pop_menu_file_list(vf), nullptr, vf->listview);
 }
 
-static void vf_pop_menu_rename_cb(GtkWidget *widget, gpointer data)
+static void vf_pop_menu_rename_cb(GtkWidget *, gpointer data)
 {
 	auto vf = static_cast<ViewFile *>(data);
 
 	switch (vf->type)
 	{
-	case FILEVIEW_LIST: vflist_pop_menu_rename_cb(widget, data); break;
-	case FILEVIEW_ICON: vficon_pop_menu_rename_cb(widget, data); break;
+	case FILEVIEW_LIST: vflist_pop_menu_rename_cb(vf); break;
+	case FILEVIEW_ICON: vficon_pop_menu_rename_cb(vf); break;
 	}
 }
 
@@ -441,14 +553,21 @@ static void vf_pop_menu_copy_path_cb(GtkWidget *, gpointer data)
 {
 	auto vf = static_cast<ViewFile *>(data);
 
-	file_util_copy_path_list_to_clipboard(vf_pop_menu_file_list(vf), TRUE);
+	file_util_path_list_to_clipboard(vf_pop_menu_file_list(vf), TRUE, ClipboardAction::COPY);
 }
 
 static void vf_pop_menu_copy_path_unquoted_cb(GtkWidget *, gpointer data)
 {
 	auto vf = static_cast<ViewFile *>(data);
 
-	file_util_copy_path_list_to_clipboard(vf_pop_menu_file_list(vf), FALSE);
+	file_util_path_list_to_clipboard(vf_pop_menu_file_list(vf), FALSE, ClipboardAction::COPY);
+}
+
+static void vf_pop_menu_cut_path_cb(GtkWidget *, gpointer data)
+{
+	auto vf = static_cast<ViewFile *>(data);
+
+	file_util_path_list_to_clipboard(vf_pop_menu_file_list(vf), FALSE, ClipboardAction::CUT);
 }
 
 static void vf_pop_menu_enable_grouping_cb(GtkWidget *, gpointer data)
@@ -580,27 +699,30 @@ static void vf_pop_menu_toggle_view_type_cb(GtkWidget *widget, gpointer data)
 	layout_views_set(vf->layout, vf->layout->options.dir_view_type, new_type);
 }
 
-static void vf_pop_menu_refresh_cb(GtkWidget *widget, gpointer data)
+static void vf_pop_menu_refresh_cb(GtkWidget *, gpointer data)
 {
 	auto vf = static_cast<ViewFile *>(data);
 
 	switch (vf->type)
 	{
-	case FILEVIEW_LIST: vflist_pop_menu_refresh_cb(widget, data); break;
-	case FILEVIEW_ICON: vficon_pop_menu_refresh_cb(widget, data); break;
+	case FILEVIEW_LIST: vflist_pop_menu_refresh_cb(vf); break;
+	case FILEVIEW_ICON: vficon_pop_menu_refresh_cb(vf); break;
 	}
 }
 
-static void vf_popup_destroy_cb(GtkWidget *widget, gpointer data)
+static void vf_popup_destroy_cb(GtkWidget *, gpointer data)
 {
 	auto vf = static_cast<ViewFile *>(data);
 
 	switch (vf->type)
 	{
-	case FILEVIEW_LIST: vflist_popup_destroy_cb(widget, data); break;
-	case FILEVIEW_ICON: vficon_popup_destroy_cb(widget, data); break;
+	case FILEVIEW_LIST: vflist_popup_destroy_cb(vf); break;
+	case FILEVIEW_ICON: vficon_popup_destroy_cb(vf); break;
 	}
 
+	vf->click_fd = nullptr;
+	vf->popup = nullptr;
+
 	filelist_free(vf->editmenu_fd_list);
 	vf->editmenu_fd_list = nullptr;
 }
@@ -624,6 +746,19 @@ static void vf_pop_menu_collections_cb(GtkWidget *widget, gpointer data)
 	filelist_free(selection_list);
 }
 
+static void vf_pop_menu_show_star_rating_cb(GtkWidget *, gpointer data)
+{
+	auto *vf = static_cast<ViewFile *>(data);
+
+	options->show_star_rating = !options->show_star_rating;
+
+	switch (vf->type)
+	{
+	case FILEVIEW_LIST: vflist_pop_menu_show_star_rating_cb(vf); break;
+	case FILEVIEW_ICON: vficon_pop_menu_show_star_rating_cb(vf); break;
+	}
+}
+
 GtkWidget *vf_pop_menu(ViewFile *vf)
 {
 	GtkWidget *menu;
@@ -633,18 +768,13 @@ GtkWidget *vf_pop_menu(ViewFile *vf)
 	gboolean class_archive = FALSE;
 	GtkAccelGroup *accel_group;
 
-	switch (vf->type)
-	{
-	case FILEVIEW_LIST:
-		vflist_color_set(vf, VFLIST(vf)->click_fd, TRUE);
-		active = (VFLIST(vf)->click_fd != nullptr);
-		class_archive = (VFLIST(vf)->click_fd != nullptr && VFLIST(vf)->click_fd->format_class == FORMAT_CLASS_ARCHIVE);
-		break;
-	case FILEVIEW_ICON:
-		active = (VFICON(vf)->click_fd != nullptr);
-		class_archive = (VFICON(vf)->click_fd != nullptr && VFICON(vf)->click_fd->format_class == FORMAT_CLASS_ARCHIVE);
-		break;
-	}
+	if (vf->type == FILEVIEW_LIST)
+		{
+		vflist_color_set(vf, vf->click_fd, TRUE);
+		}
+
+	active = (vf->click_fd != nullptr);
+	class_archive = (vf->click_fd != nullptr && vf->click_fd->format_class == FORMAT_CLASS_ARCHIVE);
 
 	menu = popup_menu_short_lived();
 
@@ -720,18 +850,20 @@ GtkWidget *vf_pop_menu(ViewFile *vf)
 				G_CALLBACK(vf_pop_menu_move_cb), vf);
 	menu_item_add_sensitive(menu, _("_Rename..."), active,
 				G_CALLBACK(vf_pop_menu_rename_cb), vf);
-	menu_item_add_sensitive(menu, _("_Copy path to clipboard"), active,
+	menu_item_add_sensitive(menu, _("_Copy to clipboard"), active,
 				G_CALLBACK(vf_pop_menu_copy_path_cb), vf);
-	menu_item_add_sensitive(menu, _("_Copy path unquoted to clipboard"), active,
+	menu_item_add_sensitive(menu, _("_Copy to clipboard (unquoted)"), active,
 				G_CALLBACK(vf_pop_menu_copy_path_unquoted_cb), vf);
+	menu_item_add_sensitive(menu, _("_Cut to clipboard"), active,
+				G_CALLBACK(vf_pop_menu_cut_path_cb), vf);
 	menu_item_add_divider(menu);
 	menu_item_add_icon_sensitive(menu,
-				options->file_ops.confirm_move_to_trash ? _("Move to Trash...") :
-					_("Move to Trash"), GQ_ICON_DELETE, active,
+				options->file_ops.confirm_move_to_trash ? _("Move selection to Trash...") :
+					_("Move selection to Trash"), GQ_ICON_DELETE, active,
 				G_CALLBACK(vf_pop_menu_move_to_trash_cb), vf);
 	menu_item_add_icon_sensitive(menu,
-				options->file_ops.confirm_delete ? _("_Delete...") :
-					_("_Delete"), GQ_ICON_DELETE_SHRED, active,
+				options->file_ops.confirm_delete ? _("_Delete selection...") :
+					_("_Delete selection"), GQ_ICON_DELETE_SHRED, active,
 				G_CALLBACK(vf_pop_menu_delete_cb), vf);
 	menu_item_add_divider(menu);
 
@@ -769,27 +901,12 @@ GtkWidget *vf_pop_menu(ViewFile *vf)
 
 	switch (vf->type)
 	{
-	case FILEVIEW_LIST:
-		menu_item_add_check(menu, _("Show _thumbnails"), VFLIST(vf)->thumbs_enabled,
-				    G_CALLBACK(vflist_pop_menu_thumbs_cb), vf);
-		break;
-	case FILEVIEW_ICON:
-		menu_item_add_check(menu, _("Show filename _text"), VFICON(vf)->show_text,
-				    G_CALLBACK(vficon_pop_menu_show_names_cb), vf);
-		break;
+	case FILEVIEW_LIST: vflist_pop_menu_add_items(vf, menu); break;
+	case FILEVIEW_ICON: vficon_pop_menu_add_items(vf, menu); break;
 	}
 
-	switch (vf->type)
-	{
-	case FILEVIEW_LIST:
-		menu_item_add_check(menu, _("Show star rating"), options->show_star_rating,
-				    G_CALLBACK(vflist_pop_menu_show_star_rating_cb), vf);
-		break;
-	case FILEVIEW_ICON:
-		menu_item_add_check(menu, _("Show star rating"), options->show_star_rating,
-				    G_CALLBACK(vficon_pop_menu_show_star_rating_cb), vf);
-		break;
-	}
+	menu_item_add_check(menu, _("Show star rating"), options->show_star_rating,
+	                    G_CALLBACK(vf_pop_menu_show_star_rating_cb), vf);
 
 	menu_item_add_icon(menu, _("Re_fresh"), GQ_ICON_REFRESH, G_CALLBACK(vf_pop_menu_refresh_cb), vf);
 
@@ -824,14 +941,14 @@ gboolean vf_set_fd(ViewFile *vf, FileData *dir_fd)
 	return ret;
 }
 
-static void vf_destroy_cb(GtkWidget *widget, gpointer data)
+static void vf_destroy_cb(GtkWidget *, gpointer data)
 {
 	auto vf = static_cast<ViewFile *>(data);
 
 	switch (vf->type)
 	{
-	case FILEVIEW_LIST: vflist_destroy_cb(widget, data); break;
-	case FILEVIEW_ICON: vficon_destroy_cb(widget, data); break;
+	case FILEVIEW_LIST: vflist_destroy_cb(vf); break;
+	case FILEVIEW_ICON: vficon_destroy_cb(vf); break;
 	}
 
 	if (vf->popup)
@@ -885,7 +1002,7 @@ static void vf_marks_tooltip_ok_cb(GenericDialog *gd, gpointer data)
 	generic_dialog_close(gd);
 }
 
-void vf_marks_filter_on_icon_press(GtkEntry *, GtkEntryIconPosition, GdkEvent *, gpointer userdata)
+static void vf_marks_filter_on_icon_press(GtkEntry *, GtkEntryIconPosition, GdkEvent *, gpointer userdata)
 {
 	auto mte = static_cast<MarksTextEntry *>(userdata);
 
@@ -954,9 +1071,6 @@ static void vf_file_filter_save_cb(GtkWidget *, gpointer data)
 	auto vf = static_cast<ViewFile *>(data);
 	gchar *entry_text;
 	gchar *remove_text = nullptr;
-	gchar *index_text = nullptr;
-	gboolean text_found = FALSE;
-	gint i;
 
 	entry_text = g_strdup(gq_gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(vf->file_filter.combo)))));
 
@@ -977,15 +1091,13 @@ static void vf_file_filter_save_cb(GtkWidget *, gpointer data)
 		{
 		if (entry_text[0] != '\0')
 			{
-			for (i = 0; i < vf->file_filter.count; i++)
-				{
-				if (index_text)
-					{
-					g_free(index_text);
-					}
-				gtk_combo_box_set_active(GTK_COMBO_BOX(vf->file_filter.combo), i);
-				index_text = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(vf->file_filter.combo));
+			gboolean text_found = FALSE;
 
+			for (gint i = 0; i < vf->file_filter.count; i++)
+				{
+				gtk_combo_box_set_active(GTK_COMBO_BOX(vf->file_filter.combo), i);
+
+				g_autofree gchar *index_text = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(vf->file_filter.combo));
 				if (g_strcmp0(index_text, entry_text) == 0)
 					{
 					text_found = TRUE;
@@ -993,7 +1105,6 @@ static void vf_file_filter_save_cb(GtkWidget *, gpointer data)
 					}
 				}
 
-			g_free(index_text);
 			if (!text_found)
 				{
 				history_list_add_to_key("file_filter", entry_text, 10);
@@ -1090,7 +1201,6 @@ static gboolean vf_file_filter_class_set_all_cb(GtkWidget *widget, gpointer data
 	auto vf = static_cast<ViewFile *>(data);
 	GtkWidget *parent;
 	GList *children;
-	GtkWidget *child;
 	gint i;
 	gboolean state;
 
@@ -1111,15 +1221,13 @@ static gboolean vf_file_filter_class_set_all_cb(GtkWidget *widget, gpointer data
 	i = 0;
 	parent = gtk_widget_get_parent(widget);
 	children = gtk_container_get_children(GTK_CONTAINER(parent));
-	while (children)
+	for (GList *work = children; work; work = work->next)
 		{
-		child = static_cast<GtkWidget *>(children->data);
 		if (i < FILE_FORMAT_CLASSES)
 			{
-			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(child), state);
+			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(work->data), state);
 			}
 		i++;
-		children = children->next;
 		}
 	g_list_free(children);
 	vf_refresh(vf);
@@ -1287,8 +1395,8 @@ ViewFile *vf_new(FileViewType type, FileData *dir_fd)
 
 	switch (type)
 	{
-	case FILEVIEW_LIST: vf = vflist_new(vf, dir_fd); break;
-	case FILEVIEW_ICON: vf = vficon_new(vf, dir_fd); break;
+	case FILEVIEW_LIST: vf = vflist_new(vf); break;
+	case FILEVIEW_ICON: vf = vficon_new(vf); break;
 	}
 
 	vf_dnd_init(vf);
@@ -1339,8 +1447,8 @@ static gdouble vf_thumb_progress(ViewFile *vf)
 
 	switch (vf->type)
 	{
-	case FILEVIEW_LIST: vflist_thumb_progress_count(vf->list, &count, &done); break;
-	case FILEVIEW_ICON: vficon_thumb_progress_count(vf->list, &count, &done); break;
+	case FILEVIEW_LIST: vflist_thumb_progress_count(vf->list, count, done); break;
+	case FILEVIEW_ICON: vficon_thumb_progress_count(vf->list, count, done); break;
 	}
 
 	DEBUG_1("thumb progress: %d of %d", done, count);
@@ -1353,10 +1461,10 @@ static gdouble vf_read_metadata_in_idle_progress(ViewFile *vf)
 	gint done = 0;
 
 	switch (vf->type)
-		{
-		case FILEVIEW_LIST: vflist_read_metadata_progress_count(vf->list, &count, &done); break;
-		case FILEVIEW_ICON: vficon_read_metadata_progress_count(vf->list, &count, &done); break;
-		}
+	{
+	case FILEVIEW_LIST: vflist_read_metadata_progress_count(vf->list, count, done); break;
+	case FILEVIEW_ICON: vficon_read_metadata_progress_count(vf->list, count, done); break;
+	}
 
 	return static_cast<gdouble>(done) / count;
 }
@@ -1612,21 +1720,6 @@ void vf_marks_set(ViewFile *vf, gboolean enable)
 
 	vf_refresh_idle(vf);
 }
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-function"
-void vf_star_rating_set_unused(ViewFile *vf, gboolean enable)
-{
-	if (options->show_star_rating == enable) return;
-	options->show_star_rating = enable;
-
-	switch (vf->type)
-		{
-		case FILEVIEW_LIST: vflist_star_rating_set(vf, enable); break;
-		case FILEVIEW_ICON: vficon_star_rating_set(vf, enable); break;
-		}
-	vf_refresh_idle(vf);
-}
-#pragma GCC diagnostic pop
 
 guint vf_marks_get_filter(ViewFile *vf)
 {
